@@ -1,4 +1,4 @@
-/*  $Id: prolog_clause.pl,v 1.9 2006/08/29 13:47:51 jan Exp $
+/*  $Id: prolog_clause.pl,v 1.13 2006/10/22 15:36:58 jan Exp $
 
     Part of SWI-Prolog
 
@@ -122,7 +122,8 @@ unify_args(I, Arity, T1, T2) :-
 alternate_syntax(prolog, _,    true,
 			       true).
 alternate_syntax(pce_class, M, pce_expansion:push_compile_operators(M),
-			       pce_expansion:pop_compile_operators).
+			       pce_expansion:pop_compile_operators) :-
+	current_prolog_flag(xpce, true).
 alternate_syntax(system, _,    style_check(+dollar),
 			       style_check(-dollar)).
 
@@ -249,6 +250,15 @@ unify_clause(:->(Head, Body), (PlHead :- PlBody), TermPos0, TermPos) :- !,
 					% XPCE get-methods
 unify_clause(:<-(Head, Body), (PlHead :- PlBody), TermPos0, TermPos) :- !,
 	pce_method_clause(Head, Body, PlHead, PlBody, TermPos0, TermPos).
+					% Unit test clauses
+unify_clause((TH :- Body),
+	     (_:'unit body'(_, _) :- !, Body),
+	     TP0, TP) :-
+	(   TH = test(_,_)
+	;   TH = test(_)
+	), !,
+	TP0 = term_position(F,T,FF,FT,[HP,BP]),
+	TP  = term_position(F,T,FF,FT,[HP,term_position(0,0,0,0,[FF-FT,BP])]).
 					% module:head :- body
 unify_clause((Head :- Read),
 	     (Head :- _M:Compiled), TermPos0, TermPos) :-
@@ -258,6 +268,20 @@ unify_clause((Head :- Read),
 				 [ PH,
 				   term_position(0,0,0,0,[0-0,PB])
 				 ]).
+unify_clause(Read, Compiled1, TermPos0, TermPos) :-
+	Read = (_ --> List, _),
+	is_list(List),
+	expand_term(Read, Compiled2),
+	Compiled2 = (DH :- _),
+	functor(DH, _, Arity),
+	DArg is Arity - 1,
+	arg(DArg, DH, List),
+	nonvar(List),
+	TermPos0 = term_position(F,T,FF,FT,[ HP,
+					     term_position(_,_,_,_,[_,BP])
+					   ]), !,
+	TermPos1 = term_position(F,T,FF,FT,[ HP, BP ]),
+	match_module(Compiled2, Compiled1, TermPos1, TermPos).
 					% general term-expansion
 unify_clause(Read, Compiled1, TermPos0, TermPos) :-
 	expand_term(Read, Compiled2),
@@ -301,7 +325,7 @@ a --> { x, y, z }.
 
 %%	ubody(+Read, +Decompiled, +TermPosRead, -TermPosForDecompiled)
 %
-%	@param Read		Clause read after term-expansion
+%	@param Read		Clause read _after_ expand_term/2
 %	@param Decompiled	Decompiled clause
 %	@param TermPosRead	Sub-term positions of source
 
@@ -322,16 +346,17 @@ ubody(B0, B,
       term_position(F,T,F,T,[A])) :- !,
 	ubody(B0, B, A0, A).
 ubody(C0, C, P0, P) :-
+	nonvar(C0), nonvar(C),
 	C0 = (_,_), C = (_,_), !,
-      conj(C0, P0, GL, PL),
-      mkconj(C, P, GL, PL).
+	conj(C0, P0, GL, PL),
+	mkconj(C, P, GL, PL).
 ubody(X0, X,
       term_position(F,T,FF,TT,PA0),
       term_position(F,T,FF,TT,PA)) :-
-      meta(X0), !,
-      X0 =.. [_|A0],
-      X  =.. [_|A],
-      ubody_list(A0, A, PA0, PA).
+	meta(X0), !,
+	X0 =.. [_|A0],
+	X  =.. [_|A],
+	ubody_list(A0, A, PA0, PA).
 	
 ubody_list([], [], [], []).
 ubody_list([G0|T0], [G|T], [PA0|PAT0], [PA|PAT]) :-
@@ -356,7 +381,9 @@ conj(A, P, [A|TG], TG, [P|TP], TP).
 mkconj(Goal, Pos, GoalList, PosList) :-
 	mkconj(Goal, Pos, GoalList, [], PosList, []).
 
-mkconj((A,B), term_position(0,0,0,0,[PA,PB]), GL, TG, PL, TP) :- !,
+mkconj(Conj, term_position(0,0,0,0,[PA,PB]), GL, TG, PL, TP) :-
+	nonvar(Conj),
+	Conj = (A,B), !,
 	mkconj(A, PA, GL, TGA, PL, TPA),
 	mkconj(B, PB, TGA, TG, TPA, TP).
 mkconj(A0, P0, [A|TG], TG, [P|TP], TP) :-
