@@ -1,11 +1,11 @@
-/*  $Id: nonblockio.c,v 1.10 2005/12/27 20:32:29 jan Exp $
+/*  $Id: nonblockio.c,v 1.15 2007/01/13 17:14:46 jan Exp $
 
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2007, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -123,12 +123,13 @@ leave the details to this function.
 #include <sys/types.h>
 #include <assert.h>
 #include <string.h>
-#ifdef WIN32
+#ifdef __WINDOWS__
 #include <malloc.h>
 #endif
 
-#ifndef WIN32
+#ifndef __WINDOWS__
 #define closesocket(n) close((n))	/* same on Unix */
+typedef int SOCKET;
 #endif
 
 #ifndef SD_SEND
@@ -173,11 +174,11 @@ static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 typedef struct _plsocket
 { int		    magic;		/* SOCK_MAGIC */
   struct _plsocket *next;		/* next in list */
-  int		    socket;		/* The OS socket */
+  SOCKET	    socket;		/* The OS socket */
   int		    flags;		/* Misc flags */
   IOSTREAM *	    input;		/* input stream */
   IOSTREAM *	    output;		/* output stream */
-#ifdef WIN32
+#ifdef __WINDOWS__
   nbio_request	    request;		/* our request */
   DWORD		    thread;		/* waiting thread */
   DWORD		    error;		/* error while executing request */
@@ -187,30 +188,30 @@ typedef struct _plsocket
   { struct
     { struct sockaddr_in addr;		/* accepted address */
       int addrlen;			/* address length */
-      int slave;			/* descriptor of slave */
+      SOCKET slave;			/* descriptor of slave */
     } accept;
     struct
     { struct sockaddr_in addr;		/* accepted address */
-      int addrlen;			/* address length */
+      size_t addrlen;			/* address length */
     } connect;
     struct
     { int bytes;			/* byte count */
       char *buffer;			/* the buffer */
-      int size;				/* buffer size */
+      size_t size;			/* buffer size */
     } read;
     struct
     { int bytes;			/* byte count */
       char *buffer;			/* the buffer */
       int written;
-      int size;				/* buffer size */
+      size_t size;			/* buffer size */
     } write;
   } rdata;
 #endif
 } plsocket;
 
-static plsocket *lookupSocket(int socket);
-#ifdef WIN32
-static plsocket   *lookupExistingSocket(int socket);
+static plsocket *lookupSocket(SOCKET socket);
+#ifdef __WINDOWS__
+static plsocket   *lookupExistingSocket(SOCKET socket);
 static const char *WinSockError(unsigned long eno);
 #endif
 
@@ -250,7 +251,7 @@ nbio_debug(int level)
 		 *	  COMPATIBILITY		*
 		 *******************************/
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 static UINT WM_SOCKET	= WM_APP+20;
 static UINT WM_REQUEST  = WM_APP+21;
 static UINT WM_READY	= WM_APP+22;
@@ -509,7 +510,7 @@ doRequest(plsocket *s)
 
 	if ( connect(s->socket,
 		     (struct sockaddr*)&s->rdata.connect.addr,
-		     s->rdata.connect.addrlen) )
+		     (int)s->rdata.connect.addrlen) )
 	{ s->error = WSAGetLastError();
 	  
 	  switch(s->error)
@@ -571,7 +572,7 @@ doRequest(plsocket *s)
 
 	s->rdata.read.bytes = recv(s->socket,
 				   s->rdata.read.buffer,
-				   s->rdata.read.size,
+				   (int)s->rdata.read.size,
 				   0);
 	if ( s->rdata.read.bytes < 0 )
 	{ s->error = WSAGetLastError();
@@ -598,7 +599,7 @@ doRequest(plsocket *s)
 	DEBUG(2, Sdprintf("send() %d bytes\n", s->rdata.write.size));
 	n = send(s->socket,
 		 s->rdata.write.buffer + s->rdata.write.written,
-		 s->rdata.write.size - s->rdata.write.written,
+		 (int)(s->rdata.write.size - s->rdata.write.written),
 		 0);
 	DEBUG(2, Sdprintf("Wrote %d bytes\n", n));
 	if ( n < 0 )
@@ -622,8 +623,8 @@ doRequest(plsocket *s)
 }
 
 
-static int WINAPI
-socket_wnd_proc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
+static LRESULT WINAPI
+socket_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 { if ( message == WM_REQUEST )
   { plsocket **s = (plsocket **)lParam;
     int i, n = (int)wParam;
@@ -778,7 +779,7 @@ startSocketThread()
   return TRUE;
 }
 
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 wait_socket() is the Unix way  to  wait   for  input  on  the socket. By
@@ -800,7 +801,7 @@ wait_socket(plsocket *s, int fd)
       tv.tv_sec = 0;
       tv.tv_usec = 250000;
       
-      select(fd+1, &rfds, NULL, NULL, &tv) != 0;
+      select(fd+1, &rfds, NULL, NULL, &tv);
       return TRUE;
     } else
     { return PL_dispatch(fd, PL_DISPATCH_WAIT);
@@ -839,7 +840,7 @@ nbio_select(int n,
 { return select(n, readfds, writefds, exceptfds, timeout); 
 }
 
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 
 		 /*******************************
@@ -856,9 +857,9 @@ static atom_t ATOM_loopback;
 static plsocket *sockets;
 static int initialised = FALSE;		/* Windows only */
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 static plsocket *
-lookupExistingSocket(int socket)
+lookupExistingSocket(SOCKET socket)
 { plsocket *p;
 
   LOCK();
@@ -875,7 +876,7 @@ lookupExistingSocket(int socket)
 
 
 static plsocket *
-lookupSocket(int socket)
+lookupSocket(SOCKET socket)
 { plsocket *p;
 
   LOCK();
@@ -895,7 +896,7 @@ lookupSocket(int socket)
   p->socket = socket;
   p->flags  = SOCK_DISPATCH;		/* by default, dispatch */
   p->magic  = SOCK_MAGIC;
-#ifdef WIN32
+#ifdef __WINDOWS__
   p->w32_flags = 0;
   p->request   = REQ_NONE;
 #endif
@@ -948,7 +949,7 @@ freeSocket(int socket)
 		 *	      ERRORS		*
 		 *******************************/
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 The code in BILLY_GETS_BETTER is, according to various documents the
 right code, but it doesn't work, so we do it by hand.
@@ -1051,7 +1052,7 @@ WinSockError(unsigned long error)
 }
 
 #endif /*BILLY_GETS_BETTER*/
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 #ifdef HAVE_H_ERRNO
 typedef struct
@@ -1098,7 +1099,7 @@ nbio_error(int code, nbio_error_map mapid)
   }
 
   {
-#ifdef WIN32
+#ifdef __WINDOWS__
   msg = WinSockError(code);
 #else
 
@@ -1117,7 +1118,7 @@ nbio_error(int code, nbio_error_map mapid)
   } else
 #endif
     msg = strerror(code);
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
   PL_unify_term(except,
 		CompoundArg("error", 2),
@@ -1149,7 +1150,7 @@ nbio_init(const char *module)
   ATOM_broadcast  = PL_new_atom("broadcast");
   ATOM_loopback   = PL_new_atom("loopback");
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 { WSADATA WSAData;
 
   hinstance = GetModuleHandle(module);
@@ -1167,7 +1168,7 @@ nbio_init(const char *module)
   }
   startSocketThread();
 }
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
   UNLOCK();
   return TRUE;
@@ -1178,7 +1179,7 @@ int
 nbio_cleanup(void)
 { if ( initialised )
   {
-#ifdef WIN32
+#ifdef __WINDOWS__
     WSACleanup();
 #endif
   }
@@ -1195,7 +1196,7 @@ socket(-Socket)
 
 int
 nbio_socket(int domain, int type, int protocol)
-{ int sock;
+{ SOCKET sock;
   plsocket *s;
 	
   assert(initialised);
@@ -1210,12 +1211,12 @@ nbio_socket(int domain, int type, int protocol)
     return -1;
   }
 
-#ifdef WIN32
+#ifdef __WINDOWS__
   WSAAsyncSelect(sock, State()->hwnd, WM_SOCKET,
 		 FD_READ|FD_WRITE|FD_ACCEPT|FD_CONNECT|FD_CLOSE);
 #endif
 
-  return sock;
+  return (int)sock;
 }
 
 
@@ -1238,7 +1239,7 @@ nbio_closesocket(int socket)
     }
   } else
   {
-#ifdef WIN32
+#ifdef __WINDOWS__
     if ( true(s, SOCK_CONNECT) )
       shutdown(socket, SD_SEND);
 #endif
@@ -1465,7 +1466,12 @@ nbio_unify_ip4(term_t Ip, unsigned long hip)
 
 int
 nbio_bind(int socket, struct sockaddr *my_addr, size_t addrlen)
-{ if ( bind(socket, my_addr, addrlen) )
+{
+#ifdef __WINDOWS__
+  if ( bind(socket, my_addr, (int)addrlen) )
+#else
+  if ( bind(socket, my_addr, addrlen) )
+#endif
   { nbio_error(errno, TCP_ERRNO);
     return -1;
   }
@@ -1484,8 +1490,8 @@ nbio_connect(int socket,
        
   s = lookupSocket(socket);
   
-#ifdef WIN32
-  if ( connect(socket, serv_addr, addrlen) )
+#ifdef __WINDOWS__
+  if ( connect(socket, serv_addr, (int)addrlen) )
   { s->error = WSAGetLastError();
 
     if ( s->error == WSAEWOULDBLOCK )
@@ -1500,7 +1506,7 @@ nbio_connect(int socket,
       return -1;
     }
   }
-#else /*!WIN32*/
+#else /*!__WINDOWS__*/
   for(;;)
   { if ( connect(socket, serv_addr, addrlen) )
     { if ( need_retry(errno) )
@@ -1523,13 +1529,16 @@ nbio_connect(int socket,
 
 int
 nbio_accept(int master, struct sockaddr *addr, socklen_t *addrlen)
-{ int slave;
+{ SOCKET slave;
   plsocket *m;
 	
   m = lookupSocket(master);
 
-#ifdef WIN32
-  slave = accept(master, addr, addrlen);
+#ifdef __WINDOWS__
+{ int alen;
+  slave = accept(master, addr, &alen);
+  *addrlen = alen;
+}
 
   if ( slave == SOCKET_ERROR )
   { m->error = WSAGetLastError();
@@ -1550,7 +1559,7 @@ nbio_accept(int master, struct sockaddr *addr, socklen_t *addrlen)
     }
   }
 
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 
   for(;;)
   { if ( !wait_socket(m, master) )
@@ -1578,9 +1587,9 @@ nbio_accept(int master, struct sockaddr *addr, socklen_t *addrlen)
       nbio_setopt(slave, TCP_NONBLOCK);
   }
 
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
   
-  return slave;
+  return (int)slave;
 }
 
 
@@ -1601,10 +1610,10 @@ nbio_listen(int socket, int backlog)
 		 *	  IO-STREAM STUFF	*
 		 *******************************/
 
-#define fdFromHandle(p) ((int)((long)(p)))
+#define fdFromHandle(p) ((int)((intptr_t)(p)))
 
-int
-nbio_read(int socket, char *buf, int bufSize)
+ssize_t
+nbio_read(int socket, char *buf, size_t bufSize)
 { plsocket *s = lookupSocket(socket);
   int n;
 
@@ -1614,9 +1623,9 @@ nbio_read(int socket, char *buf, int bufSize)
     return -1;
   }
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 
-  n = recv(socket, buf, bufSize, 0);
+  n = recv(socket, buf, (int)bufSize, 0);
   if ( n < 0 )
   { if ( WSAGetLastError() == WSAEWOULDBLOCK )
     { s->rdata.read.buffer = buf;
@@ -1633,7 +1642,7 @@ nbio_read(int socket, char *buf, int bufSize)
       errno = EIO;			/* TBD: map s->error to POSIX errno */
   }
 
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 
   for(;;)
   { if ( !wait_socket(s, socket) )
@@ -1654,7 +1663,7 @@ nbio_read(int socket, char *buf, int bufSize)
     break;
   }
 
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
   if ( n == 0 )
     set(s, SOCK_EOF_SEEN);
@@ -1662,16 +1671,17 @@ nbio_read(int socket, char *buf, int bufSize)
   return n;
 }
 
-int
-nbio_write(int socket, char *buf, int bufSize)
-{ int len = bufSize;
+
+ssize_t
+nbio_write(int socket, char *buf, size_t bufSize)
+{ size_t len = bufSize;
   char *str = buf;
 
-#ifdef WIN32
-  while( len > 0 )
-  { int n;
+#ifdef __WINDOWS__
+  while( len != 0 )
+  { ssize_t n;
 
-    n = send(socket, str, len, 0);
+    n = send(socket, str, (int)len, 0);
     if ( n < 0 )
     { int error = WSAGetLastError();
 
@@ -1700,7 +1710,7 @@ nbio_write(int socket, char *buf, int bufSize)
       return -1;
   }
 
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 
   while( len > 0 )
   { int n;
@@ -1721,7 +1731,7 @@ nbio_write(int socket, char *buf, int bufSize)
     str += n;
   }
 
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
   return bufSize;
 }
@@ -1735,7 +1745,7 @@ nbio_close_input(int socket)
   DEBUG(2, Sdprintf("[%d]: nbio_close_input(%d)\n", PL_thread_self(), socket));
   s->input = NULL;
   s->flags &= ~SOCK_INSTREAM;
-#ifdef WIN32
+#ifdef __WINDOWS__
   if ( false(s, SOCK_LISTEN) )
   { if ( shutdown(socket, SD_RECEIVE) == SOCKET_ERROR )
       Sdprintf("shutdown(%d, SD_RECEIVE) failed: %s\n",
@@ -1761,13 +1771,13 @@ nbio_close_output(int socket)
   if ( s->output )
   { s->output = NULL;
     s->flags &= ~SOCK_OUTSTREAM;
-#if WIN32
+#if __WINDOWS__
     if ( shutdown(socket, SD_SEND) == SOCKET_ERROR )
     {
 #ifdef O_DEBUG
       if ( debugging )
       { const char *msg;
-#ifdef WIN32
+#ifdef __WINDOWS__
 	msg = WinSockError(WSAGetLastError());
 #else
         msg = strerror(errno);
