@@ -1,4 +1,4 @@
-/*  $Id: db4pl.c,v 1.15 2004/09/21 14:22:23 jan Exp $
+/*  $Id: db4pl.c,v 1.19 2007/01/23 19:16:30 jan Exp $
 
     Part of SWI-Prolog
 
@@ -82,6 +82,14 @@ static int     opt_transactions;	/* are transactions enabled? */
 
 #define mkfunctor(n, a) PL_new_functor(PL_new_atom(n), a)
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+TBD: Thread-safe version
+
+	- Deal with transactions and threads
+	- Safe register/unregister
+
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 #ifdef WIN32
 #define NOSIG(code) { code; }
 #else
@@ -320,7 +328,7 @@ get_long_ex(term_t t, long *v)
 static int
 get_size_ex(term_t t, long *v)
 { if ( get_long_ex(t, v) )
-  { if ( v >= 0 )
+  { if ( *v >= 0 )
       return TRUE;
 
     return pl_error(ERR_DOMAIN, "not_less_than_zero", t);
@@ -1120,8 +1128,12 @@ typedef struct _server_info
 
 
 static void
-pl_db_error(const char *prefix, char *buffer)
-{ Sdprintf("%s%s\n", prefix, buffer);
+#if DB_VERSION_MAJOR >= 4 && DB_VERSION_MINOR >= 3
+pl_db_error(const DB_ENV *dbenv, const char *prefix, const char *msg)
+#else
+pl_db_error(const char *prefix, char *msg)
+#endif
+{ Sdprintf("%s%s\n", prefix, msg);
 }
 
 
@@ -1331,6 +1343,7 @@ static foreign_t
 pl_db_atom(term_t handle, term_t atom, term_t id)
 { dbh *db;
   atom_t a;
+  long lv;
   atomid_t aid;
     
   if ( !get_db(handle, &db) )
@@ -1341,8 +1354,10 @@ pl_db_atom(term_t handle, term_t atom, term_t id)
       return FALSE;
 
     return PL_unify_integer(id, aid);
-  } else if ( PL_get_long(id, &aid) )
-  { if ( !pl_atom_from_db(db, aid, &a) )
+  } else if ( PL_get_long(id, &lv) )
+  { aid = (atomid_t)lv;
+
+    if ( !pl_atom_from_db(db, aid, &a) )
       return FALSE;
 
     return PL_unify_atom(atom, a);
