@@ -1,4 +1,4 @@
-/*  $Id: latex2html.pl,v 1.66 2006/08/29 10:25:57 jan Exp $
+/*  $Id: latex2html.pl,v 1.68 2007/01/08 09:27:03 jan Exp $
 
     Part of SWI-Prolog
 
@@ -1499,7 +1499,7 @@ translate_section(Level, -, TeXTitle,
 	  Tell,
 	  Header,
 	  #h(Level, #label(RefName,
-			   [ #span('sec-nr', Tag),
+			   [ #span('sec-nr', Tag), ' ',
 			     #span('sec-title', Title)
 			   ]))
 	], NodeFile) :- !,
@@ -1603,7 +1603,7 @@ tableofcontents(TagPrefix, #div(toc, Sections)) :-
 
 section_html(TagPrefix,
 	     [ #div(Class, #lref(sec, Ref,
-				 [ #span('sec-nr', Tag),
+				 [ #span('sec-nr', Tag), ' ',
 				   #span('sec-title', Title)
 				 ]))
 	     ]) :-
