@@ -1,4 +1,4 @@
-/*  $Id: tex.c,v 1.7 2004/09/21 14:22:24 jan Exp $
+/*  $Id: tex.c,v 1.9 2007/01/24 14:29:16 jan Exp $
 
     Part of SWI-Prolog
 
@@ -1397,7 +1397,10 @@ parseTeX(Input fd, CallBack func, void *ctx)
 	} while(!wbreak(c));
 	*s = EOS;
 	t.type = TOK_WORD;
-	t.value.string = buf;
+	if ( streq(buf, "[]") )
+	  t.value.string = "\\[]";
+	else
+	  t.value.string = buf;
 	(*func)(&t, ctx);
 
 	break;
@@ -1698,7 +1701,7 @@ put_token(Token t, void *ctx)
       pendingblank = (pp->last_type == TOK_SPACE);
       outputBlank(pp);			/* as space! */
       if ( pendingblank &&		/* no blanks in input: concatenate! */
-	   strlen(t->value.string) + pp->line_pos > pp->right_margin )
+	   (int)strlen(t->value.string) + pp->line_pos > pp->right_margin )
 	nl(pp);
       output(pp, "%s", t->value.string);
       break;
@@ -2528,7 +2531,7 @@ put_html_token(Token t, void *ctx)
       pendingblank = (pp->last_type == TOK_SPACE);
       outputBlank(pp);			/* as space! */
       if ( pendingblank &&		/* no blanks in input: concatenate! */
-	   strlen(t->value.string) + pp->line_pos > pp->right_margin )
+	   (int)strlen(t->value.string) + pp->line_pos > pp->right_margin )
 	nl(pp);
       output_html(pp, t->value.string);
       break;
