@@ -1,4 +1,4 @@
-/* $Id: porter_stem.c,v 1.7 2006/06/20 15:04:32 jan Exp $
+/* $Id: porter_stem.c,v 1.11 2007/01/23 14:29:09 jan Exp $
 
    This is the Porter stemming algorithm, coded up in ANSI C by the
    author. It may be be regarded as canonical, in that it follows the
@@ -355,11 +355,11 @@ static int unaccent(const char *in, size_t len, char *out, size_t size);
 static foreign_t
 pl_stem(term_t t_in, term_t t_stem)
 { char *word;
-  unsigned int len, end;
+  size_t len, end;
   char *f, *t, *s, *ew;
   char buf[1024];
   char plain[1024];
-  int l;
+  long l;
 
   if ( !PL_get_nchars(t_in, &len, &word, CVT_ALL|CVT_EXCEPTION) )
   { if ( PL_is_number(t_in) )
@@ -371,7 +371,7 @@ pl_stem(term_t t_in, term_t t_stem)
   for(f=word, t=s; f<ew; )
     *t++ = tolower(*f++);
 
-  if ( (l=unaccent(s, t-s, plain, sizeof(plain))) < (int)sizeof(plain) )
+  if ( (l=unaccent(s, t-s, plain, sizeof(plain))) < (long)sizeof(plain) )
   { if ( l >= 0 )
     { if ( s != buf )
 	PL_free(s);
@@ -385,7 +385,7 @@ pl_stem(term_t t_in, term_t t_stem)
     s = s2;
   }
 
-  end = stem(s, 0, len - 1);
+  end = stem(s, 0, (int)(len - 1));
   s[end + 1] = '\0';
   
   PL_unify_atom_chars(t_stem, s);
@@ -450,9 +450,9 @@ unaccent(const char *in, size_t len, char *out, size_t size)
     *to = '\0';
 
   if ( changes == 0 )
-    return out-to;			/* no change: negative */
+    return (int)(out-to);		/* no change: negative */
 
-  return to-out;
+  return (int)(to-out);
 }
 
 
@@ -460,8 +460,8 @@ static foreign_t
 pl_unaccent(term_t from, term_t to)
 { char buf[1024];
   char *f;
-  unsigned int fl;
   int len;
+  size_t fl;
 
   if ( !PL_get_nchars(from, &fl, &f, CVT_ALL|CVT_EXCEPTION) )
     return FALSE;
@@ -576,7 +576,7 @@ unify_token(const char *s, size_t len, toktype type, void *closure)
 
     switch(type)
     { case TOK_INT:
-#ifdef WIN32
+#ifdef __WINDOWS__
       { long val = strtol(s, &ep, 10);
 
 	return PL_unify_integer(l->head, val);
@@ -605,7 +605,7 @@ unify_token(const char *s, size_t len, toktype type, void *closure)
 static foreign_t
 pl_tokenize(term_t text, term_t tokens)
 { char *s;
-  unsigned int len;
+  size_t len;
   list l;
 
   if ( !PL_get_nchars(text, &len, &s, CVT_ALL|CVT_EXCEPTION) )
@@ -665,7 +665,7 @@ unify_stem(const char *s, size_t len, toktype type, void *closure)
 static foreign_t
 pl_atom_to_stem_list(term_t text, term_t stems)
 { char *s;
-  unsigned int len;
+  size_t len;
   list l;
 
   if ( !PL_get_nchars(text, &len, &s, CVT_ALL) )
