@@ -1,4 +1,4 @@
-/*  $Id: odbc.c,v 1.66 2006/08/15 10:20:39 jan Exp $
+/*  $Id: odbc.c,v 1.74 2007/01/19 16:11:32 jan Exp $
 
     Part of SWI-Prolog
 
@@ -34,10 +34,14 @@ This module is based on pl_odbc.{c,pl},   a  read-only ODBC interface by
 Stefano  De  Giorgi  (s.degiorgi@tin.it).
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 #include <windows.h>
 #define HAVE_MKTIME 1
 #define HAVE_GMTIME 1
+#if (_MSC_VER >= 1400)
+#define HAVE_SQLLEN 1			/* should depend on SDK version? */
+#define HAVE_SQLULEN 1
+#endif
 #else
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -252,7 +256,7 @@ typedef struct
   SQLSMALLINT  NumCols;			/* # columns */
   SQLSMALLINT  NumParams;		/* # parameters */
   functor_t    db_row;			/* Functor for row */
-  unsigned int sqllen;			/* length of statement */
+  SQLINTEGER   sqllen;			/* length of statement */
   char        *sqltext;			/* statement text */
   unsigned     flags;			/* general flags */
   nulldef     *null;			/* Prolog null value */
@@ -284,7 +288,7 @@ static struct
 #define CTX_GOT_QLEN	0x0400		/* got SQL_MAX_QUALIFIER_NAME_LEN */
 #define CTX_NOAUTO	0x0800		/* fetch by hand */
 
-#define FND_SIZE(n)	((int)&((findall*)NULL)->codes[n])
+#define FND_SIZE(n)	((size_t)&((findall*)NULL)->codes[n])
 
 #define true(s, f)	((s)->flags & (f))
 #define false(s, f)	!true(s, f)
@@ -315,16 +319,28 @@ odbc_report(HENV henv, HDBC hdbc, HSTMT hstmt, RETCODE rc)
   SQLINTEGER native;			/* was DWORD */
   SQLCHAR message[SQL_MAX_MESSAGE_LENGTH];
   SWORD   msglen;
+  RETCODE rce;
   term_t  msg = PL_new_term_ref();
 
-  if ( SQLError(henv, hdbc, hstmt, state, &native, message,
-		sizeof(message), &msglen) == SQL_SUCCESS )
-  { PL_unify_term(msg, PL_FUNCTOR, FUNCTOR_odbc3,
-			 PL_CHARS,   state,
-		         PL_INTEGER, (long)native,
-		         PL_NCHARS,  msglen, message);
-  } else if ( rc != SQL_ERROR )
-    return TRUE;
+  switch ( (rce=SQLError(henv, hdbc, hstmt, state, &native, message,
+			 sizeof(message), &msglen)) )
+  { case SQL_NO_DATA_FOUND:
+    case SQL_SUCCESS_WITH_INFO:
+      if ( rc != SQL_ERROR )
+	return TRUE;
+      /*FALLTHROUGH*/
+    case SQL_SUCCESS:
+      PL_unify_term(msg, PL_FUNCTOR, FUNCTOR_odbc3,
+			   PL_CHARS,   state,
+		           PL_INTEGER, (long)native,
+		           PL_NCHARS,  msglen, message);
+      break;
+    case SQL_INVALID_HANDLE:
+      return PL_warning("ODBC INTERNAL ERROR: Invalid handle in error\n");
+    default:
+      if ( rc != SQL_ERROR )
+	return TRUE;
+  }
 
   switch(rc)
   { case SQL_SUCCESS_WITH_INFO:
@@ -564,7 +580,7 @@ int formatted_string(+Fmt-[Arg...], *len, char **out)
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static int
-formatted_string(term_t in, int *len, char **out)
+formatted_string(term_t in, size_t *len, char **out)
 { term_t av = PL_new_term_refs(3);
   static predicate_t format;
   IOSTREAM *fd = Sopenmem(out, len, "w");
@@ -813,7 +829,7 @@ compile_arg(compile_info *info, term_t t)
 	  for(i=0; i<sizeof(double)/sizeof(code); i++)
 	    ADDCODE(info, v.ascode[i]);
 	} else				/* string */
-	{ unsigned int len;
+	{ size_t len;
 	  char *s, *cp;
 
 	  PL_get_string_chars(t, &s, &len);
@@ -1159,8 +1175,8 @@ static foreign_t
 pl_odbc_connect(term_t tdsource, term_t cid, term_t options)
 {  atom_t dsn;
    const char *dsource;			/* odbc data source */
-   char *uid = "";			/* user id */
-   char *pwd = "";			/* password */
+   char *uid = NULL;			/* user id */
+   char *pwd = NULL;			/* password */
    atom_t alias = 0;			/* alias-name */
    atom_t open = 0;			/* open next connection */
    RETCODE rc;				/* result code for ODBC functions */
@@ -1229,9 +1245,12 @@ pl_odbc_connect(term_t tdsource, term_t cid, term_t options)
    dsource = PL_atom_chars(dsn);
 
    if ( !henv )
-     SQLAllocEnv(&henv);		/* Allocate an environment handle */
+   { if ( (rc=SQLAllocEnv(&henv)) != SQL_SUCCESS )
+       return PL_warning("Could not initialise SQL environment");
+   }
 
-   SQLAllocConnect(henv, &hdbc);	/* Allocate a connection handle */
+   if ( (rc=SQLAllocConnect(henv, &hdbc)) != SQL_SUCCESS )
+     return odbc_report(henv, NULL, NULL, rc);
    /* Connect to a data source. */
    rc = SQLConnect(hdbc, (SQLCHAR *)dsource, SQL_NTS,
                          (SQLCHAR *)uid,     SQL_NTS,
@@ -1271,7 +1290,7 @@ odbc_disconnect(+Connection)
 
 #define TRY_CN(cn, action) \
 	{ RETCODE rc = action; \
-	  if ( rc == SQL_ERROR ) \
+	  if ( rc != SQL_SUCCESS ) \
 	    return odbc_report(henv, cn->hdbc, NULL, rc); \
 	}
 
@@ -1570,6 +1589,7 @@ odbc_end_transaction(term_t conn, term_t action)
 static context *
 new_context(connection *cn)
 { context *ctxt = odbc_malloc(sizeof(context));
+  RETCODE rc;
 
   if ( !ctxt )
     return NULL;
@@ -1580,7 +1600,10 @@ new_context(connection *cn)
   ctxt->null = cn->null;
   ctxt->flags = cn->flags;
   ctxt->max_nogetdata = cn->max_nogetdata;
-  SQLAllocStmt(cn->hdbc, &ctxt->hstmt);
+  if ( (rc=SQLAllocStmt(cn->hdbc, &ctxt->hstmt)) != SQL_SUCCESS )
+  { odbc_report(henv, cn->hdbc, NULL, rc);
+    return NULL;
+  }
   statistics.statements_created++;
 
   return ctxt;
@@ -1766,21 +1789,21 @@ code to synchronise this problem.
 
 static int
 get_sql_text(context *ctxt, term_t tquery)
-{ unsigned int qlen;
+{ size_t qlen;
   char *q;
 
   if ( PL_is_functor(tquery, FUNCTOR_minus2) )
   { qlen = 0;
     q = NULL;
 
-    if ( !formatted_string(tquery, (int*)&qlen, &q) )
+    if ( !formatted_string(tquery, &qlen, &q) )
       return FALSE;
     ctxt->sqltext = q;
-    ctxt->sqllen = qlen;
+    ctxt->sqllen = (SQLINTEGER)qlen;
     set(ctxt, CTX_SQLMALLOCED);
   } else if ( PL_get_nchars(tquery, &qlen, &q, CVT_ATOM|CVT_STRING|BUF_MALLOC))
   { ctxt->sqltext = q;
-    ctxt->sqllen = qlen;
+    ctxt->sqllen = (SQLINTEGER)qlen;
     set(ctxt, CTX_SQLMALLOCED);
   } else
     return type_error(tquery, "atom_or_format");
@@ -2178,7 +2201,8 @@ pl_odbc_query(term_t dsn, term_t tquery, term_t trow, term_t options,
       if ( !get_connection(dsn, &cn) )
 	return FALSE;
          
-      ctxt = new_context(cn);
+      if ( !(ctxt = new_context(cn)) )
+	return FALSE;
       if ( !get_sql_text(ctxt, tquery) )
       { free_context(ctxt);
 	return FALSE;
@@ -2220,7 +2244,8 @@ odbc_tables(term_t dsn, term_t row, control_t handle)
       if ( !get_connection(dsn, &cn) )
 	return FALSE;
 
-      ctxt = new_context(cn);
+      if ( !(ctxt = new_context(cn)) )
+	return FALSE;
       ctxt->null = NULL;		/* use default $null$ */
       set(ctxt, CTX_TABLES);
       TRY(ctxt,
@@ -2249,7 +2274,7 @@ pl_odbc_column(term_t dsn, term_t db, term_t row, control_t handle)
   { case PL_FIRST_CALL:
     { connection *cn;
       context *ctxt;
-      unsigned int len;
+      size_t len;
       char *s;
 
       if ( !PL_get_nchars(db, &len, &s, CVT_ATOM|CVT_STRING) )
@@ -2257,7 +2282,8 @@ pl_odbc_column(term_t dsn, term_t db, term_t row, control_t handle)
 
       if ( !get_connection(dsn, &cn) )
 	return FALSE;
-      ctxt = new_context(cn);
+      if ( !(ctxt = new_context(cn)) )
+	return FALSE;
       ctxt->null = NULL;		/* use default $null$ */
       set(ctxt, CTX_COLUMNS);
       TRY(ctxt,
@@ -2304,7 +2330,8 @@ odbc_types(term_t dsn, term_t sqltype, term_t row, control_t handle)
 
       if ( !get_connection(dsn, &cn) )
 	return FALSE;
-      ctxt = new_context(cn);
+      if ( !(ctxt = new_context(cn)) )
+	return FALSE;
       ctxt->null = NULL;		/* use default $null$ */
       TRY(ctxt,
 	  SQLGetTypeInfo(ctxt->hstmt, type),
@@ -2522,7 +2549,7 @@ declare_parameters(context *ctxt, term_t parms)
   { atom_t name;
     int arity;
     SWORD sqlType, fNullable;
-    UDWORD cbColDef = 0;
+    SQLULEN cbColDef = 0;
     SWORD plType = SQL_PL_DEFAULT;
     SQLLEN *vlenptr = NULL;		/* pointer to length */
 
@@ -2580,7 +2607,7 @@ declare_parameters(context *ctxt, term_t parms)
 	  }
 	  params->length_ind = cbColDef;
 	} else				/* unknown, use SQLPutData() */
-	{ params->ptr_value = (PTR)pn;
+	{ params->ptr_value = (PTR)(long)pn;
 	  params->len_value = SQL_LEN_DATA_AT_EXEC(0);
 	  DEBUG(2, Sdprintf("Using SQLPutData() for column %d\n", pn));
 	}
@@ -2640,7 +2667,8 @@ odbc_prepare(term_t dsn, term_t sql, term_t parms, term_t qid, term_t options)
   if ( !get_connection(dsn, &cn) )
     return FALSE;
 
-  ctxt = new_context(cn);
+  if ( !(ctxt = new_context(cn)) )
+    return FALSE;
   if ( !get_sql_text(ctxt, sql) )
   { free_context(ctxt);
     return FALSE;
@@ -2798,7 +2826,7 @@ try_null(context *ctxt, parameter *prm, term_t val, const char *expected)
 
 
 static int
-get_parameter_text(term_t t, parameter *prm, unsigned int *len, char **s)
+get_parameter_text(term_t t, parameter *prm, size_t *len, char **s)
 { unsigned int flags = CVT_ATOM|CVT_STRING;
   const char *expected = "text";
 
@@ -2861,7 +2889,7 @@ bind_parameters(context *ctxt, term_t parms)
       case SQL_C_CHAR:
       case SQL_C_BINARY:
       { SQLLEN len;
-	unsigned int l;
+	size_t l;
 	char *s;
 
 					/* check for NULL */
@@ -2941,8 +2969,8 @@ odbc_execute(term_t qid, term_t args, term_t row, control_t handle)
       { PTR token;
 
 	if ( (ctxt->rc = SQLParamData(ctxt->hstmt, &token)) == SQL_NEED_DATA )
-	{ parameter *p = &ctxt->params[(int)token - 1];
-	  unsigned int len;
+	{ parameter *p = &ctxt->params[(long)token - 1];
+	  size_t len;
 	  char *s;
 
 	  if ( is_sql_null(p->put_data, ctxt->null) )
@@ -3368,7 +3396,7 @@ as the wrong number of pad bytes for the first part of the data.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static void
-put_chars(term_t val, int plTypeID, int len, const char *chars)
+put_chars(term_t val, int plTypeID, size_t len, const char *chars)
 { switch( plTypeID )
   { case SQL_PL_DEFAULT:
     case SQL_PL_ATOM:
@@ -3406,7 +3434,7 @@ pl_put_column(context *c, int nth, term_t col)
   if ( !p->ptr_value )			/* use SQLGetData() */
   { char buf[256];
     char *data = buf;
-    SDWORD len;
+    SQLLEN len;
 
     DEBUG(2, Sdprintf("Fetching value for column %d using SQLGetData()\n",
 		      nth+1));
@@ -3425,8 +3453,8 @@ pl_put_column(context *c, int nth, term_t col)
 			  "Please report to bugs@swi-prolog.org");
       } else if ( len >= (SDWORD)sizeof(buf) )
       { int pad = p->cTypeID == SQL_C_CHAR ? 1 : 0;
-	int todo = len-sizeof(buf)+2*pad;
-	SDWORD len2;
+	size_t todo = len-sizeof(buf)+2*pad;
+	SQLLEN len2;
 	char *ep;
 	int part = 2;
 
