@@ -1,4 +1,4 @@
-/*  $Id: doc_http.pl,v 1.28 2006/08/31 09:44:00 jan Exp $
+/*  $Id: doc_http.pl,v 1.39 2006/11/13 13:25:14 jan Exp $
 
     Part of SWI-Prolog
 
@@ -33,20 +33,26 @@
 	  [ doc_server/1,		% ?Port
 	    doc_server/2,		% ?Port, +Options
 	    doc_browser/0,
-	    doc_browser/1		% +What
+	    doc_browser/1,		% +What
+	    doc_server_root/1		% -Root
 	  ]).
 :- use_module(library(pldoc)).
 :- use_module(library('http/thread_httpd')).
 :- use_module(library('http/http_parameters')).
 :- use_module(library('http/html_write')).
 :- use_module(library('http/mimetype')).
-:- use_module(library('debug')).
-:- use_module(library('lists')).
+:- use_module(library(debug)).
+:- use_module(library(lists)).
+:- use_module(library(url)).
+:- use_module(library(socket)).
+:- use_module(library(option)).
 :- use_module(pldoc(doc_process)).
+:- use_module(pldoc(doc_htmlsrc)).
 :- use_module(pldoc(doc_html)).
 :- use_module(pldoc(doc_index)).
 :- use_module(pldoc(doc_search)).
 :- use_module(pldoc(doc_man)).
+:- use_module(pldoc(doc_wiki)).
 
 /** <module> Documentation server
 
@@ -75,6 +81,13 @@ _after_ library(pldoc) has been loaded.
 %
 %		* deny(HostOrIP)
 %		See allow(HostOrIP).
+%		
+%		* edit(Bool)
+%		Allow editing from localhost connections? Default:
+%		=true=.
+%		
+%		* root(Path)
+%		Path of the root.  Default is /
 %	
 %	The predicate doc_server/1 is defined as below, which provides a
 %	good default for development.
@@ -93,14 +106,17 @@ _after_ library(pldoc) has been loaded.
 doc_server(Port) :-
 	doc_server(Port,
 		   [ workers(1),
-		     allow(localhost)
+		     allow(localhost),
+		     allow(ip(127,0,0,1)) % Windows ip-->host often fails
 		   ]).
 
 doc_server(Port, _) :-
 	doc_current_server(Port), !.
 doc_server(Port, Options) :-
 	prepare_editor,
-	auth_options(Options, ServerOptions),
+	auth_options(Options, Options1),
+	root_option(Options1, Options2),
+	edit_option(Options2, ServerOptions),
 	append(ServerOptions,		% Put provides options first,
 	       [ port(Port),		% so they override our defaults
 		 timeout(60),
@@ -129,11 +145,13 @@ doc_browser(Spec) :-
 	format(string(URL), 'http://localhost:~w~w', [Port, Request]),
 	www_open_url(URL).
 
-browser_url([], '') :- !.
+browser_url([], Root) :- !,
+	doc_server_root(Root).
 browser_url(Name/Arity, URL) :- !,
+	doc_server_root(Root),
 	format(string(S), '~q/~w', [Name, Arity]),
 	www_form_encode(S, Enc),
-	format(string(URL), '/man?predicate=~w', [Enc]).
+	format(string(URL), '~wman?predicate=~w', [Root, Enc]).
 
 %%	prepare_editor
 %
@@ -154,14 +172,15 @@ doc_reply(Request) :-
 
 do_reply(Request) :-
 	memberchk(peer(Peer), Request),
+	select(path(Path0), Request, Request1),
 	(   allowed_peer(Peer)
-	->  memberchk(path(Path), Request),
-	    debug(pldoc, 'HTTP ~q', [Path]),
-	    (	reply(Path, Request)
+	->  debug(pldoc, 'HTTP ~q', [Path0]),
+	    (	normalise_path(Path0, Path),
+		reply(Path, [path(Path)|Request1])
 	    ->	true
-	    ;	throw(http_reply(not_found(Path)))
+	    ;	throw(http_reply(not_found(Path0)))
 	    )		      
-	;   throw(http_reply(forbidden(/)))
+	;   throw(http_reply(forbidden(Path0)))
 	).
 
 
@@ -199,55 +218,108 @@ auth_option(allow(From)) :-
 auth_option(deny(From)) :-
 	assert(deny_from(From)).
 
-%%	match_ip(+Spec, +Peer) is semidet.
+%%	match_peer(:RuleSet, +PlusMin, +Peer) is semidet.
 %
-%	True if Peer is covered by Spec.
+%	True if Peer is covered by the   ruleset RuleSet. Peer is a term
+%	ip(A,B,C,D). RuleSet is a predicate with   one  argument that is
+%	either  a  partial  ip  term,  a    hostname  or  a  domainname.
+%	Domainnames start with a '.'.
+%	
+%	@param PlusMin	Positive/negative test.  If IP->Host fails, a
+%		       	positive test fails, while a negative succeeds.
+%			I.e. deny('.com') succeeds for unknown IP
+%			addresses.
 
-match_ip(X, X).
-match_ip(Name, IP) :-
-	is_ip(IP),
-	tcp_host_to_address(Host0, IP),
-	downcase_atom(Host0, Host),
-	(   Name == Host
-	->  true
-	;   sub_atom(Name, 0, _, _, '.'),
-	    sub_atom(Host, _, _, 0, Name)
+match_peer(Spec, _, Peer) :-
+	call(Spec, Peer), !.
+match_peer(Spec, PM, Peer) :-
+	(   call(Spec, HOrDom), atom(HOrDom)
+	->  (   catch(tcp_host_to_address(Host, Peer), E, true),
+	        var(E)
+	    ->	call(Spec, HostOrDomain),
+		atom(HostOrDomain),
+		(   sub_atom(HostOrDomain, 0, _, _, '.')
+		->  sub_atom(Host, _, _, 0, HostOrDomain)
+		;   HostOrDomain == Host
+		)
+	    ;   PM == (+)
+	    ->	!, fail
+	    ;	true
+	    )
 	).
-
-is_ip(ip(A,B,C,D)) :-
-	integer(A),
-	integer(B),
-	integer(C),
-	integer(D).
-
-%%	deny(+Peer) is semidet.
-%%	allow(+Peer) is semidet.
-
-deny(Peer) :-
-	deny_from(From),
-	match_ip(From, Peer), !.
-
-allow(Peer) :-
-	allow_from(From),
-	match_ip(From, Peer), !.
+	
+%%	allowed_peer(+Peer) is semidet.
+%
+%	True if Peer is allowed according to the rules.
 
 allowed_peer(Peer) :-
-	deny(Peer), !,
-	allow(Peer).
+	match_peer(deny_from, -, Peer), !,
+	match_peer(allow_from, +, Peer).
 allowed_peer(Peer) :-
 	allow_from(_), !,
-	allow(Peer).
+	match_peer(allow_from, +, Peer).
 allowed_peer(_).
 
 
-%%	local(+Request) is semidet.
+:- dynamic
+	can_edit/1.
+
+%%	allow_edit(+Request) is semidet.
 %
-%	True if the request comes from localhost.
+%	True if, given Request, we allow editing sources.
 
-local(Request) :-
+allow_edit(_) :-
+	can_edit(false), !, 
+	fail.
+allow_edit(Request) :-
 	memberchk(peer(Peer), Request),
-	match_ip(localhost, Peer).
+	match_peer(localhost, +, Peer).
 
+localhost(ip(127,0,0,1)).
+localhost(localhost).
+
+edit_option(Options0, Options) :-
+	select_option(edit(Bool), Options0, Options), !,
+	assert(can_edit(Bool)).
+edit_option(Options, Options).
+
+
+		 /*******************************
+		 *	       ROOT		*
+		 *******************************/
+
+:- dynamic
+	root/1.
+
+root_option(Options0, Options) :-
+	select_option(root(Root), Options0, Options), !,
+	assert(root(Root)).
+root_option(Options, Options).
+	
+%%	doc_server_root(?Root) is semidet.
+%
+%	True if Root is the root of our documentation server. Default is
+%	=|/|=. Can be set with the =root= option of doc_server/1.
+
+doc_server_root(Root) :-
+	(   root(Root0)
+	->  Root = Root0
+	;   Root = /
+	).
+
+
+%%	normalise_path(+Path0, -NormalPath) is det.
+%
+%	Make paths relative to / if it was moved.
+
+normalise_path(Path0, Path) :-
+	doc_server_root(Root),
+	(   doc_server_root(/)
+	->  Path = Path0
+	;   atom_concat(Root, Rest, Path0)
+	->  atom_concat(/, Rest, Path)
+	).
+	
 
 		 /*******************************
 		 *	    USER REPLIES	*
@@ -295,7 +367,7 @@ reply('/file', Request) :-
 %	Start SWI-Prolog editor on file
 
 reply('/edit', Request) :-
-	local(Request), !,
+	allow_edit(Request), !,
 	http_parameters(Request,
 			[ file(File,     [optional(true)]),
 			  module(Module, [optional(true)]),
@@ -321,16 +393,16 @@ reply('/edit', _Request) :-
 
 %	/directory?dir=Dir
 %	
-%	Give index of directory
+%	Give index of directory.  Mapped to /doc/Dir/index.html.
 
 reply('/directory', Request) :-
 	http_parameters(Request,
 			[ dir(Dir, [])
 			]),
 	(   allowed_directory(Dir)
-	->  edit_options(Request, EditOptions),
-	    format('Content-type: text/html~n~n'),
-	    doc_for_dir(Dir, current_output, EditOptions)
+	->  format(string(IndexFile), '~w/index.html', [Dir]),
+	    doc_file_href(IndexFile, HREF),
+	    throw(http_reply(moved(HREF)))
 	;   throw(http_reply(forbidden(Dir)))
 	).
 
@@ -345,6 +417,17 @@ allowed_directory(Dir) :-
 	working_directory(Dir, Dir).
 
 
+%%	allowed_file(+File) is semidet.
+%
+%	True if we are allowed to serve File.  Currently means the
+%	directory must be allowed.
+
+allowed_file(File) :-
+	absolute_file_name(File, Canonical),
+	file_directory_name(Canonical, Dir),
+	allowed_directory(Dir).
+
+
 %	/doc/Path
 %	
 %	Reply documentation of file. Path is   the  absolute path of the
@@ -365,8 +448,9 @@ reply(ReqPath, Request) :-
 	).
 
 documentation(Path, _Request) :-
-	file_base_name(Path, 'pldoc.css'), !,
-	reply_file(pldoc('pldoc.css')).
+	file_base_name(Path, Base),
+	file(_, Base), !,			% serve pldoc.css, etc.
+	reply_file(pldoc(Base)).
 documentation(Path, Request) :-
 	Index = '/index.html',
 	sub_atom(Path, _, _, 0, Index), 
@@ -378,24 +462,43 @@ documentation(Path, Request) :-
 	    doc_for_dir(Dir, current_output, EditOptions)
 	;   throw(http_reply(forbidden(Dir)))
 	).
+documentation(File, _Request) :-
+	(   file_name_extension(_, txt, File)
+	;   file_base_name(File, Base),
+	    autolink_file(Base, wiki)
+	),
+	(   allowed_file(File)
+	->  true
+	;   throw(http_reply(forbidden(File)))
+	),
+	format('Content-type: text/html~n~n'),
+	doc_for_wiki_file(File, current_output, []).
 documentation(Path, Request) :-
 	http_parameters(Request,
 			[ public_only(Public),
-			  reload(Reload)
+			  reload(Reload),
+			  source(Source)
 			],
 			[ attribute_declarations(param)
 			]),
 	pl_file(Path, File),
+	(   allowed_file(File)
+	->  true
+	;   throw(http_reply(forbidden(File)))
+	),
 	(   Reload == true
-	->  load_files(File, [if(changed)])
+	->  load_files(File, [if(changed), imports([])])
 	;   true
 	),
 	edit_options(Request, EditOptions),
 	format('Content-type: text/html~n~n'),
-	doc_for_file(File, current_output,
-		     [ public_only(Public)
-		     | EditOptions
-		     ]).
+	(   Source == true
+	->  source_to_html(File, stream(current_output), [])
+	;   doc_for_file(File, current_output,
+			 [ public_only(Public)
+			 | EditOptions
+			 ])
+	).
 
 
 %%	edit_options(+Request, -Options) is det.
@@ -404,7 +507,7 @@ documentation(Path, Request) :-
 %	localhost.
 
 edit_options(Request, [edit(true)]) :-
-	local(Request), !.
+	allow_edit(Request), !.
 edit_options(_, []).
 
 
@@ -440,14 +543,16 @@ reply(Path, _Request) :-
 	file(Path, LocalFile),
 	reply_file(pldoc(LocalFile)).
 
-file('/pldoc.css',   'pldoc.css').
-file('/pldoc.js',    'pldoc.js').
-file('/edit.gif',    'edit.gif').
-file('/up.gif',      'up.gif').
-file('/zoomin.gif',  'zoomin.gif').
-file('/zoomout.gif', 'zoomout.gif').
-file('/reload.gif',  'reload.gif').
-file('/favicon.ico', 'favicon.ico').
+file('/pldoc.css',     'pldoc.css').
+file('/pllisting.css', 'pllisting.css').
+file('/pldoc.js',      'pldoc.js').
+file('/edit.gif',      'edit.gif').
+file('/up.gif',	       'up.gif').
+file('/source.gif',    'source.gif').
+file('/zoomin.gif',    'zoomin.gif').
+file('/zoomout.gif',   'zoomout.gif').
+file('/reload.gif',    'reload.gif').
+file('/favicon.ico',   'favicon.ico').
 
 
 %	/man?predicate=PI
@@ -556,6 +661,10 @@ param(public_only,
 	default(true)
       ]).
 param(reload,
+      [ oneof([true,false]),
+	default(false)
+      ]).
+param(source,
       [ oneof([true,false]),
 	default(false)
       ]).
@@ -569,7 +678,7 @@ param(reload,
 	prolog:message/3.
 
 prolog:message(pldoc(server_started(Port))) -->
-	[ 'Started Prolog Documentatiuon server at port ~w'-[Port], nl,
+	[ 'Started Prolog Documentation server at port ~w'-[Port], nl,
 	  'You may access the server at http://localhost:~w/'-[Port]
 	].
 
