@@ -103,7 +103,7 @@ summary:
 #include <string.h>
 #include <assert.h>
 #include "avl.h"		/* public types for avl trees */
-#ifdef WIN32
+#ifdef __WINDOWS__
 #define inline __inline
 #endif
 
@@ -116,10 +116,10 @@ summary:
 
 /* ckalloc(size) -- allocate space; check for success */
 static void *
-ckalloc(int size)
+ckalloc(size_t size)
 { void *ptr;
 
-  if ( !(ptr = malloc((size_t) size)) )
+  if ( !(ptr = malloc(size)) )
   { fprintf(stderr, "Unable to allocate storage.");
     exit(1);
   }
@@ -433,7 +433,7 @@ memswap(void *p1, void *p2, size_t size)
   char buf[256];
 
   while(size > 0)
-  { int bytes = (size > sizeof(buf) ? sizeof(buf) : size);
+  { size_t bytes = (size > sizeof(buf) ? sizeof(buf) : size);
 
     memcpy(buf, s1, bytes);
     memcpy(s1, s2, bytes);
@@ -683,6 +683,9 @@ void *
 avlfindfirst(AVL_TREE tree, void *data, avl_enum *e)
 { AVLtree node = tree->root;
 
+  if ( !node )
+    return NULL;
+
   e->tree    = tree;
   e->current = 0;
   
@@ -788,7 +791,7 @@ avlinit(AVL_TREE tree,
   tree->destroy = destroy;
   tree->alloc = alloc;
   tree->free = free;
-  tree->isize = isize;
+  tree->isize = (int)isize;
   tree->client_data = cdata;
   tree->count = 0;
 
