@@ -1,11 +1,11 @@
-/*  $Id: rdf_db.h,v 1.45 2006/03/14 09:53:04 jan Exp $
+/*  $Id: rdf_db.h,v 1.48 2007/01/13 21:23:51 jan Exp $
 
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2006, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -82,6 +82,13 @@ typedef struct list
 } list;
 
 
+typedef struct bitmatrix
+{ int width;
+  int heigth;
+  int bits[1];
+} bitmatrix;
+
+
 #define DISTINCT_DIRECT 0		/* for ->distinct_subjects, etc */
 #define DISTINCT_SUB    1
 
@@ -92,6 +99,8 @@ typedef struct predicate
   struct predicate *root;		/* Root of property tree */
   struct predicate *next;		/* next in hash-table */
   struct predicate *oldroot;		/* from previous run */
+  bitmatrix	   *reachable;		/* Reachability matrix on root */
+  int		    label;		/* Numeric label */
   int		    visited;		/* loop detection */
   struct predicate *inverse_of;		/* my inverse predicate */
   unsigned 	    transitive : 1;	/* P(a,b)&P(b,c) --> P(a,c) */
@@ -122,7 +131,7 @@ typedef struct literal
     double	real;
     struct
     { record_t  record;
-      unsigned int len;
+      size_t	len;
     } term;				/* external record */
   } value;
   atom_t	type_or_lang;		/* Type or language for literals */
@@ -208,7 +217,7 @@ typedef struct rdf_db
   int		gc_count;		/* # garbage collections */
   double	rehash_time;		/* time spent in rehash */
   double	gc_time;		/* time spent in GC */
-  long		core;			/* core in use */
+  size_t	core;			/* core in use */
   predicate   **pred_table;		/* Hash-table of predicates */
   int		pred_table_size;	/* #entries in the table */
   int		pred_count;		/* #predicates */
