@@ -1,11 +1,11 @@
-/*  $Id: rdf_edit.pl,v 1.44 2006/08/31 09:50:20 jan Exp $
+/*  $Id: rdf_edit.pl,v 1.47 2007/01/26 09:39:54 jan Exp $
 
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
     E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2007, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -76,6 +76,7 @@
 :- use_module(library(broadcast)).
 :- use_module(library(lists)).
 :- use_module(library(debug)).
+:- use_module(library(url)).
 
 :- meta_predicate
 	rdfe_transaction(:),
@@ -90,12 +91,19 @@
 	unmodified_md5/2,		% Path, MD5
 	snapshot_file/1.		% File
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+/** <module> RDF edit layer
 This library provides a number of functions on top of the rdf_db module:
 
     * Broadcast modifications
     * Provide undo/redo
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+@tbd	This library must be rewritten using rdf_monitor/3.  This allows
+	using edit layer without having to choose between rdf_ and rdfe_
+	predicates.
+
+@see	rdf_persistency.pl provides reliable persistency, but without
+	changes boardcasting and undo/redo.
+*/
 
 :- rdf_meta
 	rdfe_assert(r,r,o),
@@ -187,7 +195,8 @@ delete(Subject) :-
 		 *	   FILE HANDLING	*
 		 *******************************/
 
-%%	rdfe_load(+File)
+%%	rdfe_load(+File) is det.
+%%	rdfe_load(+File, +Options) is det.
 %	
 %	Load an RDF file and record this action including version information
 %	to facilitate reliable reload.
@@ -385,7 +394,7 @@ rdfe_transaction(Goal, Name) :-
 	    fail
 	).
 	    
-%	rdfe_begin_transaction/0
+%%	rdfe_begin_transaction
 %	
 %	Start a transaction.  This is followed by either rdfe_end_transaction
 %	or rdfe_rollback.  Transactions may be nested.
@@ -497,7 +506,7 @@ action(subject(_)).
 action(predicate(_)).
 action(object(_)).
 
-%	rdfe_undo
+%%	rdfe_undo
 %	
 %	Undo a (toplevel) transaction. More calls do further undo. The
 %	`Undone' actions are re-added to the undo log, so the user can
@@ -538,7 +547,7 @@ undo_previous(TID, Undone) :-
 last_transaction(TID) :-
 	undo_log([TID|_], _, _, _, _), !.
 
-%	rdfe_redo
+%%	rdfe_redo
 %	
 %	Start a redo-session
 
@@ -630,40 +639,53 @@ user_transaction_member(Update, Subject, Predicate, Object,
 %	
 %	Set properties on the file.  Options is one of
 %	
-%%		access(ro/rw)
-%%		default(all/fallback)
+%		* access(ro/rw)
+%		* default(all/fallback)
 
-rdfe_set_file_property(File0, access(Access)) :- !,
-	absolute_file_name(File0, File),
-	retractall(rdf_source_permission(File, _)),
-	assert(rdf_source_permission(File, Access)),
-	broadcast(rdf_file_property(File, access(Access))).
-rdfe_set_file_property(File0, default(Type)) :-
-	absolute_file_name(File0, File),
-	rdfe_set_file_property(File, access(rw)), % must be writeable
+rdfe_set_file_property(File, access(Access)) :- !,
+	to_url(File, URL),
+	retractall(rdf_source_permission(URL, _)),
+	assert(rdf_source_permission(URL, Access)),
+	broadcast(rdf_file_property(URL, access(Access))).
+rdfe_set_file_property(File, default(Type)) :-
+	to_url(File, URL),
+	rdfe_set_file_property(URL, access(rw)), % must be writeable
 	retractall(rdf_current_default_file(_,_)),
-	assert(rdf_current_default_file(File, Type)),
-	broadcast(rdf_file_property(File, default(Type))).
+	assert(rdf_current_default_file(URL, Type)),
+	broadcast(rdf_file_property(URL, default(Type))).
 
 
-%%	rdfe_get_file_property(?File, ?Option)
+%%	rdfe_get_file_property(+FileOrURL, ?Option).
+%%	rdfe_get_file_property(-URL, ?Option).
 %	
-%	Fetch file properties set with rdf_set_file_property/2.
+%	Fetch file properties set with rdfe_set_file_property/2.
 
-rdfe_get_file_property(File, access(Access)) :-
-	rdf_source(File),
-	(   rdf_source_permission(File, Access0)
+rdfe_get_file_property(FileOrURL, access(Access)) :-
+	(   ground(FileOrURL)
+	->  to_url(FileOrURL, URL)
+	;   rdf_source(URL),
+	    FileOrURL = URL
+	),
+	(   rdf_source_permission(URL, Access0)
 	->  Access0 = Access
-	;   access_file(File, write)
-	->  assert(rdf_source_permission(File, rw)),
+	;   access_file(URL, write)
+	->  assert(rdf_source_permission(URL, rw)),
 	    Access = rw
-	;   assert(rdf_source_permission(File, ro)),
+	;   assert(rdf_source_permission(URL, ro)),
 	    Access = ro
 	).
-rdfe_get_file_property(File, default(Default)) :-
-	(   rdf_current_default_file(File, Default)
+rdfe_get_file_property(FileOrURL, default(Default)) :-
+	ground(FileOrURL),
+	to_url(FileOrURL, URL),
+	(   rdf_current_default_file(URL, Default)
 	->  true
-	;   File = user,
+	;   FileOrURL = user,
+	    Default = fallback
+	).
+rdfe_get_file_property(URL, default(Default)) :-
+	(   rdf_current_default_file(URL, Default)
+	->  true
+	;   URL = user,
 	    Default = fallback
 	).
 
@@ -680,21 +702,41 @@ check_file_protection(Error) :-
 	).
 
 
+%%	to_url(+Spec, -URL) is det.
+%
+%	Convert a specification into a URL.
+
+to_url(URL, URL) :-
+	atom(URL),
+	sub_atom(URL, B, _, _, '://'),
+	sub_atom(URL, 0, B, _, Protocol),
+	url_protocol(Protocol), !.
+to_url(File, URL) :-
+	file_name_to_url(File, URL).
+
+
+url_protocol(file).
+url_protocol(http).
+url_protocol(https).
+url_protocol(ftp).
+url_protocol(ftps).
+
+
 		 /*******************************
 		 *	     MODIFIED		*
 		 *******************************/
 
-%%	rdfe_is_modified(?File)
+%%	rdfe_is_modified(?Source)
 %	
-%	True if facts have been added, deleted or updated that have File
-%	as `payload'.
+%	True if facts have been added, deleted or updated that have
+%	Source as `payload'.
 
-rdfe_is_modified(File) :-
-	rdf_source(File),
-	rdf_md5(File, MD5),
-	(   unmodified_md5(File, UnmodifiedMD5)
+rdfe_is_modified(Source) :-
+	rdf_source(Source),
+	rdf_md5(Source, MD5),
+	(   unmodified_md5(Source, UnmodifiedMD5)
 	->  true
-	;   rdf_db:rdf_source(File, _Time, _Triples, UnmodifiedMD5)
+	;   rdf_db:rdf_source(Source, _Time, _Triples, UnmodifiedMD5)
 	),
 	UnmodifiedMD5 \== MD5.
 
@@ -703,6 +745,10 @@ rdfe_clear_modified :-
 	forall(rdf_source(File),
 	       rdfe_clear_modified(File)).
 
+%%	rdfe_clear_modified(+DB) is det.
+%
+%	Consider the current state of DB as _unmodified_.
+
 rdfe_clear_modified(File) :-
 	atom(File),
 	retractall(unmodified_md5(File, _)),
@@ -733,7 +779,7 @@ rdfe_set_watermark(Name) :-
 		 *	       RESET		*
 		 *******************************/
 
-%	rdfe_reset/0
+%%	rdfe_reset
 %	
 %	Clear database, undo, namespaces and journalling info.
 
@@ -744,6 +790,10 @@ rdfe_reset :-
 	rdf_reset_db,
 	broadcast(rdf_reset).
 
+%%	rdfe_reset_journal
+%
+%	If a journal is open, close it using rdfe_close_journal/0
+
 rdfe_reset_journal :-
 	(   rdfe_current_journal(_)
 	->  rdfe_close_journal
@@ -770,6 +820,20 @@ rdfe_reset_undo :-
 
 journal_version(1).
 
+%%	rdfe_open_journal(+File, +Mode) is det.
+%
+%	Open a journal writing to File in Mode.  Mode is one of 
+%	
+%		* read
+%		Open and replay the journal
+%		
+%		* write
+%		Delete current journal and create a fresh one
+%		
+%		* append
+%		Read and replay the existing journal and append new
+%		modifications to the File.
+
 rdfe_open_journal(_, _) :-		% already open
 	journal(_, _, _), !.
 rdfe_open_journal(File, read) :- !,
@@ -835,6 +899,11 @@ journal_comment(resume, Time) :-
 	       /* Resumed: ~w\n\
 	       */~n~n', [String]).
 
+%%	rdfe_close_journal
+%
+%	Close  the  journal.  Automatically  called    from  at  program
+%	termination from at_halt/1.
+
 rdfe_close_journal :-
 	get_time(T),
 	SecTime is round(T),
