@@ -1,4 +1,4 @@
-/*  $Id: ssl4pl.c,v 1.5 2006/07/27 09:35:06 jan Exp $
+/*  $Id: ssl4pl.c,v 1.10 2007/01/21 13:29:23 jan Exp $
 
     Part of SWI-Prolog
 
@@ -25,10 +25,9 @@
 
 #include <SWI-Stream.h>
 #include <SWI-Prolog.h>
-#include "ssllib.h"
 #include <assert.h>
 #include <string.h>
-#ifdef WIN32
+#ifdef __WINDOWS__
 #  include <io.h>
 #  include <winsock2.h>
 #else
@@ -37,6 +36,7 @@
 #  include <netinet/in.h>
 #  include <unistd.h>
 #endif
+#include "ssllib.h"
 
 #ifdef _REENTRANT
 #include <pthread.h>
@@ -285,7 +285,7 @@ pl_pem_passwd_hook(PL_SSL *config, char *buf, int size)
   term_t av = PL_new_term_refs(2);
   predicate_t pred = (predicate_t) config->pl_ssl_cb_pem_passwd_data;
   char *passwd = NULL;
-  unsigned int len;
+  size_t len;
     
   /*
    * hook(+SSL, -Passwd)
@@ -294,7 +294,7 @@ pl_pem_passwd_hook(PL_SSL *config, char *buf, int size)
   unify_conf(av+0, config);
   if ( PL_call_predicate(NULL, PL_Q_NORMAL, pred, av) )
   { if ( PL_get_nchars(av+1, &len, &passwd, CVT_ALL) )
-    { if ( len >= size )
+    { if ( len >= (unsigned int)size )
 	PL_warning("pem_passwd too long");
       else
 	memcpy(buf, passwd, len);
@@ -355,7 +355,7 @@ tcp_init()
   }
   initialised = TRUE;
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 { WSADATA WSAData;
 
   if ( WSAStartup(MAKEWORD(2,0), &WSAData) )
@@ -366,7 +366,7 @@ tcp_init()
    * startSocketThread();
    */
 }
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 #ifdef _REENTRANT
   if ( !ssl_thread_setup() )
@@ -503,7 +503,7 @@ pl_ssl_accept(term_t config, term_t sock_inst, term_t peer)
 { PL_SSL *conf;
   int si;
   struct sockaddr_in sa_client;
-  size_t             client_len = sizeof(sa_client);
+  socklen_t          client_len = sizeof(sa_client);
 
   if ( !get_conf(config, &conf) )
     return FALSE;
@@ -540,6 +540,8 @@ pl_ssl_control(PL_SSL_INSTANCE *instance, int action, void *data)
       *p = instance->sock;
       return 0;
     }
+    case SIO_FLUSHOUTPUT:
+      return 0;
     default:
       return -1;
   }
