@@ -1,4 +1,4 @@
-/*  $Id: ssllib.c,v 1.8 2006/03/15 08:34:47 jan Exp $
+/*  $Id: ssllib.c,v 1.11 2007/01/21 13:39:34 jan Exp $
 
     Part of SWI-Prolog
 
@@ -25,7 +25,7 @@
 #include <string.h>
 #include <assert.h>
 #include <stdarg.h>
-#ifdef WIN32
+#ifdef __WINDOWS__
 #  include <io.h>
 #  include <winsock2.h>
 #else
@@ -60,7 +60,7 @@
 #define Serror          stderr
 #define Svfprintf       vfprintf
 static int PL_handle_signals(void) { return 0; }
-#ifndef WIN32
+#ifndef __WINDOWS__
 #define closesocket	close
 #endif
 #endif  /* __SWI_PROLOG__ */
@@ -618,7 +618,7 @@ ssl_cb_pem_passwd(char *buf, int size, int rwflag, void *userdata)
     }
 
     if (passwd) {
-        if ((len = strlen(passwd)) < size) {
+        if ((len = (int)strlen(passwd)) < size) {
             strcpy(buf, passwd);
         } else {
             len = 0;
@@ -1162,13 +1162,13 @@ ssl_socket(PL_SSL *config)
 }
 
 int
-ssl_accept(PL_SSL *config, void *addr, size_t *addrlen)
+ssl_accept(PL_SSL *config, void *addr, socklen_t *addrlen)
 /*
  * We set the TCP layer to accept mode,
  */
 {
     struct sockaddr_in sa_client;
-    size_t             client_len;
+    socklen_t          client_len;
 
     if ( !addr ) {
 	addr = (struct  sockaddr *)&sa_client;
@@ -1290,6 +1290,17 @@ pthreads_locking_callback(int mode, int type, const char *file, int line)
 }
 
 
+/*  From OpenSSL manual:
+
+    id_function(void) is a function that returns a thread ID. It is not
+    needed on Windows nor on platforms where getpid() returns a different
+    ID for each thread (most notably Linux).
+
+    As for pthreads_win32 version 2, the thread identifier is no longer
+    integral, we are going to test this claim from the manual
+*/
+
+#ifndef __WINDOWS__
 static unsigned long
 pthreads_thread_id(void)
 { unsigned long ret;
@@ -1297,6 +1308,7 @@ pthreads_thread_id(void)
   ret=(unsigned long)pthread_self();
   return(ret);
 }
+#endif
 
 
 int
@@ -1310,7 +1322,9 @@ ssl_thread_setup(void)
     pthread_mutex_init(&(lock_cs[i]), NULL);
   }
 			       
+#ifndef __WINDOWS__
   CRYPTO_set_id_callback(pthreads_thread_id);
+#endif
   CRYPTO_set_locking_callback(pthreads_locking_callback);
 
   return TRUE;
