@@ -1,4 +1,4 @@
-/*  $Id: window.pl,v 1.69 2004/09/21 14:22:25 jan Exp $
+/*  $Id: window.pl,v 1.72 2006/04/11 16:52:09 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -346,7 +346,7 @@ human_accelerator(Key, Text) :-
 	accel_cache(Key, Text), !.
 human_accelerator(Key, Text) :-
 	new(S, string('%s', Key)),
-	send(regex('\\\\C-\\(.\\)'), for_all, S,
+	send(regex('\\\\C-(.)'), for_all, S,
 	     message(@arg1, replace, @arg2, 'Control-\\1 ')),
 	send(regex('\\\\e'), for_all, S,
 	     message(@arg1, replace, @arg2, 'Alt-')),
@@ -472,7 +472,7 @@ event(D, Ev:event) :->
 	(   get(D, prompter, Prompter),
 	    Prompter \== @nil
 	->  send(D, editor_event, Ev)
-	;   send(D, send_super, event, Ev)
+	;   send_super(D, event, Ev)
 	).
 
 
@@ -806,6 +806,7 @@ mode(E, ModeName:mode_name) :->
 	    ),
 	    send(E?text_buffer, mode, ModeName),
 	    send(E, syntax, Mode?syntax),
+	    send(E, bindings, Mode?bindings),
 	    send(E, setup_mode),
 	    send(E?device, setup_mode),
 	    send(E, report, status, 'Switched to ``%s'''' mode', ModeName)
@@ -891,17 +892,18 @@ last_buffer(E, TB:text_buffer) :<-
 		 *          UTILITIES		*
 		 *******************************/
 
-looking_at(E, Re:regex, Where:[int]) :->
+looking_at(E, Re:regex, Where:[int], End:[int]) :->
 	"Test if regex macthes from the caret"::
 	(   Where == @default
 	->  get(E, caret, C)
 	;   C = Where
 	),
 	get(E, text_buffer, TB),
-	send(Re, match, TB, C).
+	send(Re, match, TB, C, End).
 
 :- pce_end_class.
 
+
 		 /*******************************
 		 *	    EMACS MODES		*
 		 *******************************/
