@@ -1,4 +1,4 @@
-/*  $Id: event.c,v 1.46 2006/08/11 08:01:15 jan Exp $
+/*  $Id: event.c,v 1.47 2007/01/23 19:22:17 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -163,7 +163,7 @@ considerLocStillEvent()
       return;
     if ( instanceOfObject(last_window, ClassWindow) &&
 	 !onFlag(last_window, F_FREED|F_FREEING) &&
-	 last_x > 0 && last_y > 0 )
+	 valInt(last_x) > 0 && valInt(last_y) > 0 )
     { ServiceMode(is_service_window(last_window),
 		  { AnswerMark mark;
 		    EventObj e;
