@@ -1,4 +1,4 @@
-/*  $Id: image.c,v 1.50 2004/09/21 14:22:25 jan Exp $
+/*  $Id: image.c,v 1.56 2007/01/14 16:36:15 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -211,7 +211,7 @@ storeImage(Image image, FileObj file)
   if ( isNil(image->file) )
     return ws_store_image(image, file);
   else
-  { putc('O', file->fd);
+  { Sputc('O', file->fd);
     succeed;
   }
 }
@@ -575,7 +575,7 @@ pixelImage(Image image, Int X, Int Y, Any val)
 
   if ( inImage(image, X, Y) )
   { if ( (image->kind == NAME_bitmap && !instanceOfObject(val, ClassBool)) ||
-	 !instanceOfObject(val, ClassColour) )
+	 (image->kind == NAME_pixmap && !instanceOfObject(val, ClassColour)) )
       return errorPce(image, NAME_pixelMismatch, val);
     
     CHANGING_IMAGE(image,
@@ -644,18 +644,23 @@ invertImage(Image image)
 static Any
 getPixelImage(Image image, Int x, Int y)
 { if ( inImage(image, x, y) )
-  { unsigned long pixel;
+  { Any result;
     d_image(image, 0, 0, valInt(image->size->w), valInt(image->size->h));
-    pixel = r_get_pixel(valInt(x), valInt(y));
+
+    if ( image->kind == NAME_bitmap )
+      result = (r_get_mono_pixel(valInt(x), valInt(y)) ? ON : OFF);
+    else
+    { unsigned long pixel;
+
+      pixel = r_get_pixel(valInt(x), valInt(y));
+      if ( pixel == NoPixel )
+	result = FAIL;
+      else
+	result = ws_pixel_to_colour(image->display, pixel);
+    }
     d_done();
 
-    DEBUG(NAME_image, Cprintf("pixel = %ld\n", pixel));
-    if ( pixel == NoPixel )
-      fail;
-    if ( image->kind == NAME_bitmap )
-      answer((pixel == 0 ? OFF : ON));
-    
-    answer(ws_pixel_to_colour(image->display, pixel));
+    answer(result);
   }
   
   fail;
@@ -885,7 +890,7 @@ getPostscriptFormatImage(Image image)
 		*       PREDEFINED IMAGES	*
 		********************************/
 
-#if defined(WIN32) || defined(HAVE_LIBXPM)
+#if defined(__WINDOWS__) || defined(HAVE_LIBXPM)
 #define XPM_PCEIMAGE 1			/* use an XPM image */
 #endif
 
@@ -1014,11 +1019,11 @@ standardImages(void)
   stdImage(NAME_olCycleImage, NULL,
 	   ol_cycle_bits, ol_cycle_width, ol_cycle_height);
   stdImage(NAME_treeExpandedImage, NULL,
-	   enode_bits, enode_width, enode_height);
+	   (char*)enode_bits, enode_width, enode_height);
   stdImage(NAME_treeCollapsedImage, NULL,
-	   cnode_bits, cnode_width, cnode_height);
+	   (char*)cnode_bits, cnode_width, cnode_height);
   stdImage(NAME_intItemImage, &INT_ITEM_IMAGE,
-	   intarrows_bits, intarrows_width, intarrows_height);
+	   (char*)intarrows_bits, intarrows_width, intarrows_height);
 #ifdef XPM_PCEIMAGE
   stdXPMImage(NAME_pceImage,	     NULL,		  pce16_xpm);
   stdXPMImage(NAME_hadjustTileImage, NULL,		  hadjusttile_xpm);
@@ -1150,7 +1155,7 @@ static senddecl send_image[] =
      NAME_pixel, "Set pixel at x-y to bool or colour"),
   SM(NAME_setPixel, 2, T_xAint_yAint, setPixelImage,
      NAME_pixel, "Set pixel at x-y (to 1 or foreground)"),
-  SM(NAME_DrawPostScript, 0, NULL, drawPostScriptImage,
+  SM(NAME_DrawPostScript, 1, "{head,body}", drawPostScriptImage,
      NAME_postscript, "Create PostScript"),
   SM(NAME_Xclose, 1, "display", XcloseImage,
      NAME_x, "Destroy associated window-system resources"),
