@@ -1,4 +1,4 @@
-/*  $Id: scrollbar.c,v 1.39 2003/10/27 14:35:27 jan Exp $
+/*  $Id: scrollbar.c,v 1.40 2007/01/14 16:36:15 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -25,7 +25,7 @@
 #include <h/kernel.h>
 #include <h/graphics.h>
 
-#ifdef __WIN32__			/* timer broken in Win95 */
+#ifdef __WINDOWS__			/* timer broken in Win95 */
 #define SCROLLTIMER_USE_SLEEP 1
 #endif
 
