@@ -1,4 +1,4 @@
-/*  $Id: interface.h,v 1.173 2006/08/24 10:29:26 jan Exp $
+/*  $Id: interface.h,v 1.184 2007/01/26 13:39:44 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -31,7 +31,7 @@
 		********************************/
 
 #ifndef PCE_VERSION
-#define PCE_VERSION "6.6.18, July 2006"
+#define PCE_VERSION "6.6.27, January 2007"
 #endif
 
 #ifndef OS_VERSION
@@ -69,7 +69,7 @@ typedef void *		PceClass;	/* PCE's view of a class */
 typedef void *		PceHostData;	/* PCE's view of a host data handle */
 
 #define INT_MASK_SHIFT	1
-#define PCE_MAX_INT	((long)((1L<<(sizeof(void*)*8 - INT_MASK_SHIFT-1))-1))
+#define PCE_MAX_INT	((intptr_t)(((uintptr_t)1<<(sizeof(void*)*8 - INT_MASK_SHIFT-1))-1))
 #define PCE_MIN_INT	(-(PCE_MAX_INT-1))
 
 typedef long		AnswerMark;	/* Mark on AnswerStack */
@@ -167,8 +167,8 @@ typedef union
 
 __pce_export int    pceToC(PceObject datum, PceCValue *rval);
 __pce_export int    pceToCReference(PceObject datum, PceCValue *rval);
-__pce_export char * pceCharArrayToCA(PceObject datum, unsigned int *len);
-__pce_export wchar_t * pceCharArrayToCW(PceObject datum, unsigned int *len);
+__pce_export char * pceCharArrayToCA(PceObject datum, size_t *len);
+__pce_export wchar_t * pceCharArrayToCW(PceObject datum, size_t *len);
 __pce_export int    pceIsString(PceObject datum);
 __pce_export void * pcePointerToC(PceObject datum);
 __pce_export PceHostData  CtoHostData(PceClass class, void *handle, int flags);
@@ -391,25 +391,25 @@ __pce_export int pceInitialise(int handles, const char *home,
 		*           C --> PCE		*
 		********************************/
 
-__pce_export PceObject	cToPceName_nA(const char *, unsigned int len);
-__pce_export PceObject	cToPceName_nW(const wchar_t *, unsigned int len);
-__pce_export PceObject	cToPceInteger(long);
+__pce_export PceObject	cToPceName_nA(const char *, size_t len);
+__pce_export PceObject	cToPceName_nW(const wchar_t *, size_t len);
+__pce_export PceObject	cToPceInteger(intptr_t);
 __pce_export PceObject	cToPceReal(double);
 __pce_export PceObject	cToPceStringA(PceName assoc,
-				      const char *text, unsigned int len,
+				      const char *text, size_t len,
 				      int translate);
 __pce_export PceObject	cToPceStringW(PceName assoc,
-				      const wchar_t *text, unsigned int len,
+				      const wchar_t *text, size_t len,
 				      int translate);
 __pce_export PceObject	cToPceAssoc(const char *);
-__pce_export PceObject	cToPceReference(unsigned long);
+__pce_export PceObject	cToPceReference(uintptr_t);
 __pce_export PceObject	cToPcePointer(void *ptr);
 __pce_export int	pceLock(PceObject);
 
 __pce_export PceObject	cToPceTmpCharArray(const char *text);
 __pce_export void	donePceTmpCharArray(PceObject);
 
-__pce_export int	pceExistsReference(unsigned long);
+__pce_export int	pceExistsReference(uintptr_t ref);
 __pce_export char *	pcePPReference(PceObject ref);
 __pce_export int	pceExistsAssoc(PceName assoc);
 __pce_export PceObject  pceObjectFromName(PceName assoc);
@@ -459,8 +459,8 @@ __pce_export char *	Cgetline(char *line, int size);
 		 *******************************/
 
 #ifndef PCE_INCLUDED
-__pce_export void *	pceMalloc(int size);
-__pce_export void *	pceRealloc(void *ptr, int size);
+__pce_export void *	pceMalloc(size_t size);
+__pce_export void *	pceRealloc(void *ptr, size_t size);
 __pce_export void	pceFree(void *ptr);
 #endif
 
@@ -501,8 +501,8 @@ __pce_export void	pceGetMethod(PceClass class,
 
 __pce_export int	pceOpen(PceObject obj, int flags, void *encoding);
 __pce_export int	pceClose(int handle);
-__pce_export int	pceWrite(int handle, const char *buf, int size);
-__pce_export int	pceRead(int handle, char *buf, int size);
+__pce_export ssize_t	pceWrite(int handle, const char *buf, size_t size);
+__pce_export ssize_t	pceRead(int handle, char *buf, size_t size);
 __pce_export long	pceSeek(int handle, long offset, int whence);
 __pce_export int	pceControl(int handle, int cmd, void *closure);
 __pce_export const char *pceOsError();
