@@ -1,4 +1,4 @@
-/*  $Id: interface.c,v 1.42 2005/05/03 14:03:11 jan Exp $
+/*  $Id: interface.c,v 1.46 2007/01/18 12:13:21 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -48,7 +48,7 @@
 		********************************/
 
 Any
-cToPceInteger(long int i)
+cToPceInteger(intptr_t i)
 { Int n = toInt(i);
 
   if ( valInt(n) != i )
@@ -67,7 +67,7 @@ cToPceReal(double f)
 
 
 Any
-cToPceStringA(Name assoc, const char *s, unsigned int len, int translate)
+cToPceStringA(Name assoc, const char *s, size_t len, int translate)
 { Any str;
   string ss;
   Any c;
@@ -91,7 +91,7 @@ cToPceStringA(Name assoc, const char *s, unsigned int len, int translate)
 
 
 Any
-cToPceStringW(Name assoc, const wchar_t *s, unsigned int len, int translate)
+cToPceStringW(Name assoc, const wchar_t *s, size_t len, int translate)
 { Any str;
   string ss;
   Any c;
@@ -128,7 +128,7 @@ cToPceName(const char *text)
 
 
 Any
-cToPceName_nA(const char *text, unsigned int len)
+cToPceName_nA(const char *text, size_t len)
 { if ( text )
   { string s;
 
@@ -141,7 +141,7 @@ cToPceName_nA(const char *text, unsigned int len)
 
 
 Any
-cToPceName_nW(const wchar_t *text, unsigned int len)
+cToPceName_nW(const wchar_t *text, size_t len)
 { return WCToName(text, len);
 }
 
@@ -181,7 +181,7 @@ pceObjectFromName(PceName name)
 
 
 Any
-cToPceReference(unsigned long val)
+cToPceReference(uintptr_t val)
 { Instance rval = longToPointer(val);
 
   if ( rval &&
@@ -194,7 +194,7 @@ cToPceReference(unsigned long val)
 
 
 int
-pceExistsReference(unsigned long ref)
+pceExistsReference(uintptr_t ref)
 { Any addr = longToPointer(ref);
 
   if ( !isProperObject(addr) || isFreedObj(addr) )
@@ -212,7 +212,7 @@ pcePPReference(PceObject ref)
 
     if ( rval[0] != '@' )
     { char tmp[256];
-      sprintf(tmp, "@%ld", valInt(ref));
+      sprintf(tmp, "@" INTPTR_FORMAT, valInt(ref));
       return save_string(tmp);
     } else
       return rval;
@@ -408,7 +408,7 @@ pceIsString(Any val)
 
 
 char *
-pceCharArrayToCA(Any val, unsigned int *len)
+pceCharArrayToCA(Any val, size_t *len)
 { if ( instanceOfObject(val, ClassCharArray) )
   { CharArray ca = val;
 
@@ -425,7 +425,7 @@ pceCharArrayToCA(Any val, unsigned int *len)
 
 
 wchar_t *
-pceCharArrayToCW(Any val, unsigned int *len)
+pceCharArrayToCW(Any val, size_t *len)
 { if ( instanceOfObject(val, ClassCharArray) )
   { CharArray ca = val;
 
@@ -906,13 +906,13 @@ pceSetProfileHooks(pce_profile_hooks *hooks)
 #undef pceFree
 
 void *
-pceMalloc(int size)
+pceMalloc(size_t size)
 { return (*TheCallbackFunctions.malloc)(size);
 }
 
 
 void *
-pceRealloc(void *ptr, int size)
+pceRealloc(void *ptr, size_t size)
 { return (*TheCallbackFunctions.realloc)(ptr, size);
 }
 
