@@ -1,4 +1,4 @@
-/*  $Id: directory.c,v 1.49 2005/07/07 15:09:33 jan Exp $
+/*  $Id: directory.c,v 1.52 2007/01/23 19:22:17 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -315,7 +315,7 @@ getParentDirectory(Directory d)
 static Chain
 getRootsDirectory(Directory dir)
 { Chain ch = answerObject(ClassChain, EAV);
-#ifdef WIN32
+#ifdef __WINDOWS__
   char buf[MAXPATHLEN];
   extern int get_logical_drive_strings(int, char *);
 
@@ -374,7 +374,7 @@ getFileNameDirectory(Directory d, Name name)
   else
   { const char *dfn = nameToUTF8(d->path);
     size_t dfnl = strlen(dfn);
-    int maxl = strlen(fn) + dfnl + 2;
+    size_t maxl = strlen(fn) + dfnl + 2;
     LocalArray(char, buf, maxl);
 
     memcpy(buf, dfn, dfnl);
@@ -776,7 +776,7 @@ absolutePath(const char *file, char *path, size_t buflen)
 
   canonisePath(path);
 
-  return strlen(path);
+  return (int)strlen(path);
 }
 
 
@@ -811,7 +811,7 @@ GETENV(const wchar_t *var, size_t len)
 int
 expandFileNameW(const wchar_t *pattern, wchar_t *bin, size_t binlen)
 { wchar_t *expanded = bin;
-  int size = 0;
+  size_t size = 0;
   wint_t c;
 
   binlen--;				/* space for EOS */
@@ -872,7 +872,7 @@ expandFileNameW(const wchar_t *pattern, wchar_t *bin, size_t binlen)
     }
 #endif
 
-    size += (l = wcslen(value));
+    size += (int)(l = wcslen(value));
     if ( size >= binlen )
     { ExpandProblem = CtoName("Name too long");
       return -1;
@@ -901,7 +901,7 @@ nouser:
 	    { ExpandProblem = CtoName("Unknown variable");
 	      return -1;
 	    }
-	    size += (l = wcslen(value));
+	    size += (l = (int)wcslen(value));
 	    if ( size >= binlen )
 	    { errno = ENAMETOOLONG;
 	      return -1;
