@@ -1,4 +1,4 @@
-/*  $Id: process.c,v 1.29 2004/09/21 14:22:25 jan Exp $
+/*  $Id: process.c,v 1.34 2007/01/14 16:36:16 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -82,7 +82,7 @@ reduced to the facility to terminate the inferior process.
 #include <sys/ioctl.h>
 #endif
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #define environ _environ	/* declared in STDLIB.H */
 #else
 extern char **environ;		/* Unix version */
@@ -419,19 +419,34 @@ initEnvironment(Process p)
 { if ( notNil(p->environment) )
   { int i = 0;
     Cell cell;
-    Name fmt = CtoName("%s=%s");
+    char **env;
 
-    environ = pceMalloc(sizeof(char *) *
-			(valInt(p->environment->attributes->size) + 1));
+    env = malloc(sizeof(char *) *
+		 (valInt(p->environment->attributes->size) + 1));
 
     for_cell(cell, p->environment->attributes)
     { Attribute a = cell->value;
-      StringObj str = answerObject(ClassString, fmt, a->name, a->value, EAV);
+      String ns = &((CharArray)a->name)->data;
+      String vs = &((CharArray)a->value)->data;
 
-      environ[i++] = save_string(strName(str));
-      doneObject(str);
+      if ( isstrA(ns) && isstrA(vs) )
+      { int len = ns->size + 1 + vs->size + 1;
+	char *e;
+
+	e = malloc(len);
+	memcpy(e, ns->s_textA, ns->size);
+	e[ns->size] = '=';
+	memcpy(e+ns->size+1, vs->s_textA, vs->size);
+	e[len-1] = EOS;
+
+	env[i++] = e;
+      } else
+      { Cprintf("Ignored wide string in environment\n");
+      }
     }
-    environ[i] = NULL;
+    env[i] = NULL;
+
+    environ = env;
   }
 }
 
@@ -443,17 +458,18 @@ getEnvironmentProcess(Process p)
 
     assign(p, environment, newObject(ClassSheet, EAV));
     for(; *env; env++)
-    { char buf[LINESIZE];
+    { char *e = *env;
       char *q;
-      int l;
 
       DEBUG(NAME_environment, Cprintf("env = %s\n", *env));
-      if ( (q=strchr(*env, '=')) )
-      { strncpy(buf, *env, (l = q - *env));
-	buf[l] = EOS;
-	valueSheet(p->environment, CtoName(buf), CtoName(&q[1]));
+      if ( (q=strchr(e, '=')) )
+      { string ns, vs;
+
+	str_set_n_ascii(&ns, q-e, e);
+	str_set_n_ascii(&vs, strlen(q+1), q+1);
+	valueSheet(p->environment, StringToName(&ns), StringToName(&vs));
       } else
-	valueSheet(p->environment, CtoName(*env), CtoName(""));
+	valueSheet(p->environment, CtoName(*env), NAME_);
     }
   }
 
@@ -494,7 +510,7 @@ openProcess(Process p, CharArray cmd, int argc, CharArray *argv)
 
   if ( isNil(p->pid) )
   { if ( p->use_tty == ON )
-    { int master, slave;
+    { int master = -1, slave = -1;
       int pid;
 #if USE_GRANTPT
       char *line = NULL;
@@ -512,15 +528,28 @@ openProcess(Process p, CharArray cmd, int argc, CharArray *argv)
 	fail;
       }
 
+#ifdef stderr
+      DEBUG(NAME_process, fprintf(stderr, "Ready to fork\n"));
+#endif
+
       if ( (pid = fork()) == 0 )	/* child process */
       { int i, argc;
 	char **argv;
-	int maxfd = getdtablesize();
+	int maxfd = getFileDesCount();
+
+#ifdef stderr
+	DEBUG(NAME_process, fprintf(stderr, "Child: maxfd = %d\n", maxfd));
+#endif
 
 	if ( notDefault(p->directory) )
 	  cdDirectory(p->directory);
+#ifdef stderr
+	DEBUG(NAME_process, fprintf(stderr, "CD ok\n"));
+#endif
 	initEnvironment(p);
-	DEBUG(NAME_process, Cprintf("Environment initialised\n"));
+#ifdef stderr
+	DEBUG(NAME_process, fprintf(stderr, "Environment initialised\n"));
+#endif
 #ifdef HAVE_SETSID
 	if ( setsid() < 0 )
 	  Cprintf("[PCE: setsid() failed: %s]\n", strName(OsError()));
@@ -615,11 +644,10 @@ openProcess(Process p, CharArray cmd, int argc, CharArray *argv)
 	  cdDirectory(p->directory);
 	initEnvironment(p);
 
-	for(i = getdtablesize()-1; i >= 0; i--)
-	  if ( i != wrfd[0] && i != rdfd[1] )
+	for(i = getFileDesCount()-1; i >= 0; i--)
+	{ if ( i != wrfd[0] && i != rdfd[1] )
 	    close(i);
-
-
+	}
 
 	dup2(wrfd[0], 0);
 	dup2(rdfd[1], 1);
@@ -659,7 +687,7 @@ openProcess(Process p, CharArray cmd, int argc, CharArray *argv)
 }
 
 #else /*HAVE_FORK*/
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 extern status openProcess(Process p, CharArray cmd, int argc, CharArray *argv);
 #endif
 #endif /*HAVE_FORK*/
