@@ -1,4 +1,4 @@
-/*  $Id: socket.c,v 1.34 2003/05/05 11:17:47 jan Exp $
+/*  $Id: socket.c,v 1.41 2007/01/18 12:13:23 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#ifdef __CYGWIN32__
+#ifdef __CYGWIN__
 #define __USE_W32_SOCKETS
 #define HAVE_WINSOCK 1
 #endif
@@ -40,6 +40,7 @@
 #include "mswinsock.h"
 #include <io.h>
 #include <fcntl.h>
+typedef size_t socklen_t;
 #else /*HAVE_WINSOCK*/
 
 #define HAVE_SYS_UN_H 1
@@ -52,8 +53,12 @@
 #endif
 #include <errno.h>
 extern int errno;
+#undef NAME_MAX				/* conflict */
 
 #define SOCKET int
+#ifndef HAVE_SOCKLEN_T
+#define socklen_t size_t
+#endif
 #endif /*WINSOCK*/
 
 #define create PCEcreate		/* avoid conflict */
@@ -320,8 +325,8 @@ cloneSocket(Socket s, Socket clone)
 
 static StringObj
 getPrintNameSocket(Socket s)
-{ char buf[256];
-  int sz = sizeof(buf);
+{ string tmp;
+  StringObj sobj;
   Any av[3];
   Name fmt;
   int an;
@@ -341,9 +346,11 @@ getPrintNameSocket(Socket s)
     fmt = CtoName("%s(%s)");
   }
 
-  swritefv(buf, &sz, (CharArray) fmt, an, av);
+  str_writefv(&tmp, (CharArray) fmt, an, av);
+  sobj = StringToString(&tmp);
+  str_unalloc(&tmp);
 
-  answer(CtoString(buf));
+  answer(sobj);
 }
 
 
@@ -475,7 +482,11 @@ bindSocket(Socket s, Bool reuse)
 	   (instanceOfObject(s->address, ClassTuple) &&
 	    ((Tuple)s->address)->second == ZERO) )
       { struct sockaddr_in addr;
-	int len = sizeof(addr);
+#ifdef __WINDOWS__
+	DWORD len = sizeof(addr);
+#else
+	socklen_t len = sizeof(addr);
+#endif
 
 	if ( getsockname(SocketHandle(s), (struct sockaddr *) &addr, &len) )
 	{ return errorPce(s, NAME_socket, NAME_getsockname, SockError());
@@ -505,7 +516,7 @@ acceptSocket(Socket s)
 #ifdef UNIX_DOMAIN_SOCKETS
   if ( s->domain == NAME_unix )
   { struct sockaddr_un address;
-    int len = sizeof(address);
+    socklen_t len = sizeof(address);
 
     if ( (id2=accept(SocketHandle(s), (struct sockaddr *) &address, &len))<0 )
       errorPce(s, NAME_socket, NAME_accept, SockError());
@@ -514,7 +525,11 @@ acceptSocket(Socket s)
   } else /*if ( s->domain == NAME_inet )*/
 #endif
   { struct sockaddr_in address;
-    int len = sizeof(address);
+#ifdef __WINDOWS__
+    DWORD len = sizeof(address);
+#else
+    socklen_t len = sizeof(address);
+#endif
 
     if ( (id2=accept(SocketHandle(s), (struct sockaddr *) &address, &len))<0 )
       errorPce(s, NAME_socket, NAME_accept, SockError());
@@ -649,7 +664,7 @@ getPeerNameSocket(Socket s)
   if ( s->domain == NAME_unix )
   { struct sockaddr_un *address;
     char buf[MAX_UN_ADDRESS_LEN + sizeof(address->sun_family)];
-    int len;
+    socklen_t len;
 
     address = (struct sockaddr_un *) buf;
     len = sizeof(buf);
@@ -663,7 +678,11 @@ getPeerNameSocket(Socket s)
   } else /* if ( s->domain = NAME_inet ) */
 #endif /*UNIX_DOMAIN_SOCKETS*/
   { struct sockaddr_in address;
-    int len = sizeof(address);
+#ifdef __WINDOWS__
+    DWORD len = sizeof(address);
+#else
+    socklen_t len = sizeof(address);
+#endif
     int port;
     unsigned long addr;
     char aname[3*4+4];
