@@ -1,4 +1,4 @@
-/*  $Id: pl-alloc.c,v 1.100 2004/09/21 14:22:25 jan Exp $
+/*  $Id: pl-alloc.c,v 1.121 2007/01/23 20:39:40 jan Exp $
 
     Part of SWI-Prolog
 
@@ -29,7 +29,7 @@
 #endif
 
 #ifndef ALLOC_DEBUG
-#if defined(_DEBUG) && defined(WIN32)
+#if defined(_DEBUG) && defined(__WINDOWS__)
 #define ALLOC_DEBUG 1
 #else
 #define ALLOC_DEBUG 0
@@ -96,10 +96,14 @@ GD.  Rules:
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #ifndef ALIGN_SIZE
-#if defined(__sgi) && !defined(__GNUC__)
+#if (defined(__sgi) && !defined(__GNUC__)) || defined __sparc__ || defined __sparc64__
 #define ALIGN_SIZE sizeof(double)
 #else
-#define ALIGN_SIZE sizeof(long)
+#ifdef __ia64__
+#define ALIGN_SIZE sizeof(intptr_t[2])
+#else
+#define ALIGN_SIZE sizeof(intptr_t)
+#endif
 #endif
 #endif
 #define ALLOC_MIN  sizeof(Chunk)
@@ -134,20 +138,20 @@ void *core_allocHeap__LD(size_t n ARG_LD);
 
 void
 freeHeap__LD(void *mem, size_t n ARG_LD)
-{ long *p = mem;
+{ intptr_t *p = mem;
 
-  assert(p[-1] == (long)n);
+  assert(p[-1] == (intptr_t)n);
   assert(p[-2] == INUSE_MAGIC);
   p[-2] = FREE_MAGIC;			/* trap double free */
   memset(mem, ALLOC_FREE_MAGIC, n);
 
-  core_freeHeap__LD(p-3, n+3*sizeof(long) PASS_LD);
+  core_freeHeap__LD(p-3, n+3*sizeof(intptr_t) PASS_LD);
 }
 
 
 void *
 allocHeap__LD(size_t n ARG_LD)
-{ long *p = core_allocHeap__LD(n+3*sizeof(long) PASS_LD);
+{ intptr_t *p = core_allocHeap__LD(n+3*sizeof(intptr_t) PASS_LD);
 
   p += 3;
   p[-1] = n;
@@ -174,7 +178,7 @@ freeToPool(AllocPool pool, void *mem, size_t n, int islocal)
   
   pool->allocated -= n;
   DEBUG(9, Sdprintf("freed %ld bytes at %ld\n",
-		    (unsigned long)n, (unsigned long)p));
+		    (uintptr_t)n, (uintptr_t)p));
 
   n /= ALIGN_SIZE;
   p->next = pool->free_chains[n];
@@ -244,7 +248,7 @@ which normally calls malloc().
 static void
 leftoverToChains(AllocPool pool)
 { if ( pool->free >= sizeof(struct chunk) )
-  { unsigned m = pool->free/ALIGN_SIZE;
+  { size_t m = pool->free/ALIGN_SIZE;
     Chunk ch = (Chunk)pool->space;
 
     assert(m <= ALLOCFAST/ALIGN_SIZE);
@@ -457,15 +461,24 @@ mergeAllocPool(AllocPool to, AllocPool from)
       i < (ALLOCFAST/ALIGN_SIZE);
       i++, t++, f++)
   { if ( *f )
-    { Chunk c = *t;
+    { if ( to->free_count[i] )
+      { if ( to->free_count[i] <= from->free_count[i] )
+	{ Chunk c = *t;
+		 
+	  while(c->next)
+	    c = c->next;
+	  c->next = *f; 
+	} else
+	{ Chunk c = *f;
 
-      if ( c )
-      { while(c->next)			/* find end of chain */
-	  c = c->next;
-	c->next = *f;
+	  while(c->next)
+	    c = c->next;
+	  c->next = *t;
+	  *t = *f;
+	}
       } else
 	*t = *f;
-
+	 
       to->free_count[i] += from->free_count[i];
       from->free_count[i] = 0;
 
@@ -653,7 +666,7 @@ outOfStack(Stack s, stack_overflow_action how)
 }
 
 
-volatile void
+void
 outOfCore()
 { fatalError("Could not allocate memory: %s", OsError());
 }
@@ -676,7 +689,7 @@ static inline word
 __consPtr(void *p, int ts)
 {
   GET_LD
-  unsigned long v = (unsigned long) p;
+  uintptr_t v = (uintptr_t) p;
 
   v -= base_addresses[ts&STG_MASK];
   assert(v < MAXTAGGEDPTR && !(v&0x3));
@@ -721,13 +734,13 @@ inline  as  it is simple and usualy very time critical.
 #if O_SHIFT_STACKS
 
 Word
-allocGlobal__LD(int n ARG_LD)
+allocGlobal__LD(size_t n ARG_LD)
 { Word result;
 
-  if ( roomStack(global) < (long) (n * sizeof(word)) )
+  if ( roomStack(global) < (intptr_t) (n * sizeof(word)) )
   { growStacks(NULL, NULL, NULL, FALSE, TRUE, FALSE);
 
-    if ( roomStack(global) < (long) (n * sizeof(word)) )
+    if ( roomStack(global) < (intptr_t) (n * sizeof(word)) )
       outOfStack((Stack) &LD->stacks.global, STACK_OVERFLOW_FATAL);
   }
 
@@ -738,10 +751,10 @@ allocGlobal__LD(int n ARG_LD)
 }
 
 Word
-allocGlobalNoShift__LD(int n ARG_LD)
+allocGlobalNoShift__LD(size_t n ARG_LD)
 { Word result;
 
-  if ( roomStack(global) < (long) (n * sizeof(word)) )
+  if ( roomStack(global) < (intptr_t) (n * sizeof(word)) )
   { assert(0);				/* TBD: see copy_term/2 */
     return NULL;
   }
@@ -755,7 +768,7 @@ allocGlobalNoShift__LD(int n ARG_LD)
 #else
 
 static inline Word
-__allocGlobal(int n ARG_LD)
+__allocGlobal(size_t n ARG_LD)
 { Word result = gTop;
 
   requireStack(global, n * sizeof(word));
@@ -764,7 +777,7 @@ __allocGlobal(int n ARG_LD)
   return result;
 }
 
-Word allocGlobal__LD(int n ARG_LD)
+Word allocGlobal__LD(size_t n ARG_LD)
 { return __allocGlobal(n PASS_LD);
 }
 
@@ -803,14 +816,38 @@ newTerm(void)
 		 *******************************/
 
 word
-globalLong(long l ARG_LD)
-{ Word p = allocGlobal(3);
-  word r = consPtr(p, TAG_INTEGER|STG_GLOBAL);
-  word m = mkIndHdr(1, TAG_INTEGER);
+globalLong(int64_t l ARG_LD)
+{ Word p;
+  word r, m;
+
+#if SIZEOF_VOIDP == 8
+  p = allocGlobal(3);
+  r = consPtr(p, TAG_INTEGER|STG_GLOBAL);
+  m = mkIndHdr(1, TAG_INTEGER);
 
   *p++ = m;
   *p++ = l;
   *p   = m;
+#else
+#if SIZEOF_VOIDP == 4
+  p = allocGlobal(4);
+  r = consPtr(p, TAG_INTEGER|STG_GLOBAL);
+  m = mkIndHdr(2, TAG_INTEGER);
+
+  *p++ = m;
+#ifdef WORDS_BIGENDIAN
+  *p++ = (word)(l>>32);
+  *p++ = (word)l;
+#else
+  *p++ = (word)l;
+  *p++ = (word)(l>>32);
+#endif
+  *p   = m;
+#else
+#error "FIXME: Unsupported sizeof intptr_t."
+#endif
+#endif
+
   
   return r;
 }
@@ -820,42 +857,115 @@ globalLong(long l ARG_LD)
 		 *    OPERATIONS ON STRINGS	*
 		 *******************************/
 
-int
-sizeString(word w)
-{ GET_LD
-  word m  = *((Word)addressIndirect(w));
-  int wn  = wsizeofInd(m);
-  int pad = padHdr(m);
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+To distinguish between byte and wide strings,   the system adds a 'B' or
+'W' in front of the real string. For   a  'W', the following 3 bytes are
+ignored to avoid alignment restriction problems.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-  return wn*sizeof(word) - pad;
-}
-
-
-word
-globalNString(long l, const char *s)
-{ GET_LD
-  int lw = (l+sizeof(word))/sizeof(word);
-  int pad = (lw*sizeof(word) - l);
+static Word
+allocString(size_t len ARG_LD)
+{ size_t lw = (len+sizeof(word))/sizeof(word);
+  int pad = (int)(lw*sizeof(word) - len);
   Word p = allocGlobal(2 + lw);
-  word r = consPtr(p, TAG_STRING|STG_GLOBAL);
   word m = mkStrHdr(lw, pad);
-  int i;
-  char *q;
 
-  *p++ = m;
-  memcpy(p, s, l);
-  for(q=(char *)p+l, i=0; i<pad; i++)
-    *q++ = EOS;
-  p += lw;
-  *p = m;
-  
-  return r;
+  p[0]    = m;
+  p[lw]   = 0L;				/* zero the pad bytes */
+  p[lw+1] = m;
+
+  return p;
 }
 
 
 word
-globalString(const char *s)
-{ return globalNString(strlen(s), s);
+globalString(size_t len, const char *s)
+{ GET_LD
+  Word p = allocString(len+1 PASS_LD);
+  char *q = (char *)&p[1];
+
+  *q++ = 'B';
+  memcpy(q, s, len);
+  
+  return consPtr(p, TAG_STRING|STG_GLOBAL);
+}
+
+
+word
+globalWString(size_t len, const pl_wchar_t *s)
+{ GET_LD
+  const pl_wchar_t *e = &s[len];
+  const pl_wchar_t *p;
+  Word g;
+
+  for(p=s; p<e; p++)
+  { if ( *p > 0xff )
+      break;
+  }
+
+  if ( p == e )				/* 8-bit string */
+  { unsigned char *t;
+
+    g = allocString(len+1 PASS_LD);
+    t = (unsigned char *)&g[1];
+    *t++ = 'B';
+    for(p=s; p<e; )
+      *t++ = *p++ & 0xff;
+  } else				/* wide string */
+  { char *t;
+    pl_wchar_t *w;
+
+    g = allocString((len+1)*sizeof(pl_wchar_t) PASS_LD);
+    t = (char *)&g[1];
+    w = (pl_wchar_t*)t;
+    w[0] = 0;
+    *t = 'W';
+    memcpy(&w[1], s, len*sizeof(pl_wchar_t));
+  }
+
+  return consPtr(g, TAG_STRING|STG_GLOBAL);
+}
+
+
+char *
+getCharsString__LD(word w, size_t *len ARG_LD)
+{ Word p = valPtr(w);
+  word m = *p;
+  size_t wn  = wsizeofInd(m);
+  size_t pad = padHdr(m);
+  char *s;
+
+  if ( len )
+    *len = wn*sizeof(word) - pad - 1;	/* -1 for the 'B' */
+  
+  s = (char *)&p[1];
+
+  if ( *s == 'B' )
+    return s+1;
+
+  assert(*s == 'W');
+  return NULL;
+}
+
+
+pl_wchar_t *
+getCharsWString__LD(word w, size_t *len ARG_LD)
+{ Word p = valPtr(w);
+  word m = *p;
+  size_t wn  = wsizeofInd(m);
+  size_t pad = padHdr(m);
+  char *s;
+  pl_wchar_t *ws;
+
+  s = (char *)&p[1];
+  if ( *s != 'W' )
+    return NULL;
+  
+  if ( len )
+    *len = ((wn*sizeof(word) - pad)/sizeof(pl_wchar_t)) - 1;
+  
+  ws = (pl_wchar_t *)&p[1];
+  return ws+1;
 }
 
 
@@ -923,6 +1033,32 @@ globalReal(double d)
 }
 
 
+		 /*******************************
+		 *	  64-BIT INTEGERS	*
+		 *******************************/
+
+#ifdef INT64_ALIGNMENT
+
+int64_t					/* take care of alignment! */
+valBignum__LD(word w ARG_LD)
+{ Word p = valIndirectP(w);
+  union
+  { int64_t i;
+    word w[WORDS_PER_INT64];
+  } val;
+  
+#if ( SIZEOF_VOIDP == 4 )
+  val.w[0] = p[0];
+  val.w[1] = p[1];
+#else
+#error "Unsupported int64_t alignment conversion"
+#endif
+
+  return val.i;
+}
+
+#endif
+
 		 /*******************************
 		 *  GENERIC INDIRECT OPERATIONS	*
 		 *******************************/
@@ -934,9 +1070,9 @@ equalIndirect(word w1, word w2)
   Word p2 = addressIndirect(w2);
   
   if ( *p1 == *p2 )
-  { int n = wsizeofInd(*p1);
+  { size_t n = wsizeofInd(*p1);
     
-    while( --n >= 0 )
+    while( n-- > 0 )
     { if ( *++p1 != *++p2 )
 	fail;
     }
@@ -958,12 +1094,12 @@ globalIndirect(word w)
 { GET_LD
   Word p = addressIndirect(w);
   word t = *p;
-  int  n = wsizeofInd(t);
+  size_t  n = wsizeofInd(t);
   Word h = allocGlobal((n+2));
   Word hp = h;
   
   *hp = t;
-  while(--n >= 0)
+  while(n-- > 0)
     *++hp = *++p;
   *++hp = t;
 
@@ -976,12 +1112,12 @@ globalIndirectFromCode(Code *PC)
 { GET_LD
   Code pc = *PC;
   word m = *pc++;
-  int  n = wsizeofInd(m);
+  size_t n = wsizeofInd(m);
   Word p = allocGlobal(n+2);
   word r = consPtr(p, tag(m)|STG_GLOBAL);
 
   *p++ = m;
-  while(--n >= 0)
+  while(n-- > 0)
     *p++ = *pc++;
   *p++ = m;
 
@@ -997,9 +1133,9 @@ equalIndirectFromCode(word a, Code *PC)
   Word pa = addressIndirect(a);
 
   if ( *pc == *pa )
-  { int  n = wsizeofInd(*pc);
+  { size_t n = wsizeofInd(*pc);
 
-    while(--n >= 0)
+    while(n-- > 0)
     { if ( *++pc != *++pa )
 	fail;
     }
@@ -1023,13 +1159,17 @@ equalIndirectFromCode(word a, Code *PC)
 
 char *
 store_string(const char *s)
-{ GET_LD
-  char *copy = (char *)allocHeap(strlen(s)+2);
+{ if ( s )
+  { GET_LD
+    char *copy = (char *)allocHeap(strlen(s)+2);
 
-  *copy++ = CHAR_INUSE;
-  strcpy(copy, s);
+    *copy++ = CHAR_INUSE;
+    strcpy(copy, s);
 
-  return copy;
+    return copy;
+  } else
+  { return NULL;
+  }
 }
 
 
@@ -1048,11 +1188,16 @@ remove_string(char *s)
 
 char *
 store_string(const char *s)
-{ GET_LD
-  char *copy = (char *)allocHeap(strlen(s)+1);
-
-  strcpy(copy, s);
-  return copy;
+{ if ( s )
+  { GET_LD
+  
+    char *copy = (char *)allocHeap(strlen(s)+1);
+    
+    strcpy(copy, s);
+    return copy;
+  } else
+  { return NULL;
+  }
 }
 
 
@@ -1073,7 +1218,7 @@ distribution over these atoms.  Note that size equals 2^n.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 int
-unboundStringHashValue(const char *t, unsigned int len)
+unboundStringHashValue(const char *t, size_t len)
 { unsigned int value = 0;
   unsigned int shift = 5;
 
@@ -1099,7 +1244,7 @@ with the GNU C-library lead to undefined symbols.  Therefore we define
 them in SWI-Prolog so that we can also give consistent warnings.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#ifndef xmalloc				/* for use with dmalloc */
+#if !defined(xmalloc) && defined(O_XMALLOC)
 
 void *
 xmalloc(size_t size)
@@ -1139,7 +1284,7 @@ void *
 PL_malloc(size_t size)
 { void *mem;
 
-  if ( !size )
+  if ( size == 0 )
     return NULL;
   if ( (mem = malloc(size)) )
     return mem;
