@@ -1,4 +1,4 @@
-/*  $Id: pl-arith.c,v 1.104 2006/08/27 15:24:14 jan Exp $
+/*  $Id: pl-arith.c,v 1.118 2007/01/23 14:21:59 jan Exp $
 
     Part of SWI-Prolog
 
@@ -46,7 +46,7 @@ day.
 #include "pl-incl.h"
 #ifdef HAVE_FLOAT_H
 #include <float.h>
-#ifdef WIN32
+#ifdef __WINDOWS__
 #define isnan(x) _isnan(x)
 #endif
 #endif
@@ -68,7 +68,7 @@ day.
 #define M_E (2.7182818284590452354)
 #endif
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 #define LL(x) x ## i64
 #else
 #define LL(x) x ## LL
@@ -128,7 +128,7 @@ static int		registerFunction(ArithFunction f, int index);
 static int		getCharExpression(term_t t, Number r ARG_LD);
 static int		ar_sign_i(Number n1);
 static int		ar_add(Number n1, Number n2, Number r);
-static int		ar_add_ui(Number n, long val);
+static int		ar_add_ui(Number n, intptr_t val);
 static int		ar_minus(Number n1, Number n2, Number r);
 
 
@@ -145,10 +145,18 @@ clearInteger(Number n)
 #endif
 }
 
+
+typedef struct
+{ number low;
+  number high;
+  int hinf;
+} between_state;
+
+
 static
 PRED_IMPL("between", 3, between, PL_FA_NONDETERMINISTIC)
 { GET_LD
-  number *state;
+  between_state *state;
   term_t low = A1;
   term_t high = A2;
   term_t n = A3;
@@ -162,9 +170,11 @@ PRED_IMPL("between", 3, between, PL_FA_NONDETERMINISTIC)
 	  return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, low);
 	if ( !PL_get_number(high, &h) || !intNumber(&h) )
 	{ if ( PL_is_inf(high) )
+	  { h.type = V_INTEGER;		/* make clearInteger() safe */
 	    hinf = TRUE;
-	  else
-	    return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, high);
+	  } else
+	  { return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, high);
+	  }
 	}
 
 					/* between(+,+,+) */
@@ -201,9 +211,10 @@ PRED_IMPL("between", 3, between, PL_FA_NONDETERMINISTIC)
 	  succeed;
 	}
 
-	state = allocHeap(sizeof(number)*2);
-	cpNumber(&state[0], &l);
-	cpNumber(&state[1], &h);
+	state = allocHeap(sizeof(*state));
+	cpNumber(&state->low, &l);
+	cpNumber(&state->high, &h);
+	state->hinf = hinf;
 	clearInteger(&l);
 	clearInteger(&h);
 	ForeignRedoPtr(state);
@@ -211,18 +222,19 @@ PRED_IMPL("between", 3, between, PL_FA_NONDETERMINISTIC)
     case FRG_REDO:
       { state = CTX_PTR;
 
-	ar_add_ui(&state[0], 1L);
-	PL_unify_number(n, &state[0]);
-	if ( cmpNumbers(&state[0], &state[1]) == 0 )
+	ar_add_ui(&state->low, 1L);
+	PL_unify_number(n, &state->low);
+	if ( !state->hinf &&
+	     cmpNumbers(&state->low, &state->high) == 0 )
 	  goto cleanup;
 	ForeignRedoPtr(state);
       }
     case FRG_CUTTED:
       { state = CTX_PTR;
       cleanup:
-	clearInteger(&state[0]);
-	clearInteger(&state[1]);
-	freeHeap(state,  sizeof(number)*2);
+	clearInteger(&state->low);
+	clearInteger(&state->high);
+	freeHeap(state, sizeof(*state));
       }
     default:;
       succeed;
@@ -742,19 +754,22 @@ int arithChar(Word p)
 
 int
 arithChar(Word p ARG_LD)
-{ int chr;
-  deRef(p);
+{ deRef(p);
   
   if ( isInteger(*p) )
-  { chr = valInt(*p);
+  { intptr_t chr = valInt(*p);
 
-    if ( chr >= 0 && chr < 256 )
-      return chr;
+    if ( chr >= 0 && chr <= 0x10ffff )	/* UNICODE_MAX */
+      return (int)chr;
   } else if ( isAtom(*p) )
-  { Atom a = atomValue(*p);
-    
-    if ( true(a->type, PL_BLOB_TEXT) && a->length == 1 )
-      return a->name[0] & 0xff;		/* ASCII! */
+  { PL_chars_t txt;
+
+    if ( get_atom_text(*p, &txt) && txt.length == 1 )
+    { if ( txt.encoding == ENC_WCHAR )
+	return txt.text.w[0];
+      else
+	return txt.text.t[0]&0xff;
+    }
   }
 
   PL_error(NULL, 0, NULL, ERR_TYPE,
@@ -863,7 +878,7 @@ promoteIntNumber(Number n)
 		*********************************/
 
 static int
-ar_add_ui(Number n, long add)
+ar_add_ui(Number n, intptr_t add)
 { switch(n->type)
   { case V_INTEGER:
     { int64_t r = n->value.i + add;
@@ -872,10 +887,10 @@ ar_add_ui(Number n, long add)
 	   (r > 0 && add < 0 && n->value.i < 0) )
       { if ( !promoteIntNumber(n) )
 	  fail;
+      } else
+      { n->value.i = r;
+	succeed;
       }
-
-      n->value.i = r;
-      succeed;
     }
 #ifdef O_GMP
     case V_MPZ:
@@ -1038,7 +1053,8 @@ msb64(int64_t i)
 
 static int
 ar_shift(Number n1, Number n2, Number r, int dir) 
-{ long shift;
+{ GET_LD
+  intptr_t shift;
   const char *plop = (dir < 0 ? "<<" : ">>");
 
   if ( !toIntegerNumber(n1) ) 
@@ -1046,21 +1062,24 @@ ar_shift(Number n1, Number n2, Number r, int dir)
   if ( !toIntegerNumber(n2) ) 
     return PL_error(plop, 2, NULL, ERR_AR_TYPE, ATOM_integer, n2); 
 
+  if ( ar_sign_i(n1) == 0 )		/* shift of 0 is always 0 */
+  { r->value.i = 0;
+    r->type = V_INTEGER;
+  }
+
   switch(n2->type)
   { case V_INTEGER:
-      if ( n2->value.i < LONG_MIN )
-	shift = LONG_MIN;
-      else if ( n2->value.i > LONG_MAX )
-	shift = LONG_MAX;
+      if ( n2->value.i < LONG_MIN  ||
+	   n2->value.i > LONG_MAX )
+	return (int)outOfStack((Stack)&LD->stacks.global, STACK_OVERFLOW_RAISE);
       else
-	shift = (long)n2->value.i;
+	shift = (intptr_t)n2->value.i;
       break;
 #ifdef O_GMP
     case V_MPZ:
-      if ( mpz_cmp_si(n2->value.mpz, LONG_MIN) < 0 )
-	shift = LONG_MIN;
-      else if ( mpz_cmp_si(n2->value.mpz, LONG_MAX) > 0 )
-	shift = LONG_MAX;
+      if ( mpz_cmp_si(n2->value.mpz, LONG_MIN) < 0 ||
+	   mpz_cmp_si(n2->value.mpz, LONG_MAX) > 0 )
+	return (int)outOfStack((Stack)&LD->stacks.global, STACK_OVERFLOW_RAISE);
       else
 	shift = mpz_get_si(n2->value.mpz);
       break;
@@ -1070,12 +1089,17 @@ ar_shift(Number n1, Number n2, Number r, int dir)
       fail;
   }
 
+  if ( shift < 0 )
+  { shift = -shift;
+    dir = -dir;
+  }
+
   switch(n1->type) 
   { case V_INTEGER: 
       if ( dir < 0 )
       {
 #ifdef O_GMP				/* msb() is 0..63 */
-	if ( msb64(n1->value.i) + shift >= (sizeof(int64_t)*8-1) )
+	if ( msb64(n1->value.i) + shift >= (int)(sizeof(int64_t)*8-1) )
 	{ promoteToMPZNumber(n1);
 	  goto mpz;
 	} else
@@ -1093,8 +1117,13 @@ ar_shift(Number n1, Number n2, Number r, int dir)
       r->type = V_MPZ; 
       mpz_init(r->value.mpz); 
       if ( dir < 0 )
+      { intptr_t msb = (mpz_sizeinbase(n1->value.mpz, 2)-1+shift);
+
+	if ( spaceStack(global)-1024 < msb/8 )
+	  return (int)outOfStack((Stack)&LD->stacks.global, STACK_OVERFLOW_RAISE);
+	
 	mpz_mul_2exp(r->value.mpz, n1->value.mpz, shift); 
-      else
+      } else
 	mpz_fdiv_q_2exp(r->value.mpz, n1->value.mpz, shift); 
       succeed; 
 #endif
@@ -1204,13 +1233,13 @@ ar_pow(Number n1, Number n2, Number r)
 { 
 #ifdef O_GMP
   if ( intNumber(n1) && intNumber(n2) )
-  { unsigned long exp;
+  { uintptr_t exp;
 
     switch(n2->type)
     { case V_INTEGER:
 	if ( n2->value.i < 0 || n2->value.i > LONG_MAX )
 	  goto doreal;
-	exp = (unsigned long)n2->value.i;
+	exp = (uintptr_t)n2->value.i;
 	break;
       case V_MPZ:
 	if ( mpz_sgn(n2->value.mpz) < 0 ||
@@ -1229,7 +1258,7 @@ ar_pow(Number n1, Number n2, Number r)
     switch(n1->type)
     { case V_INTEGER:
 	if ( n1->value.i >= 0L && n1->value.i <= LONG_MAX )
-	{ mpz_ui_pow_ui(r->value.mpz, (long)n1->value.i, exp);
+	{ mpz_ui_pow_ui(r->value.mpz, (intptr_t)n1->value.i, exp);
 	  succeed;
 	} else
 	{ promoteToMPZNumber(n1);
@@ -1865,7 +1894,8 @@ ar_lsb(Number n1, Number r)
 
 static int
 popcount64(int64_t i)
-{ int c, j;
+{ int c;
+  size_t j;
   int64_t m = LL(1);
 
   for(j=0,c=0; j<sizeof(i)*8; j++, m<<=1)
@@ -2028,9 +2058,17 @@ ar_integer(Number n1, Number r)
     }
 #endif
     case V_REAL:
-    { if ( n1->value.f < PLMAXINT && n1->value.f > PLMININT )
-      { r->value.i = (n1->value.f > 0 ? (int64_t)(n1->value.f + 0.5)
-			              : (int64_t)(n1->value.f - 0.5));
+    { if ( n1->value.f <= PLMAXINT && n1->value.f >= PLMININT )
+      { if ( n1->value.f > 0 )
+	{ r->value.i = (int64_t)(n1->value.f + 0.5);
+	  if ( r->value.i < 0 )		/* Why can this happen? */
+	    r->value.i = PLMAXINT;
+	} else
+	{ r->value.i = (int64_t)(n1->value.f - 0.5);
+	  if ( r->value.i > 0 )
+	    r->value.i = PLMININT;
+	}
+
 	r->type = V_INTEGER;
 	succeed;
       }
@@ -2086,14 +2124,12 @@ ar_floor(Number n1, Number r)
     case V_REAL:
     {
 #ifdef HAVE_FLOOR
-      double f = floor(n1->value.f);
-      if ( f >= (double)PLMININT && f <= (double)PLMAXINT )
-      { r->type = V_INTEGER;
-	r->value.i = (int64_t)f;
-      } else
+      r->type = V_REAL;
+      r->value.f = floor(n1->value.f);
+      if ( !toIntegerNumber(r) )
       {
 #ifdef O_GMP
-	mpz_init_set_d(r->value.mpz, f);
+	mpz_init_set_d(r->value.mpz, n1->value.f);
 	r->type = V_MPZ;
 #else
 	return PL_error("floor", 1, NULL, ERR_EVALUATION, ATOM_int_overflow);
@@ -2147,14 +2183,12 @@ ar_ceil(Number n1, Number r)
     case V_REAL:
     {
 #ifdef HAVE_CEIL
-       double f = ceil(n1->value.f);
-       if ( f >= (double)PLMININT && f <= (double)PLMAXINT )
-       { r->type = V_INTEGER;
-	 r->value.i = (int64_t)f;
-       } else
+       r->type = V_REAL;
+       r->value.f = ceil(n1->value.f);
+       if ( !toIntegerNumber(r) )
        {
 #ifdef O_GMP
-         mpz_init_set_d(r->value.mpz, f);
+         mpz_init_set_d(r->value.mpz, n1->value.f);
 	 r->type = V_MPZ;
 #else
          return PL_error("ceil", 1, NULL, ERR_EVALUATION, ATOM_int_overflow);
@@ -2198,8 +2232,9 @@ ar_float_fractional_part(Number n1, Number r)
 #ifdef O_GMP
     case V_MPZ:
 #endif
-      cpNumber(r, n1);
-      break;
+      r->value.i = 0;
+      r->type = V_INTEGER;
+      succeed;
 #ifdef O_GMP
     case V_MPQ:
       r->type = V_MPQ;
@@ -2281,13 +2316,36 @@ ar_truncate(Number n1, Number r)
 
 static int
 ar_random(Number n1, Number r)
-{ if ( !toIntegerNumber(n1) )
+{ uint64_t bound;
+
+  if ( !toIntegerNumber(n1) )
     return PL_error("random", 1, NULL, ERR_AR_TYPE, ATOM_integer, n1);
 
-  if ( n1->value.i < 1 )
-    return mustBePositive("random", 1, n1);
+  switch(n1->type)
+  {
+#ifdef O_GMP
+    case V_MPZ:
+    { int64_t i;
 
-  r->value.i = (uint64_t)_PL_Random() % (uint64_t)n1->value.i;
+      if ( !mpz_to_int64(n1->value.mpz, &i) )
+	return PL_error("random", 1, NULL, ERR_REPRESENTATION, ATOM_integer);
+      if ( i < 1 )
+	return mustBePositive("random", 1, n1);
+      bound = (uint64_t)i;
+      break;
+    }
+#endif
+    case V_INTEGER:
+      if ( n1->value.i < 1 )
+	return mustBePositive("random", 1, n1);
+      bound = (uint64_t)n1->value.i;
+      break;
+    default:
+      assert(0);
+      fail;
+  }
+
+  r->value.i = _PL_Random() % bound;
   r->type = V_INTEGER;
 
   succeed;
@@ -2359,7 +2417,7 @@ PRED_IMPL("$arithmetic_function", 2, arithmetic_function, PL_FA_TRANSPARENT)
   ArithFunction f, a, *ap;
   Module m = NULL;
   term_t head = PL_new_term_ref();
-  int v;
+  size_t v;
   int index, rc;
 
   PL_strip_module(A1, &m, head);
@@ -2469,7 +2527,7 @@ PRED_IMPL("$prolog_arithmetic_function", 2, prolog_arithmetic_function,
       i = 0;
       break;
     case FRG_REDO:
-      i = CTX_INT;
+      i = (int)CTX_INT;
       break;
     case FRG_CUTTED:
     default:
@@ -2477,7 +2535,7 @@ PRED_IMPL("$prolog_arithmetic_function", 2, prolog_arithmetic_function,
   }
 
   tmp = PL_new_term_ref();
-  mx = entriesBuffer(function_array, ArithFunction);
+  mx = (int)entriesBuffer(function_array, ArithFunction);
 
   for( ; i<mx; i++ )
   { ArithFunction f = FunctionFromIndex(i);
@@ -2490,7 +2548,7 @@ PRED_IMPL("$prolog_arithmetic_function", 2, prolog_arithmetic_function,
 		       PL_FUNCTOR, FUNCTOR_colon2,
 		         PL_ATOM, f->module->name,
 		         PL_TERM, tmp) &&
-	 PL_unify_integer(A2, (long)f->index) )
+	 PL_unify_integer(A2, (intptr_t)f->index) )
     { if ( ++i == mx )
 	succeed;
       ForeignRedoInt(i);
@@ -2575,7 +2633,7 @@ static const ar_funcdef ar_funcdefs[] = {
 
 static int
 registerFunction(ArithFunction f, int index)
-{ int i = entriesBuffer(function_array, ArithFunction);
+{ int i = (int)entriesBuffer(function_array, ArithFunction);
 
   if ( index )
   { if ( index != i )
@@ -2604,7 +2662,7 @@ registerBuiltinFunctions()
   memset(f, 0, size * sizeof(struct arithFunction));
 
   for(d = ar_funcdefs, n=0; n<size; n++, f++, d++)
-  { int v = functorHashValue(d->functor, ARITHHASHSIZE);
+  { int v = (int)functorHashValue(d->functor, ARITHHASHSIZE);
 
     f->functor  = d->functor;
     f->function = d->function;
