@@ -1,4 +1,4 @@
-/*  $Id: pl-atom.c,v 1.69 2006/07/31 12:10:50 jan Exp $
+/*  $Id: pl-atom.c,v 1.78 2007/01/24 14:29:16 jan Exp $
 
     Part of SWI-Prolog
 
@@ -32,9 +32,9 @@ Implementation issues
 
 There are two parts in the atom   administration. One is a dynamic array
 (called buffer) atom_array, which is there to   find  the atoms back. An
-atom as it appears is a long   of the form (n<<LMASK_BITS)|TAG_ATOM. The
+atom as it appears is a intptr_t   of the form (n<<LMASK_BITS)|TAG_ATOM. The
 atom  structure  is  located  by  getting  the  n-th  pointer  from  the
-atom_array dynamic array.  See atomValue() for translating the long into
+atom_array dynamic array.  See atomValue() for translating the intptr_t into
 the address of the structure.
 
 Next, there is a hash-table, which is a normal `open' hash-table mapping
@@ -119,9 +119,9 @@ This is a hard problem. I think the   best  solution is to add something
 like PL_thread_signal_async(int tid, void (*f)(void)) and call this from
 the invoking thread on all other threads.   These  thread will then scan
 their stacks and mark any references from their. Next they can carry on,
-as long as the invoking thread keeps   the  atom mutex locked during the
+as intptr_t as the invoking thread keeps   the  atom mutex locked during the
 whole atom garbage collection process. This   implies  the thread cannot
-create any atoms as long as the collection is going on.
+create any atoms as intptr_t as the collection is going on.
 
 We do have to define some mechanism to   know  all threads are done with
 their marking.
@@ -284,7 +284,7 @@ another atom.
 
 static void
 registerAtom(Atom a)
-{ int n = entriesBuffer(&atom_array, Atom);
+{ size_t n = entriesBuffer(&atom_array, Atom);
 #ifdef O_ATOMGC				/* try to find a hole! */
   Atom *ap = baseBuffer(&atom_array, Atom);
   Atom *ep = ap+n;
@@ -295,7 +295,7 @@ registerAtom(Atom a)
     { n = p - ap;
       *p = a;
       a->atom = (n<<LMASK_BITS)|TAG_ATOM;
-      if ( indexAtom(a->atom) != (unsigned long)n )
+      if ( indexAtom(a->atom) != (uintptr_t)n )
       {	/* TBD: user-level exception */
 	fatalError("Too many (%d) atoms", n);
       }
@@ -318,7 +318,7 @@ registerAtom(Atom a)
 		 *******************************/
 
 word
-lookupBlob(const char *s, unsigned int length, PL_blob_t *type, int *new)
+lookupBlob(const char *s, size_t length, PL_blob_t *type, int *new)
 { int v0, v;
   ulong oldheap;
   Atom a;
@@ -393,7 +393,7 @@ lookupBlob(const char *s, unsigned int length, PL_blob_t *type, int *new)
   }
 
 #ifdef O_ATOMGC
-  if ( GD->atoms.margin > 0 &&
+  if ( GD->atoms.margin != 0 &&
        GD->statistics.atoms == GD->atoms.non_garbage + GD->atoms.margin )
     PL_raise(SIG_ATOM_GC);
 #endif
@@ -414,7 +414,7 @@ lookupBlob(const char *s, unsigned int length, PL_blob_t *type, int *new)
 
 
 word
-lookupAtom(const char *s, unsigned int length)
+lookupAtom(const char *s, size_t length)
 { int new;
 
   return lookupBlob(s, length, &text_atom, &new);
@@ -530,11 +530,13 @@ Mark an atom from the stacks.  We must be prepared to handle fake-atoms!
 
 void
 markAtom(atom_t a)
-{ unsigned int i = indexAtom(a);
+{ size_t i = indexAtom(a);
   Atom ap;
 
   if ( i >= entriesBuffer(&atom_array, Atom) )
     return;				/* not an atom */
+  if ( i < GD->atoms.builtin )
+    return;				/* locked range */
 
   ap = fetchBuffer(&atom_array, i, Atom);
 
@@ -557,7 +559,7 @@ know we trapped a bug.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static void
-destroyAtom(Atom *ap, unsigned long mask ARG_LD)
+destroyAtom(Atom *ap, uintptr_t mask ARG_LD)
 { Atom a = *ap;
   Atom *ap2 = &atomTable[a->hash_value & mask];
 
@@ -607,7 +609,7 @@ collectAtoms(void)
   Atom *ap  = ap0 + GD->atoms.builtin;
   Atom *ep  = ap0 + entriesBuffer(&atom_array, Atom);
   int hole_seen = FALSE;
-  unsigned long mask = atom_buckets-1;
+  uintptr_t mask = atom_buckets-1;
 
   ap--;
   while(++ap < ep)
@@ -628,7 +630,8 @@ collectAtoms(void)
 	GD->atoms.no_hole_before = ap-ap0;
       }
     } else
-      a->references &= ~ATOM_MARKED_REFERENCE;
+    { a->references &= ~ATOM_MARKED_REFERENCE;
+    }
   }
 }
 
@@ -643,9 +646,9 @@ word
 pl_garbage_collect_atoms()
 { GET_LD
   int verbose = trueFeature(TRACE_GC_FEATURE);
-  long oldcollected = GD->atoms.collected;
-  long oldheap = GD->statistics.heap;
-  long freed;
+  intptr_t oldcollected = GD->atoms.collected;
+  intptr_t oldheap = GD->statistics.heap;
+  intptr_t freed;
   double t;
   sigset_t set;
 
@@ -736,25 +739,39 @@ resetAtoms()
 
 void
 PL_register_atom(atom_t a)
-{ Atom p;
+{ 
 #ifdef O_ATOMGC
-  LOCK();
-  p = atomValue(a);
-  p->references++;
-  UNLOCK();
+  size_t index = indexAtom(a);
+
+  if ( index >= GD->atoms.builtin )
+  { Atom p;
+
+    LOCK();
+    p = fetchBuffer(&atom_array, index, Atom);
+    p->references++;
+    UNLOCK();
+  }
 #endif
 }
 
-
 void
 PL_unregister_atom(atom_t a)
-{ Atom p;
+{ 
 #ifdef O_ATOMGC
-  LOCK();
-  p = atomValue(a);
-  p->references--;
-  assert(p->references >= 0);
-  UNLOCK();
+  size_t index = indexAtom(a);
+
+  if ( index >= GD->atoms.builtin )
+  { Atom p;
+
+    LOCK();
+    p = fetchBuffer(&atom_array, index, Atom);
+    p->references--;
+    if ( p->references == (unsigned)-1 )
+    { Sdprintf("OOPS: -1 references to %s\n", p->name);
+      trap_gdb();
+    }
+    UNLOCK();
+  }
 #endif
 }
 
@@ -770,8 +787,8 @@ rehashAtoms()
 { GET_LD
   Atom *oldtab   = atomTable;
   int   oldbucks = atom_buckets;
-  long mx = entriesBuffer(&atom_array, Atom);
-  unsigned long mask;
+  intptr_t mx = entriesBuffer(&atom_array, Atom);
+  uintptr_t mask;
   Atom *ap, *ep;
 
   startCritical;
@@ -786,7 +803,7 @@ rehashAtoms()
       ap < ep;
       ap++)
   { Atom a = *ap;
-    int v = a->hash_value & mask;
+    size_t v = a->hash_value & mask;
 
     a->next = atomTable[v];
     atomTable[v] = a;
@@ -800,10 +817,10 @@ rehashAtoms()
 word
 pl_atom_hashstat(term_t idx, term_t n)
 { GET_LD
-  int i, m;
+  long i, m;
   Atom a;
   
-  if ( !PL_get_integer(idx, &i) || i < 0 || i >= atom_buckets )
+  if ( !PL_get_long(idx, &i) || i < 0 || i >= (long)atom_buckets )
     fail;
   for(m = 0, a = atomTable[i]; a; a = a->next)
     m++;
@@ -822,7 +839,7 @@ registerBuiltinAtoms()
   GD->statistics.atoms = size;
 
   for(s = atoms; *s; s++, a++)
-  { int len = strlen(*s);
+  { size_t len = strlen(*s);
     int v0 = unboundStringHashValue(*s, len);
     int v = v0 & (atom_buckets-1);
 
@@ -881,7 +898,7 @@ cleanupAtoms(void)
 
 
 static word
-current_blob(term_t a, term_t type, frg_code call, int i ARG_LD)
+current_blob(term_t a, term_t type, frg_code call, intptr_t i ARG_LD)
 { atom_t type_name = 0;
 
   switch( call )
@@ -978,7 +995,7 @@ allAlpha(const char *s)
 
 static int
 extendAtom(char *prefix, bool *unique, char *common)
-{ long i, mx = entriesBuffer(&atom_array, Atom);
+{ intptr_t i, mx = entriesBuffer(&atom_array, Atom);
   Atom a;
   bool first = TRUE;
   int lp = (int) strlen(prefix);
@@ -1042,7 +1059,7 @@ compareMatch(const void *m1, const void *m2)
 
 static bool
 extend_alternatives(char *prefix, struct match *altv, int *altn)
-{ long i, mx = entriesBuffer(&atom_array, Atom);
+{ intptr_t i, mx = entriesBuffer(&atom_array, Atom);
   Atom a;
   char *as;
   int l;
@@ -1147,7 +1164,7 @@ alnum_text(PL_chars_t *txt)
 static int
 atom_generator(PL_chars_t *prefix, PL_chars_t *hit, int state)
 { GET_LD
-  long i, mx = entriesBuffer(&atom_array, Atom);
+  intptr_t i, mx = entriesBuffer(&atom_array, Atom);
 
 #ifdef O_PLMT
   if ( !key )
@@ -1159,7 +1176,7 @@ atom_generator(PL_chars_t *prefix, PL_chars_t *hit, int state)
   else
   {
 #ifdef O_PLMT
-    i = (long)pthread_getspecific(key);
+    i = (intptr_t)pthread_getspecific(key);
 #else
     i = LD->atoms.generator;
 #endif
@@ -1215,7 +1232,7 @@ PL_atom_generator(const char *prefix, int state)
 pl_wchar_t *
 PL_atom_generator_w(const pl_wchar_t *prefix,
 		    pl_wchar_t *buffer,
-		    unsigned int buflen,
+		    size_t buflen,
 		    int state)
 { PL_chars_t txt, hit;
 
