@@ -1,4 +1,4 @@
-/*  $Id: pl-ctype.c,v 1.36 2006/07/14 12:43:22 jan Exp $
+/*  $Id: pl-ctype.c,v 1.39 2007/01/23 19:22:19 jan Exp $
 
     Part of SWI-Prolog
 
@@ -80,7 +80,7 @@ fiscsymf(wint_t chr)
 
 static int
 iseof(wint_t chr)
-{ return chr == -1;
+{ return chr == (wint_t)-1;
 }
 
 static int
@@ -105,12 +105,12 @@ isquote(wint_t chr)
 
 static int
 fupper(wint_t chr)
-{ return iswlower(chr) ? towupper(chr) : -1;
+{ return iswlower(chr) ? (int)towupper(chr) : -1;
 }
 
 static int
 flower(wint_t chr)
-{ return iswupper(chr) ? towlower(chr) : -1;
+{ return iswupper(chr) ? (int)towlower(chr) : -1;
 }
 
 static int
@@ -163,7 +163,7 @@ fdigit(wint_t chr)
 
 static int
 rdigit(wint_t d)
-{ if ( d >= 0 && d <= 9 )
+{ if ( (int)d >= 0 && d <= 9 )
     return d+'0';
   return -1;
 }
@@ -185,7 +185,7 @@ fxdigit(wint_t chr)
 
 static int
 rxdigit(wint_t d)
-{ if ( d >= 0 && d <= 9 )
+{ if ( (int)d >= 0 && d <= 9 )
     return d+'0';
   if ( d >= 10 && d <= 15 )
     return d-10+'a';
@@ -208,7 +208,7 @@ mkfunction(iswupper)
 mkfunction(iswpunct)
 mkfunction(iswspace)
 
-const static char_type char_types[] =
+static const char_type char_types[] =
 { { ATOM_alnum,		fiswalnum },
   { ATOM_alpha,		fiswalpha },
   { ATOM_csym,		fiscsym },
@@ -607,6 +607,9 @@ initLocale()
   if ( !setlocale(LC_TIME, "") )
   { DEBUG(0, Sdprintf("Failed to set LC_TIME locale\n"));
   }
+  if ( !setlocale(LC_COLLATE, "") )
+  { DEBUG(0, Sdprintf("Failed to set LC_COLLATE locale\n"));
+  }
 }
 
 typedef struct
