@@ -1,4 +1,4 @@
-/*  $Id: pl-ext.c,v 1.151 2006/08/27 12:31:14 jan Exp $
+/*  $Id: pl-ext.c,v 1.154 2007/01/13 12:49:07 jan Exp $
 
     Part of SWI-Prolog
 
@@ -33,7 +33,7 @@ See how the system reacts on segmentation faults.
 
 static word
 pl_crash()
-{ long *lp = NULL;
+{ intptr_t *lp = NULL;
 
   Sdprintf("You asked for it ... Writing to address 0\n");
 
@@ -103,7 +103,7 @@ static const PL_extension foreigns[] = {
   FRG("file_name_extension",	3, pl_file_name_extension,	0),
   FRG("prolog_to_os_filename",	2, pl_prolog_to_os_filename,	0),
   FRG("$mark_executable",	1, pl_mark_executable,		0),
-#ifdef __WIN32__
+#ifdef __WINDOWS__
   FRG("win_exec",		2, pl_win_exec,			0),
   FRG("win_module_file",	2, pl_win_module_file,		0),
 #endif
@@ -433,7 +433,7 @@ struct extension_cell
 static char *
 dupStr(const char *str)
 { if (str)
-  { int len = strlen(str)+1;
+  { size_t len = strlen(str)+1;
     char *m = PL_malloc(len);
     memcpy(m, str, len);
     return m;
@@ -590,7 +590,7 @@ initBuildIns(void)
 #ifdef O_GVAR
   REG_PLIST(gvar);
 #endif
-#ifdef __WIN32__
+#ifdef __WINDOWS__
   REG_PLIST(win);
 #endif
 
