@@ -1,4 +1,4 @@
-/*  $Id: pl-feature.c,v 1.48 2006/07/25 08:55:54 jan Exp $
+/*  $Id: pl-feature.c,v 1.55 2007/01/13 13:17:45 jan Exp $
 
     Part of SWI-Prolog
 
@@ -26,7 +26,7 @@
 #include "pl-incl.h"
 #include "pl-ctype.h"
 #include <ctype.h>
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #include <process.h>			/* getpid() */
 #endif
 
@@ -47,7 +47,7 @@ is unix.
 
 In  the  multi-threading  version,   features    have   to   be  changed
 thread-local. Therefore two feature-tables have   been defined: a global
-one which is used as long as there is   only one thread, and a local one
+one which is used as intptr_t as there is   only one thread, and a local one
 that is used to write changes to after multiple threads exist. On thread
 creation this table is copied from  the   parent  and on destruction the
 local table is destroyed.  Note  that   the  flag-mask  for  fast access
@@ -85,13 +85,13 @@ C-interface for defining features.  Depending on the type, the
 following arguments are to be provided:
 
     FT_BOOL	TRUE/FALSE, *_FEATURE
-    FT_INTEGER  long
+    FT_INTEGER  intptr_t
     FT_ATOM	const char *
     FT_TERM	a term
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static int
-indexOfBoolMask(unsigned long mask)
+indexOfBoolMask(uintptr_t mask)
 { int i=1;
   
   if ( !mask )
@@ -134,7 +134,7 @@ defFeature(const char *name, int flags, ...)
   switch(type)
   { case FT_BOOL:
     { int  val           = va_arg(args, int);
-      unsigned long mask = va_arg(args, unsigned long);
+      uintptr_t mask = va_arg(args, uintptr_t);
 
       if ( s && mask && f->index < 0 )		/* type definition */
       { f->index = indexOfBoolMask(mask);
@@ -156,7 +156,7 @@ defFeature(const char *name, int flags, ...)
       break;
     }
     case FT_INTEGER:
-    { long val = va_arg(args, long);
+    { intptr_t val = va_arg(args, intptr_t);
       f->value.i = val;
       break;
     }
@@ -341,7 +341,7 @@ set_feature_unlocked(term_t key, term_t value)
   } else				/* define new feature */
   { feature *f = allocHeap(sizeof(*f));
     atom_t a;
-    long i;
+    int64_t i;
 
     f->index = -1;
     if ( PL_get_atom(value, &a) )
@@ -351,7 +351,7 @@ set_feature_unlocked(term_t key, term_t value)
 	f->flags = FT_ATOM;
       f->value.a = a;
       PL_register_atom(a);
-    } else if ( PL_get_long(value, &i) )
+    } else if ( PL_get_int64(value, &i) )
     { f->flags = FT_INTEGER;
       f->value.i = i;
     } else
@@ -383,7 +383,7 @@ set_feature_unlocked(term_t key, term_t value)
       { return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_bool, value);
       }
       if ( f->index > 0 )
-      { unsigned long mask = 1L << (f->index-1);
+      { uintptr_t mask = 1L << (f->index-1);
 
 	if ( val )
 	  setFeatureMask(mask);
@@ -443,14 +443,14 @@ set_feature_unlocked(term_t key, term_t value)
       break;
     }
     case FT_INTEGER:
-    { long i;
+    { int64_t i;
 
-      if ( !PL_get_long(value, &i) )
+      if ( !PL_get_int64(value, &i) )
 	return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, value);
       f->value.i = i;
 #ifdef O_ATOMGC
       if ( k == ATOM_agc_margin )
-	GD->atoms.margin = i;
+	GD->atoms.margin = (size_t)i;
 #endif
       break;
     }
@@ -523,9 +523,9 @@ unify_feature_value(Module m, atom_t key, feature *f, term_t val)
   switch(f->flags & FT_MASK)
   { case FT_BOOL:
       if ( f->index >= 0 )
-      { unsigned long mask = 1L << (f->index-1);
+      { uintptr_t mask = 1L << (f->index-1);
 
-	return PL_unify_bool_ex(val, trueFeature(mask));
+	return PL_unify_bool_ex(val, trueFeature(mask) != FALSE);
       }
       /*FALLTHROUGH*/
     case FT_ATOM:
@@ -678,6 +678,10 @@ pl_feature5(term_t key, term_t value,
       { UNLOCK();
 	ForeignRedoPtr(e);
       }
+      if ( exception_term )
+      { exception_term = 0;
+	setVar(*valTermRef(exception_bin));
+      }
       Undo(m);
     }
 
@@ -710,6 +714,9 @@ pl_feature(term_t name, term_t value, control_t h)
 #ifndef SO_EXT
 #define SO_EXT "so"
 #endif
+#ifndef SO_PATH
+#define SO_PATH "LD_LIBRARY_PATH"
+#endif
 
 void
 initFeatureTable()
@@ -725,7 +732,7 @@ void
 initFeatures()
 { defFeature("iso",  FT_BOOL, FALSE, ISO_FEATURE);
   defFeature("arch", FT_ATOM|FF_READONLY, ARCH);
-#if __WIN32__
+#if __WINDOWS__
   defFeature("windows",	FT_BOOL|FF_READONLY, TRUE, 0);
 #endif
   defFeature("version",	FT_INTEGER|FF_READONLY, PLVERSION);
@@ -760,8 +767,9 @@ initFeatures()
   defFeature("agc_margin",FT_INTEGER,	       GD->atoms.margin);
 #endif
 #if defined(HAVE_DLOPEN) || defined(HAVE_SHL_LOAD) || defined(EMULATE_DLOPEN)
-  defFeature("open_shared_object",	FT_BOOL|FF_READONLY, TRUE, 0);
-  defFeature("shared_object_extension",	FT_ATOM|FF_READONLY, SO_EXT);
+  defFeature("open_shared_object",	  FT_BOOL|FF_READONLY, TRUE, 0);
+  defFeature("shared_object_extension",	  FT_ATOM|FF_READONLY, SO_EXT);
+  defFeature("shared_object_search_path", FT_ATOM|FF_READONLY, SO_PATH);
 #endif
 #if O_DYNAMIC_STACKS
   defFeature("dynamic_stacks",	FT_BOOL|FF_READONLY, TRUE, 0);
@@ -841,7 +849,7 @@ initFeatures()
 	     trueFeature(SIGNALS_FEATURE), SIGNALS_FEATURE);
   defFeature("readline", FT_BOOL/*|FF_READONLY*/, FALSE, 0);
 
-#if defined(__WIN32__) && defined(_DEBUG)
+#if defined(__WINDOWS__) && defined(_DEBUG)
   defFeature("kernel_compile_mode", FT_ATOM|FF_READONLY, "debug");
 #endif
 
