@@ -1,4 +1,4 @@
-/*  $Id: pl-file.c,v 1.193 2006/08/04 11:53:51 jan Exp $
+/*  $Id: pl-file.c,v 1.202 2007/01/23 14:22:00 jan Exp $
 
     Part of SWI-Prolog
 
@@ -84,7 +84,7 @@ standardStreamIndexFromName(atom_t name)
 
   for(ap=standardStreams; *ap; ap++)
   { if ( *ap == name )
-      return (ap - standardStreams);
+      return (int)(ap - standardStreams);
   }
 
   return -1;
@@ -341,7 +341,7 @@ initIO()
   getStreamContext(Soutput);
   getStreamContext(Serror);
   for( i=0, np = standardStreams; *np; np++, i++ )
-    addHTable(streamAliases, (void *)*np, (void *)(long)i);
+    addHTable(streamAliases, (void *)*np, (void *)(intptr_t)i);
 
   GD->io_initialised = TRUE;
 }
@@ -384,9 +384,13 @@ releaseStream(IOSTREAM *s)
 
 #endif /*O_PLMT*/
 
-void
+int
 PL_release_stream(IOSTREAM *s)
-{ releaseStream(s);
+{ if ( Sferror(s) )
+    return streamStatus(s);
+
+  releaseStream(s);
+  return TRUE;
 }
 
 
@@ -436,7 +440,7 @@ get_stream_handle__LD(term_t t, IOSTREAM **s, int flags ARG_LD)
       LOCK();
     if ( (symb=lookupHTable(streamAliases, (void *)alias)) )
     { IOSTREAM *stream;
-      unsigned long n = (unsigned long)symb->value;
+      uintptr_t n = (uintptr_t)symb->value;
 
       if ( n < 6 )			/* standard stream! */
       { stream = LD->IO.streams[n];
@@ -625,7 +629,7 @@ that ignores the error. This might get hairy if the user is playing with
 these streams too.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 static int
 isConsoleStream(IOSTREAM *s)
 { int i = standardStreamIndexFromStream(s);
@@ -822,12 +826,12 @@ openFileDescriptors(unsigned char *buf, int size)
 
 
 void
-protocol(const char *str, int n)
+protocol(const char *str, size_t n)
 { GET_LD
   IOSTREAM *s;
 
   if ( LD && (s = getStream(Sprotocol)) )
-  { while( --n >= 0 )
+  { while( n-- > 0 )
       Sputcode(*str++&0xff, s);
     releaseStream(s);			/* we don not check errors */
   }
@@ -1420,7 +1424,7 @@ error:
 extern IOFUNCTIONS Smemfunctions;
 
 bool
-tellString(char **s, int *size, IOENC enc)
+tellString(char **s, size_t *size, IOENC enc)
 { GET_LD
   IOSTREAM *stream;
   
@@ -1528,7 +1532,7 @@ pl_wait_for_input(term_t Streams, term_t Available,
     e->next   = map;
     map       = e;
 
-#ifdef WIN32
+#ifdef __WINDOWS__
     FD_SET((SOCKET)fd, &fds);
 #else
     FD_SET(fd, &fds);
@@ -1587,7 +1591,7 @@ pl_wait_for_input(term_t Streams, term_t Available,
     FD_ZERO(&fds);			/* EINTR may leave fds undefined */
     for(e=map; e; e=e->next)		/* so we rebuild it to be safe */
     {
-#ifdef WIN32
+#ifdef __WINDOWS__
       FD_SET((SOCKET)e->fd, &fds);
 #else
       FD_SET(e->fd, &fds);
@@ -1632,7 +1636,7 @@ PRED_IMPL("read_pending_input", 3, read_pending_input, 0)
 
   if ( getInputStream(A1, &s) )
   { char buf[MAX_PENDING];
-    int n, i;
+    ssize_t n, i;
     Word gstore, lp, tp;
 
     if ( Sferror(s) )
@@ -2579,14 +2583,14 @@ pl_told()
 		 *	   NULL-STREAM		*
 		 *******************************/
 
-static int
-Swrite_null(void *handle, char *buf, int size)
+static ssize_t
+Swrite_null(void *handle, char *buf, size_t size)
 { return size;
 }
 
 
-static int
-Sread_null(void *handle, char *buf, int size)
+static ssize_t
+Sread_null(void *handle, char *buf, size_t size)
 { return 0;
 }
 
@@ -3262,13 +3266,14 @@ pl_seek(term_t stream, term_t offset, term_t method, term_t newloc)
     return PL_error("seek", 4, NULL, ERR_DOMAIN, ATOM_integer, offset);
 
   if ( PL_get_stream_handle(stream, &s) )
-  { if ( (new = Sseek64(s, off, whence)) < 0 )
+  { if ( Sseek64(s, off, whence) < 0 )
     { PL_error("seek", 4, OsError(), ERR_PERMISSION,
 	       ATOM_reposition, ATOM_stream, stream);
       releaseStream(s);
       fail;
     }
 
+    new = Stell64(s);
     releaseStream(s);
     return PL_unify_int64(newloc, new);
   }
@@ -3357,7 +3362,7 @@ PRED_IMPL("line_count", 2, line_count, 0)
   IOSTREAM *s;
 
   if ( getStreamWithPosition(A1, &s) )
-  { long n = s->position->lineno;
+  { intptr_t n = s->position->lineno;
 
     releaseStream(s);
     return PL_unify_integer(A2, n);
@@ -3373,7 +3378,7 @@ PRED_IMPL("line_position", 2, line_position, 0)
   IOSTREAM *s;
 
   if ( getStreamWithPosition(A1, &s) )
-  { long n = s->position->linepos;
+  { intptr_t n = s->position->linepos;
 
     releaseStream(s);
     return PL_unify_integer(A2, n);
@@ -3507,8 +3512,8 @@ typedef struct wrappedIO
 } wrappedIO;
 
 
-int
-Sread_user(void *handle, char *buf, int size)
+ssize_t
+Sread_user(void *handle, char *buf, size_t size)
 { GET_LD
   wrappedIO *wio = handle;
 
@@ -3548,8 +3553,8 @@ closeWrappedIO(void *handle)
 
 static void
 wrapIO(IOSTREAM *s,
-       int (*read)(void *, char *, int),
-       int (*write)(void *, char *, int))
+       ssize_t (*read)(void *, char *, size_t),
+       ssize_t (*write)(void *, char *, size_t))
 { wrappedIO *wio = PL_malloc(sizeof(*wio));
 
   wio->wrapped_functions = s->functions;
@@ -3618,7 +3623,7 @@ out:
 		*********************************/
 
 bool
-unifyTime(term_t t, long time)
+unifyTime(term_t t, intptr_t time)
 { return PL_unify_float(t, (double)time);
 }
 
@@ -3717,7 +3722,7 @@ pl_time_file(term_t name, term_t t)
 { char *fn;
 
   if ( PL_get_file_name(name, &fn, 0) )
-  { long time;
+  { intptr_t time;
 
     if ( (time = LastModifiedFile(fn)) == -1 )
       fail;
@@ -4192,9 +4197,9 @@ pl_copy_stream_data3(term_t in, term_t out, term_t len)
       }
     }
   } else
-  { long n;
+  { int64_t n;
 
-    if ( !PL_get_long_ex(len, &n) )
+    if ( !PL_get_int64_ex(len, &n) )
       fail;
     
     while ( n-- > 0 && (c = Sgetcode(i)) != EOF )
