@@ -1,4 +1,4 @@
-/*  $Id: pl-fli.c,v 1.164 2006/01/20 13:07:48 jan Exp $
+/*  $Id: pl-fli.c,v 1.176 2007/01/23 09:58:02 jan Exp $
 
     Part of SWI-Prolog
 
@@ -38,7 +38,7 @@
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SWI-Prolog  new-style  foreign-language  interface.   This  new  foreign
 interface is a mix of the old  interface using the ideas on term-handles
-from  Quintus  Prolog.  Term-handles  are    integers  (unsigned  long),
+from  Quintus  Prolog.  Term-handles  are    integers  (uintptr_t),
 describing the offset of the term-location relative   to the base of the
 local stack.
 
@@ -75,7 +75,7 @@ Prolog int) is used by the garbage collector to update the stack frames.
 #define valHandleP(h)		valTermRef(h)
 
 #undef ulong
-#define ulong unsigned long
+#define ulong uintptr_t
 
 static inline word
 valHandle__LD(term_t r ARG_LD)
@@ -205,7 +205,7 @@ PL_reset_term_refs(term_t r)
   FliFrame fr = fli_context;
 
   lTop = (LocalFrame) valTermRef(r);
-  fr->size = (Word) lTop - (Word)addPointer(fr, sizeof(struct fliFrame));
+  fr->size = (int)((Word) lTop - (Word)addPointer(fr, sizeof(struct fliFrame)));
   SECURE(if ( fr->size < 0 || fr->size > 100 )
 	   Sdprintf("Suspect foreign frame size: %d\n", fr->size));
 }
@@ -283,7 +283,7 @@ PL_new_atom(const char *s)
 
 
 atom_t
-PL_new_atom_nchars(unsigned int len, const char *s)
+PL_new_atom_nchars(size_t len, const char *s)
 { if ( !GD->initialised )
     initAtoms();
 
@@ -347,7 +347,7 @@ isUCSAtom(Atom a)
 
 
 atom_t
-lookupUCSAtom(const pl_wchar_t *s, unsigned int len)
+lookupUCSAtom(const pl_wchar_t *s, size_t len)
 { int new;
 
   return lookupBlob((const char *)s, len*sizeof(pl_wchar_t),
@@ -356,7 +356,7 @@ lookupUCSAtom(const pl_wchar_t *s, unsigned int len)
 
 
 atom_t
-PL_new_atom_wchars(unsigned int len, const wchar_t *s)
+PL_new_atom_wchars(size_t len, const wchar_t *s)
 { PL_chars_t txt;
 
   if ( !GD->initialised )
@@ -422,7 +422,7 @@ compareUCSAtom(atom_t h1, atom_t h2)
   Atom a2 = atomValue(h2);
   const pl_wchar_t *s1 = (const pl_wchar_t*)a1->name;
   const pl_wchar_t *s2 = (const pl_wchar_t*)a2->name;
-  unsigned len = a1->length < a2->length ? a1->length : a2->length;
+  size_t len = a1->length < a2->length ? a1->length : a2->length;
 
   len /= sizeof(pl_wchar_t);
   
@@ -431,7 +431,8 @@ compareUCSAtom(atom_t h1, atom_t h2)
       return *s1 - *s2;
   }
 
-  return a1->length - a2->length;
+  return a1->length >  a2->length ? 1 :
+	 a1->length == a2->length ? 0 : -1;
 }
 
 
@@ -451,7 +452,7 @@ static atom_t
 loadUCSAtom(IOSTREAM *fd)
 { pl_wchar_t buf[256];
   pl_wchar_t *w;
-  unsigned len;
+  size_t len;
   atom_t a;
 
   w = wicGetStringUTF8(fd, &len, buf, sizeof(buf)/sizeof(pl_wchar_t));
@@ -465,11 +466,11 @@ loadUCSAtom(IOSTREAM *fd)
 
 
 int
-PL_unify_wchars(term_t t, int flags, unsigned int len, const pl_wchar_t *s)
+PL_unify_wchars(term_t t, int flags, size_t len, const pl_wchar_t *s)
 { PL_chars_t text;
   int rc;
 
-  if ( len == (unsigned int)-1 )
+  if ( len == (size_t)-1 )
     len = wcslen(s);
 
   text.text.w    = (pl_wchar_t *)s;
@@ -487,11 +488,11 @@ PL_unify_wchars(term_t t, int flags, unsigned int len, const pl_wchar_t *s)
 
 int
 PL_unify_wchars_diff(term_t t, term_t tail, int flags,
-		     unsigned int len, const pl_wchar_t *s)
+		     size_t len, const pl_wchar_t *s)
 { PL_chars_t text;
   int rc;
 
-  if ( len == (unsigned int)-1 )
+  if ( len == (size_t)-1 )
     len = wcslen(s);
 
   text.text.w    = (pl_wchar_t *)s;
@@ -507,8 +508,8 @@ PL_unify_wchars_diff(term_t t, term_t tail, int flags,
 }
 
 
-unsigned int
-PL_utf8_strlen(const char *s, unsigned int len)
+size_t
+PL_utf8_strlen(const char *s, size_t len)
 { return utf8_strlen(s, len);
 }
 
@@ -524,7 +525,7 @@ PL_atom_chars(atom_t a)
 
 
 const char *
-PL_atom_nchars(atom_t a, unsigned int *len)
+PL_atom_nchars(atom_t a, size_t *len)
 { Atom x = atomValue(a);
 
   if ( x->type != &ucs_atom )
@@ -538,7 +539,7 @@ PL_atom_nchars(atom_t a, unsigned int *len)
 
 
 const wchar_t *
-PL_atom_wchars(atom_t a, unsigned int *len)
+PL_atom_wchars(atom_t a, size_t *len)
 { Atom x = atomValue(a);
 
   if ( x->type == &ucs_atom )
@@ -596,7 +597,7 @@ PL_cvt_i_atom(term_t p, atom_t *c)
 
 
 bool
-PL_cvt_o_integer(long c, term_t p)
+PL_cvt_o_integer(intptr_t c, term_t p)
 { GET_LD
   return PL_unify_integer(p, c);
 }
@@ -774,20 +775,20 @@ normally 0). longToPointer() does the inverse operation.
 
 static inline ulong
 pointerToLong(void *ptr)
-{ ulong p   = (ulong) ptr;
-  ulong low = p & 0x3L;
+{ uintptr_t p   = (uintptr_t) ptr;
+  uintptr_t low = p & 0x3L;
 
   p -= heap_base;
   p >>= 2;
-  p |= low<<(sizeof(ulong)*8-2);
+  p |= low<<(sizeof(uintptr_t)*8-2);
   
   return p;
 }
 
 
 static inline void *
-longToPointer(ulong p)
-{ ulong low = p >> (sizeof(ulong)*8-2);
+longToPointer(uintptr_t p)
+{ uintptr_t low = p >> (sizeof(uintptr_t)*8-2);
 
   p <<= 2;
   p |= low;
@@ -905,7 +906,7 @@ PL_get_atom_chars(term_t t, char **s)
 
 
 int
-PL_get_atom_nchars(term_t t, unsigned int *len, char **s)
+PL_get_atom_nchars(term_t t, size_t *len, char **s)
 { GET_LD
   word w = valHandle(t);
 
@@ -926,7 +927,7 @@ PL_get_atom_nchars(term_t t, unsigned int *len, char **s)
 
 #ifdef O_STRING
 int
-PL_get_string(term_t t, char **s, unsigned int *len)
+PL_get_string(term_t t, char **s, size_t *len)
 { GET_LD
   word w = valHandle(t);
 
@@ -969,7 +970,7 @@ findBuffer(int flags)
 char *
 buffer_string(const char *s, int flags)
 { Buffer b = findBuffer(flags);
-  int l = strlen(s) + 1;
+  size_t l = strlen(s) + 1;
 
   addMultipleBuffer(b, s, l, char);
 
@@ -1027,7 +1028,7 @@ codes_or_chars_to_buffer(term_t l, unsigned int flags, int wide)
   { arg = argTermP(list, 0);
     deRef(arg);
     if ( isTaggedInt(*arg) )
-    { int i = valInt(*arg);
+    { intptr_t i = valInt(*arg);
       if ( i >= 0 && (wide || i < 256) )
       { type = CODES;
 	goto ok;
@@ -1046,7 +1047,7 @@ ok:
   b = findBuffer(flags);
 
   while( isList(list) )
-  { int c = -1;
+  { intptr_t c = -1;
 
     arg = argTermP(list, 0);
     deRef(arg);
@@ -1063,14 +1064,14 @@ ok:
     }
 
     if ( c < 0 || (!wide && c > 0xff) )
-    { unfindBuffer(flags);
+    { unfindBuffer(flags);		/* TBD: check unicode range */
       return NULL;
     }
 
     if ( wide )
-      addBuffer(b, c, pl_wchar_t);
+      addBuffer(b, (pl_wchar_t)c, pl_wchar_t);
     else
-      addBuffer(b, c, char);
+      addBuffer(b, (unsigned char)c, unsigned char);
 
     tail = argTermP(list, 1);
     deRef(tail);
@@ -1086,13 +1087,12 @@ ok:
 
 
 int
-PL_get_list_nchars(term_t l,
-		   unsigned int *length, char **s, unsigned int flags)
+PL_get_list_nchars(term_t l, size_t *length, char **s, unsigned int flags)
 { Buffer b;
 
   if ( (b = codes_or_chars_to_buffer(l, flags, FALSE)) )
   { char *r;
-    int len = entriesBuffer(b, char);
+    size_t len = entriesBuffer(b, char);
 
     if ( length )
       *length = len;
@@ -1120,7 +1120,7 @@ PL_get_list_chars(term_t l, char **s, unsigned flags)
 
 
 int
-PL_get_wchars(term_t l, unsigned int *length, pl_wchar_t **s, unsigned flags)
+PL_get_wchars(term_t l, size_t *length, pl_wchar_t **s, unsigned flags)
 { PL_chars_t text;
 
   if ( !PL_get_text(l, &text, flags) )
@@ -1138,7 +1138,7 @@ PL_get_wchars(term_t l, unsigned int *length, pl_wchar_t **s, unsigned flags)
 
 
 int
-PL_get_nchars(term_t l, unsigned int *length, char **s, unsigned flags)
+PL_get_nchars(term_t l, size_t *length, char **s, unsigned flags)
 { PL_chars_t text;
 
   if ( !PL_get_text(l, &text, flags) )
@@ -1188,7 +1188,11 @@ PL_get_integer__LD(term_t t, int *i ARG_LD)
 { word w = valHandle(t);
   
   if ( isTaggedInt(w) )
-  { *i = valInt(w);
+  { intptr_t val = valInt(w);
+
+    if ( val > INT_MAX || val < INT_MIN )
+      fail;
+    *i = (int)val;
     succeed;
   }
   if ( isBignum(w) )
@@ -1233,7 +1237,11 @@ PL_get_long__LD(term_t t, long *i ARG_LD)
 { word w = valHandle(t);
   
   if ( isTaggedInt(w) )
-  { *i = valInt(w);
+  { intptr_t val = valInt(w);
+
+    if ( val > LONG_MAX || val < LONG_MIN )
+      fail;
+    *i = (long)val;
     succeed;
   }
   if ( isBignum(w) )
@@ -1274,9 +1282,8 @@ PL_get_long(term_t t, long *i)
 
 
 int
-PL_get_int64(term_t t, int64_t *i)
-{ GET_LD
-  word w = valHandle(t);
+PL_get_int64__LD(term_t t, int64_t *i ARG_LD)
+{ word w = valHandle(t);
   
   if ( isTaggedInt(w) )
   { *i = valInt(w);
@@ -1306,6 +1313,15 @@ PL_get_int64(term_t t, int64_t *i)
 }
 
 
+#undef PL_get_int64
+int
+PL_get_int64(term_t t, int64_t *i)
+{ GET_LD
+  return PL_get_int64__LD(t, i PASS_LD);
+}
+#define PL_get_int64(t, i) PL_get_int64__LD(t, i PASS_LD)
+
+
 int
 PL_is_inf(term_t t)
 { GET_LD
@@ -1345,7 +1361,7 @@ PL_get_pointer__LD(term_t t, void **ptr ARG_LD)
 { int64_t p;
 
   if ( PL_get_int64(t, &p) )
-  { *ptr = longToPointer((ulong)p);
+  { *ptr = longToPointer((uintptr_t)p);
 
     succeed;
   }
@@ -1582,7 +1598,7 @@ _PL_get_xpce_reference(term_t t, xpceref_t *ref)
       }
       if ( isBignum(*p) )
       { ref->type    = PL_INTEGER;
-	ref->value.i = (long)valBignum(*p);
+	ref->value.i = (intptr_t)valBignum(*p);
 
 	goto ok;
       }
@@ -1816,7 +1832,7 @@ PL_unify_string_chars(term_t t, const char *s)
 }
 
 int
-PL_unify_string_nchars(term_t t, unsigned int len, const char *s)
+PL_unify_string_nchars(term_t t, size_t len, const char *s)
 { GET_LD
   word str = globalString(len, s);
 
@@ -1865,7 +1881,7 @@ PL_put_atom_chars(term_t t, const char *s)
 
 
 void
-PL_put_atom_nchars(term_t t, unsigned int len, const char *s)
+PL_put_atom_nchars(term_t t, size_t len, const char *s)
 { GET_LD
   atom_t a = lookupAtom(s, len);
 
@@ -1884,7 +1900,7 @@ PL_put_string_chars(term_t t, const char *s)
 
 
 void
-PL_put_string_nchars(term_t t, unsigned int len, const char *s)
+PL_put_string_nchars(term_t t, size_t len, const char *s)
 { GET_LD
   word w = globalString(len, s);
 
@@ -1893,7 +1909,7 @@ PL_put_string_nchars(term_t t, unsigned int len, const char *s)
 
 
 void
-PL_put_list_ncodes(term_t t, unsigned int len, const char *chars)
+PL_put_list_ncodes(term_t t, size_t len, const char *chars)
 { GET_LD
   
   if ( len == 0 )
@@ -1904,7 +1920,7 @@ PL_put_list_ncodes(term_t t, unsigned int len, const char *chars)
 
     for( ; len-- != 0; chars++)
     { *p++ = FUNCTOR_dot2;
-      *p++ = consInt((long)*chars & 0xff);
+      *p++ = consInt((intptr_t)*chars & 0xff);
       *p = consPtr(p+1, TAG_COMPOUND|STG_GLOBAL);
       p++;
     }
@@ -1920,7 +1936,7 @@ PL_put_list_codes(term_t t, const char *chars)
 
 
 void
-PL_put_list_nchars(term_t t, unsigned int len, const char *chars)
+PL_put_list_nchars(term_t t, size_t len, const char *chars)
 { GET_LD
   
   if ( len == 0 )
@@ -1947,14 +1963,14 @@ PL_put_list_chars(term_t t, const char *chars)
 
 
 void
-PL_put_integer__LD(term_t t, long i ARG_LD)
+PL_put_integer__LD(term_t t, intptr_t i ARG_LD)
 { setHandle(t, makeNum(i));
 }
 
 
 #undef PL_put_integer
 void
-PL_put_integer(term_t t, long i)
+PL_put_integer(term_t t, intptr_t i)
 { GET_LD
   setHandle(t, makeNum(i));
 }
@@ -2045,7 +2061,7 @@ PL_put_term(term_t t1, term_t t2)
 
 
 void
-_PL_put_xpce_reference_i(term_t t, unsigned long r)
+_PL_put_xpce_reference_i(term_t t, uintptr_t r)
 { GET_LD
   Word a = allocGlobal(2);
 
@@ -2099,7 +2115,7 @@ PL_unify_functor(term_t t, functor_t f)
     } else
     { 
 #ifdef O_SHIFT_STACKS
-      if ( roomStack(global) < (1+arity) * (long)sizeof(word) )
+      if ( roomStack(global) < (1+arity) * (intptr_t)sizeof(word) )
       { growStacks(environment_frame, NULL, NULL, FALSE, TRUE, FALSE);
 	p = valHandleP(t);
 	deRef(p);
@@ -2109,14 +2125,14 @@ PL_unify_functor(term_t t, functor_t f)
 #endif
 
       { Word a = gTop;
-	word t = consPtr(a, TAG_COMPOUND|STG_GLOBAL);
+	word to = consPtr(a, TAG_COMPOUND|STG_GLOBAL);
 
 	gTop += 1+arity;
 	*a = f;
 	while( --arity >= 0 )
 	  setVar(*++a);
 
-	bindConst(p, t);
+	bindConst(p, to);
       }
     }
 
@@ -2148,7 +2164,7 @@ PL_unify_atom_chars(term_t t, const char *chars)
 
 
 int
-PL_unify_atom_nchars(term_t t, unsigned int len, const char *chars)
+PL_unify_atom_nchars(term_t t, size_t len, const char *chars)
 { GET_LD
   atom_t a = lookupAtom(chars, len);
   int rval = unifyAtomic(t, a PASS_LD);
@@ -2160,17 +2176,17 @@ PL_unify_atom_nchars(term_t t, unsigned int len, const char *chars)
 
 
 static atom_t
-uncachedCodeToAtom(int code)
-{ if ( code < 256 )
+uncachedCodeToAtom(int chrcode)
+{ if ( chrcode < 256 )
   { char tmp[1];
 
-    tmp[0] = code;
+    tmp[0] = chrcode;
     return lookupAtom(tmp, 1);
   } else
   { pl_wchar_t tmp[1];
     int new;
 
-    tmp[0] = code;
+    tmp[0] = chrcode;
 
     return lookupBlob((const char *)tmp, sizeof(pl_wchar_t),
 		      &ucs_atom, &new);
@@ -2179,17 +2195,17 @@ uncachedCodeToAtom(int code)
 
 
 atom_t
-codeToAtom(int code)
+codeToAtom(int chrcode)
 { atom_t a;
 
-  if ( code == EOF )
+  if ( chrcode == EOF )
     return ATOM_end_of_file;
 
-  assert(code >= 0);
+  assert(chrcode >= 0);
 
-  if ( code < (1<<15) )
-  { int page  = code / 256;
-    int entry = code % 256;
+  if ( chrcode < (1<<15) )
+  { int page  = chrcode / 256;
+    int entry = chrcode % 256;
     atom_t *pv;
 
     if ( !(pv=GD->atoms.for_code[page]) )
@@ -2200,10 +2216,10 @@ codeToAtom(int code)
     }
 
     if ( !(a=pv[entry]) )
-    { a = uncachedCodeToAtom(code);
+    { a = uncachedCodeToAtom(chrcode);
     }
   } else
-  { a = uncachedCodeToAtom(code);
+  { a = uncachedCodeToAtom(chrcode);
   }
   
   return a;
@@ -2211,7 +2227,7 @@ codeToAtom(int code)
 
 
 int
-PL_unify_list_ncodes(term_t l, unsigned int len, const char *chars)
+PL_unify_list_ncodes(term_t l, size_t len, const char *chars)
 { GET_LD
   if ( PL_is_variable(l) )
   { term_t tmp = PL_new_term_ref();
@@ -2244,7 +2260,7 @@ PL_unify_list_codes(term_t l, const char *chars)
 
 
 int
-PL_unify_list_nchars(term_t l, unsigned int len, const char *chars)
+PL_unify_list_nchars(term_t l, size_t len, const char *chars)
 { GET_LD
   if ( PL_is_variable(l) )
   { term_t tmp = PL_new_term_ref();
@@ -2284,11 +2300,11 @@ flags: bitwise or of type and representation
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 int
-PL_unify_chars(term_t t, int flags, unsigned int len, const char *s)
+PL_unify_chars(term_t t, int flags, size_t len, const char *s)
 { PL_chars_t text;
   int rc;
 
-  if ( len == (unsigned int)-1 )
+  if ( len == (size_t)-1 )
     len = strlen(s);
 
   text.text.t    = (char *)s;
@@ -2308,14 +2324,14 @@ PL_unify_chars(term_t t, int flags, unsigned int len, const char *s)
 
 
 int
-PL_unify_integer__LD(term_t t, long i ARG_LD)
+PL_unify_integer__LD(term_t t, intptr_t i ARG_LD)
 { return unifyAtomic(t, makeNum(i) PASS_LD);
 }
 
 
 #undef PL_unify_integer
 int
-PL_unify_integer(term_t t, long i)
+PL_unify_integer(term_t t, intptr_t i)
 { GET_LD
   return unifyAtomic(t, makeNum(i) PASS_LD);
 }
@@ -2385,7 +2401,7 @@ PL_unify_list__LD(term_t l, term_t h, term_t t ARG_LD)
   { Word a;
 
 #ifdef O_SHIFT_STACKS
-    if ( roomStack(global) < (long)(3 * sizeof(word)) )
+    if ( roomStack(global) < (intptr_t)(3 * sizeof(word)) )
     { growStacks(environment_frame, NULL, NULL, FALSE, TRUE, FALSE);
       p = valHandleP(t);
       deRef(p);
@@ -2521,7 +2537,7 @@ cont:
       rval = PL_unify_atom_chars(t, va_arg(args, const char *));
       break;
     case PL_NCHARS:
-    { unsigned int len = va_arg(args, unsigned int);
+    { size_t len = va_arg(args, size_t);
       const char *s = va_arg(args, const char *);
 
       rval = PL_unify_atom_nchars(t, len, s);
@@ -2546,7 +2562,7 @@ cont:
     case PL_NUTF8_STRING:
     { PL_chars_t txt;
 
-      txt.length    = va_arg(args, unsigned int);
+      txt.length    = va_arg(args, size_t);
       txt.text.t    = va_arg(args, char *);
       txt.storage   = PL_CHARS_HEAP;
       txt.encoding  = ENC_UTF8;
@@ -2563,7 +2579,7 @@ cont:
     case PL_NWSTRING:
     { PL_chars_t txt;
 
-      txt.length    = va_arg(args, unsigned int);
+      txt.length    = va_arg(args, size_t);
       txt.text.w    = va_arg(args, wchar_t *);
       txt.storage   = PL_CHARS_HEAP;
       txt.encoding  = ENC_WCHAR;
@@ -2789,7 +2805,7 @@ _PL_copy_atomic(term_t t, PL_atomic_t arg) /* internal one */
 		 *******************************/
 
 int
-PL_unify_blob(term_t t, void *blob, unsigned int len, PL_blob_t *type)
+PL_unify_blob(term_t t, void *blob, size_t len, PL_blob_t *type)
 { GET_LD
   int new;
   atom_t a = lookupBlob(blob, len, type, &new);
@@ -2802,7 +2818,7 @@ PL_unify_blob(term_t t, void *blob, unsigned int len, PL_blob_t *type)
 
 
 int
-PL_put_blob(term_t t, void *blob, unsigned int len, PL_blob_t *type)
+PL_put_blob(term_t t, void *blob, size_t len, PL_blob_t *type)
 { GET_LD
   int new;
   atom_t a = lookupBlob(blob, len, type, &new);
@@ -2815,7 +2831,7 @@ PL_put_blob(term_t t, void *blob, unsigned int len, PL_blob_t *type)
 
 
 int
-PL_get_blob(term_t t, void **blob, unsigned int *len, PL_blob_t **type)
+PL_get_blob(term_t t, void **blob, size_t *len, PL_blob_t **type)
 { GET_LD
   word w = valHandle(t);
 
@@ -2837,7 +2853,7 @@ PL_get_blob(term_t t, void **blob, unsigned int *len, PL_blob_t **type)
 
 
 void *
-PL_blob_data(atom_t a, unsigned int *len, PL_blob_t **type)
+PL_blob_data(atom_t a, size_t *len, PL_blob_t **type)
 { Atom x = atomValue(a);
 
   if ( len )
@@ -3029,7 +3045,7 @@ PL_call(term_t t, Module m)
 		********************************/
 
 foreign_t
-_PL_retry(long v)
+_PL_retry(intptr_t v)
 { ForeignRedoInt(v);
 }
 
@@ -3043,7 +3059,7 @@ _PL_retry_address(void *v)
 }
 
 
-long
+intptr_t
 PL_foreign_context(control_t h)
 { return ForeignContextInt(h);
 }
@@ -3068,7 +3084,7 @@ find_query(LocalFrame fr)
     while(fr->parent)
       fr = fr->parent;
     
-    qf = (QueryFrame)addPointer(fr, -(long)offsetof(struct queryFrame, frame));
+    qf = (QueryFrame)addPointer(fr, -(intptr_t)offsetof(struct queryFrame, frame));
 
     return qf;
   }
@@ -3097,7 +3113,7 @@ PL_throw(term_t exception)
   assert(exception_term);
 
   if ( QF )
-    longjmp(QF->exception_jmp_env, exception_term);
+    longjmp(QF->exception_jmp_env, 1);
 
   fail;
 }
@@ -3486,7 +3502,7 @@ PL_prompt_string(int fd)
 
 
 void
-PL_add_to_protocol(const char *buf, int n)
+PL_add_to_protocol(const char *buf, size_t n)
 { protocol(buf, n);
 }
 
@@ -3507,9 +3523,9 @@ PL_dispatch_hook(PL_dispatch_hook_t hook)
 }
 
 
-#if defined(HAVE_SELECT) && !defined(WIN32)
+#if defined(HAVE_SELECT) && !defined(__WINDOWS__)
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 #include <winsock2.h>
 #endif
 
@@ -3621,7 +3637,7 @@ PL_set_feature(const char *name, int type, ...)
       break;
     }
     case PL_INTEGER:
-    { long v = va_arg(args, long);
+    { intptr_t v = va_arg(args, intptr_t);
       defFeature(name, FT_INTEGER, v);
       break;
     }
@@ -3672,7 +3688,7 @@ PL_action(int action, ...)
 
   switch(action)
   { case PL_ACTION_TRACE:
-      rval = pl_trace();
+      rval = (int)pl_trace();
       break;
     case PL_ACTION_DEBUG:
       debugmode(DBG_ALL, NULL);
@@ -3706,7 +3722,7 @@ PL_action(int action, ...)
 #endif
       break;
     case PL_ACTION_BREAK:
-      rval = pl_break();
+      rval = (int)pl_break();
       break;
     case PL_ACTION_HALT:
     { int a = va_arg(args, int);
@@ -3716,7 +3732,7 @@ PL_action(int action, ...)
       break;
     }
     case PL_ACTION_ABORT:
-      rval = pl_abort(ABORT_NORMAL);
+      rval = (int)pl_abort(ABORT_NORMAL);
       break;
     case PL_ACTION_GUIAPP:
     { int guiapp = va_arg(args, int);
@@ -3806,15 +3822,15 @@ init_c_args()
 }
 
 
-long
+intptr_t
 PL_query(int query)
 { switch(query)
   { case PL_QUERY_ARGC:
       init_c_args();
-      return (long) c_argc;
+      return (intptr_t) c_argc;
     case PL_QUERY_ARGV:
       init_c_args();
-      return (long) c_argv;
+      return (intptr_t) c_argv;
     case PL_QUERY_MAX_INTEGER:
     case PL_QUERY_MIN_INTEGER:
       fail;				/* cannot represent (anymore) */
@@ -3824,7 +3840,7 @@ PL_query(int query)
       return PLMINTAGGEDINT;
     case PL_QUERY_GETC:
       PopTty(Sinput, &ttytab);		/* restore terminal mode */
-      return (long) Sgetchar();		/* normal reading */
+      return (intptr_t) Sgetchar();		/* normal reading */
     case PL_QUERY_VERSION:
       return PLVERSION;
     case PL_QUERY_MAX_THREADS:
@@ -3842,7 +3858,7 @@ PL_query(int query)
     }
     case PL_QUERY_USER_CPU:		/* User CPU in milliseconds */
     { double cpu = CpuTime(CPU_USER);
-      return (long)(cpu*1000.0);
+      return (intptr_t)(cpu*1000.0);
     }
     default:
       sysError("PL_query: Illegal query: %d", query);
