@@ -1,4 +1,4 @@
-/*  $Id: pl-fmt.c,v 1.67 2006/07/07 12:51:30 jan Exp $
+/*  $Id: pl-fmt.c,v 1.72 2007/01/23 14:22:00 jan Exp $
 
     Part of SWI-Prolog
 
@@ -39,8 +39,8 @@ static char *	formatFloat(int how, int arg, Number f, Buffer out);
 #define MAXRUBBER 100
 
 struct rubber
-{ int where;				/* where is rubber in output */
-  int size;				/* how big should it be */
+{ size_t where;				/* where is rubber in output */
+  size_t size;				/* how big should it be */
   pl_wchar_t pad;			/* padding character */
 };
 
@@ -48,7 +48,7 @@ typedef struct
 { IOSTREAM *out;			/* our output stream */
   int column;				/* current column */
   tmp_buffer buffer;			/* bin for characters with tabs */
-  int buffered;				/* characters in buffer */
+  size_t buffered;			/* characters in buffer */
   int pending_rubber;			/* number of not-filled ~t's */
   struct rubber rub[MAXRUBBER];
 } format_state;
@@ -119,7 +119,7 @@ arguments.  No fuzz with wide characters here.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static int
-outstring(format_state *state, const char *s, unsigned int len)
+outstring(format_state *state, const char *s, size_t len)
 { const char *q;
   const char *e = &s[len];
 
@@ -147,7 +147,7 @@ outstring0(format_state *state, const char *s)
 
 
 static int
-oututf8(format_state *state, const char *s, unsigned int len)
+oututf8(format_state *state, const char *s, size_t len)
 { const char *e = &s[len];
 
   while(s<e)
@@ -217,10 +217,10 @@ pl_format_predicate(term_t chr, term_t descr)
   if ( !format_predicates )
     format_predicates = newHTable(8);
   
-  if ( (s = lookupHTable(format_predicates, (void *)(long)c)) )
+  if ( (s = lookupHTable(format_predicates, (void *)(intptr_t)c)) )
     s->value = proc;
   else
-    addHTable(format_predicates, (void *)(long)c, proc);
+    addHTable(format_predicates, (void *)(intptr_t)c, proc);
 
   succeed;
 }
@@ -251,7 +251,7 @@ pl_current_format_predicate(term_t chr, term_t descr, control_t h)
   while( (s=advanceTableEnum(e)) )
   { Mark(m);
 
-    if ( PL_unify_integer(chr, (long)s->name) &&
+    if ( PL_unify_integer(chr, (intptr_t)s->name) &&
 	 unify_definition(descr, ((Procedure)s->value)->definition, 0, 0) )
     { ForeignRedoPtr(e);
     }
@@ -307,7 +307,7 @@ format_impl(IOSTREAM *out, term_t format, term_t Args)
 word
 pl_format3(term_t out, term_t format, term_t args)
 { redir_context ctx;
-  int rc;
+  word rc;
 
   startCritical;
   if ( !setupOutputRedirect(out, &ctx, FALSE) )
@@ -406,13 +406,13 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv)
 
 					/* Check for user defined format */
 	  if ( format_predicates &&
-	       (s = lookupHTable(format_predicates, (Void)((long)c))) )
+	       (s = lookupHTable(format_predicates, (Void)((intptr_t)c))) )
 	  { Procedure proc = (Procedure) s->value;
 	    FunctorDef fdef = proc->definition->functor;
 	    term_t av = PL_new_term_refs(fdef->arity);
 	    char buf[BUFSIZE];
 	    char *str = buf;
-	    int bufsize = BUFSIZE;
+	    size_t bufsize = BUFSIZE;
 	    unsigned int i;
 	    qid_t qid;
 
@@ -568,7 +568,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv)
 
 		  NEED_ARG;
 		  if ( state.pending_rubber )
-		  { int bufsize = BUFSIZE;
+		  { size_t bufsize = BUFSIZE;
 
 		    str = buf;
 		    tellString(&str, &bufsize, ENC_UTF8);
@@ -583,14 +583,14 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv)
 		    { IOSTREAM *old = Scurout;
 
 		      Scurout = fd;
-		      rc = (*f)(argv);
+		      rc = (int)(*f)(argv);
 		      Scurout = old;
 		      if ( !rc )
 			goto out;
 
 		      state.column = fd->position->linepos;
 		    } else
-		    { int bufsize = BUFSIZE;
+		    { size_t bufsize = BUFSIZE;
 
 		      str = buf;
 		      tellString(&str, &bufsize, ENC_UTF8);
@@ -613,11 +613,11 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv)
 		 { FMT_ERROR("not enough arguments");
 		 }
 		 if ( state.pending_rubber )
-		  { int bufsize = BUFSIZE;
+		  { size_t bufsize = BUFSIZE;
 
 		    str = buf;
 		    tellString(&str, &bufsize, ENC_UTF8);
-		    rc = pl_write_term(argv, argv+1);
+		    rc = (int)pl_write_term(argv, argv+1);
 		    toldString();
 		    if ( !rc )
 		      goto out;
@@ -630,18 +630,18 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv)
 		    { IOSTREAM *old = Scurout;
 
 		      Scurout = fd;
-		      rc = pl_write_term(argv, argv+1);
+		      rc = (int)pl_write_term(argv, argv+1);
 		      Scurout = old;
 		      if ( !rc )
 			goto out;
 
 		      state.column = fd->position->linepos;
 		    } else
-		    { int bufsize = BUFSIZE;
+		    { size_t bufsize = BUFSIZE;
 
 		      str = buf;
 		      tellString(&str, &bufsize, ENC_UTF8);
-		      rc = pl_write_term(argv, argv+1);
+		      rc = (int)pl_write_term(argv, argv+1);
 		      if ( !rc )
 			goto out;
 		      toldString();
@@ -658,7 +658,7 @@ do_format(IOSTREAM *fd, PL_chars_t *fmt, int argc, term_t argv)
 	      case '@':
 	        { char buf[BUFSIZE];
 		  char *str = buf;
-		  int bufsize = BUFSIZE;
+		  size_t bufsize = BUFSIZE;
 		  term_t ex = 0;
 		  int rval;
 
@@ -793,13 +793,13 @@ emit_rubber(format_state *state)
   const char *e = &s[entriesBuffer(&state->buffer, char)];
   struct rubber *r = state->rub;
   int rn = state->pending_rubber;
-  int j;
+  size_t j;
 
   for(j = 0; s <= e; j++)
   { int chr;
 
     if ( r->where == j && rn )
-    { int n;
+    { size_t n;
 
       for(n=0; n<r->size; n++)
       { if ( Sputcode(r->pad, state->out) < 0 )
@@ -880,7 +880,7 @@ formatNumber(bool split, int div, int radix, bool small, Number i,
       char tmp[256];
       char *buf;
 
-      if ( len+2 > sizeof(tmp) )
+      if ( len+2 > (int)sizeof(tmp) )
 	buf = PL_malloc(len+2);
       else
 	buf = tmp;
