@@ -1,4 +1,4 @@
-/*  $Id: pl-funcs.h,v 1.258 2006/08/27 12:31:14 jan Exp $
+/*  $Id: pl-funcs.h,v 1.275 2007/01/23 20:39:40 jan Exp $
 
     Part of SWI-Prolog
 
@@ -38,22 +38,25 @@ symbol lookup and relocations.
 COMMON(void) 		freeHeap__LD(void *mem, size_t n ARG_LD);
 COMMON(word) 		outOfStack(Stack s, stack_overflow_action how);
 COMMON(void)		outOfCore(void) NORETURN;
-COMMON(Word) 		allocGlobal__LD(int words ARG_LD);
-COMMON(Word) 		allocGlobalNoShift__LD(int words ARG_LD);
+COMMON(Word) 		allocGlobal__LD(size_t words ARG_LD);
+COMMON(Word) 		allocGlobalNoShift__LD(size_t words ARG_LD);
 COMMON(void *) 		allocHeap__LD(size_t n ARG_LD);
 COMMON(void) 		initMemAlloc(void);
 COMMON(void) 		cleanupMemAlloc(void);
 COMMON(void) 		mergeAllocPool(AllocPool to, AllocPool from);
 COMMON(word) 		globalFunctor(functor_t def);
-COMMON(word) 		globalString(long len, const char *s);
-COMMON(word) 		globalWString(unsigned len, const pl_wchar_t *s);
-COMMON(char *)		getCharsString__LD(word w, unsigned *len ARG_LD);
-COMMON(pl_wchar_t *)	getCharsWString__LD(word w, unsigned *len ARG_LD);
+COMMON(word) 		globalString(size_t len, const char *s);
+COMMON(word) 		globalWString(size_t len, const pl_wchar_t *s);
+COMMON(char *)		getCharsString__LD(word w, size_t *len ARG_LD);
+COMMON(pl_wchar_t *)	getCharsWString__LD(word w, size_t *len ARG_LD);
 COMMON(Word) 		newTerm(void);
 COMMON(void) 		doublecpy(void *to, void *from);
 COMMON(word) 		globalReal(real f);
 COMMON(word) 		globalLong(int64_t i ARG_LD);
 COMMON(double) 		valReal__LD(word w ARG_LD);
+#ifdef INT64_ALIGNMENT
+COMMON(int64_t)		valBignum__LD(word w ARG_LD);
+#endif
 COMMON(word) 		globalIndirect(word in);
 COMMON(int) 		equalIndirect(word r1, word r2);
 COMMON(word) 		globalIndirectFromCode(Code *PC);
@@ -63,7 +66,7 @@ COMMON(word) 		consPtr(void *p, int ts);
 #endif
 COMMON(char *) 		store_string(const char *s);
 COMMON(void) 		remove_string(char *s);
-COMMON(int) 		unboundStringHashValue(const char *t, unsigned int l);
+COMMON(int) 		unboundStringHashValue(const char *t, size_t l);
 #ifndef xmalloc
 COMMON(void *) 		xmalloc(size_t size);
 COMMON(void *) 		xrealloc(void *mem, size_t size);
@@ -92,8 +95,8 @@ COMMON(word) 		pl_unify_with_occurs_check(term_t t1, term_t t2);
 COMMON(fid_t) 		PL_open_signal_foreign_frame(void);
 
 /* pl-atom.c */
-COMMON(word) 		lookupAtom(const char *s, unsigned int len);
-COMMON(word) 		lookupBlob(const char *s, unsigned int len,
+COMMON(word) 		lookupAtom(const char *s, size_t len);
+COMMON(word) 		lookupBlob(const char *s, size_t len,
 				   PL_blob_t *type, int *new);
 COMMON(word) 		pl_atom_hashstat(term_t i, term_t n);
 COMMON(void) 		initAtoms(void);
@@ -183,8 +186,8 @@ COMMON(word) 		parseSaveProgramOptions(term_t args,
 			bool *tty, bool *standalone);
 
 /* pl-index.c */
-COMMON(int) 		cardinalityPattern(unsigned long pattern);
-COMMON(void) 		getIndex(Word argv, unsigned long pattern, int card,
+COMMON(int) 		cardinalityPattern(uintptr_t pattern);
+COMMON(void) 		getIndex(Word argv, uintptr_t pattern, int card,
 			 struct index *
 			 ARG_LD);
 COMMON(word) 		getIndexOfTerm(term_t t);
@@ -220,12 +223,14 @@ COMMON(int) 		PL_error(const char *pred, int arity, const char *msg,
 			 int id, ...);
 COMMON(char *) 		tostr(char *buf, const char *fmt, ...);
 COMMON(void) 		printMessage(atom_t severity, ...);
-COMMON(int) 		PL_get_nchars_ex(term_t t, unsigned int *len, char **s,
+COMMON(int) 		PL_get_nchars_ex(term_t t, size_t *len, char **s,
 					 unsigned int flags);
 COMMON(int) 		PL_get_chars_ex(term_t t, char **s, unsigned int flags);
 COMMON(int) 		PL_get_atom_ex(term_t t, atom_t *a);
 COMMON(int) 		PL_get_integer_ex(term_t t, int *i);
 COMMON(int) 		PL_get_long_ex(term_t t, long *i);
+COMMON(int) 		PL_get_int64_ex(term_t t, int64_t *i);
+COMMON(int) 		PL_get_intptr_ex(term_t t, intptr_t *i);
 COMMON(int) 		PL_get_bool_ex(term_t t, int *i);
 COMMON(int) 		PL_get_float_ex(term_t t, double *f);
 COMMON(int) 		PL_get_char_ex(term_t t, int *p, int eof);
@@ -242,7 +247,7 @@ COMMON(void) 		initIO(void);
 COMMON(void) 		dieIO(void);
 COMMON(void) 		closeFiles(int all);
 COMMON(int) 		openFileDescriptors(unsigned char *buf, int size);
-COMMON(void) 		protocol(const char *s, int n);
+COMMON(void) 		protocol(const char *s, size_t n);
 COMMON(bool) 		getInputStream__LD(term_t t, IOSTREAM **s ARG_LD);
 COMMON(bool) 		getOutputStream(term_t t, IOSTREAM **s);
 COMMON(bool) 		streamStatus(IOSTREAM *s);
@@ -263,7 +268,7 @@ COMMON(word) 		pl_noprotocol(void);
 COMMON(bool) 		seeString(const char *s);
 COMMON(bool) 		seeingString(void);
 COMMON(bool) 		seenString(void);
-COMMON(bool) 		tellString(char **s, int *size, IOENC enc);
+COMMON(bool) 		tellString(char **s, size_t *size, IOENC enc);
 COMMON(bool) 		toldString(void);
 COMMON(word) 		pl_wait_for_input(term_t streams, term_t avail,
 					  term_t tmo);
@@ -304,7 +309,7 @@ COMMON(word) 		pl_current_output(term_t stream);
 COMMON(word) 		pl_source_location(term_t file, term_t line);
 COMMON(word) 		pl_at_end_of_stream1(term_t stream);
 COMMON(word) 		pl_at_end_of_stream0();
-COMMON(bool) 		unifyTime(term_t t, long time);
+COMMON(bool) 		unifyTime(term_t t, intptr_t time);
 COMMON(word) 		pl_time_file(term_t name, term_t t);
 COMMON(word) 		pl_size_file(term_t name, term_t len);
 COMMON(word) 		pl_size_stream(term_t stream, term_t len);
@@ -327,7 +332,7 @@ COMMON(word) 		pl_file_dir_name(term_t f, term_t b);
 COMMON(word) 		pl_file_name_extension(term_t base, term_t ext, term_t full);
 COMMON(word) 		pl_prolog_to_os_filename(term_t pl, term_t os);
 COMMON(foreign_t) 	pl_mark_executable(term_t path);
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 COMMON(word) 		pl_make_fat_filemap(term_t dir);
 #endif
 COMMON(word) 		pl_copy_stream_data3(term_t in, term_t out, term_t len);
@@ -368,16 +373,17 @@ COMMON(void) 		_PL_get_arg__LD(int index, term_t t, term_t a ARG_LD);
 COMMON(term_t) 		PL_new_term_ref__LD(ARG1_LD);
 COMMON(term_t) 		PL_new_term_refs__LD(int n ARG_LD);
 COMMON(int) 		PL_unify__LD(term_t t1, term_t t2 ARG_LD);
-COMMON(int) 		PL_unify_integer__LD(term_t t1, long i ARG_LD);
+COMMON(int) 		PL_unify_integer__LD(term_t t1, intptr_t i ARG_LD);
 COMMON(int) 		PL_get_atom__LD(term_t t1, atom_t *a ARG_LD);
 COMMON(void) 		PL_put_atom__LD(term_t t1, atom_t a ARG_LD);
-COMMON(void) 		PL_put_integer__LD(term_t t1, long i ARG_LD);
+COMMON(void) 		PL_put_integer__LD(term_t t1, intptr_t i ARG_LD);
 COMMON(int) 		PL_is_atomic__LD(term_t t ARG_LD);
 COMMON(int) 		PL_is_functor__LD(term_t t, functor_t f ARG_LD);
 COMMON(int) 		PL_is_variable__LD(term_t t ARG_LD);
 COMMON(int) 		PL_strip_module__LD(term_t q, module_t *m, term_t t ARG_LD);
 COMMON(int) 		PL_get_integer__LD(term_t t, int *i ARG_LD);
 COMMON(int) 		PL_get_long__LD(term_t t, long *i ARG_LD);
+COMMON(int) 		PL_get_int64__LD(term_t t, int64_t *i ARG_LD);
 COMMON(int) 		PL_get_pointer__LD(term_t t, void **ptr ARG_LD);
 COMMON(void) 		PL_put_term__LD(term_t t1, term_t t2 ARG_LD);
 COMMON(int) 		PL_get_functor__LD(term_t t, functor_t *f ARG_LD);
@@ -390,7 +396,7 @@ COMMON(void) 		PL_cons_list__LD(term_t l, term_t head, term_t tail
 					 ARG_LD);
 COMMON(int)		PL_is_inf(term_t t);
 COMMON(int)		isUCSAtom(Atom a);
-COMMON(atom_t)		lookupUCSAtom(const pl_wchar_t *s, unsigned int len);
+COMMON(atom_t)		lookupUCSAtom(const pl_wchar_t *s, size_t len);
 COMMON(Buffer)		codes_or_chars_to_buffer(term_t l, unsigned int flags,
 						 int wide);
 COMMON(Buffer)		findBuffer(int flags);
@@ -426,7 +432,7 @@ COMMON(void) 		blockGC(ARG1_LD);
 COMMON(void) 		unblockGC(ARG1_LD);
 COMMON(Word) 		findGRef(int n);
 COMMON(int) 		growStacks(LocalFrame fr, Choice ch, Code PC,
-			   long lminfree, long gminfree, long tminfree);
+			   intptr_t lminfree, intptr_t gminfree, intptr_t tminfree);
 COMMON(void) 		clearUninitialisedVarsFrame(LocalFrame, Code);
 COMMON(word) 		check_foreign(void);	/* O_SECURE stuff */
 COMMON(void) 		markAtomsOnStacks(PL_local_data_t *ld);
@@ -496,7 +502,7 @@ COMMON(uint64_t)	_PL_Random(void);
 COMMON(char *) 		canonisePath(char *path);
 COMMON(char *) 		OsPath(const char *plpath, char *ospath);
 COMMON(char *) 		PrologPath(const char *ospath, char *plpath, size_t len);
-COMMON(long) 		LastModifiedFile(char *f);
+COMMON(intptr_t) 		LastModifiedFile(char *f);
 COMMON(bool) 		ExistsFile(const char *path);
 COMMON(bool) 		AccessFile(const char *path, int mode);
 COMMON(bool) 		ExistsDirectory(const char *path);
@@ -519,10 +525,10 @@ COMMON(char *) 		DeRefLink(const char *link, char *buf);
 COMMON(bool) 		ChDir(const char *path);
 COMMON(atom_t) 		TemporaryFile(const char *id);
 COMMON(int) 		hasConsole(void);
-COMMON(struct)  tm *	LocalTime(long int *t, struct tm *r);
+COMMON(struct tm *)	LocalTime(long *t, struct tm *r);
 COMMON(Char) 		GetChar(void);
-COMMON(char *) 		getenv3(const char *, char *buf, unsigned int buflen);
-COMMON(int) 		getenvl(const char *);
+COMMON(size_t) 		getenv3(const char *, char *buf, size_t buflen);
+COMMON(char *) 		Getenv(const char *, char *buf, size_t buflen);
 COMMON(int) 		Setenv(char *name, char *value);
 COMMON(int) 		Unsetenv(char *name);
 COMMON(int) 		System(char *cmd);
@@ -609,7 +615,7 @@ COMMON(foreign_t) 	pl_current_predicate1(term_t spec, control_t ctx);
 COMMON(ClauseRef) 	assertProcedure(Procedure proc, Clause clause,
 				int where ARG_LD);
 COMMON(bool) 		abolishProcedure(Procedure proc, Module module);
-COMMON(bool) 		retractClauseProcedure(Procedure proc, Clause clause ARG_LD);
+COMMON(bool) 		retractClauseDefinition(Definition def, Clause clause ARG_LD);
 COMMON(void) 		freeClause(Clause c ARG_LD);
 COMMON(void) 		freeClauseRef(ClauseRef c ARG_LD);
 COMMON(ClauseRef) 	newClauseRef(Clause cl ARG_LD);
@@ -629,7 +635,7 @@ COMMON(word) 		pl_set_predicate_attribute(term_t pred, term_t k, term_t v);
 COMMON(void) 		reindexDefinition(Definition def);
 COMMON(void) 		redefineProcedure(Procedure proc, SourceFile sf);
 COMMON(void) 		startConsult(SourceFile f);
-COMMON(void) 		indexDefinition(Definition def, unsigned long pattern);
+COMMON(void) 		indexDefinition(Definition def, long pattern);
 COMMON(word) 		pl_index(term_t pred);
 COMMON(SourceFile) 	lookupSourceFile(atom_t name);
 COMMON(SourceFile) 	indexToSourceFile(int index);
@@ -706,10 +712,10 @@ COMMON(void) 		unblockSignal(int sig);
 COMMON(void) 		blockSignal(int sig);
 COMMON(void) 		resetSignals(void);
 COMMON(void) 		cleanupSignals(void);
-COMMON(int) 		initPrologStacks(long local,
-				 long global,
-				 long trail,
-				 long argument);
+COMMON(int) 		initPrologStacks(intptr_t local,
+				 intptr_t global,
+				 intptr_t trail,
+				 intptr_t argument);
 COMMON(void) 		initPrologLocalData(void);
 COMMON(void) 		deallocateStacks(void);
 COMMON(bool) 		restoreStack(Stack s);
@@ -721,7 +727,7 @@ COMMON(void) 		freeLocalData(PL_local_data_t *ld);
 COMMON(word) 		pl_trim_stacks(void);
 COMMON(word) 		pl_limit_stack(term_t s, term_t l);
 COMMON(word) 		pl_stack_parameter(term_t s, term_t k, term_t o, term_t n);
-COMMON(void) 		ensureRoomStack(Stack s, int n);
+COMMON(void) 		ensureRoomStack(Stack s, size_t n);
 COMMON(int) 		_PL_get_signum(term_t sig, int *n);
 
 /* pl-sys.c */
@@ -777,7 +783,6 @@ COMMON(char *) 		predicateName(Definition def);
 COMMON(word) 		notImplemented(char *name, int arity);
 COMMON(word) 		setBoolean(int *flag, term_t o, term_t n);
 COMMON(word) 		setInteger(int *val, term_t old, term_t new);
-COMMON(word) 		setLong(long *val, term_t old, term_t new);
 COMMON(bool) 		strprefix(const char *string, const char *prefix);
 COMMON(bool) 		strpostfix(const char *string, const char *postfix);
 COMMON(bool) 		stripostfix(const char *string, const char *postfix);
@@ -821,7 +826,7 @@ COMMON(void)		wicPutNum(int64_t n, IOSTREAM *fd);
 COMMON(int64_t)		wicGetNum(IOSTREAM *fd);
 COMMON(void)		wicPutStringW(const pl_wchar_t *w, size_t len, 
 				      IOSTREAM *fd);
-COMMON(pl_wchar_t*)	wicGetStringUTF8(IOSTREAM *fd, unsigned *length,
+COMMON(pl_wchar_t*)	wicGetStringUTF8(IOSTREAM *fd, size_t *length,
 					 pl_wchar_t *buf, size_t bufsize);
 
 /* pl-write.c */
@@ -919,6 +924,9 @@ COMMON(bool) 		systemMode(bool accept);
 /* pl-thread.c */
 COMMON(foreign_t) 	pl_with_mutex(term_t mutex, term_t goal);
 COMMON(foreign_t) 	pl_thread_self(term_t self);
+#ifdef O_PLMT
+COMMON(int)		unify_thread_id(term_t id, PL_thread_info_t *info);
+#endif
 COMMON(int) 		enableThreads(int enable);
 
 
