@@ -1,4 +1,4 @@
-/*  $Id: pl-list.c,v 1.23 2006/07/31 12:10:50 jan Exp $
+/*  $Id: pl-list.c,v 1.26 2007/01/08 13:29:24 jan Exp $
 
     Part of SWI-Prolog
 
@@ -56,7 +56,7 @@ pl_length(term_t list, term_t l)
   }
 
   if ( PL_is_variable(l) )
-  { long n;
+  { intptr_t n;
   
     if ( (n=lengthList(list, FALSE)) >= 0 )
       return PL_unify_integer(l, n);
@@ -64,6 +64,10 @@ pl_length(term_t list, term_t l)
     fail;			/* both variables: generate in Prolog */
   }
   
+				/* large integer; we cannot represent anyway */
+  if ( PL_is_integer(l) )
+    return outOfStack((Stack)&LD->stacks.global, STACK_OVERFLOW_RAISE);
+
   return PL_error("length", 2, NULL, ERR_TYPE, ATOM_integer, l);
 }  
 
@@ -288,7 +292,7 @@ prolog_list_to_sort_list(term_t t, int key, list *lp, Word *end)
   int n = lengthList(t, TRUE);
   Word l;
   list p;
-  long minfree;
+  intptr_t minfree;
 
   if ( n < 0 )
     fail;				/* not a proper list */
@@ -385,7 +389,7 @@ pl_nat_sort(term_t in, term_t out, int remove_dups, int compare_keys ARG_LD)
 
 
 static
-PRED_IMPL("ok_sort", 2, sort, 0)
+PRED_IMPL("sort", 2, sort, 0)
 { PRED_LD
 
   return pl_nat_sort(A1, A2, TRUE, FALSE PASS_LD);
@@ -393,7 +397,7 @@ PRED_IMPL("ok_sort", 2, sort, 0)
 
 
 static
-PRED_IMPL("ok_msort", 2, msort, 0)
+PRED_IMPL("msort", 2, msort, 0)
 { PRED_LD
 
   return pl_nat_sort(A1, A2, FALSE, FALSE PASS_LD);
@@ -401,7 +405,7 @@ PRED_IMPL("ok_msort", 2, msort, 0)
 
 
 static
-PRED_IMPL("ok_keysort", 2, keysort, 0)
+PRED_IMPL("keysort", 2, keysort, 0)
 { PRED_LD
 
   return pl_nat_sort(A1, A2, FALSE, TRUE PASS_LD);
