@@ -1,4 +1,4 @@
-/*  $Id: pl-main.c,v 1.162 2006/02/22 12:07:06 jan Exp $
+/*  $Id: pl-main.c,v 1.170 2007/01/22 11:52:23 jan Exp $
 
     Part of SWI-Prolog
 
@@ -43,7 +43,7 @@ option  parsing,  initialisation  and  handling  of errors and warnings.
 #include "morecore.c"
 #endif
 
-#if defined(_DEBUG) && defined(WIN32)
+#if defined(_DEBUG) && defined(__WINDOWS__)
 #include <crtdbg.h>
 #endif
 
@@ -67,7 +67,7 @@ static const char exec_vars[512] = EXECVARMAGIC;
 static const char *
 exec_var(const char *name)
 { const char *s=exec_vars + strlen(EXECVARMAGIC);
-  int l = strlen(name);
+  size_t l = strlen(name);
 
   while(s < exec_vars+sizeof(exec_vars))
   { if ( strncmp(name, s, l) == 0 && s[l] == '=' )
@@ -90,15 +90,15 @@ findHome(char *symbols)
   const char *val;
   
   if ( (val  = exec_var("homevar")) &&
-       (home = getenv3(val, envbuf, sizeof(envbuf))) &&
+       (home = Getenv(val, envbuf, sizeof(envbuf))) &&
        (home = PrologPath(home, plp, sizeof(plp))) )
     return store_string(home);
   if ( (val = exec_var("home")) &&
        (home = PrologPath(val, plp, sizeof(plp))) )
     return store_string(home);
 
-  if ( !(home = getenv3("SWI_HOME_DIR", envbuf, sizeof(envbuf))) )
-    home = getenv3("SWIPL", envbuf, sizeof(envbuf));
+  if ( !(home = Getenv("SWI_HOME_DIR", envbuf, sizeof(envbuf))) )
+    home = Getenv("SWIPL", envbuf, sizeof(envbuf));
   if ( home &&
        (home = PrologPath(home, plp, sizeof(plp))) &&
        ExistsDirectory(home) )
@@ -114,7 +114,7 @@ findHome(char *symbols)
 
     if ( (fd = Sopen_file(buf, "r")) )
     { if ( Sfgets(buf, sizeof(buf), fd) )
-      { int l = strlen(buf);
+      { size_t l = strlen(buf);
 
 	while(l > 0 && buf[l-1] <= ' ')
 	  l--;
@@ -176,16 +176,16 @@ defaultSystemInitFile(char *a0)
     *s++ = *base++;
   *s = EOS;
 
-  if ( strlen(buf) > 0 )
+  if ( buf[0] != EOS )
     return store_string(buf);
 
   return store_string("pl");
 }
 
 
-#define MEMAREA_INVALID_SIZE (unsigned long)(~0L)
+#define MEMAREA_INVALID_SIZE (uintptr_t)(~0L)
 
-static unsigned long
+static uintptr_t
 memarea_limit(const char *s)
 { number n;
   unsigned char *q;
@@ -195,13 +195,16 @@ memarea_limit(const char *s)
     { case 'k':
       case 'K':
       case EOS:
-	return (long)n.value.i K;
+	return (intptr_t)n.value.i K;
       case 'm':
       case 'M':
-	return (long)n.value.i K K;
+	return (intptr_t)n.value.i K K;
+      case 'g':
+      case 'G':
+	return (intptr_t)n.value.i K K K;
       case 'b':
       case 'B':
-	return (long)n.value.i;
+	return (intptr_t)n.value.i;
     }
   }
 
@@ -220,8 +223,8 @@ setupGNUEmacsInferiorMode()
   char *s;
   int val;
 
-  if ( ((s = getenv3("EMACS", envbuf, sizeof(envbuf))) && streq(s, "t")) ||
-       ((s = getenv3("INFERIOR", envbuf, sizeof(envbuf))) && streq(s, "yes")) )
+  if ( ((s = Getenv("EMACS", envbuf, sizeof(envbuf))) && s[0]) ||
+       ((s = Getenv("INFERIOR", envbuf, sizeof(envbuf))) && streq(s, "yes")) )
   { clearFeatureMask(TTY_CONTROL_FEATURE);
     val = TRUE;
   } else
@@ -248,7 +251,7 @@ initPaths()
   
     systemDefaults.home	       = findHome(symbols);
   
-#ifdef __WIN32__			/* we want no module but the .EXE */
+#ifdef __WINDOWS__			/* we want no module but the .EXE */
     GD->paths.module	       = store_string(symbols);
     symbols = findExecutable(NULL, plp);
     DEBUG(2, Sdprintf("Executable: %s\n", symbols));
@@ -258,7 +261,7 @@ initPaths()
   } else
   { systemDefaults.home	       = findHome(NULL);
     GD->options.systemInitFile = store_string("none");
-#ifdef __WIN32__			/* we want no module but the .EXE */
+#ifdef __WINDOWS__			/* we want no module but the .EXE */
     GD->paths.module	       = store_string("libpl.dll");
 #endif
   }
@@ -287,7 +290,7 @@ initDefaults()
   systemDefaults.toplevel    = "prolog";
   systemDefaults.notty       = NOTTYCONTROL;
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
   getDefaultsFromRegistry();
 #endif
 
@@ -388,7 +391,7 @@ initDefaultOptions()
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Does the commandline option parsing.  Actually   we  should  use the GNU
-getopt package and deal nicely with long   arguments  as well as shorts,
+getopt package and deal nicely with intptr_t   arguments  as well as shorts,
 but these options are  too  widely  used   as  they  are  to change them
 overnight. Returns -1 on error.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -405,7 +408,7 @@ parseCommandLineOptions(int argc0, char **argv, int *compile)
     if ( streq(s, "-" ) )		/* pl <plargs> -- <app-args> */
       break;
     if ( *s == '-' )
-      continue;				/* don't handle --long=value */
+      continue;				/* don't handle --intptr_t=value */
 
     if ( streq(s, "tty") )	/* +/-tty */
     { if ( s[-1] == '+' )
@@ -460,7 +463,7 @@ parseCommandLineOptions(int argc0, char **argv, int *compile)
 	case 'T':
 	case 'A':
 	case 'H':
-        { unsigned long size = memarea_limit(&s[1]);
+        { uintptr_t size = memarea_limit(&s[1]);
 	  
 	  if ( size == MEMAREA_INVALID_SIZE )
 	    return -1;
@@ -509,10 +512,10 @@ Find the resource database.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #ifndef BOOTFILE		/* normally delivered through config.h */
-#if SIZEOF_LONG == 4
+#if SIZEOF_VOIDP == 4
 #define BOOTFILE "boot32.prc"
 #else
-#if SIZEOF_LONG == 8
+#if SIZEOF_VOIDP == 8
 #define BOOTFILE "boot64.prc"
 #else
 #define BOOTFILE "boot.prc"
@@ -530,7 +533,7 @@ openResourceDB(int argc, char **argv)
 
   if ( !GD->bootsession )
   { 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
     if ( GD->paths.module &&
 	 !streq(GD->paths.module, GD->paths.executable) &&
 	 (rc = rc_open_archive(GD->paths.module, flags)) )
@@ -695,7 +698,7 @@ script_argv(int argc, char **argv)
     char *av[MAXARGV];
     int  an = 0;
 
-#ifdef WIN32
+#ifdef __WINDOWS__
     if ( argc == 2 )
     { char tmp[MAXPATHLEN];
       char dir[MAXPATHLEN];
@@ -779,7 +782,7 @@ PL_initialise(int argc, char **argv)
   bool compile = FALSE;
   const char *rcpath = "<none>";
 
-#if defined(_DEBUG) && defined(WIN32) && 0
+#if defined(_DEBUG) && defined(__WINDOWS__) && 0
   _CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF|
 		 _CRTDBG_CHECK_CRT_DF|
 		 //_CRTDBG_CHECK_ALWAYS_DF| 	/* very expensive */
@@ -899,7 +902,7 @@ properly on Linux. Don't bother with it.
     }
     if ( Sclose(s) != 0 || !rc_save_archive(GD->resourceDB, NULL) )
     { 
-#if defined(__WINDOWS__) || defined(__WIN32__)
+#ifdef __WINDOWS__
       PlMessage("Failed to save system resources: %s", rc_strerror(rc_errno));
 #else
       Sfprintf(Serror,
@@ -908,7 +911,7 @@ properly on Linux. Don't bother with it.
 #endif
       PL_halt(1);
     }
-#if defined(__WINDOWS__) || defined(__WIN32__)
+#ifdef __WINDOWS__
     PlMessage("Boot compilation has created %s", rcpathcopy);
 #else
     Sfprintf(Serror,
@@ -969,7 +972,7 @@ usage()
     "    7) %s [options] [-o output] -b bootfile -c file ...\n",
     "Options:\n",
     "    -x state         Start from state (must be first)\n",
-    "    -[LGTA]size[KM]  Specify {Local,Global,Trail,Argument} limits\n",
+    "    -[LGTA]size[KMG] Specify {Local,Global,Trail,Argument} limits\n",
     "    -t toplevel      Toplevel goal\n",
     "    -g goal          Initialisation goal\n",
     "    -f file          User initialisation file\n",
@@ -1165,6 +1168,8 @@ PL_cleanup(int rval)
   if ( GD->initialised )
   { fid_t cid = PL_open_foreign_frame();
     predicate_t proc = PL_predicate("$run_at_halt", 0, "system");
+
+    DEBUG(5, Sdprintf("Running at_halt hooks\n"));
     PL_call_predicate(MODULE_system, FALSE, proc, 0);
     PL_discard_foreign_frame(cid);
   }
@@ -1175,7 +1180,7 @@ PL_cleanup(int rval)
   for(h = GD->os.on_halt_list; h; h = h->next)
     (*h->function)(rval, h->argument);
 
-#if defined(__WINDOWS__) || defined(__WIN32__)
+#ifdef __WINDOWS__
   if ( rval != 0 && !hasConsole() )
     PlMessage("Exit status is %d", rval);
 #endif
@@ -1329,7 +1334,7 @@ action:
 void
 vfatalError(const char *fm, va_list args)
 {
-#if defined(__WINDOWS__) || defined(__WIN32__)
+#ifdef __WINDOWS__
   char msg[500];
   Ssprintf(msg, "[FATAL ERROR:\n\t");
   Svsprintf(&msg[strlen(msg)], fm, args);
