@@ -1,4 +1,4 @@
-/*  $Id: pl-modul.c,v 1.49 2004/09/30 10:41:36 jan Exp $
+/*  $Id: pl-modul.c,v 1.51 2007/01/13 12:49:07 jan Exp $
 
     Part of SWI-Prolog
 
@@ -293,7 +293,7 @@ PRED_IMPL("import_module", 2, import_module,
       i = 0;
       break;
     case FRG_REDO:
-      i = ForeignContextInt(PL__ctx);
+      i = (int)ForeignContextInt(PL__ctx);
       break;
     default:
       succeed;
@@ -588,7 +588,7 @@ declareModule(atom_t name, SourceFile sf, int line)
 
     obj = PL_new_term_ref();
     PL_put_atom(obj, name);
-    Ssprintf(msg, "Alread loaded from %s",
+    Ssprintf(msg, "Already loaded from %s",
 	     atom_summary(module->file->name, 100));
     return PL_error("module", 2, msg, ERR_PERMISSION,
 		    ATOM_redefine, ATOM_module, obj);
