@@ -1,4 +1,4 @@
-/*  $Id: pl-nt.c,v 1.42 2004/09/21 14:22:26 jan Exp $
+/*  $Id: pl-nt.c,v 1.67 2007/01/13 12:49:07 jan Exp $
 
     Part of SWI-Prolog
 
@@ -22,11 +22,18 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#if defined(__WINDOWS__) || defined(__WIN32__) || defined(WIN32)
+#ifdef __WINDOWS__
 #define _WIN32_WINNT 0x0400
+#if (_MSC_VER >= 1300)
+#include <winsock2.h>			/* Needed on VC8 */
 #include <windows.h>
+#else
+#include <windows.h>			/* Needed for MSVC 5&6 */
+#include <winsock2.h>
+#endif
 
 #include "pl-incl.h"
+#include "pl-utf8.h"
 #include <crtdbg.h>
 #include <process.h>
 #include "pl-ctype.h"
@@ -86,17 +93,6 @@ PlMessage(const char *fm, ...)
   va_end(args);
 }
 
-		 /*******************************
-		 *	       WIN32		*
-		 *******************************/
-
-int
-iswin32s()
-{ if( GetVersion() & 0x80000000 && (GetVersion() & 0xFF) ==3)
-    return TRUE;
-  else
-    return FALSE;
-}
 
 
 		 /*******************************
@@ -222,7 +218,7 @@ CpuTime(cputime_kind which)
     t = (double)p->dwHighDateTime * (4294967296.0 * ntick nano);
     t += (double)p->dwLowDateTime  * (ntick nano);
   } else				/* '95, Windows 3.1/win32s */
-  { extern long clock_wait_ticks;
+  { extern intptr_t clock_wait_ticks;
 
     t = (double) (clock() - clock_wait_ticks) / (double) CLOCKS_PER_SEC;
   }
@@ -234,7 +230,7 @@ CpuTime(cputime_kind which)
 char *
 findExecutable(const char *module, char *exe)
 { int n;
-  char buf[MAXPATHLEN];
+  wchar_t wbuf[MAXPATHLEN];
   HMODULE hmod;
 
   if ( module )
@@ -249,15 +245,12 @@ findExecutable(const char *module, char *exe)
   } else
     hmod = NULL;
 
-  if ( (n = GetModuleFileName(hmod, buf, sizeof(buf))) > 0 )
-  { char buf2[MAXPATHLEN];
-
-    buf[n] = EOS;
-    _xos_long_file_name(buf, buf2);
-
-    strcpy(exe, buf2);
+  if ( (n = GetModuleFileNameW(hmod, wbuf, MAXPATHLEN)) > 0 )
+  { wbuf[n] = EOS;
+    return _xos_long_file_name_toA(wbuf, exe, MAXPATHLEN);
   } else if ( module )
-  { PrologPath(module, buf);
+  { char buf[MAXPATHLEN];
+    PrologPath(module, buf, sizeof(buf));
 
     strcpy(exe, buf);
   } else
@@ -318,60 +311,90 @@ get_showCmd(term_t show, int *cmd)
 
 
 
-static char *
-win_exec(const char *cmd, UINT show)
-{ char *msg;
+static int
+win_exec(size_t len, const wchar_t *cmd, UINT show)
+{ STARTUPINFOW startup;
+  PROCESS_INFORMATION info;
   int rval;
+  wchar_t *wcmd;
 
-  switch((rval = WinExec(cmd, show)))
-  { case 0:
-      msg = "Not enough memory";
-      break;
-    case ERROR_BAD_FORMAT:
-      msg = "Bad format";
-      break;
-    case ERROR_FILE_NOT_FOUND:
-      msg = "File not found";
-      break;
-    case ERROR_PATH_NOT_FOUND:
-      msg = "Path not found";
-      break;
-    default:
-      if ( rval > 31 )
-	msg = NULL;
-      else
-	msg = "Unknown error";
-      break;
+  memset(&startup, 0, sizeof(startup));
+  startup.cb = sizeof(startup);
+  startup.wShowWindow = show;
+
+					/* ensure 0-terminated */
+  wcmd = PL_malloc((len+1)*sizeof(wchar_t));
+  memcpy(wcmd, cmd, len*sizeof(wchar_t));
+  wcmd[len] = 0;
+  
+  rval = CreateProcessW(NULL,		/* app */
+			wcmd,
+			NULL, NULL,	/* security */
+			FALSE,		/* inherit handles */
+			0,		/* flags */
+			NULL,		/* environment */
+			NULL,		/* Directory */
+			&startup,
+			&info);		/* process info */
+  PL_free(wcmd);
+
+  if ( rval )
+  { CloseHandle(info.hProcess);
+    CloseHandle(info.hThread);
+
+    succeed;
+  } else
+  { term_t tmp = PL_new_term_ref();
+      
+    PL_unify_wchars(tmp, PL_ATOM, len, cmd);
+    return PL_error(NULL, 0, WinError(), ERR_SHELL_FAILED, tmp);
   }
+}
 
-  return msg;
+
+static void
+utf8towcs(wchar_t *o, const char *src)
+{ for( ; *src; )
+  { int wc;
+
+    src = utf8_get_char(src, &wc);
+    *o++ = wc;
+  }
+  *o = 0;
 }
 
 
 int
-System(char *command)
-{ STARTUPINFO sinfo;
+System(char *command)			/* command is a UTF-8 string */
+{ STARTUPINFOW sinfo;
   PROCESS_INFORMATION pinfo;
   int shell_rval;
+  size_t len;
+  wchar_t *wcmd;
 
   memset(&sinfo, 0, sizeof(sinfo));
   sinfo.cb = sizeof(sinfo);
 
-  if ( CreateProcess(NULL,			/* module */
-		     command,			/* command line */
-		     NULL,			/* Security stuff */
-		     NULL,			/* Thread security stuff */
-		     FALSE,			/* Inherit handles */
-		     NORMAL_PRIORITY_CLASS,	/* flags */
-		     NULL,			/* environment */
-		     NULL,			/* CWD */
-		     &sinfo,			/* startup info */
-		     &pinfo) )			/* process into */
+  len = utf8_strlen(command, strlen(command));
+  wcmd = PL_malloc((len+1)*sizeof(wchar_t));
+  utf8towcs(wcmd, command);
+
+  if ( CreateProcessW(NULL,			/* module */
+		      wcmd,			/* command line */
+		      NULL,			/* Security stuff */
+		      NULL,			/* Thread security stuff */
+		      FALSE,			/* Inherit handles */
+		      NORMAL_PRIORITY_CLASS,	/* flags */
+		      NULL,			/* environment */
+		      NULL,			/* CWD */
+		      &sinfo,			/* startup info */
+		      &pinfo) )			/* process into */
   { BOOL rval;
     DWORD code;
 
     CloseHandle(pinfo.hThread);			/* don't need this */
-      
+    PL_free(wcmd);
+
     do
     { MSG msg;
 
@@ -387,7 +410,9 @@ System(char *command)
     shell_rval = (rval == TRUE ? code : -1);
     CloseHandle(pinfo.hProcess);
   } else
+  { PL_free(wcmd);
     return shell_rval = -1;
+  }
 
   return shell_rval;
 }
@@ -395,17 +420,13 @@ System(char *command)
 
 word
 pl_win_exec(term_t cmd, term_t how)
-{ char *s;
+{ wchar_t *s;
+  size_t len;
   UINT h;
 
-  if ( PL_get_chars_ex(cmd, &s, CVT_ALL) &&
+  if ( PL_get_wchars(cmd, &len, &s, CVT_ALL|CVT_EXCEPTION) &&
        get_showCmd(how, &h) )
-  { char *msg = win_exec(s, h);
-
-    if ( msg )
-      return warning("win_exec/2: %s", msg);
-    else
-      succeed;
+  { return win_exec(len, s, h);
   } else
     fail;
 }
@@ -437,18 +458,19 @@ static const shell_error se_errors[] =
 
 static int
 win_shell(term_t op, term_t file, term_t how)
-{ char *o, *f;
+{ size_t lo, lf;
+  wchar_t *o, *f;
   UINT h;
   HINSTANCE instance;
 
-  if ( !PL_get_chars_ex(op,   &o, CVT_ALL|BUF_RING) ||
-       !PL_get_chars_ex(file, &f, CVT_ALL|BUF_RING) ||
+  if ( !PL_get_wchars(op,   &lo, &o, CVT_ALL|CVT_EXCEPTION|BUF_RING) ||
+       !PL_get_wchars(file, &lf, &f, CVT_ALL|CVT_EXCEPTION|BUF_RING) ||
        !get_showCmd(how, &h) )
     fail;
        
-  instance = ShellExecute(NULL, o, f, NULL, NULL, h);
+  instance = ShellExecuteW(NULL, o, f, NULL, NULL, h);
 
-  if ( (long)instance <= 32 )
+  if ( (intptr_t)instance <= 32 )
   { const shell_error *se;
 
     for(se = se_errors; se->message; se++)
@@ -474,36 +496,6 @@ PRED_IMPL("win_shell", 3, win_shell3, 0)
 }
 
 
-char *
-getenv3(const char *name, char *buf, unsigned int len)
-{ if ( GetEnvironmentVariable(name, buf, (DWORD)len) )
-    return buf;
-  
-  return NULL;
-}
-
-/* What does this return if the variable is not defined?
-*/
-
-int
-getenvl(const char *name)
-{ return GetEnvironmentVariable(name, NULL, 0);
-}
-
-#if _DEBUG
-void
-initHeapDebug(void)
-{ int tmpFlag = _CrtSetDbgFlag( _CRTDBG_REPORT_FLAG );
-
-  if ( !(tmpFlag & _CRTDBG_CHECK_ALWAYS_DF) )
-  { /*PlMessage("Setting malloc() debugging");*/
-    tmpFlag |= _CRTDBG_CHECK_ALWAYS_DF;
-    _CrtSetDbgFlag(tmpFlag);
-  } /*else
-    PlMessage("Malloc debugging already set");*/
-}
-#endif
-
 foreign_t
 pl_win_module_file(term_t module, term_t file)
 { char buf[MAXPATHLEN];
@@ -522,8 +514,8 @@ pl_win_module_file(term_t module, term_t file)
 		 *	  WINDOWS MESSAGES	*
 		 *******************************/
 
-int
-PL_win_message_proc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
+LRESULT
+PL_win_message_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
 #ifdef O_PLMT
   if ( hwnd == NULL &&
@@ -556,10 +548,19 @@ interface.
 static char *dlmsg;
 
 void *
-dlopen(const char *file, int flags)
+dlopen(const char *file, int flags)	/* file is in UTF-8 */
 { HINSTANCE h;
+  size_t len = utf8_strlen(file, strlen(file));
+  wchar_t *wfile = alloca((len+1)*sizeof(wchar_t));
 
-  if ( (h = LoadLibrary(file)) )
+  if ( !wfile )
+  { dlmsg = "No memory";
+    return NULL;
+  }
+
+  utf8towcs(wfile, file);
+
+  if ( (h = LoadLibraryW(wfile)) )
   { dlmsg = "No Error";
     return (void *)h;
   }
@@ -598,19 +599,22 @@ dlclose(void *handle)
 
 #endif /*EMULATE_DLOPEN*/
 
+
 		 /*******************************
 		 *	      REGISTRY		*
 		 *******************************/
 
+#define wstreq(s,q) (wcscmp((s), (q)) == 0)
+
 static HKEY
-reg_open_key(const char *which, int create)
+reg_open_key(const wchar_t *which, int create)
 { HKEY key = HKEY_CURRENT_USER;
   DWORD disp;
   LONG rval;
 
   while(*which)
-  { char buf[256];
-    char *s;
+  { wchar_t buf[256];
+    wchar_t *s;
     HKEY tmp;
 
     for(s=buf; *which && !(*which == '/' || *which == '\\'); )
@@ -619,22 +623,22 @@ reg_open_key(const char *which, int create)
     if ( *which )
       which++;
 
-    if ( streq(buf, "HKEY_CLASSES_ROOT") )
+    if ( wstreq(buf, L"HKEY_CLASSES_ROOT") )
     { key = HKEY_CLASSES_ROOT;
       continue;
-    } else if ( streq(buf, "HKEY_CURRENT_USER") )
+    } else if ( wstreq(buf, L"HKEY_CURRENT_USER") )
     { key = HKEY_CURRENT_USER;
       continue;
-    } else if ( streq(buf, "HKEY_LOCAL_MACHINE") )
+    } else if ( wstreq(buf, L"HKEY_LOCAL_MACHINE") )
     { key = HKEY_LOCAL_MACHINE;
       continue;
-    } else if ( streq(buf, "HKEY_USERS") )
+    } else if ( wstreq(buf, L"HKEY_USERS") )
     { key = HKEY_USERS;
       continue;
     }
 
     DEBUG(2, Sdprintf("Trying %s\n", buf));
-    if ( RegOpenKeyEx(key, buf, 0L, KEY_READ, &tmp) == ERROR_SUCCESS )
+    if ( RegOpenKeyExW(key, buf, 0L, KEY_READ, &tmp) == ERROR_SUCCESS )
     { RegCloseKey(key);
       key = tmp;
       continue;
@@ -643,7 +647,7 @@ reg_open_key(const char *which, int create)
     if ( !create )
       return NULL;
 
-    rval = RegCreateKeyEx(key, buf, 0, "", 0,
+    rval = RegCreateKeyExW(key, buf, 0, L"", 0,
 			  KEY_ALL_ACCESS, NULL, &tmp, &disp);
     RegCloseKey(key);
     if ( rval == ERROR_SUCCESS )
@@ -657,32 +661,38 @@ reg_open_key(const char *which, int create)
 
 #define MAXREGSTRLEN 1024
 
-foreign_t
-pl_get_registry_value(term_t Key, term_t Name, term_t Value)
+static
+PRED_IMPL("win_registry_get_value", 3, win_registry_get_value, 0) 
 { DWORD type;
   BYTE  data[MAXREGSTRLEN];
   DWORD len = sizeof(data);
-  char *k;
-  char *name;
+  size_t klen, namlen;
+  wchar_t *k, *name;
   HKEY key;
 
-  if ( !PL_get_chars_ex(Key, &k, CVT_ATOM) ||
-       !PL_get_chars_ex(Name, &name, CVT_ATOM) )
+  term_t Key = A1;
+  term_t Name = A2;
+  term_t Value = A3;
+
+  if ( !PL_get_wchars(Key, &klen, &k, CVT_ATOM|CVT_EXCEPTION) ||
+       !PL_get_wchars(Name, &namlen, &name, CVT_ATOM|CVT_ATOM) )
     return FALSE;
   if ( !(key=reg_open_key(k, FALSE)) )
     return PL_error(NULL, 0, NULL, ERR_EXISTENCE, PL_new_atom("key"), Key);
 
   DEBUG(9, Sdprintf("key = %p, name = %s\n", key, name));
-  if ( RegQueryValueEx(key, name, NULL, &type, data, &len) == ERROR_SUCCESS )
+  if ( RegQueryValueExW(key, name, NULL, &type, data, &len) == ERROR_SUCCESS )
   { RegCloseKey(key);
 
     switch(type)
     { case REG_SZ:
-	return PL_unify_atom_chars(Value, data);
+	return PL_unify_wchars(Value, PL_ATOM,
+			       len/sizeof(wchar_t)-1, (wchar_t*)data);
       case REG_DWORD:
 	return PL_unify_integer(Value, *((DWORD *)data));
       default:
-	return warning("get_registry_value/2: Unknown registery-type");
+	warning("get_registry_value/2: Unknown registery-type: %d", type);
+        fail;
     }
   }
 
@@ -732,14 +742,19 @@ void
 getDefaultsFromRegistry()
 { HKEY key;
 
-  if ( (key = reg_open_key("HKEY_LOCAL_MACHINE/Software/SWI/Prolog", FALSE)) )
+  if ( (key = reg_open_key(L"HKEY_LOCAL_MACHINE/Software/SWI/Prolog", FALSE)) )
   { setStacksFromKey(key);
     RegCloseKey(key);
   }
-  if ( (key = reg_open_key("HKEY_CURRENT_USER/Software/SWI/Prolog", FALSE)) )
+  if ( (key = reg_open_key(L"HKEY_CURRENT_USER/Software/SWI/Prolog", FALSE)) )
   { setStacksFromKey(key);
     RegCloseKey(key);
   }
+
+#ifdef O_CYCLIC
+  if ( GD->defaults.argument == DEFARGUMENT )
+    GD->defaults.argument = GD->defaults.global/4;
+#endif
 }
 
 		 /*******************************
@@ -749,6 +764,7 @@ getDefaultsFromRegistry()
 BeginPredDefs(win)
   PRED_DEF("win_shell", 2, win_shell2, 0)
   PRED_DEF("win_shell", 3, win_shell3, 0)
+  PRED_DEF("win_registry_get_value", 3, win_registry_get_value, 0) 
 EndPredDefs
 
 #endif /*__WINDOWS__*/
