@@ -1,4 +1,4 @@
-/*  $Id: pl-op.c,v 1.29 2004/09/21 14:22:26 jan Exp $
+/*  $Id: pl-op.c,v 1.36 2007/01/12 21:12:29 jan Exp $
 
     Part of SWI-Prolog
 
@@ -337,7 +337,7 @@ of matching operators and (on backtracking) return the matching ones.
 static void
 addOpToBuffer(Buffer b, atom_t name, int type, int priority)
 { opdef *op = baseBuffer(b, opdef);
-  int mx    = entriesBuffer(b, opdef);
+  int mx    = (int)entriesBuffer(b, opdef);
   int i;
   opdef new;
 
@@ -455,10 +455,11 @@ current_op(Module m, int inherit,
     }
     default:
       assert(0);
+      fail;				/*NOTREACHED*/
   }
 
   Mark(mrk);
-  mx = entriesBuffer(b, opdef);
+  mx = (int)entriesBuffer(b, opdef);
   match = baseBuffer(b, opdef) + e->index;
   for(; e->index++<mx; match++)
   { if ( match->priority == 0 )		/* canceled operator */
@@ -511,19 +512,20 @@ static const opdef operators[] = {
   OP(ATOM_plus,		OP_FX,		500),	/* + */
   OP(ATOM_plus,		OP_YFX,		500),
   OP(ATOM_comma,	OP_XFY,	       1000),	/* , */
-  OP(ATOM_minus,	OP_FX,		500),	/* - */
+  OP(ATOM_minus,	OP_FY,		200),	/* - */
   OP(ATOM_minus,	OP_YFX,		500),
   OP(ATOM_grammar,	OP_XFX,	       1200),	/* --> */
   OP(ATOM_ifthen,	OP_XFY,	       1050),	/* -> */
   OP(ATOM_softcut,	OP_XFY,	       1050),	/* *-> */
   OP(ATOM_divide,	OP_YFX,		400),	/* / */
   OP(ATOM_div,		OP_YFX,		400),	/* // */
+  OP(ATOM_rdiv,		OP_YFX,		400),	/* rdiv */
   OP(ATOM_and,		OP_YFX,		500),	/* /\ */
   OP(ATOM_colon,	OP_XFY,		600),	/* : */
   OP(ATOM_prove,	OP_FX,	       1200),	/* :- */
   OP(ATOM_prove,	OP_XFX,	       1200),
   OP(ATOM_semicolon,	OP_XFY,	       1100),	/* ; */
-  OP(ATOM_bar,		OP_XFY,	       1100),	/* | */
+  OP(ATOM_bar,		OP_XFY,	       1105),	/* | */
   OP(ATOM_smaller,	OP_XFX,		700),	/* < */
   OP(ATOM_lshift,	OP_YFX,		400),	/* << */
   OP(ATOM_equals,	OP_XFX,		700),	/* = */
@@ -535,13 +537,12 @@ static const opdef operators[] = {
   OP(ATOM_ar_not_equal,	OP_XFX,		700),	/* =\= */
   OP(ATOM_larger,	OP_XFX,		700),	/* > */
   OP(ATOM_rshift,	OP_YFX,		400),	/* >> */
-  OP(ATOM_obtain,	OP_FX,		500),	/* ? */
   OP(ATOM_query,	OP_FX,	       1200),	/* ?- */
   OP(ATOM_at_smaller,	OP_XFX,		700),	/* @< */
   OP(ATOM_at_smaller_eq,OP_XFX,		700),	/* @=< */
   OP(ATOM_at_larger,	OP_XFX,		700),	/* @> */
   OP(ATOM_at_larger_eq,	OP_XFX,		700),	/* @>= */
-  OP(ATOM_backslash,	OP_FX,		500),	/* \ */
+  OP(ATOM_backslash,	OP_FY,		200),	/* \ */
   OP(ATOM_not_provable,	OP_FY,		900),	/* \+ */
   OP(ATOM_or,		OP_YFX,		500),	/* \/ */
   OP(ATOM_not_equals,	OP_XFX,		700),	/* \= */
@@ -555,6 +556,7 @@ static const opdef operators[] = {
   OP(ATOM_volatile,	OP_FX,	       1150), 	/* volatile */
   OP(ATOM_thread_local,	OP_FX,	       1150), 	/* thread_local */
   OP(ATOM_initialization,OP_FX,	       1150), 	/* initialization */
+  OP(ATOM_thread_initialization,OP_FX, 1150), 	/* thread_initialization */
   OP(ATOM_is,		OP_XFX,		700),	/* is */
   OP(ATOM_mod,		OP_YFX,		400),	/* mod */
   OP(ATOM_rem,		OP_YFX,		400),	/* rem */
@@ -592,7 +594,7 @@ pl_builtin_op(term_t prec, term_t type, term_t name, control_t h)
       i = 0;
       break;
     case FRG_REDO:
-      i = ForeignContextInt(h);
+      i = (int)ForeignContextInt(h);
       break;
     case FRG_CUTTED:
     default:
