@@ -1,4 +1,4 @@
-/*  $Id: pl-os.c,v 1.182 2006/07/31 12:10:50 jan Exp $
+/*  $Id: pl-os.c,v 1.199 2007/01/13 12:49:07 jan Exp $
 
     Part of SWI-Prolog
 
@@ -75,7 +75,7 @@
 #include <dos.h>
 #endif
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 #define STAT_TYPE struct _stat
 #else
 #define STAT_TYPE struct stat
@@ -101,7 +101,7 @@ static char *	Which(const char *program, char *fullname);
 		 *	       GLOBALS		*
 		 *******************************/
 #ifdef HAVE_CLOCK
-long clock_wait_ticks;
+intptr_t clock_wait_ticks;
 #endif
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -133,7 +133,7 @@ initOs(void)
   DEBUG(1, Sdprintf("OS:initEnviron() ...\n"));
   initEnviron();
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
   set(&features, FILE_CASE_PRESERVING_FEATURE);
 #else
   set(&features, FILE_CASE_FEATURE);
@@ -179,7 +179,7 @@ char *
 OsError(void)
 {
 #ifdef HAVE_STRERROR
-#ifdef __WIN32__
+#ifdef __WINDOWS__
   return strerror(_xos_errno());
 #else
   return strerror(errno);
@@ -218,7 +218,7 @@ static char errmsg[64];
     consult a file or to execute a query.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#ifndef __WIN32__			/* defined in pl-nt.c */
+#ifndef __WINDOWS__			/* defined in pl-nt.c */
 
 #ifdef HAVE_TIMES
 #include <sys/times.h>
@@ -286,10 +286,10 @@ CpuTime(cputime_kind which)
 #endif
 }
 
-#endif /*__WIN32__*/
+#endif /*__WINDOWS__*/
 
 void
-PL_clock_wait_ticks(long waited)
+PL_clock_wait_ticks(intptr_t waited)
 {
 #ifdef HAVE_CLOCK
   clock_wait_ticks += waited;
@@ -301,6 +301,12 @@ double
 WallTime(void)
 { double stime;
 
+#if HAVE_CLOCK_GETTIME
+  struct timespec tp;
+
+  clock_gettime(CLOCK_REALTIME, &tp);
+  stime = (double)tp.tv_sec + (double)tp.tv_nsec/1000000000.0;
+#else
 #ifdef HAVE_GETTIMEOFDAY
   struct timeval tp;
 
@@ -315,6 +321,7 @@ WallTime(void)
 #else
   stime = (double)time((time_t *)NULL);
 #endif
+#endif
 #endif
 
   return stime;
@@ -366,7 +373,7 @@ FreeMemory(void)
     uint64_t _PL_Random()
 
     Return a random number. Used for arithmetic only. More trouble. On
-    some systems (WIN32) the seed of rand() is thread-local, while on
+    some systems (__WINDOWS__) the seed of rand() is thread-local, while on
     others it is global.  We appear to have the choice between
 
     	# srand()/rand()
@@ -382,26 +389,26 @@ FreeMemory(void)
 
 static void
 initRandom(void)
-{ long init;
+{ unsigned int seed;
 
-#ifdef __WIN32__
-  init = GetTickCount();
+#ifdef __WINDOWS__
+  seed = (unsigned int)GetTickCount();
 #else
 #ifdef HAVE_GETTIMEOFDAY
   struct timeval tp;
 
   gettimeofday(&tp, NULL);
-  init = tp.tv_sec + tp.tv_usec;
+  seed = (unsigned int)(tp.tv_sec + tp.tv_usec);
 #else
-  init = (long)time((time_t *) NULL);
+  seed = (unsigned int)time((time_t *) NULL);
 #endif
 #endif
 
 #ifdef HAVE_SRANDOM
-  srandom(init);
+  srandom(seed);
 #else
 #ifdef HAVE_SRAND
-  srand(init);
+  srand(seed);
 #endif
 #endif
 }
@@ -414,7 +421,7 @@ _PL_Random(void)
   }
 
 #ifdef HAVE_RANDOM
-#if SIZEOF_LONG == 4
+#if SIZEOF_VOIDP == 4
   { uint64_t l = random();
     
     l ^= (uint64_t)random()<<32;
@@ -472,7 +479,7 @@ struct tempfile
 #define tmpfile_tail (GD->os._tmpfile_tail)
 
 #ifndef DEFTMPDIR
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #define DEFTMPDIR "c:/tmp"
 #else
 #define DEFTMPDIR "/tmp"
@@ -486,8 +493,8 @@ TemporaryFile(const char *id)
   char envbuf[MAXPATHLEN];
   char *tmpdir;
 
-  if ( !((tmpdir = getenv3("TEMP", envbuf, sizeof(envbuf))) ||
-	 (tmpdir = getenv3("TMP",  envbuf, sizeof(envbuf)))) )
+  if ( !((tmpdir = Getenv("TEMP", envbuf, sizeof(envbuf))) ||
+	 (tmpdir = Getenv("TMP",  envbuf, sizeof(envbuf)))) )
     tmpdir = DEFTMPDIR;
 
 #ifdef __unix__
@@ -498,7 +505,7 @@ TemporaryFile(const char *id)
 }
 #endif
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 { char *tmp;
   static int temp_counter = 0;
 
@@ -565,7 +572,7 @@ macros to deal with 16-bit machines, but are not  defined  as  functions
 here.   Some  more  specific things SWI-Prolog wants to know about files
 are defined here:
 
-    long LastModifiedFile(path)
+    intptr_t LastModifiedFile(path)
 	 char *path;
 
     Returns the last time `path' has been modified.  Used by the  source
@@ -724,7 +731,7 @@ OsPath(const char *p, char *buf)
 }
 #endif /* O_XOS */
 
-long
+intptr_t
 LastModifiedFile(char *f)
 { char tmp[MAXPATHLEN];
 
@@ -734,13 +741,13 @@ LastModifiedFile(char *f)
   if ( statfunc(OsPath(f, tmp), &buf) < 0 )
     return -1;
 
-  return (long)buf.st_mtime;
+  return (intptr_t)buf.st_mtime;
 #endif
 
 #if tos
 #define DAY	(24*60*60L)
   static int msize[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
-  long t;
+  intptr_t t;
   int n;
   struct ffblk buf;
   struct dz
@@ -1049,7 +1056,7 @@ initExpand(void)
 #ifdef O_CANONISE_DIRS
 { char envbuf[MAXPATHLEN];
 
-  if ( (cpaths = getenv3("CANONICAL_PATHS", envbuf, sizeof(envbuf))) )
+  if ( (cpaths = Getenv("CANONICAL_PATHS", envbuf, sizeof(envbuf))) )
   { char buf[MAXPATHLEN];
 
     while(*cpaths)
@@ -1069,9 +1076,9 @@ initExpand(void)
     }
   }
 
-  if ( (dir = getenv3("HOME", envbuf, sizeof(envbuf))) ) canoniseDir(dir);
-  if ( (dir = getenv3("PWD",  envbuf, sizeof(envbuf))) ) canoniseDir(dir);
-  if ( (dir = getenv3("CWD",  envbuf, sizeof(envbuf))) ) canoniseDir(dir);
+  if ( (dir = Getenv("HOME", envbuf, sizeof(envbuf))) ) canoniseDir(dir);
+  if ( (dir = Getenv("PWD",  envbuf, sizeof(envbuf))) ) canoniseDir(dir);
+  if ( (dir = Getenv("CWD",  envbuf, sizeof(envbuf))) ) canoniseDir(dir);
 }
 #endif
 }
@@ -1264,22 +1271,36 @@ cleanupExpand(void)
 
 static char *
 canoniseFileName(char *path)
-{ char *out = path, *in = path;
+{ char *out = path, *in = path, *start = path;
   char *osave[100];
   int  osavep = 0;
 
+#ifdef O_HASDRIVES			/* C: */
+  if ( in[1] == ':' && isLetter(in[0]) )
+  { in += 2;
+
+    out = start = in;
+  }
+#endif
+#ifdef O_HASSHARES			/* //host/ */
+  if ( in[0] == '/' && in[1] == '/' && isAlpha(in[2]) )
+  { char *s;
+
+    for(s = in+3; *s && isAlpha(*s); s++)
+      ;
+    if ( *s == '/' )
+    { in = out = s+1;
+      start = in-1; 
+    }
+  }
+#endif
+
   while( in[0] == '/' && in[1] == '.' && in[2] == '.' && in[3] == '/' )
     in += 3;
   while( in[0] == '.' && in[1] == '/' )
     in += 2;
   if ( in[0] == '/' )
     *out++ = '/';
-#ifdef O_HASSHARES
-  if ( in[1] == '/' )
-  { in++;
-    *out++ = '/';
-  }
-#endif
   osave[osavep++] = out;
 
   while(*in)
@@ -1298,15 +1319,19 @@ canoniseFileName(char *path)
 	  { *out = EOS;
 	    return path;
 	  }
-	  if ( in[2] == '.' &&		/* delete /foo/../ */
-	       (in[3] == '/' || in[3] == EOS) && osavep > 0 )
-	  { out = osave[--osavep];
-	    in += 3;
-	    if ( in[0] == EOS && out > path )
-	    { out[-1] = EOS;		/* delete trailing / */
-	      return path;
+	  if ( in[2] == '.' && (in[3] == '/' || in[3] == EOS) )
+	  { if ( osavep > 0 )		/* delete /foo/../ */
+	    { out = osave[--osavep];
+	      in += 3;
+	      if ( in[0] == EOS && out > start+1 )
+	      { out[-1] = EOS;		/* delete trailing / */
+		return path;
+	      }
+	      goto again;
+	    } else if (	start[0] == '/' && out == start+1 )
+	    { in += 3;
+	      goto again;
 	    }
-	    goto again;
 	  }
 	}
       }
@@ -1412,7 +1437,7 @@ expandVars(const char *pattern, char *expanded, int maxlen)
       if ( !(value = GD->os.myhome) )
       { char envbuf[MAXPATHLEN];
 
-	if ( (value = getenv3("HOME", envbuf, sizeof(envbuf))) &&
+	if ( (value = Getenv("HOME", envbuf, sizeof(envbuf))) &&
 	     (value = PrologPath(value, wordbuf, sizeof(wordbuf))) )
 	{ GD->os.myhome = store_string(value);
 	} else
@@ -1481,7 +1506,7 @@ expandVars(const char *pattern, char *expanded, int maxlen)
 	  if ( var[0] == EOS )
 	    goto def;
 	  LOCK();
-	  value = getenv3(var, envbuf, sizeof(envbuf));
+	  value = Getenv(var, envbuf, sizeof(envbuf));
 	  if ( value == (char *) NULL )
 	  { if ( fileerrors )
 	    { term_t name = PL_new_term_ref();
@@ -1579,17 +1604,21 @@ ExpandOneFile(const char *spec, char *file)
 
 #ifdef O_HASDRIVES
 
+#define IS_DIR_SEPARATOR(c) ((c) == '/' || (c) == '\\')
+
 int
-IsAbsolutePath(const char *p)		/* /d:/ or d:/ */
+IsAbsolutePath(const char *p)				/* /d:/ */
 { if ( p[0] == '/' && p[2] == ':' && isLetter(p[1]) &&
        (p[3] == '/' || p[3] == '\0') )
     succeed;
 
-  if ( p[1] == ':' && isLetter(p[0]) && (p[2] == '/' || p[2] == '\0') )
+  if ( p[1] == ':' && isLetter(p[0]) &&			/* d:/ or d:\ */
+       (IS_DIR_SEPARATOR(p[2]) || p[2] == '\0') )
     succeed;
 
 #ifdef O_HASSHARES
-  if ( p[0] == '/' && p[1] == '/' )	/* //host/share */
+  if ( (p[0] == '/' && p[1] == '/') ||	/* //host/share */
+       (p[0] == '\\' && p[1] == '\\') )	/* \\host\share */
     succeed;
 #endif
 
@@ -1599,10 +1628,10 @@ IsAbsolutePath(const char *p)		/* /d:/ or d:/ */
 
 static inline int
 isDriveRelativePath(const char *p)	/* '/...' */
-{ return p[0] == '/' && !IsAbsolutePath(p);
+{ return IS_DIR_SEPARATOR(p[0]) && !IsAbsolutePath(p);
 }
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #undef mkdir
 #include <direct.h>
 #define mkdir _xos_mkdir
@@ -1614,7 +1643,7 @@ GetCurrentDriveLetter()
 #ifdef OS2
   return _getdrive();
 #endif
-#ifdef __WIN32__
+#ifdef __WINDOWS__
   return _getdrive() + 'a' - 1;
 #endif
 #ifdef __WATCOMC__
@@ -1855,7 +1884,7 @@ ChDir(const char *path)
   AbsoluteFile(path, tmp);
 
   if ( chdir(ospath) == 0 )
-  { int len;
+  { size_t len;
 
     len = strlen(tmp);
     if ( len == 0 || tmp[len-1] != '/' )
@@ -1880,7 +1909,7 @@ ChDir(const char *path)
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     struct tm *LocalTime(time, struct tm *r)
-	      long *time;
+	      intptr_t *time;
 
     Convert time in Unix internal form (seconds since Jan 1 1970) into a
     structure providing easier access to the time.
@@ -1901,13 +1930,13 @@ ChDir(const char *path)
 	int	tm_isdst;	/ * daylight saving time info * /
     };
 
-    long Time()
+    intptr_t Time()
 
     Return time in seconds after Jan 1 1970 (Unix' time notion).
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 struct tm *
-LocalTime(long int *t, struct tm *r)
+LocalTime(long *t, struct tm *r)
 {
 #if defined(_REENTRANT) && defined(HAVE_LOCALTIME_R)
   return localtime_r(t, r);
@@ -1940,9 +1969,9 @@ ResetStdin()
     GD->os.org_terminal = *Sinput->functions;
 }
 
-static int
-Sread_terminal(void *handle, char *buf, int size)
-{ long h = (long)handle;
+static ssize_t
+Sread_terminal(void *handle, char *buf, size_t size)
+{ intptr_t h = (intptr_t)handle;
   int fd = (int)h;
   source_location oldsrc = LD->read_source;
 
@@ -2175,49 +2204,58 @@ PopTty(IOSTREAM *s, ttybuf *buf)
 		*********************************/
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Simple  library  to  manipulate  the  Unix  environment.   The  modified
-environment  will  be  passed  to  child  processes  and the can also be
-requested via getenv/2 from Prolog.  Functions
+Simple  library  to  manipulate  the    OS   environment.  The  modified
+environment will be passed to  child  processes   and  the  can  also be
+requested via getenv/2 from Prolog. Functions
 
-    char *Setenv(name, value)
+    int Setenv(name, value)
          char *name, *value;
 	
-    Set the Unix environment variable with name `name'.   If  it  exists
+    Set the OS environment variable with name `name'.   If  it  exists
     its  value  is  changed, otherwise a new entry in the environment is
     created.  The return value is a pointer to the old value, or NULL if
     the variable is new.
 
-    char *Unsetenv(name)
+    int Unsetenv(name)
          char *name;
 
     Delete a variable from the environment.  Return  value  is  the  old
     value, or NULL if the variable did not exist.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#ifndef __WIN32__
-char *
-getenv3(const char *name, char *buf, unsigned int len)
-{ char *s = getenv(name);
+size_t
+getenv3(const char *name, char *buf, size_t len)
+{
+#if O_XOS
+  return _xos_getenv(name, buf, len);
+#else
+  char *s = getenv(name);
+  size_t l;
 
-  if ( s && strlen(s) < len )
-  { strcpy(buf, s);
+  if ( s )
+  { if ( (l=strlen(s)) < len )
+      memcpy(buf, s, l+1);
+    else if ( len > 0 )
+      buf[0] = EOS;                     /* empty string if not fit */
 
-    return buf;
+    return l;
   }
 
+  return (size_t)-1;
+#endif
+}
+
+
+char *
+Getenv(const char *name, char *buf, size_t len)
+{ size_t l = getenv3(name, buf, len);
+
+  if ( l != (size_t)-1 && l < len )
+    return buf;
+
   return NULL;
 }
 
-int
-getenvl(const char *name)
-{ char *s;
-
-  if ( (s = getenv(name)) )
-    return strlen(s);
-
-  return -1;
-}
-#endif
 
 #if defined(HAVE_PUTENV) || defined(HAVE_SETENV)
 
@@ -2226,17 +2264,24 @@ Setenv(char *name, char *value)
 { 
 #ifdef HAVE_SETENV
   if ( setenv(name, value, TRUE) != 0 )
-    return PL_error("setenv", 2, NULL, ERR_NOMEM);
+    return PL_error(NULL, 0, MSG_ERRNO, ERR_SYSCALL, "setenv");
 #else
-  char *buf = alloca(strlen(name) + strlen(value) + 2);
+  char *buf;
+
+  if ( *name == '\0' || strchr(name, '=') != NULL )
+  { errno = EINVAL;
+    return PL_error(NULL, 0, MSG_ERRNO, ERR_SYSCALL, "setenv");
+  }
+
+  buf = alloca(strlen(name) + strlen(value) + 2);
 
   if ( buf )
   { Ssprintf(buf, "%s=%s", name, value);
 
     if ( putenv(store_string(buf)) < 0 )
-      return PL_error("setenv", 2, NULL, ERR_NOMEM);
+      return PL_error(NULL, 0, MSG_ERRNO, ERR_SYSCALL, "setenv");
   } else
-    return PL_error("setenv", 2, NULL, ERR_NOMEM);
+    return PL_error(NULL, 0, NULL, ERR_NOMEM);
 #endif
   succeed;
 }
@@ -2245,7 +2290,12 @@ int
 Unsetenv(char *name)
 {
 #ifdef HAVE_UNSETENV
+#ifdef VOID_UNSETENV
   unsetenv(name);
+#else
+  if ( unsetenv(name) < 0 )
+    return PL_error(NULL, 0, MSG_ERRNO, ERR_SYSCALL, "unsetenv");
+#endif
 
   succeed;
 #else
@@ -2540,14 +2590,14 @@ System(char *cmd)
 The routine system_via_shell() has been written by Tom Demeijer.  Thanks!
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define _SHELL_P ((long *)0x4f6L)
+#define _SHELL_P ((intptr_t *)0x4f6L)
 #define SHELL_OK (do_sys != 0)
 
 int cdecl (*do_sys)(const char *cmd); /* Parameter on stack ! */
 
 static int
 system_via_shell(const char *cmd)
-{ long oldssp;
+{ intptr_t oldssp;
 
   oldssp = Super((void *)0L);
   do_sys = (void (*))*_SHELL_P;
@@ -2656,7 +2706,7 @@ System(char *command)
 #endif
 
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #define SPECIFIC_SYSTEM 1
 
 					/* definition in pl-nt.c */
@@ -2682,7 +2732,7 @@ char *command;
     Return the path name of the executable of SWI-Prolog.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#ifndef __WIN32__			/* Win32 version in pl-nt.c */
+#ifndef __WINDOWS__			/* Win32 version in pl-nt.c */
 
 char *
 findExecutable(const char *av0, char *buffer)
@@ -2726,7 +2776,7 @@ findExecutable(const char *av0, char *buffer)
 
   return strcpy(buffer, file ? file : buf);
 }
-#endif /*__WIN32__*/
+#endif /*__WINDOWS__*/
 
 
 #ifdef __unix__
@@ -2749,7 +2799,7 @@ okToExec(const char *s)
 #define PATHSEP ','
 #endif
 
-#if defined(OS2) || defined(__DOS__) || defined(__WINDOWS__) || defined(__WIN32__)
+#if defined(OS2) || defined(__DOS__) || defined(__WINDOWS__)
 #define EXEC_EXTENSIONS { ".exe", ".com", ".bat", ".cmd", NULL }
 #define PATHSEP ';'
 #endif
@@ -2845,7 +2895,7 @@ Which(const char *program, char *fullname)
     point,  expressing  the  time  to sleep in seconds.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #define PAUSE_DONE 1			/* see pl-nt.c */
 #endif
 
@@ -2861,7 +2911,7 @@ Pause(real t)
     succeed;
 
   req.tv_sec = (time_t) t;
-  req.tv_nsec = (long)((t - floor(t)) * 1000000000);
+  req.tv_nsec = (intptr_t)((t - floor(t)) * 1000000000);
 
   for(;;)
   { rc = nanosleep(&req, &req);
@@ -2881,7 +2931,11 @@ Pause(real t)
 
 int
 Pause(real t)
-{ usleep((unsigned long)(t * 1000000.0));
+{
+  if ( t <= 0.0 )
+    return;
+
+  usleep((uintptr_t)(t * 1000000.0));
 
   return TRUE;
 }
@@ -2900,8 +2954,8 @@ Pause(real time)
     return;
 
   if ( time < 60.0 )		/* select() is expensive. Does it make sense */
-  { timeout.tv_sec = (long) time;
-    timeout.tv_usec = (long)(time * 1000000) % 1000000;
+  { timeout.tv_sec = (intptr_t) time;
+    timeout.tv_usec = (intptr_t)(time * 1000000) % 1000000;
     select(32, NULL, NULL, NULL, &timeout);
     
     return TRUE;
@@ -2973,9 +3027,9 @@ Pause(real t)
 
 int
 Pause(real t)
-{ long wait = (long)(t * 200.0);
-  long start_tick = clock();
-  long end_tick = wait + start_tick;
+{ intptr_t wait = (intptr_t)(t * 200.0);
+  intptr_t start_tick = clock();
+  intptr_t end_tick = wait + start_tick;
 
   while( clock() < end_tick )
   { if ( kbhit() )
@@ -2998,3 +3052,4 @@ Pause(real t)
 { return notImplemented("sleep", 1);
 }
 #endif
+
