@@ -1,4 +1,4 @@
-/*  $Id: pl-prims.c,v 1.182 2006/07/07 12:51:30 jan Exp $
+/*  $Id: pl-prims.c,v 1.191 2007/01/23 14:22:00 jan Exp $
 
     Part of SWI-Prolog
 
@@ -26,7 +26,7 @@
 #include "pl-incl.h"
 #include "pl-ctype.h"
 #undef ulong
-#define ulong unsigned long
+#define ulong uintptr_t
 
 #undef LD
 #define LD LOCAL_LD
@@ -328,7 +328,7 @@ Should this be int64_t for compatibility?
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static bool
-termHashValue(word term, long *hval ARG_LD)
+termHashValue(word term, intptr_t *hval ARG_LD)
 { for(;;)
   { switch(tag(term))
     { case TAG_VAR:
@@ -338,7 +338,7 @@ termHashValue(word term, long *hval ARG_LD)
 	*hval += atomValue(term)->hash_value;
         succeed;
       case TAG_STRING:
-      { unsigned len;
+      { size_t len;
 	char *s;
 
 	s = getCharsString(term, &len);
@@ -351,7 +351,7 @@ termHashValue(word term, long *hval ARG_LD)
 	{ *hval += valInt(term);
 	} else
 	{ Word p = valIndirectP(term);
-#if SIZEOF_LONG == 4
+#if SIZEOF_VOIDP == 4
 	  *hval += p[0]^p[1];
 #else
 	  *hval += p[0];
@@ -360,8 +360,8 @@ termHashValue(word term, long *hval ARG_LD)
         succeed;
       case TAG_FLOAT:
       { int i;
-	long *p = (long *)valIndirectP(term);
-	long h = *p;
+	intptr_t *p = (intptr_t *)valIndirectP(term);
+	intptr_t h = *p;
 
 	for(p++, i=WORDS_PER_DOUBLE-1; --i >= 0; )
 	  h ^= *p++;
@@ -416,7 +416,7 @@ static
 PRED_IMPL("hash_term", 2, hash_term, 0)
 { PRED_LD
   Word p = valTermRef(A1);
-  long hraw = 0L;
+  intptr_t hraw = 0L;
   Word *m = aTop;
   int rc;
 
@@ -455,17 +455,18 @@ compareAtoms(atom_t w1, atom_t w2)
   { if ( a1->type->compare )
     { return (*a1->type->compare)(w1, w2);
     } else
-    { int l   = (a1->length <= a2->length ? a1->length : a2->length);
+    { size_t l   = (a1->length <= a2->length ? a1->length : a2->length);
       int v;
 
       if ( (v=memcmp(a1->name, a2->name, l)) != 0 )
 	return v;
-      return (int)a1->length - (int)a2->length;
+      return a1->length == a2->length ? 0 :
+	     a1->length < a2->length ? -1 : 1;
     }
   } else if ( true(a1->type, PL_BLOB_TEXT) &&
 	      true(a2->type, PL_BLOB_TEXT) )
   { PL_chars_t t1, t2;
-    unsigned len;
+    size_t len;
 
     get_atom_text(w1, &t1);
     get_atom_text(w2, &t2);
@@ -481,7 +482,7 @@ compareAtoms(atom_t w1, atom_t w2)
 static int
 compareStrings(word w1, word w2 ARG_LD)
 { PL_chars_t t1, t2;
-  unsigned len;
+  size_t len;
 
   get_string_text(w1, &t1 PASS_LD);
   get_string_text(w2, &t2 PASS_LD);
@@ -516,7 +517,7 @@ tests (alphabetical order) are skipped and the call returns NOTEQ.
 static int
 do_compare(Word p1, Word p2, int eq ARG_LD)
 { word w1, w2;
-  int t1, t2;
+  word t1, t2;
 
 tail_recursion:
   deRef(p1);
@@ -1018,7 +1019,7 @@ PRED_IMPL("arg", 3, arg, PL_FA_NONDETERMINISTIC)
       return PL_error("arg", 3, NULL, ERR_TYPE, ATOM_integer, n);
     }
     case FRG_REDO:
-    { int argn = CTX_INT + 1;
+    { int argn = (int)CTX_INT + 1;
       term_t a = PL_new_term_ref();
 
       PL_get_name_arity(term, &name, &arity);
@@ -1270,7 +1271,7 @@ start:
     }
 
 #ifdef O_SHIFT_STACKS
-    if ( roomStack(global) < 2 * (long)sizeof(word) )
+    if ( roomStack(global) < 2 * (intptr_t)sizeof(word) )
     { growStacks(environment_frame, NULL, NULL, FALSE, TRUE, FALSE);
       p = valTermRef(t);
       deRef(p);
@@ -1991,7 +1992,7 @@ static word
 x_chars(const char *pred, term_t atom, term_t string, int how)
 { char *s;
   pl_wchar_t *ws;
-  unsigned int len;
+  size_t len;
   int arg1;
 
   if ( (how & X_NUMBER) )
@@ -2096,7 +2097,7 @@ pl_char_code(term_t atom, term_t chr)
 { PL_chars_t txt;
   int n;
 
-  if ( PL_get_text(atom, &txt, CVT_ATOM) && txt.length == 1 )
+  if ( PL_get_text(atom, &txt, CVT_ATOM|CVT_STRING) && txt.length == 1 )
   { if ( txt.encoding == ENC_WCHAR )
       return PL_unify_integer(chr, txt.text.w[0]);
     else
@@ -2116,7 +2117,7 @@ pl_char_code(term_t atom, term_t chr)
 static
 PRED_IMPL("atom_number", 2, atom_number, 0)
 { char *s;
-  unsigned len;
+  size_t len;
 
   if ( PL_get_nchars(A1, &len, &s, CVT_ATOM|CVT_STRING) )
   { number n;
@@ -2135,6 +2136,34 @@ PRED_IMPL("atom_number", 2, atom_number, 0)
 }
 
 
+static
+PRED_IMPL("collation_key", 2, collation_key, 0)
+{ wchar_t *s;
+  size_t len;
+  wchar_t buf[256];
+  size_t buflen = sizeof(buf)/sizeof(wchar_t);
+  wchar_t *o = buf;
+  size_t n;
+
+  if ( !PL_get_wchars(A1, &len, &s, CVT_ATOM|CVT_STRING|CVT_EXCEPTION) )
+    fail;
+  for(;;)
+  { if ( (n=wcsxfrm(o, s, buflen)) < buflen )
+    { int rc = PL_unify_wchars(A2, PL_STRING, n, o);
+
+      if ( o != buf )
+	PL_free(o);
+
+      return rc;
+    } else
+    { assert(o == buf);
+      buflen = n+1;
+      o = PL_malloc(buflen*sizeof(wchar_t));
+    }
+  }
+}
+
+
 static word
 concat(const char *pred,
        term_t a1, term_t a2, term_t a3, 
@@ -2198,7 +2227,7 @@ concat(const char *pred,
       return PL_unify_text_range(a1, &t3, 0, L3-L2, otype);
     fail;
   } else				/* -, -, + */
-  { unsigned int at_n;
+  { size_t at_n;
     mark m;
 
     switch ( ForeignControl(ctx) )
@@ -2254,7 +2283,7 @@ pl_atom_concat(term_t a1, term_t a2, term_t a3, control_t ctx)
 static int
 split_atom(term_t list, term_t sep, term_t atom)
 { PL_chars_t st, at;
-  int i, last;
+  size_t i, last;
   term_t tail = PL_copy_term_ref(list);
   term_t head = PL_new_term_ref();
 
@@ -2264,7 +2293,7 @@ split_atom(term_t list, term_t sep, term_t atom)
        !PL_get_text(sep, &st, CVT_ATOMIC) )
     return -1;
 
-  for(last=i=0; i<=(int)at.length-(int)st.length; )
+  for(last=i=0; (ssize_t)i<=(ssize_t)(at.length-st.length); )
   { if ( PL_cmp_text(&st, 0, &at, i, st.length) == 0 )
     { if ( !PL_unify_list(tail, head, tail) ||
 	   !PL_unify_text_range(head, &at, last, i-last, PL_ATOM) )
@@ -2301,7 +2330,7 @@ append_text_to_buffer(Buffer b, PL_chars_t *txt, IOENC *enc)
       addBuffer(b, chr, pl_wchar_t);
     }
   } else				/* promote our buffer */
-  { int len = entriesBuffer(b, char);
+  { size_t len = entriesBuffer(b, char);
     unsigned char *tmp = PL_malloc(len);
     const unsigned char *s = tmp;
     const unsigned char *e = &s[len];
@@ -2404,7 +2433,7 @@ word
 pl_apropos_match(term_t a1, term_t a2)
 { char *s1=NULL, *s2=NULL;
   pl_wchar_t *w1=NULL, *w2=NULL;
-  unsigned int l1, l2;
+  size_t l1, l2;
 
   if ( PL_get_chars(a1, &s1, CVT_ALL|BUF_RING) &&
        PL_get_chars(a2, &s2, CVT_ALL) )
@@ -2429,7 +2458,7 @@ pl_apropos_match(term_t a1, term_t a2)
 
     for (i2=0; i2<l2; i2++)
     { for(q=w1, s=w2+i2; q<eq && s<es; q++, s++)
-      { if ( *q != *s && *q != towlower(*s) )
+      { if ( *q != *s && *q != (pl_wchar_t)towlower(*s) )
 	  break;
       }
       if ( q == eq )
@@ -2475,17 +2504,17 @@ enum sub_type
 
 typedef struct
 { enum sub_type type;			/* Type of enumeration */
-  int n1;				/* 1-st state id */
-  int n2;				/* 2-nd state id */
-  int n3;
+  size_t n1;				/* 1-st state id */
+  size_t n2;				/* 2-nd state id */
+  size_t n3;
 } sub_state;
 
 
 static int
-get_positive_integer_or_unbound(term_t t, int *v)
-{ int i;
+get_positive_integer_or_unbound(term_t t, ssize_t *v)
+{ long i;
 
-  if ( PL_get_integer(t, &i) )
+  if ( PL_get_long(t, &i) )		/* TBD: should be ssize_t */
   { if ( i < 0 )
       PL_error(NULL, 0, NULL, ERR_DOMAIN,
 	       ATOM_not_less_than_zero, t);
@@ -2509,7 +2538,7 @@ sub_text(term_t atom,
 	 control_t h,
 	 int type)			/* PL_ATOM or PL_STRING */
 { PL_chars_t ta, ts;			/* the strings */
-  int b = -1, l = -1, a = -1;		/* the integers */
+  ssize_t b = -1, l = -1, a = -1;	/* the integers */
   sub_state *state;			/* non-deterministic state */
   atom_t expected = (type == PL_STRING ? ATOM_string : ATOM_atom);
   int match;
@@ -2545,7 +2574,7 @@ sub_text(term_t atom,
 	  fail;
 	}
 	if ( a >= 0 )			/* after given: test */
-	{ int off = la-a-ls;
+	{ ssize_t off = la-a-ls;
 
 	  if ( off >= 0 && PL_cmp_text(&ta, (unsigned)off, &ts, 0, ls) == 0 )
 	  { return (PL_unify_integer(len, ls) &&
@@ -2932,8 +2961,8 @@ pl_depth_limit_true(term_t limit, term_t olimit, term_t oreached,
       if ( PL_get_long_ex(limit, &l) &&
 	   PL_get_long_ex(olimit, &ol) &&
 	   PL_get_long_ex(oreached, &or) )
-      { long clevel = levelFrame(environment_frame) - 1;
-	long used = depth_reached - clevel - 1;
+      { intptr_t clevel = levelFrame(environment_frame) - 1;
+	intptr_t used = depth_reached - clevel - 1;
 	Choice ch;
 
 	depth_limit   = ol;
@@ -3037,9 +3066,9 @@ QP_STATISTICS is defined. The compatibility   is pretty complete, except
 the `atoms' key that is defined by both and this ambiguous.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static long
+static intptr_t
 heapUsed(void)
-{ long heap = GD->statistics.heap;	/* Big allocations */
+{ intptr_t heap = GD->statistics.heap;	/* Big allocations */
   
   heap += GD->alloc_pool.allocated;	/* global small allocations */
 #ifdef O_PLMT
@@ -3064,17 +3093,17 @@ qp_statistics__LD(atom_t key, int64_t v[], PL_local_data_t *ld)
   if ( key == ATOM_runtime )
   { v[0] = (int64_t)(LD->statistics.user_cputime * 1000.0);
     v[1] = v[0] - LD->statistics.last_cputime;
-    LD->statistics.last_cputime = (long)v[0];
+    LD->statistics.last_cputime = (intptr_t)v[0];
     vn = 2;
   } else if ( key == ATOM_system_time )
   { v[0] = (int64_t)(LD->statistics.system_cputime * 1000.0);
     v[1] = v[0] - LD->statistics.last_systime;
-    LD->statistics.last_systime = (long)v[0];
+    LD->statistics.last_systime = (intptr_t)v[0];
     vn = 2;
   } else if ( key == ATOM_real_time )
   { v[0] = (int64_t)WallTime();
     v[1] = v[0] - LD->statistics.last_walltime;
-    LD->statistics.last_walltime = (long)v[0];
+    LD->statistics.last_walltime = (intptr_t)v[0];
     vn = 2;
   } else if ( key == ATOM_memory || key == ATOM_core )
   { v[0] = UsedMemory();
@@ -3362,7 +3391,7 @@ PRED_IMPL("$option", 3, option, PL_FA_NONDETERMINISTIC)
 	for( ; optdefs[index].name; index++ )
 	{ switch( optdefs[index].type )
 	  { case CMDOPT_LONG:
-	    { long *val = optdefs[index].address;
+	    { intptr_t *val = optdefs[index].address;
 
 	      if ( !PL_unify_integer(old, *val) )
 		continue;
@@ -3384,7 +3413,7 @@ PRED_IMPL("$option", 3, option, PL_FA_NONDETERMINISTIC)
       }
       break;
     case FRG_REDO:
-      index = CTX_INT;
+      index = (int)CTX_INT;
       goto next;
     case FRG_CUTTED:
       succeed;
@@ -3439,14 +3468,14 @@ set_pl_option(const char *name, const char *value)
   { if ( streq(name, d->name) )
     { switch(d->type)
       { case CMDOPT_LONG:
-	{ long *val = d->address;
+	{ intptr_t *val = d->address;
 	  number n;
 	  unsigned char *q;
 
 	  if ( str_number((unsigned char *)value, &q, &n, FALSE) &&
 	       *q == EOS &&
 	       intNumber(&n) )
-	  { *val = (long)n.value.i;
+	  { *val = (intptr_t)n.value.i;
 	    succeed;
 	  }
 	  fail;
@@ -3535,6 +3564,7 @@ BeginPredDefs(prims)
   PRED_DEF("$depth_limit_false",  3, depth_limit_false, 0)
 #endif
   PRED_DEF("atom_number", 2, atom_number, 0)
+  PRED_DEF("collation_key", 2, collation_key, 0)
   PRED_DEF("statistics", 2, statistics, 0)
   PRED_DEF("$option", 3, option, PL_FA_NONDETERMINISTIC)
   PRED_DEF("$style_check", 2, style_check, 0)
