@@ -1,4 +1,4 @@
-/*  $Id: pl-pro.c,v 1.74.2.1 2005/01/17 09:07:43 jan Exp $
+/*  $Id: pl-pro.c,v 1.81 2007/01/12 13:20:52 jan Exp $
 
     Part of SWI-Prolog
 
@@ -60,7 +60,7 @@ pl_break1(term_t goal)
 
   IOSTREAM *inSave  = Scurin;
   IOSTREAM *outSave = Scurout;
-  long skipSave     = debugstatus.skiplevel;
+  intptr_t skipSave     = debugstatus.skiplevel;
   int  suspSave     = debugstatus.suspendTrace;
   int  traceSave;
   debug_type debugSave;
@@ -96,7 +96,7 @@ word
 pl_notrace1(term_t goal)
 { bool rval;
 
-  long	     skipSave  = debugstatus.skiplevel;
+  intptr_t	     skipSave  = debugstatus.skiplevel;
   bool	     traceSave = debugstatus.tracing;
 
   rval = callProlog(NULL, goal, PL_Q_NODEBUG, NULL);
@@ -178,6 +178,7 @@ little choice.
 static word
 pl_throw_abort()
 { pl_notrace();
+  Sreset();
 
   if ( LD->critical > 0 )		/* abort in critical region: delay */
   { LD->aborted = TRUE;
@@ -254,7 +255,7 @@ resetProlog()
     emptyStacks();
 
 #ifdef O_LIMIT_DEPTH
-  depth_limit   = (unsigned long)DEPTH_NO_LIMIT;
+  depth_limit   = (uintptr_t)DEPTH_NO_LIMIT;
 #endif
 
   gc_status.blocked    = 0;
@@ -380,7 +381,7 @@ printk(char *fm, ...)
   trap_gdb();
 }
 
-static long check_marked;
+static intptr_t check_marked;
 
 #define mark(p)		(*(p) |= MARK_MASK, check_marked++)
 #define unmark(p)	(*(p) &= ~MARK_MASK, check_marked--)
@@ -474,12 +475,15 @@ last_arg:
     if ( isReal(*p) )
       return key+(word) valReal(*p);
     if ( isString(*p) )
-    { long sz, len;
+    { size_t sz, len;
+      char *s;
 
-      if ( (sz=sizeString(*p)) != (len=strlen(valString(*p))) )
+      s = getCharsString(*p, &sz);
+
+      if ( sz != (len=strlen(s)) )
       { if ( sz < len )
 	  printk("String has inconsistent length: 0x%x", *p);
-	else if ( valString(*p)[sz] )
+	else if ( s[sz] )
 	  printk("String not followed by NUL-char: 0x%x", *p);
 /*	else
 	  printf("String contains NUL-chars: 0x%x", *p);
@@ -492,8 +496,8 @@ last_arg:
   }
 
   if ( isAtom(*p) )
-  { unsigned long idx;
-    unsigned long mx = entriesBuffer(&atom_array, Atom);
+  { uintptr_t idx;
+    uintptr_t mx = entriesBuffer(&atom_array, Atom);
 
     assert(!is_marked(p));
     if ( storage(*p) != STG_STATIC )
