@@ -1,4 +1,4 @@
-/*  $Id: pl-proc.c,v 1.152 2006/07/27 19:25:05 jan Exp $
+/*  $Id: pl-proc.c,v 1.156 2007/01/13 12:49:07 jan Exp $
 
     Part of SWI-Prolog
 
@@ -181,7 +181,7 @@ hasClausesDefinition(Definition def)
     { GET_LD
       ClauseRef c;
 #ifdef O_LOGICAL_UPDATE
-      unsigned long generation;
+      uintptr_t generation;
       LocalFrame fr = environment_frame;
       if ( fr )
 	generation = generationFrame(fr);
@@ -934,7 +934,7 @@ removeClausesProcedure(Procedure proc, int sfindex, int fromfile)
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Unlink a clause from the  definition,  both   from  the  index table and
 clause-chain. The clause itself is not  deleted,   this  task is left to
-retractClauseProcedure().
+retractClauseDefinition().
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static void
@@ -977,10 +977,8 @@ the definition is always referenced.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 bool
-retractClauseProcedure(Procedure proc, Clause clause ARG_LD)
-{ Definition def = getProcDefinition(proc);
-
-  LOCKDYNDEF(def);
+retractClauseDefinition(Definition def, Clause clause ARG_LD)
+{ LOCKDYNDEF(def);
   assert(true(def, DYNAMIC));
   if ( true(clause, ERASED) )
   { UNLOCKDYNDEF(def);
@@ -1702,7 +1700,7 @@ pl_retract(term_t term, control_t h)
     r0 = PL_new_term_refs(0);
     while( cref )
     { if ( decompile(cref->clause, cl, 0) )
-      { retractClauseProcedure(proc, cref->clause PASS_LD);
+      { retractClauseDefinition(getProcDefinition(proc), cref->clause PASS_LD);
 	if ( !next )
 	{ PL_reset_term_refs(r0);
 	  leaveDefinition(def);
@@ -1774,7 +1772,7 @@ pl_retractall(term_t head)
   r0 = PL_new_term_refs(0);
   while( cref )
   { if ( decompileHead(cref->clause, thehead) )
-      retractClauseProcedure(proc, cref->clause PASS_LD);
+      retractClauseDefinition(def, cref->clause PASS_LD);
 
     PL_reset_term_refs(r0);
     Undo(m);
@@ -1854,7 +1852,7 @@ pl_abolish1(term_t spec)		/* Name/Arity */
 }
 
 
-static unsigned long
+static uintptr_t
 attribute_mask(atom_t key)
 {
 #define TRACE_ANY (TRACE_CALL|TRACE_REDO|TRACE_EXIT|TRACE_FAIL)
@@ -1890,7 +1888,7 @@ pl_get_predicate_attribute(term_t pred,
   functor_t fd;
   atom_t key;
   Module module = (Module) NULL;
-  unsigned long att;
+  uintptr_t att;
   term_t head = PL_new_term_ref();
   
   if ( !PL_strip_module(pred, &module, head) ||
@@ -2103,7 +2101,7 @@ pl_set_predicate_attribute(term_t pred,
   Definition def;
   atom_t key;
   int val, rc;
-  unsigned long att;
+  uintptr_t att;
 
   if ( !PL_get_atom(what, &key) )
     return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_atom, what);
@@ -2243,7 +2241,7 @@ reindexDefinition(Definition def)
 
 
 void
-indexDefinition(Definition def, unsigned long pattern)
+indexDefinition(Definition def, long pattern)
 { clear(def, AUTOINDEX);
 
   if ( (def->indexPattern & ~NEED_REINDEX) != 0x1L &&
@@ -2355,7 +2353,7 @@ registerSourceFile(SourceFile f)
 { if ( !GD->files.source_files.base )
     initBuffer(&GD->files.source_files);
 
-  f->index = entriesBuffer(&GD->files.source_files, SourceFile) + 1;
+  f->index = (int)entriesBuffer(&GD->files.source_files, SourceFile) + 1;
     
   addBuffer(&GD->files.source_files, f, SourceFile);
 }
@@ -2406,7 +2404,7 @@ lookupSourceFile(atom_t name)
 
 SourceFile
 indexToSourceFile(int index)
-{ int n = entriesBuffer(&GD->files.source_files, SourceFile);
+{ int n = (int)entriesBuffer(&GD->files.source_files, SourceFile);
 
   index--;
   if ( index >= 0 && index < n )
@@ -2499,7 +2497,7 @@ redefineProcedure(Procedure proc, SourceFile sf)
 
 word
 pl_make_system_source_files(void)
-{ int i, n = entriesBuffer(&GD->files.source_files, SourceFile);
+{ int i, n = (int)entriesBuffer(&GD->files.source_files, SourceFile);
 
 
   for(i=0; i<n; i++)
@@ -2581,14 +2579,14 @@ word
 pl_time_source_file(term_t file, term_t time, control_t h)
 { GET_LD
   int index;
-  int mx = entriesBuffer(&GD->files.source_files, SourceFile);
+  int mx = (int)entriesBuffer(&GD->files.source_files, SourceFile);
 
   switch( ForeignControl(h) )
   { case FRG_FIRST_CALL:
       index = 0;
       break;
     case FRG_REDO:
-      index = ForeignContextInt(h);
+      index = (int)ForeignContextInt(h);
       break;
     case FRG_CUTTED:
     default:
