@@ -1,4 +1,4 @@
-/*  $Id: pl-prof.c,v 1.53 2006/07/27 09:22:42 jan Exp $
+/*  $Id: pl-prof.c,v 1.57 2007/01/13 13:17:46 jan Exp $
 
     Part of SWI-Prolog
 
@@ -53,7 +53,7 @@ static PL_prof_type_t *types[MAX_PROF_TYPES] = { &prof_default_type };
 #define PROFNODE_MAGIC 0x7ae38f24
 
 typedef struct call_node
-{ long 		    magic;		/* PROFNODE_MAGIC */
+{ intptr_t 		    magic;		/* PROFNODE_MAGIC */
   struct call_node *parent;
   void *            handle;		/* handle to procedure-id */
   PL_prof_type_t   *type;
@@ -84,26 +84,30 @@ activateProfiler(int active ARG_LD)
 }
 
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 MS-Windows version
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static void profile(long ticks, PL_local_data_t *ld);
+static void profile(intptr_t ticks, PL_local_data_t *ld);
+
+#if (_MSC_VER < 1400)
+typedef DWORD DWORD_PTR;
+#endif
 
 static LARGE_INTEGER last_profile;
 static HANDLE	     mythread;
 static PL_local_data_t *my_LD;
 static UINT	     timer;
-static long	     virtual_events;
-static long	     events;
+static intptr_t	     virtual_events;
+static intptr_t	     events;
 
-static long
+static intptr_t
 prof_new_ticks(HANDLE thread)
 { FILETIME created, exit, kernel, user;
   LARGE_INTEGER u;
-  long ticks;
+  intptr_t ticks;
 
   if ( !GetThreadTimes(thread,
 		       &created,
@@ -115,7 +119,7 @@ prof_new_ticks(HANDLE thread)
   u.LowPart  = user.dwLowDateTime;
   u.HighPart = user.dwHighDateTime;
 
-  ticks = (long)((u.QuadPart - last_profile.QuadPart)/10240);
+  ticks = (intptr_t)((u.QuadPart - last_profile.QuadPart)/10240);
   last_profile = u;
 
   virtual_events += ticks;
@@ -125,8 +129,8 @@ prof_new_ticks(HANDLE thread)
 }
 
 static void CALLBACK
-callTimer(UINT id, UINT msg, DWORD dwuser, DWORD dw1, DWORD dw2)
-{ long newticks;
+callTimer(UINT id, UINT msg, DWORD_PTR dwuser, DWORD_PTR dw1, DWORD_PTR dw2)
+{ intptr_t newticks;
 
   SuspendThread(mythread);		/* stop thread to avoid trouble */
   if ( (newticks = prof_new_ticks(mythread)) )
@@ -163,7 +167,7 @@ startProfiler(void)
   rval = timeSetEvent(10,
 		      5,		/* resolution (milliseconds) */
 		      callTimer,
-		      (DWORD)0,
+		      (DWORD_PTR)0,
 		      TIME_PERIODIC);
   if ( rval )
     timer = rval;
@@ -192,7 +196,7 @@ stopItimer(void)
   }
 }
 
-#else /*__WIN32__*/
+#else /*__WINDOWS__*/
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 POSIX version
@@ -245,7 +249,7 @@ stopItimer(void)
   }
 }
 
-#endif /*__WIN32__*/
+#endif /*__WINDOWS__*/
 
 static bool
 stopProfiler(void)
@@ -255,7 +259,7 @@ stopProfiler(void)
 
   stopItimer();
   activateProfiler(FALSE PASS_LD);
-#ifndef __WIN32__
+#ifndef __WINDOWS__
   set_sighandler(SIGPROF, SIG_IGN);
 #endif
 
@@ -777,10 +781,10 @@ clear the flags again.
 #define LD LOCAL_LD
 
 static void
-#ifdef __WIN32__
-profile(long count, PL_local_data_t *__PL_ld)
+#ifdef __WINDOWS__
+profile(intptr_t count, PL_local_data_t *__PL_ld)
 { 
-#else /*__WIN32__*/
+#else /*__WINDOWS__*/
 profile(int sig)
 { GET_LD
 
@@ -795,7 +799,7 @@ profile(int sig)
   signal(SIGPROF, profile);
 #endif
 
-#endif /*__WIN32__*/
+#endif /*__WINDOWS__*/
   LD->profile.ticks += count;
   
   if ( accounting )
