@@ -1,4 +1,4 @@
-/*  $Id: pl-rec.c,v 1.91 2005/09/05 19:41:05 jan Exp $
+/*  $Id: pl-rec.c,v 1.97 2007/01/23 14:22:00 jan Exp $
 
     Part of SWI-Prolog
 
@@ -131,7 +131,7 @@ cleanRecordList(RecordList rl)
 typedef struct
 { tmp_buffer code;			/* code buffer */
   tmp_buffer vars;			/* variable pointers */
-  int	     size;			/* size on global stack */
+  size_t     size;			/* size on global stack */
   uint	     nvars;			/* # variables */
   int	     external;			/* Allow for external storage */
 } compile_info, *CompileInfo;
@@ -180,15 +180,15 @@ independent.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static inline void
-addUintBuffer(Buffer b, uint val)
+addUintBuffer(Buffer b, size_t val)
 { if ( !(val & ~0x7f) )
-    addBuffer(b, val, uchar);
+    addBuffer(b, (uchar)val, uchar);
   else
   { int zips = ((sizeof(val))*8+7-1)/7 - 1;
     int leading = TRUE;
 
     for(; zips >= 0; zips--)
-    { uint d = (val >> zips*7) & 0x7f;
+    { uint d = (uint)((val >> zips*7) & 0x7f);
 
       if ( d || !leading )
       { if ( zips != 0 )
@@ -202,13 +202,13 @@ addUintBuffer(Buffer b, uint val)
 
 
 static inline void
-addSizeInt(CompileInfo info, uint val)
+addSizeInt(CompileInfo info, size_t val)
 { addUintBuffer((Buffer)&info->code, val);
 }
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Add a signed long value. First byte   is  number of bytes, remaining are
+Add a signed intptr_t value. First byte   is  number of bytes, remaining are
 value-bytes, starting at most-significant.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
@@ -272,7 +272,7 @@ addWord(CompileInfo info, word w)
 
 
 static inline void
-addChars(CompileInfo info, int len, const char *data)
+addChars(CompileInfo info, size_t len, const char *data)
 { addSizeInt(info, len);
 
   addMultipleBuffer(&info->code, data, len, char);
@@ -282,11 +282,9 @@ addChars(CompileInfo info, int len, const char *data)
 static inline void
 addAtomValue(CompileInfo info, atom_t name)
 { Atom a = atomValue(name);
-  uint len = a->length;
 
   addSizeInt(info, a->length);
-
-  addMultipleBuffer(&info->code, a->name, len, char);
+  addMultipleBuffer(&info->code, a->name, a->length, char);
 }
 
 
@@ -335,7 +333,7 @@ right_recursion:
 
   switch(tag(w))
   { case TAG_VAR:
-    { long n = info->nvars++;
+    { intptr_t n = info->nvars++;
 
       *p = (n<<7)|TAG_ATOM|STG_GLOBAL;
       addBuffer(&info->vars, p, Word);
@@ -346,7 +344,7 @@ right_recursion:
     }
 #if O_ATTVAR
     case TAG_ATTVAR:
-    { long n = info->nvars++;
+    { intptr_t n = info->nvars++;
       Word ap = valPAttVar(w);
 
       if ( isEmptyBuffer(&info->code) )
@@ -368,7 +366,7 @@ right_recursion:
 #endif
     case TAG_ATOM:
     { if ( storage(w) == STG_GLOBAL )	/* this is a variable */
-      { long n = ((long)(w) >> 7);
+      { intptr_t n = ((intptr_t)(w) >> 7);
 
 	addOpCode(info, PL_TYPE_VARIABLE);
 	addSizeInt(info, n);
@@ -412,10 +410,10 @@ right_recursion:
       return;
     }
     case TAG_STRING:
-    { Word f  = addressIndirect(w);
-      int n   = wsizeofInd(*f);
-      int pad = padHdr(*f);		/* see also getCharsString() */
-      long l  = n*sizeof(word)-pad;
+    { Word f     = addressIndirect(w);
+      size_t n   = wsizeofInd(*f);
+      size_t pad = padHdr(*f);		/* see also getCharsString() */
+      size_t l   = n*sizeof(word)-pad;
 
       info->size += n+2;
       addOpCode(info, PL_TYPE_STRING);
@@ -451,7 +449,7 @@ right_recursion:
 	*aTop++ = (Word)f;
 	*aTop++ = (Word)f->definition;
 	f->definition = (functor_t)consInt(info->size);
-	assert(valInt(f->definition) == info->size); /* overflow test */
+	assert(valInt(f->definition) == (intptr_t)info->size); /* overflow test */
       }
 #endif
       
@@ -506,8 +504,8 @@ compileTermToHeap__LD(term_t t, int flags ARG_LD)
 { compile_info info;
   Record record;
   Word *p;
-  int size;
-  int rsize = SIZERECORD(flags);
+  size_t size;
+  size_t rsize = SIZERECORD(flags);
 #if O_CYCLIC
   Word *m = aTop;
 #endif
@@ -538,9 +536,9 @@ compileTermToHeap__LD(term_t t, int flags ARG_LD)
   
   size = rsize + sizeOfBuffer(&info.code);
   record = allocHeap(size);
-  record->gsize = info.size;
+  record->gsize = (unsigned int)info.size; /* only 28-bit */
   record->nvars = info.nvars;
-  record->size  = size;
+  record->size  = (int)size;
   record->flags = flags;
   if ( flags & R_DUPLICATE )
   { record->references = 1;
@@ -568,7 +566,7 @@ compileTermToHeap__LD(term_t t, int flags ARG_LD)
 
 #define REC_SZMASK  (REC_32|REC_64)	/* SIZE_MASK */
 
-#if SIZEOF_LONG == 8
+#if SIZEOF_VOIDP == 8
 #define REC_SZ REC_64
 #else
 #define REC_SZ REC_32
@@ -578,7 +576,7 @@ compileTermToHeap__LD(term_t t, int flags ARG_LD)
 #define REC_COMPAT(m)	(((m)&(REC_VMASK|REC_SZMASK)) == REC_HDR)
 
 char *
-PL_record_external(term_t t, unsigned int *len)
+PL_record_external(term_t t, size_t *len)
 { GET_LD
   compile_info info;
   Word p, *vp;
@@ -610,7 +608,7 @@ PL_record_external(term_t t, unsigned int *len)
     addInt64(&info, v);
 
   ret_primitive:
-    scode = sizeOfBuffer(&info.code);
+    scode = (int)sizeOfBuffer(&info.code);
     rec = allocHeap(scode);
     memcpy(rec, info.code.base, scode);
     discardBuffer(&info.code);
@@ -634,7 +632,7 @@ PL_record_external(term_t t, unsigned int *len)
   while(--n >= 0)
     setVar(**vp++);
   discardBuffer(&info.vars);
-  scode = sizeOfBuffer(&info.code);
+  scode = (int)sizeOfBuffer(&info.code);
   if ( info.nvars == 0 )
     first |= REC_GROUND;
 
@@ -648,7 +646,7 @@ PL_record_external(term_t t, unsigned int *len)
   addUintBuffer((Buffer)&hdr, info.size);	/* size on stack */
   if ( info.nvars > 0 )
     addUintBuffer((Buffer)&hdr, info.nvars);	/* Number of variables */
-  shdr = sizeOfBuffer(&hdr);
+  shdr = (int)sizeOfBuffer(&hdr);
   
   rec = allocHeap(shdr + scode);
   memcpy(rec, hdr.base, shdr);
@@ -813,12 +811,12 @@ fetchChars(CopyInfo b, unsigned len, Word to)
 
 static void
 copy_record(Word p, CopyInfo b ARG_LD)
-{ long tag;
+{ intptr_t tag;
 
 right_recursion:
   switch( (tag = fetchOpCode(b)) )
   { case PL_TYPE_VARIABLE:
-    { long n = fetchSizeInt(b);
+    { intptr_t n = fetchSizeInt(b);
 
       if ( b->vars[n] )
       { if ( p > b->vars[n] )		/* ensure the reference is in the */
@@ -843,7 +841,7 @@ right_recursion:
     }
 #if O_ATTVAR
     case PL_TYPE_ATTVAR:
-    { long n = fetchSizeInt(b);
+    { intptr_t n = fetchSizeInt(b);
 
       *p = consPtr(b->gstore, TAG_ATTVAR|STG_GLOBAL);
       b->vars[n] = p;
@@ -869,7 +867,7 @@ right_recursion:
       return;
     }
     case PL_TYPE_INTEGER:
-    { int i;
+    { size_t i;
       union
       { int64_t i64;
 	word    w[WORDS_PER_PLINT];
@@ -932,7 +930,7 @@ right_recursion:
     }
 #endif
   { word fdef;
-    long arity;
+    int arity;
     case PL_TYPE_COMPOUND:
 
       fdef = fetchWord(b);
@@ -949,7 +947,7 @@ right_recursion:
     case PL_TYPE_EXT_COMPOUND:
     { atom_t name;
 
-      arity = fetchSizeInt(b);
+      arity = (int)fetchSizeInt(b);
       fetchAtom(b, &name);
       fdef = lookupFunctorDef(name, arity);
 
@@ -1081,7 +1079,7 @@ right_recursion:
       goto right_recursion;
     }
     case PL_TYPE_EXT_COMPOUND:
-    { long arity = fetchSizeInt(b);
+    { intptr_t arity = fetchSizeInt(b);
 
       skipAtom(b);
       while(--arity > 0)
@@ -1148,7 +1146,7 @@ unref_cont:
     case TAG_ATOM:
       if ( storage(w) == STG_GLOBAL )
       { if ( stag == PL_TYPE_VARIABLE )
-	{ uint n = (uint)((unsigned long)(w) >> 7);
+	{ uint n = (uint)((uintptr_t)(w) >> 7);
 	  uint i = fetchSizeInt(info);
 
 	  if ( i == n )
@@ -1193,12 +1191,12 @@ unref_cont:
       fail;
     case TAG_STRING:
       if ( stag == PL_TYPE_STRING )
-      { Word f  = addressIndirect(w);
-	int n   = wsizeofInd(*f);
-	int pad = padHdr(*f);		/* see also getCharsString() */
-	uint l  = n*sizeof(word)-pad;
+      { Word f      = addressIndirect(w);
+	size_t n    = wsizeofInd(*f);
+	size_t pad  = padHdr(*f);		/* see also getCharsString() */
+	size_t l    = n*sizeof(word)-pad;
+	size_t llen = fetchSizeInt(info);
 
-	uint llen = fetchSizeInt(info);
 	if ( llen == l &&
 	     memcmp((char *)(f+1), info->data, l) == 0 )
 	{ info->data += l;
@@ -1247,7 +1245,7 @@ unref_cont:
       } else if ( stag == PL_TYPE_EXT_COMPOUND )
       { Functor f = valueTerm(w);
 	FunctorDef fd = valueFunctor(f->definition);
-	long arity = fetchSizeInt(info);
+	intptr_t arity = fetchSizeInt(info);
 	atom_t name;
 
 	if ( (unsigned)arity != fd->arity )
@@ -1291,7 +1289,7 @@ structuralEqualArg1OfRecord(term_t t, Record r ARG_LD)
   copy_info info;
   int n, rval, navars;
   Word *p;
-  long stag;
+  intptr_t stag;
 
   DEBUG(3, Sdprintf("structuralEqualArg1OfRecord() of ");
 	   PL_write_term(Serror, t, 1200, PL_WRT_ATTVAR_WRITE);
@@ -1308,7 +1306,7 @@ structuralEqualArg1OfRecord(term_t t, Record r ARG_LD)
   if ( stag == PL_TYPE_COMPOUND )
     skipBuf(&info, word);
   else if ( stag == PL_TYPE_EXT_COMPOUND )
-  { skipBuf(&info, long);		/* arity */
+  { skipBuf(&info, intptr_t);		/* arity */
     skipAtom((CopyInfo)&info);		/* name */
   } else
     assert(0);
@@ -1685,13 +1683,13 @@ pl_erase(term_t ref)
   record = ptr;
   if ( isClause(record) )
   { Clause clause = (Clause) record;
+    Definition def = getProcDefinition(clause->procedure);
   
-    if ( true(clause->procedure->definition, LOCKED) &&
-	 false(clause->procedure->definition, DYNAMIC|P_THREAD_LOCAL) )
-      PL_error("erase", 1, NULL, ERR_PERMISSION,
-	       ATOM_clause, ATOM_erase, ref);
+    if ( !true(def, DYNAMIC) )
+      return PL_error("erase", 1, NULL, ERR_PERMISSION,
+		      ATOM_clause, ATOM_erase, ref);
 
-    return retractClauseProcedure(clause->procedure, clause PASS_LD);
+    return retractClauseDefinition(def, clause PASS_LD);
   }
   
   LOCK();
