@@ -1,4 +1,4 @@
-/*  $Id: pl-setup.c,v 1.144 2004/09/28 13:44:17 jan Exp $
+/*  $Id: pl-setup.c,v 1.156 2007/01/23 14:22:00 jan Exp $
 
     Part of SWI-Prolog
 
@@ -32,7 +32,7 @@
 #endif
 
 #undef ulong
-#define ulong unsigned long
+#define ulong uintptr_t
 #undef max
 #define max(a,b) ((a) > (b) ? (a) : (b))
 
@@ -48,7 +48,7 @@ access.   Finally  it holds the code to handle signals transparently for
 foreign language code or packages with which Prolog was linked together.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static   int allocStacks(long local, long global, long trail, long argument);
+static   int allocStacks(intptr_t local, intptr_t global, intptr_t trail, intptr_t argument);
 forwards void initSignals(void);
 
 #undef I
@@ -75,6 +75,7 @@ setupProlog(void)
 
   startCritical;
   initCharTypes();
+  initForeign();
 #if HAVE_SIGNAL
   DEBUG(1, Sdprintf("Prolog Signal Handling ...\n"));
   if ( trueFeature(SIGNALS_FEATURE) )
@@ -109,6 +110,8 @@ setupProlog(void)
   initBuildIns();
   DEBUG(1, Sdprintf("Operators ...\n"));
   initOperators();
+  DEBUG(1, Sdprintf("GMP ...\n"));
+  initGMP();
   DEBUG(1, Sdprintf("Arithmetic ...\n"));
   initArith();
   DEBUG(1, Sdprintf("Tracer ...\n"));
@@ -133,7 +136,7 @@ void
 initPrologLocalData(void)
 {
 #ifdef O_LIMIT_DEPTH
-  depth_limit   = (unsigned long)DEPTH_NO_LIMIT;
+  depth_limit   = (uintptr_t)DEPTH_NO_LIMIT;
   depth_reached = 0;
 #endif
 
@@ -164,12 +167,7 @@ some day.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #if HAVE_SIGNAL
-
-#ifdef __WIN32__
-#define HAVE_SIGNALS !iswin32s()
-#else
 #define HAVE_SIGNALS 1
-#endif
 
 #define PLSIG_PREPARED 0x01		/* signal is prepared */
 #define PLSIG_THROW    0x02		/* throw signal(num, name) */
@@ -341,9 +339,13 @@ interruptHandler() in pl-trace.c which in   turn re-routes the interrupt
 to the main thread.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+#undef LD
+#define LD LOCAL_LD
+
 static void
 dispatch_signal(int sig, int sync)
-{ SigHandler sh = &GD->sig_handlers[sig];
+{ GET_LD
+  SigHandler sh = &GD->sig_handlers[sig];
   fid_t fid;
   LocalFrame lTopSave;
   int saved_current_signal;
@@ -357,6 +359,11 @@ dispatch_signal(int sig, int sync)
   }
 #endif
 
+  DEBUG(1, Sdprintf("Got signal %d in thread %d (=%d) %s\n",
+		    sig, LD->thread.info->pl_tid,
+		    pthread_self(),
+		    sync ? " (sync)" : " (async)"));
+	
   lTopSave = lTop;
   saved_current_signal = LD->current_signal;
 
@@ -438,6 +445,8 @@ dispatch_signal(int sig, int sync)
   unblockGC(PASS_LD1);
 }
 
+#undef LD
+#define LD GLOBAL_LD
 
 static void
 pl_signal_handler(int sig)
@@ -870,37 +879,37 @@ pl_on_signal(term_t sig, term_t name, term_t old, term_t new)
 		 *******************************/
 
 static void
-enforce_limit(long *size, long maxarea, const char *name)
+enforce_limit(intptr_t *size, intptr_t maxarea, const char *name)
 { if ( *size == 0 )
     *size = maxarea;
-  else if ( *size > MAXTAGGEDPTR+1 )
-  { if ( *size != PLMAXINT )		/* user demanded maximum */
+  else if ( *size > (intptr_t)(MAXTAGGEDPTR+1) )
+  { if ( *size != LONG_MAX )		/* user demanded maximum */
       Sdprintf("WARNING: Maximum stack size for %s stack is %d MB\n",
 	       name, (MAXTAGGEDPTR+1) / (1 MB));
-    *size = MAXTAGGEDPTR;
+    *size = MAXTAGGEDPTR+1;
   }
 }
 
 
 int
-initPrologStacks(long local, long global, long trail, long argument)
+initPrologStacks(intptr_t local, intptr_t global, intptr_t trail, intptr_t argument)
 { ulong maxarea;
 
   maxarea = MAXTAGGEDPTR+1;		/* MAXTAGGEDPTR = 0x..fff.. */
   if ( maxarea > 1024 MB )		/* 64-bit machines */
     maxarea = 1024 MB;
 
-  enforce_limit(&local,	   maxarea, "local");
-  enforce_limit(&global,   maxarea, "global");
-  enforce_limit(&trail,	   maxarea, "trail");
-  enforce_limit(&argument, 64 MB,   "argument");
+  enforce_limit(&local,	   maxarea,  "local");
+  enforce_limit(&global,   maxarea,  "global");
+  enforce_limit(&trail,	   maxarea,  "trail");
+  enforce_limit(&argument, global/2, "argument");
 
   if ( !allocStacks(local, global, trail, argument) )
     fail;
 
-  base_addresses[STG_LOCAL]  = (unsigned long)lBase;
-  base_addresses[STG_GLOBAL] = (unsigned long)gBase;
-  base_addresses[STG_TRAIL]  = (unsigned long)tBase;
+  base_addresses[STG_LOCAL]  = (uintptr_t)lBase;
+  base_addresses[STG_GLOBAL] = (uintptr_t)gBase;
+  base_addresses[STG_TRAIL]  = (uintptr_t)tBase;
   emptyStacks();
 
   DEBUG(1, Sdprintf("base_addresses[STG_LOCAL] = %p\n",
@@ -956,7 +965,7 @@ emptyStacks()
 #if O_DYNAMIC_STACKS
 
 static void init_stack(Stack s, char *name,
-		       caddress base, long limit, long minsize);
+		       caddress base, intptr_t limit, intptr_t minsize);
 static void gcPolicy(Stack s, int policy);
 
 #ifndef NO_SEGV_HANDLING
@@ -1006,14 +1015,14 @@ todays Unix systems or VirtualAlloc() and friends in Win32.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #include <errno.h>
-#ifndef WIN32
+#ifndef __WINDOWS__
 extern int errno;
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 static int size_alignment;	/* Stack sizes must be aligned to this */
 
-static long
-align_size(long int x)
+static intptr_t
+align_size(intptr_t x)
 { return x % size_alignment ? (x / size_alignment + 1) * size_alignment : x;
 }
 
@@ -1100,7 +1109,7 @@ get_map_fd()
 static void
 mapOrOutOf(Stack s)
 { ulong incr;
-  long  newroom;
+  intptr_t  newroom;
 
   if ( s->top > s->max )
     incr = ROUND(((ulong)s->top - (ulong)s->max), size_alignment);
@@ -1139,8 +1148,8 @@ mapOrOutOf(Stack s)
 
 #ifdef NO_SEGV_HANDLING
 void
-ensureRoomStack(Stack s, int bytes)
-{ while((char *)s->max - (char *)s->top < (int)bytes)
+ensureRoomStack(Stack s, size_t bytes)
+{ while((char *)s->max <= &((char *)s->top)[bytes])
     mapOrOutOf(s);
 }
 #endif
@@ -1149,10 +1158,10 @@ ensureRoomStack(Stack s, int bytes)
 static void
 unmap(Stack s)
 { caddress top  = (s->top > s->min ? s->top : s->min);
-  caddress addr = (caddress) align_size((long) top + size_alignment);
+  caddress addr = (caddress) align_size((intptr_t) top + size_alignment);
 
   if ( addr < s->max )
-  { long len = (char *)s->max - (char *)addr;
+  { intptr_t len = (char *)s->max - (char *)addr;
 
 #ifdef MAP_FIXED
     munmap(addr, len);
@@ -1173,17 +1182,17 @@ unmap(Stack s)
 /* mmap() version */
 
 static int
-allocStacks(long local, long global, long trail, long argument)
+allocStacks(intptr_t local, intptr_t global, intptr_t trail, intptr_t argument)
 { caddress lbase, gbase, tbase, abase;
-  long glsize;
-  long lsep, tsep;
-  long minglobal   = 4*SIZEOF_LONG K;
-  long minlocal    = 2*SIZEOF_LONG K;
-  long mintrail    = 2*SIZEOF_LONG K;
-  long minargument = 1*SIZEOF_LONG K;
+  intptr_t glsize;
+  intptr_t lsep, tsep;
+  intptr_t minglobal   = 4*SIZEOF_VOIDP K;
+  intptr_t minlocal    = 2*SIZEOF_VOIDP K;
+  intptr_t mintrail    = 2*SIZEOF_VOIDP K;
+  intptr_t minargument = 1*SIZEOF_VOIDP K;
   
   size_alignment = getpagesize();
-  while(size_alignment < 4*SIZEOF_LONG K)
+  while(size_alignment < 4*SIZEOF_VOIDP K)
     size_alignment *= 2;
 
 #ifndef HAVE_MAP_ANON
@@ -1205,10 +1214,10 @@ allocStacks(long local, long global, long trail, long argument)
   trail    = max(trail,    mintrail + STACK_SIGNAL);
   argument = max(argument, minargument + STACK_SIGNAL);
 
-  local    = (long) align_size(local);	/* Round up to page boundary */
-  global   = (long) align_size(global);
-  trail    = (long) align_size(trail);
-  argument = (long) align_size(argument);
+  local    = (intptr_t) align_size(local);	/* Round up to page boundary */
+  global   = (intptr_t) align_size(global);
+  trail    = (intptr_t) align_size(trail);
+  argument = (intptr_t) align_size(argument);
   glsize   = global+tsep+local+lsep;
 
   tbase = mmap(NULL, trail+tsep,    MAPPROTECT, MAP_FLAGS, mapfd, 0L);
@@ -1258,8 +1267,8 @@ Free stacks for the current Prolog thread
 
 void
 freeStacks(PL_local_data_t *ld)
-{ long lsep, tsep;
-  long tlen, alen, glen;
+{ intptr_t lsep, tsep;
+  intptr_t tlen, alen, glen;
 
 #ifdef NO_SEGV_HANDLING
   lsep = tsep = 0;
@@ -1303,7 +1312,7 @@ freeStacks(PL_local_data_t *ld)
 static void
 mapOrOutOf(Stack s)
 { ulong incr;
-  long  newroom;
+  intptr_t  newroom;
 
   if ( s->top > s->max )
     incr = ROUND(((ulong)s->top - (ulong)s->max), size_alignment);
@@ -1342,8 +1351,8 @@ mapOrOutOf(Stack s)
 
 #ifdef NO_SEGV_HANDLING
 void
-ensureRoomStack(Stack s, int bytes)
-{ while((char *)s->max - (char *)s->top < (int)bytes)
+ensureRoomStack(Stack s, size_t bytes)
+{ while((char *)s->max <= &((char *)s->top)[bytes])
     mapOrOutOf(s);
 }
 #endif
@@ -1351,7 +1360,7 @@ ensureRoomStack(Stack s, int bytes)
 static void
 unmap(Stack s)
 { caddress top  = (s->top > s->min ? s->top : s->min);
-  caddress addr = (caddress) align_size((long) top + size_alignment);
+  caddress addr = (caddress) align_size((intptr_t) top + size_alignment);
 
   if ( addr < s->max )
   { if ( !VirtualFree(addr, (ulong)s->max - (ulong)addr, MEM_DECOMMIT) )
@@ -1364,15 +1373,15 @@ unmap(Stack s)
 /* Windows VirtualAlloc() version */
 
 static int
-allocStacks(long local, long global, long trail, long argument)
+allocStacks(intptr_t local, intptr_t global, intptr_t trail, intptr_t argument)
 { caddress lbase, gbase, tbase, abase;
-  long glsize;
-  long lsep, tsep;
+  intptr_t glsize;
+  intptr_t lsep, tsep;
   SYSTEM_INFO info;
-  long minglobal   = 4*SIZEOF_LONG K;
-  long minlocal    = 2*SIZEOF_LONG K;
-  long mintrail    = 2*SIZEOF_LONG K;
-  long minargument = 1*SIZEOF_LONG K;
+  intptr_t minglobal   = 4*SIZEOF_VOIDP K;
+  intptr_t minlocal    = 2*SIZEOF_VOIDP K;
+  intptr_t mintrail    = 2*SIZEOF_VOIDP K;
+  intptr_t minargument = 1*SIZEOF_VOIDP K;
 
   GetSystemInfo(&info);
   size_alignment = info.dwPageSize;
@@ -1389,10 +1398,10 @@ allocStacks(long local, long global, long trail, long argument)
   trail    = max(trail,    mintrail + STACK_SIGNAL);
   argument = max(argument, minargument + STACK_SIGNAL);
 
-  local    = (long) align_size(local);	/* Round up to page boundary */
-  global   = (long) align_size(global);
-  trail    = (long) align_size(trail);
-  argument = (long) align_size(argument);
+  local    = (intptr_t) align_size(local);	/* Round up to page boundary */
+  global   = (intptr_t) align_size(global);
+  trail    = (intptr_t) align_size(trail);
+  argument = (intptr_t) align_size(argument);
   glsize   = global+tsep+local+lsep;
 
   tbase = VirtualAlloc(NULL, trail+tsep,    MEM_RESERVE, PAGE_READWRITE);
@@ -1509,7 +1518,7 @@ _PL_segv_handler(int sig)
 		    roomStack(global), roomStack(local), roomStack(trail)));
 
   for(i=0; i<N_STACKS; i++)
-  { long r = (ulong)stacka[i].max - (ulong)stacka[i].top;
+  { intptr_t r = (ulong)stacka[i].max - (ulong)stacka[i].top;
 
     if ( r < size_alignment )
     { DEBUG(1, Sdprintf("Mapped %s stack (free was %d)\n", stacka[i].name, r));
@@ -1527,7 +1536,7 @@ _PL_segv_handler(int sig)
 #endif /*NO_SEGV_HANDLING*/
 
 static void
-init_stack(Stack s, char *name, caddress base, long limit, long minsize)
+init_stack(Stack s, char *name, caddress base, intptr_t limit, intptr_t minsize)
 { s->name       = name;
   s->base       = s->max = s->top = base;
   s->limit	= addPointer(base, limit);
@@ -1587,7 +1596,7 @@ pl_trim_stacks()
 		*    SIMPLE STACK ALLOCATION    *
 		*********************************/
 
-forwards void init_stack(Stack, char *, long, long, long);
+forwards void init_stack(Stack, char *, intptr_t, intptr_t, intptr_t);
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 On systems that do not allow us to get access to the MMU (or that do not
@@ -1609,7 +1618,7 @@ word
 pl_stack_parameter(term_t name, term_t key, term_t old, term_t new)
 { atom_t a, k;
   Stack stack = NULL;
-  long *value = NULL;
+  intptr_t *value = NULL;
 
   if ( PL_get_atom(name, &a) )
   { if ( a == ATOM_local )
@@ -1631,12 +1640,16 @@ pl_stack_parameter(term_t name, term_t key, term_t old, term_t new)
   if ( !value )
     return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_stack_parameter, key);
 
-  return setLong(value, old, new);
+  if ( !PL_unify_int64(old, *value) ||
+       !PL_get_intptr_ex(new, value) )
+    fail;
+
+  succeed;
 }
 
 
 static void
-init_stack(Stack s, char *name, long size, long limit, long minfree)
+init_stack(Stack s, char *name, intptr_t size, intptr_t limit, intptr_t minfree)
 { if ( s->base == NULL )
   { fatalError("Not enough core to allocate stacks");
     return;
@@ -1656,21 +1669,21 @@ init_stack(Stack s, char *name, long size, long limit, long minfree)
 /* malloc() version */
 
 static int
-allocStacks(long local, long global, long trail, long argument)
-{ long old_heap = GD->statistics.heap;
-  long minglobal   = 25*SIZEOF_LONG K;
-  long minlocal    = 4*SIZEOF_LONG K;
-  long mintrail    = 4*SIZEOF_LONG K;
-  long minargument = 1*SIZEOF_LONG K;
+allocStacks(intptr_t local, intptr_t global, intptr_t trail, intptr_t argument)
+{ intptr_t old_heap = GD->statistics.heap;
+  intptr_t minglobal   = 25*SIZEOF_VOIDP K;
+  intptr_t minlocal    = 4*SIZEOF_VOIDP K;
+  intptr_t mintrail    = 4*SIZEOF_VOIDP K;
+  intptr_t minargument = 1*SIZEOF_VOIDP K;
 
 #if O_SHIFT_STACKS
-  long itrail  = 8*SIZEOF_LONG K;
-  long iglobal = 50*SIZEOF_LONG K;
-  long ilocal  = 8*SIZEOF_LONG K;
+  intptr_t itrail  = 8*SIZEOF_VOIDP K;
+  intptr_t iglobal = 50*SIZEOF_VOIDP K;
+  intptr_t ilocal  = 8*SIZEOF_VOIDP K;
 #else
-  long itrail  = trail;
-  long iglobal = global;
-  long ilocal  = local;
+  intptr_t itrail  = trail;
+  intptr_t iglobal = global;
+  intptr_t ilocal  = local;
 #endif
 
   local    = max(local,    minlocal);
