@@ -1,4 +1,4 @@
-/*  $Id: pl-stream.c,v 1.130 2006/02/12 15:59:30 jan Exp $
+/*  $Id: pl-stream.c,v 1.143 2007/01/23 14:22:00 jan Exp $
 
     Part of SWI-Prolog
 
@@ -22,13 +22,13 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#if defined(WIN32) && !defined(__WIN32__)
-#define __WIN32__ 1
-#endif
-
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #include <uxnt.h>
+#ifdef WIN64
+#define MD "config/win64.h"
+#else
 #define MD "config/win32.h"
+#endif
 #include <winsock2.h>
 #include "pl-mswchar.h"
 #define CRLF_MAPPING 1
@@ -114,7 +114,7 @@ typedef wchar_t pl_wchar_t;
 
 int Slinesize = SIO_LINESIZE;		/* Sgets() buffer size */
 
-static int	S__flushbuf(IOSTREAM *s);
+static ssize_t	S__flushbuf(IOSTREAM *s);
 static void	run_close_hooks(IOSTREAM *s);
 static int	S__removebuf(IOSTREAM *s);
 
@@ -294,13 +294,15 @@ Sunlock(IOSTREAM *s)
 		 *	     FLUSH/FILL		*
 		 *******************************/
 
-static int
+/* return values: -1: error, else #bytes written */
+			 
+static ssize_t
 S__flushbuf(IOSTREAM *s)
-{ int size;
+{ ssize_t size;
   char *from = s->buffer;
 
   while ( (size = s->bufp - from) > 0 )
-  { int n = (*s->functions->write)(s->handle, from, size);
+  { ssize_t n = (*s->functions->write)(s->handle, from, size);
 
     if ( n > 0 )			/* wrote some */
     { from += n;
@@ -313,14 +315,14 @@ S__flushbuf(IOSTREAM *s)
   }
 
   if ( s->bufp - from == 0 )		/* full flush */
-  { int rc = s->bufp - s->buffer;
-
+  { ssize_t rc = s->bufp - s->buffer;
+    
     s->bufp = s->buffer;
 
     return rc;
   } else				/* partial flush */
-  { int rc = from - s->buffer;
-    int left = s->bufp - from;
+  { ssize_t rc = from - s->buffer;
+    size_t left = s->bufp - from;
 
     memmove(s->buffer, from, left);
     s->bufp = s->buffer + left;
@@ -386,7 +388,7 @@ S__fillbuf(IOSTREAM *s)
       time.tv_sec  = s->timeout / 1000;
       time.tv_usec = (s->timeout % 1000) * 1000;
       FD_ZERO(&wait);
-#ifdef WIN32
+#ifdef __WINDOWS__
       FD_SET((SOCKET)fd, &wait);
 #else
       FD_SET(fd, &wait);
@@ -422,7 +424,7 @@ S__fillbuf(IOSTREAM *s)
 
   if ( s->flags & SIO_NBUF )
   { char chr;
-    int n;
+    ssize_t n;
 
     if ( (n=(*s->functions->read)(s->handle, &chr, 1)) == 1 )
     { c = char_to_int(chr);
@@ -436,7 +438,7 @@ S__fillbuf(IOSTREAM *s)
       goto error;			/* error */
     }
   } else
-  { int n, len;
+  { size_t n, len;
 
     if ( !s->buffer )
     { if ( S__setbuf(s, NULL, 0) < 0 )
@@ -496,7 +498,7 @@ update_linepos(IOSTREAM *s, int c)
       p->lineno++;
       p->linepos = 0;
       s->flags &= ~SIO_NOLINEPOS;
-#ifdef __WIN32__
+#ifdef __WINDOWS__
       if ( s->flags & O_TEXT )
 	p->charno++;			/* writes one extra! */
 #endif
@@ -689,7 +691,7 @@ Sputcode(int c, IOSTREAM *s)
       goto simple;
     case ENC_ANSI:
     { char b[MB_LEN_MAX];
-      int n;
+      size_t n;
 
       if ( !s->mbstate )
       { if ( !(s->mbstate = malloc(sizeof(*s->mbstate))) )
@@ -697,11 +699,11 @@ Sputcode(int c, IOSTREAM *s)
 	memset(s->mbstate, 0, sizeof(*s->mbstate));
       }
 
-      if ( (n = wcrtomb(b, (wchar_t)c, s->mbstate)) < 0 )
+      if ( (n = wcrtomb(b, (wchar_t)c, s->mbstate)) == (size_t)-1 )
       { if ( reperror(c, s) < 0 )
 	  return -1;
       } else
-      { int i;
+      { size_t i;
 
 	for(i=0; i<n; i++)
 	{ if ( put_byte(b[i]&0xff, s) < 0 )
@@ -783,7 +785,7 @@ Scanrepresent(int c, IOSTREAM *s)
       char b[MB_LEN_MAX];
 
       memset(&state, 0, sizeof(state));
-      if ( wcrtomb(b, (wchar_t)c, &state) >= 0 )
+      if ( wcrtomb(b, (wchar_t)c, &state) != (size_t)-1 )
 	return 0;
       return -1;
     }
@@ -826,7 +828,7 @@ retry:
     }
     case ENC_ANSI:
     { char b[1];
-      int rc, n = 0;
+      size_t rc, n = 0;
       wchar_t wc;
 
       if ( !s->mbstate )
@@ -849,7 +851,7 @@ retry:
 	if ( (rc=mbrtowc(&wc, b, 1, s->mbstate)) == 1 )
 	{ c = wc;
 	  goto out;
-	} else if ( rc == -1 )
+	} else if ( rc == (size_t)-1 )
 	{ Sseterr(s, SIO_WARN, "Illegal multibyte Sequence");
 	  goto mberr;
 	}				/* else -2: incomplete */
@@ -908,7 +910,7 @@ retry:
     case ENC_WCHAR:
     { pl_wchar_t chr;
       char *p = (char*)&chr;
-      int n;
+      size_t n;
 
       for(n=0; n<sizeof(pl_wchar_t); n++)
       { int c1 = get_byte(s);
@@ -977,7 +979,7 @@ Sungetcode(int c, IOSTREAM *s)
       goto simple;
     case ENC_ANSI:			/* (*) See above */
     { char b[MB_LEN_MAX];
-      int n;
+      size_t n;
 
       if ( !s->mbstate )		/* do we need a seperate state? */
       { if ( !(s->mbstate = malloc(sizeof(*s->mbstate))) )
@@ -985,11 +987,11 @@ Sungetcode(int c, IOSTREAM *s)
 	memset(s->mbstate, 0, sizeof(*s->mbstate));
       }
 
-      if ( (n = wcrtomb(b, (wchar_t)c, s->mbstate)) > 0  &&
-	   s->bufp - s->unbuffer >= n )
-      { int i;
+      if ( (n = wcrtomb(b, (wchar_t)c, s->mbstate)) != (size_t)-1 &&
+	   s->bufp >= n + s->unbuffer )
+      { size_t i;
 
-	for(i=n-1; i>=0; i--)
+	for(i=n; i-- > 0; )
 	{ unget_byte(b[i], s);
 	}
 
@@ -1105,9 +1107,9 @@ Sgetw(IOSTREAM *s)
 		 *	    FREAD/FWRITE	*
 		 *******************************/
 
-int
-Sfread(void *data, int size, int elms, IOSTREAM *s)
-{ int chars = size * elms;
+size_t
+Sfread(void *data, size_t size, size_t elms, IOSTREAM *s)
+{ size_t chars = size * elms;
   char *buf = data;
 
   for( ; chars > 0; chars-- )
@@ -1123,9 +1125,9 @@ Sfread(void *data, int size, int elms, IOSTREAM *s)
 }
 
 
-int
-Sfwrite(const void *data, int size, int elms, IOSTREAM *s)
-{ int chars = size * elms;
+size_t
+Sfwrite(const void *data, size_t size, size_t elms, IOSTREAM *s)
+{ size_t chars = size * elms;
   const char *buf = data;
 
   for( ; chars > 0; chars-- )
@@ -1141,10 +1143,10 @@ Sfwrite(const void *data, int size, int elms, IOSTREAM *s)
 		 *	       PENDING		*
 		 *******************************/
 
-int
-Sread_pending(IOSTREAM *s, char *buf, int limit, int flags)
+ssize_t
+Sread_pending(IOSTREAM *s, char *buf, size_t limit, int flags)
 { int done = 0;
-  int n;
+  size_t n;
 
   if ( s->bufp >= s->limitp && (flags & SIO_RP_BLOCK) )
   { int c = S__fillbuf(s);
@@ -1181,7 +1183,7 @@ update the stream encoding.  Return value is one of
 
 typedef struct
 { IOENC encoding;
-  int bomlen;
+  unsigned int bomlen;
   const char *bom;
 } bomdef;
 
@@ -1200,7 +1202,7 @@ ScheckBOM(IOSTREAM *s)
   }
 
   for(;;)
-  { int avail = s->limitp - s->bufp;
+  { size_t avail = s->limitp - s->bufp;
     const bomdef *bd;
 
     for(bd=bomdefs; bd->bomlen; bd++)
@@ -1336,6 +1338,9 @@ Sflush(IOSTREAM *s)
 { if ( s->buffer && (s->flags & SIO_OUTPUT) )
   { if ( S__flushbuf(s) < 0 )
       return -1;
+    if ( s->functions->control &&
+	 (*s->functions->control)(s->handle, SIO_FLUSHOUTPUT, NULL) < 0 )
+      return -1;
   }
 
   return 0;
@@ -1381,8 +1386,12 @@ Ssize(IOSTREAM *s)
   }
   if ( s->functions->seek )
   { long here = Stell(s);
-    long end  = Sseek(s, 0, SIO_SEEK_END);
+    long end;
 
+    if ( Sseek(s, 0, SIO_SEEK_END) == 0 )
+      end = Stell(s);
+    else
+      end = -1;
     Sseek(s, here, SIO_SEEK_SET);
 
     return end;
@@ -1401,7 +1410,7 @@ The first part checks whether  repositioning   the  read  pointer in the
 buffer suffices to achieve the seek.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int64_t
+int
 Sseek64(IOSTREAM *s, int64_t pos, int whence)
 { if ( (s->flags & SIO_INPUT) && s->limitp > s->buffer ) /* something there */
   { int64_t now = Stell64(s);
@@ -1463,19 +1472,13 @@ update:
     s->position->charno = pos;
   }
 
-  return pos;
+  return 0;
 }
 
 
-long
+int
 Sseek(IOSTREAM *s, long pos, int whence)
-{ int64_t p2 = Sseek64(s, pos, whence);
-
-  if ( p2 <= LONG_MAX )
-    return (long) p2;
-
-  errno = EINVAL;
-  return -1;
+{ return Sseek64(s, (int64_t)pos, whence);
 }
 
 
@@ -1554,7 +1557,7 @@ Sclose(IOSTREAM *s)
     free(s->mbstate);
 
   s->flags |= SIO_CLOSING;
-#ifdef __WIN32__
+#ifdef __WINDOWS__
   if ( (s->flags & SIO_ADVLOCK) )
   { OVERLAPPED ov;
     HANDLE h = (HANDLE)_get_osfhandle((int)s->handle);
@@ -1701,7 +1704,7 @@ Svprintf(const char *fm, va_list args)
 
 int
 Svfprintf(IOSTREAM *s, const char *fm, va_list args)
-{ long printed = 0;
+{ intptr_t printed = 0;
   char buf[TMPBUFSIZE];
   int tmpbuf;
 
@@ -1790,7 +1793,7 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 	  case 'u':
 	  case 'x':
 	  case 'X':
-	  { long v = 0;			/* make compiler silent */
+	  { intptr_t v = 0;			/* make compiler silent */
 	    int64_t vl = 0;
 	    char fmbuf[8], *fp=fmbuf;
 
@@ -1799,7 +1802,7 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 		v = va_arg(args, int);
 	        break;
 	      case 1:
-		v = va_arg(args, long);
+		v = va_arg(args, intptr_t);
 	        break;
 	      case 2:
 	        vl = va_arg(args, int64_t);
@@ -1816,7 +1819,7 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 	      sprintf(fs, fmbuf, v);
 	    } else
 	    {
-#ifdef WIN32
+#ifdef __WINDOWS__
 	      strcat(fp-1, "I64");	/* Synchronise with INT64_FORMAT! */
 	      fp += strlen(fp);
 #else
@@ -1880,7 +1883,7 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 	      w++;
 	    }
 	  } else /*if ( align == A_RIGHT ) */
-	  { int w;
+	  { size_t w;
 
 	    if ( fs == fbuf )
 	      w = fe - fs;
@@ -1930,7 +1933,7 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
   }
 
   SUNLOCK(s);
-  return printed;
+  return (int)printed;
 
 error:
   SUNLOCK(s);
@@ -1981,7 +1984,7 @@ Svdprintf(const char *fm, va_list args)
 
   SLOCK(s);
   rval = Svfprintf(s, fm, args);
-#if defined(_DEBUG) && defined(WIN32)
+#if defined(_DEBUG) && defined(__WINDOWS__)
   Sputc('\0', s);
   s->bufp--;				/* `Unput' */
   OutputDebugString(s->buffer);
@@ -2058,7 +2061,7 @@ Svfscanf(IOSTREAM *s, const char *fm, va_list args)
 	  c = GET(s);
 
       switch(*fm)
-      { { long v;			/* collect value here */
+      { { intptr_t v;			/* collect value here */
 	  int negative;			/* true if < 0 */
 	  int base;			/* base for conversion */
 	  int ok;			/* successful */
@@ -2101,7 +2104,7 @@ Svfscanf(IOSTREAM *s, const char *fm, va_list args)
 	      { short *vp = va_arg(args, short *);
 		*vp = v;
 	      } else if ( tsize == SZ_LONG )
-	      { long *vp = va_arg(args, long *);
+	      { intptr_t *vp = va_arg(args, intptr_t *);
 		*vp = v;
 	      } else
 	      { int *vp = va_arg(args, int *);
@@ -2145,7 +2148,7 @@ Svfscanf(IOSTREAM *s, const char *fm, va_list args)
 	    { short *vp = va_arg(args, short *);
 	      *vp = chread;
 	    } else if ( tsize == SZ_LONG )
-	    { long *vp = va_arg(args, long *);
+	    { intptr_t *vp = va_arg(args, intptr_t *);
 	      *vp = chread;
 	    } else
 	    { int *vp = va_arg(args, int *);
@@ -2281,13 +2284,18 @@ out:
 		 *	    FILE STREAMS	*
 		 *******************************/
 
-static int
-Sread_file(void *handle, char *buf, int size)
-{ long h = (long) handle;
-  int bytes;
+static ssize_t
+Sread_file(void *handle, char *buf, size_t size)
+{ intptr_t h = (intptr_t) handle;
+  ssize_t bytes;
 
   for(;;)
-  { bytes = read((int)h, buf, size);
+  {
+#ifdef __WINDOWS__
+    bytes = read((int)h, buf, (int)size);
+#else
+    bytes = read((int)h, buf, size);
+#endif
 
     if ( bytes == -1 && errno == EINTR )
     { if ( PL_handle_signals() < 0 )
@@ -2303,13 +2311,18 @@ Sread_file(void *handle, char *buf, int size)
 }
 
 
-static int
-Swrite_file(void *handle, char *buf, int size)
-{ long h = (long) handle;
-  int bytes;
+static ssize_t
+Swrite_file(void *handle, char *buf, size_t size)
+{ intptr_t h = (intptr_t) handle;
+  ssize_t bytes;
 
   for(;;)
-  { bytes = write((int)h, buf, size);
+  {
+#ifdef __WINDOWS__
+    bytes = write((int)h, buf, (int)size);
+#else
+    bytes = write((int)h, buf, size);
+#endif
 
     if ( bytes == -1 && errno == EINTR )
     { if ( PL_handle_signals() < 0 )
@@ -2327,7 +2340,7 @@ Swrite_file(void *handle, char *buf, int size)
 
 static long
 Sseek_file(void *handle, long pos, int whence)
-{ long h = (long) handle;
+{ intptr_t h = (intptr_t) handle;
 
 					/* cannot do EINTR according to man */
   return lseek((int)h, pos, whence);
@@ -2337,7 +2350,7 @@ Sseek_file(void *handle, long pos, int whence)
 #ifdef O_LARGEFILES
 static int64_t
 Sseek_file64(void *handle, int64_t pos, int whence)
-{ long h = (long) handle;
+{ intptr_t h = (intptr_t) handle;
 
 					/* cannot do EINTR according to man */
   return lseek((int)h, pos, whence);
@@ -2347,7 +2360,7 @@ Sseek_file64(void *handle, int64_t pos, int whence)
 
 static int
 Sclose_file(void *handle)
-{ long h = (long) handle;
+{ intptr_t h = (intptr_t) handle;
   int rc;
 
   do
@@ -2360,12 +2373,12 @@ Sclose_file(void *handle)
 
 static int
 Scontrol_file(void *handle, int action, void *arg)
-{ long h = (long) handle;
+{ intptr_t h = (intptr_t) handle;
   int fd = (int)h;
 
   switch(action)
   { case SIO_GETSIZE:
-    { long *rval = arg;
+    { intptr_t *rval = arg;
       struct stat buf;
 
       if ( fstat(fd, &buf) == 0 )
@@ -2375,6 +2388,7 @@ Scontrol_file(void *handle, int action, void *arg)
       return -1;
     }
     case SIO_SETENCODING:
+    case SIO_FLUSHOUTPUT:
       return 0;
     default:
       return -1;
@@ -2453,7 +2467,7 @@ Sopen_file(const char *path, const char *how)
   int oflags = O_BINARY;
   int flags = SIO_FILE|SIO_TEXT|SIO_RECORDPOS;
   int op = *how++;
-  long lfd;
+  intptr_t lfd;
   enum {lnone=0,lread,lwrite} lock = lnone;
   IOSTREAM *s;
   IOENC enc = ENC_UNKNOWN;
@@ -2529,7 +2543,7 @@ Sopen_file(const char *path, const char *how)
       return NULL;
     }
 #else					/* we don't have locking */
-#if __WIN32__
+#if __WINDOWS__
     HANDLE h = (HANDLE)_get_osfhandle(fd);
     OVERLAPPED ov;
 
@@ -2550,7 +2564,7 @@ Sopen_file(const char *path, const char *how)
 #endif
   }
 
-  lfd = (long)fd;
+  lfd = (intptr_t)fd;
   s = Snew((void *)lfd, flags, &Sfilefunctions);
   if ( enc != ENC_UNKNOWN )
     s->encoding = enc;
@@ -2564,22 +2578,28 @@ Sopen_file(const char *path, const char *how)
 IOSTREAM *
 Sfdopen(int fd, const char *type)
 { int flags;
-  long lfd;
+  intptr_t lfd;
 
   if ( fd < 0 )
+  { errno = EINVAL;
     return NULL;
+  }
+#if defined(HAVE_FCNTL) && defined(F_GETFL)
+  if ( fcntl(fd, F_GETFL) == -1 )
+    return NULL;
+#endif
 
   if ( *type == 'r' )
     flags = SIO_FILE|SIO_INPUT|SIO_RECORDPOS;
   else
     flags = SIO_FILE|SIO_OUTPUT|SIO_RECORDPOS;
 
-  lfd = (long)fd;
+  lfd = (intptr_t)fd;
 
   return Snew((void *)lfd, flags, &Sfilefunctions);
 }
 
-/* MT: as long as s is valid, this should be ok
+/* MT: as intptr_t as s is valid, this should be ok
 */
 
 int
@@ -2587,7 +2607,7 @@ Sfileno(IOSTREAM *s)
 { int n;
 
   if ( s->flags & SIO_FILE )
-  { long h = (long)s->handle;
+  { intptr_t h = (intptr_t)s->handle;
     n = (int)h;
   } else if ( s->flags & SIO_PIPE )
   { n = fileno((FILE *)s->handle);
@@ -2664,26 +2684,34 @@ S__getiob()
 		 *******************************/
 
 #ifdef HAVE_POPEN
-#ifdef WIN32
+#ifdef __WINDOWS__
 #include "popen.c"
 
 #define popen(cmd, how) pt_popen(cmd, how)
 #define pclose(fd)	pt_pclose(fd)
 #endif
 
-static int
-Sread_pipe(void *handle, char *buf, int size)
+static ssize_t
+Sread_pipe(void *handle, char *buf, size_t size)
 { FILE *fp = handle;
 
+#ifdef __WINDOWS__
+  return read(fileno(fp), buf, (unsigned int)size);
+#else
   return read(fileno(fp), buf, size);
+#endif
 }
 
 
-static int
-Swrite_pipe(void *handle, char *buf, int size)
+static ssize_t
+Swrite_pipe(void *handle, char *buf, size_t size)
 { FILE *fp = handle;
 
+#ifdef __WINDOWS__
+  return write(fileno(fp), buf, (unsigned int)size);
+#else
   return write(fileno(fp), buf, size);
+#endif
 }
 
 
@@ -2746,12 +2774,12 @@ MT: we assume these handles are not passed between threads
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 typedef struct
-{ long	here;				/* `here' location */
-  long  size;				/* size of buffer */
-  int  *sizep;				/* pointer to size */
-  long	allocated;			/* allocated size */
-  char **buffer;			/* allocated buffer */
-  int	malloced;			/* malloc() maintained */
+{ size_t	here;			/* `here' location */
+  size_t	size;			/* size of buffer */
+  size_t       *sizep;			/* pointer to size */
+  size_t	allocated;		/* allocated size */
+  char	      **buffer;			/* allocated buffer */
+  int		malloced;		/* malloc() maintained */
 } memfile;
 
 
@@ -2761,9 +2789,9 @@ Sfree(void *ptr)			/* Windows: must free from same */
 }
 
 
-static long
-S__memfile_nextsize(long needed)
-{ long size = 512;
+static size_t
+S__memfile_nextsize(size_t needed)
+{ size_t size = 512;
 
   while ( size < needed )
     size *= 2;
@@ -2772,12 +2800,12 @@ S__memfile_nextsize(long needed)
 }
 
 
-static int
-Swrite_memfile(void *handle, char *buf, int size)
+static ssize_t
+Swrite_memfile(void *handle, char *buf, size_t size)
 { memfile *mf = handle;
 
   if ( mf->here + size + 1 >= mf->allocated )
-  { long ns = S__memfile_nextsize(mf->here + size + 1);
+  { intptr_t ns = S__memfile_nextsize(mf->here + size + 1);
     char *nb;
 
     if ( mf->allocated == 0 || !mf->malloced )
@@ -2815,14 +2843,15 @@ Swrite_memfile(void *handle, char *buf, int size)
 }
 
 
-static int
-Sread_memfile(void *handle, char *buf, int size)
+static ssize_t
+Sread_memfile(void *handle, char *buf, size_t size)
 { memfile *mf = handle;
 
   if ( size + mf->here > mf->size )
-  { size = mf->size - mf->here;
-    if ( size < 0 )
+  { if ( mf->here > mf->size )
       size = 0;
+    else
+      size = mf->size - mf->here;
   }
   
   memcpy(buf, &(*mf->buffer)[mf->here], size);
@@ -2840,16 +2869,16 @@ Sseek_memfile(void *handle, long offset, int whence)
   { case SIO_SEEK_SET:
       break;
     case SIO_SEEK_CUR:
-      offset += mf->here;
+      offset += (long)mf->here;		/* Win64: truncates */
       break;
     case SIO_SEEK_END:
-      offset = mf->size - offset;
+      offset = (long)mf->size - offset;	/* Win64 */
       break;
     default:
       errno = EINVAL;
       return -1;
   }
-  if ( offset < 0 || offset > mf->size )
+  if ( offset < 0 || offset > (long)mf->size )
   { errno = EINVAL;
     return -1;
   }
@@ -2882,7 +2911,7 @@ IOFUNCTIONS Smemfunctions =
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Sopenmem(char **buffer, int *size, const char* mode)
+Sopenmem(char **buffer, size_t *sizep, const char* mode)
     Open a memory area as a stream.  Output streams will automatically
     resized using realloc() if *size = 0 or the stream is opened with mode
     "wa".
@@ -2894,7 +2923,7 @@ Sopenmem(char **buffer, int *size, const char* mode)
     { char buf[1024];			(don't allocate for small stuff)
       char *s = buf;
       IOSTREAM *fd;
-      int size = sizeof(buf);
+      size_t size = sizeof(buf);
 
       fd = Sopenmem(&s, &size, "w");
       ...
@@ -2911,10 +2940,10 @@ and other output predicates to create strings.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 IOSTREAM *
-Sopenmem(char **buffer, int *sizep, const char *mode)
+Sopenmem(char **buffer, size_t *sizep, const char *mode)
 { memfile *mf = malloc(sizeof(memfile));
   int flags = SIO_FBUF|SIO_RECORDPOS|SIO_NOMUTEX;
-  int size;
+  size_t size;
 
   if ( !mf )
   { errno = ENOMEM;
@@ -2926,7 +2955,7 @@ Sopenmem(char **buffer, int *sizep, const char *mode)
   switch(*mode)
   { case 'r':
       flags |= SIO_INPUT;
-      if ( sizep == NULL || *sizep < 0 )
+      if ( sizep == NULL || *sizep == (size_t)-1 )
 	size = (*buffer ? strlen(*buffer) : 0);
       else
 	size = *sizep;
@@ -2964,13 +2993,13 @@ Sopenmem(char **buffer, int *sizep, const char *mode)
 /* MT: we assume these handles are not passed between threads
 */
 
-static int
-Sread_string(void *handle, char *buf, int size)
+static ssize_t
+Sread_string(void *handle, char *buf, size_t size)
 { return 0;				/* signal EOF */
 }
 
-static int
-Swrite_string(void *handle, char *buf, int size)
+static ssize_t
+Swrite_string(void *handle, char *buf, size_t size)
 { errno = ENOSPC;			/* signal error */
   return -1;
 }
@@ -3000,7 +3029,7 @@ IOFUNCTIONS Sstringfunctions =
 
 
 IOSTREAM *
-Sopen_string(IOSTREAM *s, char *buf, int size, const char *mode)
+Sopen_string(IOSTREAM *s, char *buf, size_t size, const char *mode)
 { int flags = SIO_FBUF|SIO_USERBUF;
 
   if ( !s )
@@ -3022,7 +3051,7 @@ Sopen_string(IOSTREAM *s, char *buf, int size, const char *mode)
 
   switch(*mode)
   { case 'r':
-      if ( size < 0 )
+      if ( size == (size_t)-1 )
 	size = strlen(buf);
       flags |= SIO_INPUT;
       break;
