@@ -1,4 +1,4 @@
-/*  $Id: pl-thread.c,v 1.181 2006/08/16 07:15:29 jan Exp $
+/*  $Id: pl-thread.c,v 1.189 2007/01/12 21:12:29 jan Exp $
 
     Part of SWI-Prolog
 
@@ -107,7 +107,7 @@ static sem_t sem_canceled;		/* used on halt */
 #define sem_canceled_ptr (&sem_canceled)
 #endif
 
-#ifndef WIN32
+#ifndef __WINDOWS__
 #include <signal.h>
 
 #ifdef USE_SEM_OPEN
@@ -117,7 +117,7 @@ Apple Darwin (6.6) only contains the sem_init()   function as a stub. It
 only provides named semaphores  through   sem_open().  These defines and
 my_sem_open() try to hide the details of   this as much as possible from
 the rest of the code. Note  that   we  unlink  the semaphore right after
-creating it, using the common Unix trick to keep access to it as long as
+creating it, using the common Unix trick to keep access to it as intptr_t as
 we do not close it. We assume  the   OS  will close the semaphore as the
 application terminates. All this is highly   undesirable, but it will do
 for now. The USE_SEM_OPEN define  is  set   by  configure  based  on the
@@ -166,7 +166,7 @@ static sem_t sem_mark;			/* used for atom-gc */
 #define SIG_FORALL SIGHUP
 #endif
 #define SIG_RESUME SIG_FORALL
-#endif /*!WIN32*/
+#endif /*!__WINDOWS__*/
 
 
 		 /*******************************
@@ -192,7 +192,7 @@ Some remarks:
 	General-purpose mutex.  Should only be used for simple, very
 	local tasks and may not be used to lock anything significant.
 
-    WIN32
+    __WINDOWS__
 	We use native windows CRITICAL_SECTIONS for mutexes here to
 	get the best performance, notably on single-processor hardware.
 	This is selected in pl-mutex.h based on the macro
@@ -350,7 +350,6 @@ static void	free_thread_info(PL_thread_info_t *info);
 static void	set_system_thread_id(PL_thread_info_t *info);
 static int	get_message_queue(term_t t, message_queue **queue);
 static void	cleanupLocalDefinitions(PL_local_data_t *ld);
-static int	unify_thread(term_t id, PL_thread_info_t *info);
 static pl_mutex *mutexCreate(atom_t name);
 static double   ThreadCPUTime(PL_thread_info_t *info, int which);
 
@@ -450,7 +449,7 @@ static void
 free_prolog_thread(void *data)
 { PL_local_data_t *ld = data;
   PL_thread_info_t *info;
-  int acknowlege;
+  int acknowledge;
   double time;
 
   if ( !threads_ready )
@@ -461,10 +460,13 @@ free_prolog_thread(void *data)
   LOCK();
   if ( info->status == PL_THREAD_RUNNING )
     info->status = PL_THREAD_EXITED;	/* foreign pthread_exit() */
-  acknowlege = (info->status == PL_THREAD_CANCELED);
+  acknowledge = (info->status == PL_THREAD_CANCELED);
   UNLOCK();
   DEBUG(1, Sdprintf("Freeing prolog thread %d\n", info-threads));
 
+#if O_DEBUGGER
+  callEventHook(PL_EV_THREADFINISHED, info);
+#endif
   run_thread_exit_hooks();
   cleanupLocalDefinitions(ld);
   
@@ -487,6 +489,7 @@ free_prolog_thread(void *data)
   GD->statistics.thread_cputime += time;
 
   info->thread_data = NULL;
+  info->has_tid = FALSE;		/* needed? */
   ld->thread.info = NULL;		/* avoid a loop */
   UNLOCK();
 
@@ -496,7 +499,7 @@ free_prolog_thread(void *data)
   mergeAllocPool(&GD->alloc_pool, &ld->alloc_pool);
   freeHeap(ld, sizeof(*ld));
 
-  if ( acknowlege )
+  if ( acknowledge )
     sem_post(sem_canceled_ptr);
 }
 
@@ -519,15 +522,11 @@ initPrologThreads()
   TLD_set(PL_ldata, &PL_local_data);
   PL_local_data.magic = LD_MAGIC;
   info = &threads[1];
-  info->tid = pthread_self();
   info->pl_tid = 1;
   info->thread_data = &PL_local_data;
   info->status = PL_THREAD_RUNNING;
   PL_local_data.thread.info = info;
   PL_local_data.thread.magic = PL_THREAD_MAGIC;
-#ifdef WIN32
-  info->w32id = GetCurrentThreadId();
-#endif
   set_system_thread_id(info);
   init_message_queue(&PL_local_data.thread.messages);
 
@@ -598,7 +597,7 @@ exitPrologThreads()
 	      break;			/* done so */
 	  }
 
-#ifdef WIN32
+#ifdef __WINDOWS__
   	  t->thread_data->exit_requested = TRUE;
 	  t->thread_data->pending_signals |= (1L << (SIGINT-1));
 	  PostThreadMessage(t->w32id, WM_QUIT, 0, 0);
@@ -638,16 +637,17 @@ exitPrologThreads()
       Pause(0.1);
     }
   }
+
   if ( canceled )
   { printMessage(ATOM_informational,
 		 PL_FUNCTOR_CHARS, "threads_not_died", 1,
 		   PL_INT, canceled);
   } else
   { DEBUG(1, Sdprintf("done\n"));
-  }
-
-  if ( canceled == 0 )			/* safe */
+#ifndef WIN64			/* FIXME: Hangs if nothing is printed */
     sem_destroy(sem_canceled_ptr);
+#endif
+  }
 
   threads_ready = FALSE;
 }
@@ -673,7 +673,7 @@ aliasThread(int tid, atom_t name)
 		    ERR_PERMISSION, ATOM_thread, ATOM_create, obj);
   }
 
-  addHTable(threadTable, (void *)name, (void *)(long)tid);
+  addHTable(threadTable, (void *)name, (void *)(intptr_t)tid);
   PL_register_atom(name);
   threads[tid].name = name;
 
@@ -735,7 +735,7 @@ PL_thread_self()
 }
 
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PL_w32thread_raise(DWORD id, int sig)
@@ -756,7 +756,7 @@ PL_w32thread_raise(DWORD id, int sig)
   for(i = 0, info = threads; i < MAX_THREADS; i++, info++)
   { if ( info->w32id == id && info->thread_data )
     { info->thread_data->pending_signals |= (1L << (sig-1));
-#ifdef WIN32
+#ifdef __WINDOWS__
       if ( info->w32id )
 	PostThreadMessage(info->w32id, WM_SIGNALLED, 0, 0L);
 #endif
@@ -770,7 +770,7 @@ PL_w32thread_raise(DWORD id, int sig)
   return FALSE;				/* can't find thread */
 }
 
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PL_thread_raise() is used  for  re-routing   interrupts  in  the Windows
@@ -793,7 +793,7 @@ PL_thread_raise(int tid, int sig)
     goto error;
 
   info->thread_data->pending_signals |= (1L << (sig-1));
-#ifdef WIN32
+#ifdef __WINDOWS__
   if ( info->w32id )
     PostThreadMessage(info->w32id, WM_SIGNALLED, 0, 0L);
 #endif
@@ -823,7 +823,7 @@ threadName(int id)
 }
 
 
-long
+intptr_t
 system_thread_id(PL_thread_info_t *info)
 { if ( !info )
   { if ( LD )
@@ -834,10 +834,10 @@ system_thread_id(PL_thread_info_t *info)
 #ifdef __linux__
   return info->pid;
 #else
-#ifdef WIN32
+#ifdef __WINDOWS__
   return info->w32id;
 #else
-  return (long)info->tid;
+  return (intptr_t)info->tid;
 #endif
 #endif
 }
@@ -845,10 +845,12 @@ system_thread_id(PL_thread_info_t *info)
 static void
 set_system_thread_id(PL_thread_info_t *info)
 { 
+  info->tid = pthread_self();
+  info->has_tid = TRUE;
 #ifdef HAVE_GETTID
   info->pid = gettid();
 #else
-#ifdef WIN32
+#ifdef __WINDOWS__
   info->w32id = GetCurrentThreadId();
 #endif
 #endif
@@ -941,7 +943,7 @@ pl_thread_create(term_t goal, term_t id, term_t options)
   PL_local_data_t *ldnew;
   atom_t alias = NULL_ATOM;
   pthread_attr_t attr;
-  long stack = 0;
+  intptr_t stack = 0;
   int rc;
 
   if ( !(PL_is_compound(goal) || PL_is_atom(goal)) )
@@ -1031,7 +1033,7 @@ pl_thread_create(term_t goal, term_t id, term_t options)
 		    ERR_SYSCALL, "pthread_create");
   }
 
-  return unify_thread(id, info);
+  return unify_thread_id(id, info);
 }
 
 
@@ -1048,7 +1050,7 @@ get_thread(term_t t, PL_thread_info_t **info, int warn)
     { Symbol s;
 
       if ( (s = lookupHTable(threadTable, (void *)name)) )
-	i = (int)(long)s->value;
+	i = (int)(intptr_t)s->value;
     }
   }
 
@@ -1066,8 +1068,8 @@ get_thread(term_t t, PL_thread_info_t **info, int warn)
 }
 
 
-static int
-unify_thread(term_t id, PL_thread_info_t *info)
+int
+unify_thread_id(term_t id, PL_thread_info_t *info)
 { if ( info->name )
     return PL_unify_atom(id, info->name);
 
@@ -1122,7 +1124,7 @@ unify_thread_status(term_t status, PL_thread_info_t *info)
 
 word
 pl_thread_self(term_t self)
-{ return unify_thread(self, LD->thread.info);
+{ return unify_thread_id(self, LD->thread.info);
 }
 
 
@@ -1272,16 +1274,16 @@ pl_current_thread(term_t id, term_t status, control_t h)
       return unify_thread_status(status, info);
     }
     case FRG_REDO:
-      current = ForeignContextInt(h);
+      current = (int)ForeignContextInt(h);
     redo:
       for( ; current < MAX_THREADS; current++ )
       { mark m;
 
-	if ( !threads[current].tid )
+	if ( !threads[current].has_tid )
 	   continue;
 
 	Mark(m);
-	if ( unify_thread(id, &threads[current]) &&
+	if ( unify_thread_id(id, &threads[current]) &&
 	     unify_thread_status(status, &threads[current]) )
 	  ForeignRedoInt(current+1);
 	Undo(m);
@@ -1298,11 +1300,11 @@ pl_current_thread(term_t id, term_t status, control_t h)
 Sum the amount of heap allocated through all threads allocation-pools.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-long
+intptr_t
 threadLocalHeapUsed(void)
 { int i;
   PL_thread_info_t *info;
-  long heap = 0;
+  intptr_t heap = 0;
 
   LOCK();
   for(i=0, info=threads; i<MAX_THREADS; i++, info++)
@@ -1525,7 +1527,7 @@ pl_thread_signal(term_t thread, term_t goal)
   }
   ld->pending_signals |= (1L << (SIG_THREAD_SIGNAL-1));
 
-#ifdef WIN32
+#ifdef __WINDOWS__
   if ( ld->thread.info->w32id )
     PostThreadMessage(ld->thread.info->w32id, WM_SIGNALLED, 0, 0L);
 #endif
@@ -1595,7 +1597,7 @@ freeThreadSignals(PL_local_data_t *ld)
 		 *	  MESSAGE QUEUES	*
 		 *******************************/
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Earlier implementations used pthread-win32 condition   variables.  As we
 need to dispatch messages while waiting for a condition variable we need
@@ -1713,12 +1715,12 @@ win32_cond_broadcast(win32_cond_t *cv)	/* must be holding associated mutex */
 #define cv_signal    	win32_cond_signal
 #define cv_init(cv,p)	win32_cond_init(cv)
 #define cv_destroy	win32_cond_destroy
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 #define cv_broadcast	pthread_cond_broadcast
 #define cv_signal	pthread_cond_signal
 #define cv_init(cv,p)	pthread_cond_init(cv, p)
 #define cv_destroy	pthread_cond_destroy
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This code deals with telling other threads something.  The interface:
@@ -1794,14 +1796,14 @@ cleanup_get_message(void *context)
   simpleMutexUnlock(&ctx->queue->mutex);
 }
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 
 static int
 dispatch_cond_wait(message_queue *queue)
 { return win32_cond_wait(&queue->cond_var, &queue->mutex);
 }
 
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 
 #if 0
 
@@ -1870,7 +1872,7 @@ dispatch_cond_wait(message_queue *queue)
 #endif
 
 
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 static int
 get_message(message_queue *queue, term_t msg)
@@ -2133,7 +2135,7 @@ get_message_queue(term_t t, message_queue **queue)
   { Symbol s = lookupHTable(threadTable, (void *)id);
 
     if ( s )
-    { tid = (int)(long)s->value;
+    { tid = (int)(intptr_t)s->value;
       goto thread_queue;
     }
   }
@@ -2456,7 +2458,7 @@ unify_mutex(term_t t, pl_mutex *m)
 static int
 unify_mutex_owner(term_t t, int owner)
 { if ( owner )
-    return unify_thread(t, &threads[owner]);
+    return unify_thread_id(t, &threads[owner]);
   else
     return PL_unify_nil(t);
 }
@@ -2828,7 +2830,6 @@ PL_thread_attach_engine(PL_thread_attr_t *attr)
     errno = ENOMEM;
     return -1;
   }
-  info->tid = pthread_self();		/* we are complete now */
   set_system_thread_id(info);
   if ( attr && attr->alias )
   { if ( !aliasThread(info->pl_tid, PL_new_atom(attr->alias)) )
@@ -2891,10 +2892,10 @@ detach_engine(PL_engine_t e)
 #ifdef __linux__
   info->pid = -1;
 #endif
-#ifdef WIN32
+#ifdef __WINDOWS__
   info->w32id = 0;
 #endif
-  info->tid = 0L;
+  memset(&info->tid, 0, sizeof(info->tid));
 
   TLD_set(PL_ldata, NULL);
 }
@@ -2915,7 +2916,7 @@ PL_set_engine(PL_engine_t new, PL_engine_t *old)
       { UNLOCK();
 	return PL_ENGINE_INVAL;
       }
-      if ( new->thread.info->tid )
+      if ( new->thread.info->has_tid )
       { UNLOCK();
 	return PL_ENGINE_INUSE;
       }
@@ -2927,6 +2928,7 @@ PL_set_engine(PL_engine_t new, PL_engine_t *old)
     if ( new )
     { TLD_set(PL_ldata, new);
       new->thread.info->tid = pthread_self();
+      
       set_system_thread_id(new->thread.info);
     }
   
@@ -3033,7 +3035,7 @@ PRED_IMPL("thread_statistics", 3, thread_statistics, 0)
 }
 
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 
 /* How to make the memory visible?
 */
@@ -3066,7 +3068,7 @@ ThreadCPUTime(PL_thread_info_t *info, int which)
   return 0.0;
 }
 
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 
 #define timespec_to_double(ts) \
 	(double)ts.tv_sec + (double)ts.tv_nsec/1000000000ull
@@ -3226,7 +3228,7 @@ ThreadCPUTime(PL_thread_info_t *info, int which)
   if ( (e=get_procps_entry(info->pid)) )
   { char buffer[1000];
     char *s;
-    long long ticks;
+    int64_t ticks;
     int i, n, nth = 10;			/* user time */
 
     if ( which == CPU_SYSTEM )
@@ -3309,7 +3311,7 @@ ThreadCPUTime(PL_thread_info_t *info, int which)
 #endif /*LINUX_PROCFS*/
 #endif /*LINUX_CPUCLOCKS*/
 #endif /*PTHREAD_CPUCLOCKS*/
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 
 		 /*******************************
@@ -3326,7 +3328,7 @@ We set up a semaphore and  signal   all  the  other threads. Each thread
 receiving a the SIG_MARKATOMS signal calls markAtomsOnStacks() and posts
 the semaphore. The latter performs its  job with certain heuristics, but
 must ensure it doesn't  forget  any  atoms   (a  few  too  many  is ok).
-Basically this signal handler can run whenever  necessary, as long as as
+Basically this signal handler can run whenever  necessary, as intptr_t as as
 the thread is not in a GC,  which   makes  it impossible to traverse the
 stacks.
 
@@ -3343,14 +3345,14 @@ refer to new atoms by creating them, in which case the thread will block
 or by executing an instruction that refers to the atom. In this case the
 atom is locked by the instruction anyway.
 
-[WIN32]  The  windows  case  is  entirely    different  as  we  have  no
+[__WINDOWS__]  The  windows  case  is  entirely    different  as  we  have  no
 asynchronous signals. Fortunately we  can   suspend  and resume threads.
 This makes the code a lot easier as   you can see below. Problem is that
 only one processor is doing the  job,   where  atom-gc  is a distributed
 activity in the POSIX based code.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 
 void					/* For comments, see above */
 forThreadLocalData(void (*func)(PL_local_data_t *), unsigned flags)
@@ -3394,7 +3396,7 @@ resumeThreads(void)
   }
 }
 
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 
 static void (*ldata_function)(PL_local_data_t *data);
 
@@ -3599,7 +3601,7 @@ forThreadLocalData(void (*func)(PL_local_data_t *), unsigned flags)
   ldata_function = NULL;
 }
 
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 		 /*******************************
 		 *	    PREDICATES		*
@@ -3644,7 +3646,7 @@ localiseDefinition(Definition def)
        id >= def->definition.local->size )
   { int newsize = def->definition.local ? def->definition.local->size : 1;
     LocalDefinitions new;
-    int bytes;
+    size_t bytes;
     int i=0;
 
     do
@@ -3760,7 +3762,7 @@ PL_thread_destroy_engine()
 { return FALSE;
 }
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 int
 PL_w32thread_raise(DWORD id, int sig)
 { return PL_raise(sig);
