@@ -1,4 +1,4 @@
-/*  $Id: pl-thread.h,v 1.61 2004/09/21 14:22:26 jan Exp $
+/*  $Id: pl-thread.h,v 1.70 2007/01/23 09:58:02 jan Exp $
 
     Part of SWI-Prolog
 
@@ -41,7 +41,7 @@ typedef enum
   LDATA_ANSWERED
 } ldata_status_t;
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 enum
 { SIGNAL     = 0,
   BROADCAST  = 1,
@@ -56,19 +56,20 @@ typedef struct
 
 typedef struct _PL_thread_info_t
 { int		    pl_tid;		/* Prolog thread id */
-  unsigned long	    local_size;		/* Stack sizes */
-  unsigned long	    global_size;
-  unsigned long	    trail_size;
-  unsigned long	    argument_size;
+  uintptr_t	    local_size;		/* Stack sizes */
+  uintptr_t	    global_size;
+  uintptr_t	    trail_size;
+  uintptr_t	    argument_size;
   int		    (*cancel)(int id);	/* cancel function */
   int		    open_count;		/* for PL_thread_detach_engine() */
   bool		    detached;		/* detached thread */
   int		    status;		/* PL_THREAD_* */
   pthread_t	    tid;		/* Thread identifier */
+  int		    has_tid;		/* TRUE: tid = valid */
 #ifdef __linux__
   pid_t		    pid;		/* for identifying */
 #endif
-#ifdef WIN32
+#ifdef __WINDOWS__
   unsigned long	    w32id;		/* Win32 thread HANDLE */
 #endif
   struct PL_local_data  *thread_data;	/* The thread-local data  */
@@ -81,7 +82,7 @@ typedef struct _PL_thread_info_t
 
 typedef struct message_queue
 { simpleMutex	       mutex;		/* Message queue mutex */
-#ifdef WIN32
+#ifdef __WINDOWS__
   win32_cond_t	       cond_var;
 #else
   pthread_cond_t       cond_var;	/* condition variable of queue */
@@ -134,6 +135,7 @@ extern counting_mutex _PL_mutexes[];	/* Prolog mutexes */
 #define L_TERM	       15
 #define L_GC	       16
 #define L_FOREIGN      17
+#define L_OS	       18
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 The IFMT(id, g) macro  is  used  to   bypass  mutexes  if  threading  is
@@ -240,7 +242,7 @@ may wish to write:
 		 *   NATIVE THREAD-LOCAL DATA	*
 		 *******************************/
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 typedef DWORD	TLD_KEY;
 
 #define TLD_alloc(p)	(*(p) = TlsAlloc())
@@ -323,14 +325,14 @@ extern foreign_t	pl_current_mutex(term_t mutex,
 const char *		threadName(int id);
 void			executeThreadSignals(int sig);
 foreign_t		pl_attach_xterm(term_t in, term_t out);
-long			threadLocalHeapUsed(void);
+intptr_t			threadLocalHeapUsed(void);
 int			attachConsole(void);
 Definition		localiseDefinition(Definition def);
 int			PL_mutex_lock(struct pl_mutex *m);
 int			PL_mutex_unlock(struct pl_mutex *m);
 int			PL_thread_raise(int tid, int sig);
 void			cleanupThreads();
-long			system_thread_id(PL_thread_info_t *info);
+intptr_t			system_thread_id(PL_thread_info_t *info);
 
 		 /*******************************
 		 *	 GLOBAL GC SUPPORT	*
@@ -400,6 +402,8 @@ extern void		initPrologThreads(void);
 #define makeNum(n)		makeNum__LD(n PASS_LD)
 #define getInputStream(t, s)	getInputStream__LD(t, s PASS_LD)
 #define valReal(w)		valReal__LD(w PASS_LD)
+#define getCharsString(s, l)	getCharsString__LD(s, l PASS_LD)
+#define getCharsWString(s, l)	getCharsWString__LD(s, l PASS_LD)
 #define compileTermToHeap(t, f)	compileTermToHeap__LD(t, f PASS_LD)
 #define linkVal(p)		linkVal__LD(p PASS_LD)
 #ifdef O_SHIFT_STACKS
@@ -421,6 +425,7 @@ extern void		initPrologThreads(void);
 #define PL_strip_module(q, m, t) PL_strip_module__LD(q, m, t PASS_LD)
 #define PL_get_integer(t, i)	PL_get_integer__LD(t, i PASS_LD)
 #define PL_get_long(t, i)	PL_get_long__LD(t, i PASS_LD)
+#define PL_get_int64(t, i)	PL_get_int64__LD(t, i PASS_LD)
 #define PL_get_pointer(t, ptr)	PL_get_pointer__LD(t, ptr PASS_LD)
 #define PL_put_term(t1, t2)	PL_put_term__LD(t1, t2 PASS_LD)
 #define PL_get_functor(t, f)	PL_get_functor__LD(t, f PASS_LD)
