@@ -1,4 +1,4 @@
-/*  $Id: pl-trace.c,v 1.108 2006/08/07 12:36:19 jan Exp $
+/*  $Id: pl-trace.c,v 1.114 2007/01/13 10:30:37 jan Exp $
 
     Part of SWI-Prolog
 
@@ -83,7 +83,8 @@ PL_get_frame(term_t r, LocalFrame *fr)
   if ( PL_get_long(r, &i) )
   { LocalFrame f = ((LocalFrame)((Word)lBase + i));
 
-    assert(f >= lBase && f < lTop);
+    if ( !(f >= lBase && f < lTop) )
+      fail;
     *fr = f;
 
     succeed;
@@ -1019,7 +1020,7 @@ traceInterception(LocalFrame frame, Choice bfr, int port, Code PC)
     { int pcn;
 
       if ( PC && false(frame->predicate, FOREIGN) && frame->clause )
-	pcn = PC - frame->clause->clause->codes;
+	pcn = (int)(PC - frame->clause->clause->codes);
       else
 	pcn = 0;
 
@@ -1064,7 +1065,7 @@ traceInterception(LocalFrame frame, Choice bfr, int port, Code PC)
 	{ debugstatus.retryFrame = fr;
 	  rval = ACTION_RETRY;
 	} else
-	  warning("prolog_trace_interception/3: bad argument to retry/1");
+	  PL_warning("prolog_trace_interception/3: bad argument to retry/1");
       }
     }
     PL_close_query(qid);
@@ -1135,12 +1136,12 @@ hasAlternativesFrame(LocalFrame frame)
 
 
 #ifdef O_DEBUG
-static long
+static intptr_t
 loffset(void *p)
 { if ( p == NULL )
     return 0;
 
-  assert((long)p % sizeof(word) == 0);
+  assert((intptr_t)p % sizeof(word) == 0);
   return (Word)p-(Word)lBase;
 }
 
@@ -1435,7 +1436,7 @@ pl_skip_level(term_t old, term_t new)
   }
       
   if ( PL_get_long(new, &sl) )
-  { debugstatus.skiplevel = (unsigned long) sl;
+  { debugstatus.skiplevel = sl;
     succeed;
   }
   if ( PL_get_atom(new, &a) && a == ATOM_very_deep)
@@ -1666,7 +1667,7 @@ pl_prolog_frame_attribute(term_t frame, term_t what,
 	 fr->parent &&
 	 false(fr->parent->predicate, FOREIGN) &&
 	 fr->parent->predicate != PROCEDURE_dcall1->definition )
-    { long pc = fr->programPointer - fr->parent->clause->clause->codes;
+    { intptr_t pc = fr->programPointer - fr->parent->clause->clause->codes;
 
       if ( pc < 0 )
 	trap_gdb();
@@ -1694,7 +1695,7 @@ pl_prolog_frame_attribute(term_t frame, term_t what,
 #ifdef O_LIMIT_DEPTH
     QueryFrame qf = findQuery(environment_frame);
     
-    if ( qf && (unsigned long)levelFrame(fr) > qf->saved_depth_limit )
+    if ( qf && (uintptr_t)levelFrame(fr) > qf->saved_depth_limit )
       a = ATOM_true;
     else
 #endif
@@ -1816,6 +1817,17 @@ callEventHook(int ev, ...)
 		           PL_TERM, ref);
 	break;
       }
+#ifdef O_PLMT
+      case PL_EV_THREADFINISHED:
+      { PL_thread_info_t *info = va_arg(args, PL_thread_info_t*);
+	term_t id = PL_new_term_ref();
+
+	unify_thread_id(id, info);
+	PL_unify_term(arg, PL_FUNCTOR_CHARS, "thread_finished", 1,
+		             PL_TERM, id);
+	break;
+      }
+#endif
       default:
 	warning("callEventHook(): unknown event: %d", ev);
         goto out;
