@@ -1,4 +1,4 @@
-/*  $Id: pl-util.c,v 1.29 2004/09/21 14:22:26 jan Exp $
+/*  $Id: pl-util.c,v 1.38 2007/01/23 14:22:01 jan Exp $
 
     Part of SWI-Prolog
 
@@ -24,6 +24,7 @@
 
 #include "pl-incl.h"
 #include "pl-ctype.h"
+#include "pl-utf8.h"
 
 static bool	isUserSystemPredicate(Definition def);
 
@@ -72,7 +73,9 @@ digitValue(int b, int c)
 These  functions  return  a  user-printable  name   of  a  predicate  as
 name/arity or module:name/arity. The result  is   stored  in the foreign
 buffer ring, so we are thread-safe, but   the  result needs to be copied
-before the ring is exhausted. See buffer_string() for details.
+before the ring is exhausted. See buffer_string() for details.  For wide
+character versions, we use UTF-8 encoding.  This isn't very elegant, but
+these functions are for debugging only.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 char *
@@ -81,44 +84,69 @@ procedureName(Procedure proc)
 }
 
 
+#define fetch_text(s, i) \
+	((s)->encoding == ENC_ISO_LATIN_1 ? (s)->text.t[i]&0xff \
+					  : (s)->text.w[i])
+
 const char *
 atom_summary(atom_t name, unsigned int maxlen)
-{ unsigned int len;
-  const char *nm = PL_atom_nchars(name, &len);
+{ PL_chars_t txt;
+  Buffer b;
+  size_t i;
 
-  if ( maxlen < 10 )
-    maxlen = 10;
+  if ( !get_atom_text(name, &txt) )
+    return NULL;
 
-  if ( len > maxlen )
-  { char tmp[256];
+  if ( txt.encoding == ENC_ISO_LATIN_1 && txt.length < maxlen )
+  { const unsigned char *s = (const unsigned char*) txt.text.t;
+    const unsigned char *e = &s[txt.length];
 
-    if ( maxlen > 255 )
-      maxlen = 255;
+    for( ; s<e; s++ )
+    { if ( *s >= 0x80 )
+	break;
+    }
+    if ( s == e )
+      return txt.text.t;
+  }
 
-    memcpy(tmp, nm, maxlen-6);
-    strcpy(&tmp[maxlen-6], "...");
-    memcpy(&tmp[maxlen-3], &nm[len-3], 3);
-    tmp[maxlen] = EOS;
+  b = findBuffer(BUF_RING);
+  for(i=0; i<txt.length; i++)
+  { char buf[6];
+    char *e;
 
-    return (const char *)buffer_string(tmp, BUF_RING);
-  } else
-    return nm;				/* non-printable characters? */
+    e = utf8_put_char(buf, fetch_text(&txt, i));
+    addMultipleBuffer(b, buf, e-buf, char);
+    if ( i == maxlen - 6 )
+    { addMultipleBuffer(b, "...", 3, char);
+      i = txt.length - 4;
+      maxlen = 0;			/* make sure not to trap again */
+    }
+  }
+  addBuffer(b, 0, char);
+
+  return baseBuffer(b, char);
 }
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+predicateName() returns an UTF-8  representation  of   the  name  of the
+predicate. Note that we need for the buffer 6*max summary length,
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 char *
 predicateName(Definition def)
-{ char tmp[256];
+{ char tmp[650];
+  char *e = tmp;
 
-  if ( def->module == MODULE_user || isUserSystemPredicate(def) )
-    Ssprintf(tmp, "%s/%d",
-	     atom_summary(def->functor->name, 50), 
-	     def->functor->arity);
-  else
-    Ssprintf(tmp, "%s:%s/%d",
-	     atom_summary(def->module->name, 50), 
-	     atom_summary(def->functor->name, 50), 
-	     def->functor->arity);
+  if ( def->module != MODULE_user && !isUserSystemPredicate(def) )
+  { strcpy(e, atom_summary(def->module->name, 50));
+    e += strlen(e);
+    *e++ = ':';
+  }
+  strcpy(e, atom_summary(def->functor->name, 50));
+  e += strlen(e);
+  *e++ = '/';
+  Ssprintf(e, "%d", def->functor->arity);
 
   return buffer_string(tmp, BUF_RING);
 }
@@ -163,16 +191,6 @@ setInteger(int *flag, term_t old, term_t new)
 }
 
 
-word
-setLong(long *flag, term_t old, term_t new)
-{ if ( !PL_unify_integer(old, *flag) ||
-       !PL_get_long_ex(new, flag) )
-    fail;
-
-  succeed;
-}
-
-
 		 /*******************************
 		 *	       OPTIONS		*
 		 *******************************/
@@ -189,7 +207,7 @@ Variable argument list:
 
 typedef union
 { bool *b;				/* boolean value */
-  long *l;				/* integer value */
+  long *l;				/* long value */
   int  *i;				/* integer value */
   char **s;				/* string value */
   word *a;				/* atom value */
@@ -265,7 +283,7 @@ scan_options(term_t options, int flags, atom_t optype,
 	  case OPT_LONG:
 	  { if ( !PL_get_long(val, values[n].l) )
 	    { if ( (s->type & OPT_INF) && PL_is_inf(val) )
-		*values[n].l = PLMAXINT;
+		*values[n].l = LONG_MAX;
 	      else
 		goto itemerror;
 	    }
@@ -330,7 +348,7 @@ strprefix(const char *string, const char *prefix)
 
 bool
 strpostfix(const char *string, const char *postfix)
-{ long offset = strlen(string) - strlen(postfix);
+{ intptr_t offset = strlen(string) - strlen(postfix);
 
   if ( offset < 0 )
     fail;
@@ -339,16 +357,22 @@ strpostfix(const char *string, const char *postfix)
 }
 
 
-#ifndef HAVE_STRICMP
+#ifndef HAVE_STRCASECMP
 int
-stricmp(const char *s1, const char *s2)
-{ while(*s1 && makeLower(*s1) == makeLower(*s2))
+strcasecmp(const char *s1, const char *s2)
+{
+#ifdef HAVE_STRICMP
+  return stricmp(s1, s2);
+#else
+  while(*s1 && makeLower(*s1) == makeLower(*s2))
     s1++, s2++;
   
   return makeLower(*s1) - makeLower(*s2);
+#endif
 }
 #endif
 
+
 #ifndef HAVE_STRLWR
 char *
 strlwr(char *s)
@@ -364,13 +388,96 @@ strlwr(char *s)
 
 bool
 stripostfix(const char *s, const char *e)
-{ int ls = strlen(s);
-  int le = strlen(e);
+{ size_t ls = strlen(s);
+  size_t le = strlen(e);
 
   if ( ls >= le )
-    return stricmp(&s[ls-le], e) == 0;
+    return strcasecmp(&s[ls-le], e) == 0;
 
   return FALSE;
 } 
 
 
+		 /*******************************
+		 *	MULTIBYTE STRINGS	*
+		 *******************************/
+
+typedef struct 
+{ wchar_t *wcp;
+  int	   len;
+  int	   malloced;
+} wbuf;
+
+
+#if !defined(HAVE_MBSCOLL) || !defined(HAVE_MBCASESCOLL)
+static void
+wstolower(wchar_t *w, size_t len)
+{ wchar_t *e = &w[len];
+
+  for( ; w<e; w++ )
+    *w = towlower(*w);
+}
+
+static int
+int_mbscoll(const char *s1, const char *s2, int icase)
+{ size_t l1 = strlen(s1);
+  size_t l2 = strlen(s2);
+  wchar_t *w1;
+  wchar_t *w2;
+  int ml1, ml2;
+  mbstate_t mbs;
+  int rc;
+
+  if ( l1 < 1024 && (w1 = alloca(sizeof(wchar_t)*(l1+1))) )
+  { ml1 = FALSE;
+  } else
+  { w1 = PL_malloc(sizeof(wchar_t)*(l1+1));
+    ml1 = TRUE;
+  }
+  if ( l2 < 1024 && (w2 = alloca(sizeof(wchar_t)*(l2+1))) )
+  { ml2 = FALSE;
+  } else
+  { w2 = PL_malloc(sizeof(wchar_t)*(l2+1));
+    ml2 = TRUE;
+  }
+
+  memset(&mbs, 0, sizeof(mbs));
+  if ( mbsrtowcs(w1, &s1, l1, &mbs) == (size_t)-1 )
+  { rc = -2;
+    goto out;
+  }
+  if ( mbsrtowcs(w2, &s2, l2, &mbs) == (size_t)-1 )
+  { rc = 2;
+    goto out;
+  }
+  if ( icase )
+  { wstolower(w1, l1);
+    wstolower(w2, l2);
+  }
+
+  rc = wcscoll(w1, w2);
+
+out:
+  if ( ml1 ) PL_free(w1);
+  if ( ml2 ) PL_free(w2);
+
+  return rc;
+}
+#endif
+
+
+#ifndef HAVE_MBSCOLL
+int
+mbscoll(const char *s1, const char *s2)
+{ return int_mbscoll(s1, s2, FALSE);
+}
+#endif
+
+
+#ifndef HAVE_MBSCASECOLL
+int
+mbscasecoll(const char *s1, const char *s2)
+{ return int_mbscoll(s1, s2, TRUE);
+}
+#endif
+
