@@ -1,4 +1,4 @@
-/*  $Id: pl-wic.c,v 1.132 2006/06/22 09:55:50 jan Exp $
+/*  $Id: pl-wic.c,v 1.137 2007/01/13 10:08:41 jan Exp $
 
     Part of SWI-Prolog
 
@@ -40,7 +40,7 @@ static bool	loadWicFd(IOSTREAM *);
 static bool	loadPredicate(IOSTREAM *, int skip ARG_LD);
 static bool	loadImport(IOSTREAM *, int skip ARG_LD);
 static void	saveXRBlobType(PL_blob_t *type, IOSTREAM *fd);
-static void	putString(const char *, unsigned len, IOSTREAM *);
+static void	putString(const char *, size_t len, IOSTREAM *);
 static void	putNum(int64_t, IOSTREAM *);
 static void	putReal(real, IOSTREAM *);
 static void	saveWicClause(Clause, IOSTREAM *);
@@ -263,7 +263,7 @@ popXrIdTable(ARG1_LD)
 
 
 static word
-lookupXrId(long id)
+lookupXrId(intptr_t id)
 { XrTable t = loadedXrs;
   Word array = t->table[id/SUBENTRIES];
   word value;
@@ -278,7 +278,7 @@ lookupXrId(long id)
 static void
 storeXrId(long id, word value)
 { XrTable t = loadedXrs;
-  int i = id/SUBENTRIES;
+  long i = id/SUBENTRIES;
 
   while ( i >= t->tablesize )
   { Word a = malloc(ALLOCSIZE);
@@ -360,7 +360,7 @@ getString(IOSTREAM *fd, unsigned *length)
 
 
 pl_wchar_t *
-wicGetStringUTF8(IOSTREAM *fd, unsigned *length,
+wicGetStringUTF8(IOSTREAM *fd, size_t *length,
 		 pl_wchar_t *buf, size_t bufsize)
 { size_t i, len = (size_t)wicGetNum(fd);
   IOENC oenc = fd->encoding;
@@ -393,11 +393,11 @@ static atom_t
 getAtom(IOSTREAM *fd, PL_blob_t *type ARG_LD)
 { char buf[1024];
   char *tmp, *s;
-  int len = getInt(fd);
-  int i;
+  size_t len = getInt(fd);
+  size_t i;
   atom_t a;
 
-  if ( len < (int)sizeof(buf) )
+  if ( len < sizeof(buf) )
     tmp = buf;
   else
     tmp = allocHeap(len);
@@ -451,14 +451,17 @@ getMagicString(IOSTREAM *fd, char *buf, int maxlen)
 
 static int64_t
 getInt64(IOSTREAM *fd)
-{ int64_t first = Getc(fd);
+{ int64_t first;
   int bytes, shift, b;
 
+  DEBUG(4, Sdprintf("getInt64() from %ld --> \n", Stell(fd)));
+
+  first = Getc(fd);
   if ( !(first & 0xc0) )		/* 99% of them: speed up a bit */    
   { first <<= (INT64BITSIZE-6);
     first >>= (INT64BITSIZE-6);
 
-    DEBUG(4, Sdprintf("getInt64() --> " INT64_FORMAT "\n", first));
+    DEBUG(4, Sdprintf(INT64_FORMAT "\n", first));
     return first;
   }
 
@@ -488,7 +491,7 @@ getInt64(IOSTREAM *fd)
   first <<= shift;
   first >>= shift;
   
-  DEBUG(4, Sdprintf("getInt64() --> " INT64_FORMAT "\n", first));
+  DEBUG(4, Sdprintf(INT64_FORMAT "\n", first));
   return first;
 }
 
@@ -539,7 +542,7 @@ getReal(IOSTREAM *fd)
 
 word
 getWord(IOSTREAM *s)
-{ unsigned long v;
+{ word v;
 
   v  = (Sgetc(s) & 0xff) << 24;
   v |= (Sgetc(s) & 0xff) << 16;
@@ -564,7 +567,7 @@ loadXRc(int c, IOSTREAM *fd ARG_LD)
 
   switch( c )
   { case XR_REF:
-    { long xr  = getLong(fd);
+    { intptr_t xr  = getLong(fd);
       word val = lookupXrId(xr);
 
       return val;
@@ -632,7 +635,7 @@ loadXRc(int c, IOSTREAM *fd ARG_LD)
     }
     case XR_STRING_UTF8:
     { pl_wchar_t *w;
-      unsigned len;
+      size_t len;
       pl_wchar_t buf[256];
       word s;
       
@@ -926,7 +929,7 @@ loadPredicateFlags(Definition def, IOSTREAM *fd, int skip ARG_LD)
   { int	flags = getInt(fd);
 
     if ( !skip )
-    { unsigned long lflags = 0L;
+    { uintptr_t lflags = 0L;
 
       if ( flags & PRED_SYSTEM )
 	lflags |= SYSTEM;
@@ -1135,11 +1138,11 @@ qlfFixSourcePath(const char *raw)
     
   if ( load_state->has_moved && strprefix(raw, load_state->save_dir) )
   { char *s;
-    int lensave = strlen(load_state->save_dir);
+    size_t lensave = strlen(load_state->save_dir);
     const char *tail = &raw[lensave];
 
     if ( strlen(load_state->load_dir)+1+strlen(tail)+1 > MAXPATHLEN )
-      fatalError("Path name too long: %s", raw);
+      fatalError("Path name too intptr_t: %s", raw);
 
     strcpy(buf, load_state->load_dir);
     s = &buf[strlen(buf)];
@@ -1147,7 +1150,7 @@ qlfFixSourcePath(const char *raw)
     strcpy(s, tail);
   } else
   { if ( strlen(raw)+1 > MAXPATHLEN )
-      fatalError("Path name too long: %s", raw);
+      fatalError("Path name too intptr_t: %s", raw);
     strcpy(buf, raw);
   }
 
@@ -1168,7 +1171,7 @@ the module where it is a multifile one.
 static bool
 qlfLoadSource(IOSTREAM *fd)
 { char *str = getString(fd, NULL);
-  long time = getLong(fd);
+  intptr_t time = getLong(fd);
   int issys = (Qgetc(fd) == 's') ? TRUE : FALSE;
   atom_t fname;
 
@@ -1335,12 +1338,12 @@ of a predicate together.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static Table savedXRTable;		/* saved XR entries */
-static long  savedXRTableId;		/* next id */
+static intptr_t  savedXRTableId;		/* next id */
 
-#define STR_NOLEN (~(unsigned)0)
+#define STR_NOLEN ((size_t)-1)
 
 static void
-putString(const char *s, unsigned len, IOSTREAM *fd)
+putString(const char *s, size_t len, IOSTREAM *fd)
 { const char *e;
 
   if ( len == STR_NOLEN )
@@ -1356,7 +1359,7 @@ putString(const char *s, unsigned len, IOSTREAM *fd)
 
 
 static void
-putStringW(const pl_wchar_t *s, unsigned len, IOSTREAM *fd)
+putStringW(const pl_wchar_t *s, size_t len, IOSTREAM *fd)
 { const pl_wchar_t *e;
   IOENC oenc = fd->encoding;
 
@@ -1408,14 +1411,14 @@ First byte:  bits 8&7  bits 1-6 (low order)
 		3      number of bytes following
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define PLMINLONG   ((long)(-1L<<(LONGBITSIZE-1)))
+#define PLMINLONG   ((intptr_t)(-1L<<(LONGBITSIZE-1)))
 
 static void
 putNum(int64_t n, IOSTREAM *fd)
 { int m;
   int64_t absn = (n >= 0 ? n : -n);
 
-  DEBUG(8, Sdprintf("0x%x at %ld\n", (unsigned long)n, Stell(fd)));
+  DEBUG(8, Sdprintf("0x%x at %ld\n", (uintptr_t)n, Stell(fd)));
 
   if ( n != PLMININT )
   { if ( absn < (1L << 5) )
@@ -1464,7 +1467,7 @@ putReal(real f, IOSTREAM *fd)
 
 
 static void
-putLong(unsigned long v, IOSTREAM *fd)	/* always 4 bytes */
+putLong(long v, IOSTREAM *fd)		/* always 4 bytes */
 { Sputc((v>>24)&0xff, fd);
   Sputc((v>>16)&0xff, fd);
   Sputc((v>>8)&0xff, fd);
@@ -1486,10 +1489,10 @@ savedXRPointer must be used for the pointers.
 static int
 savedXR(void *xr, IOSTREAM *fd)
 { Symbol s;
-  long id;
+  intptr_t id;
 
   if ( (s = lookupHTable(savedXRTable, xr)) )
-  { id = (long) s->value;
+  { id = (intptr_t) s->value;
     Sputc(XR_REF, fd);
     putNum(id, fd);
     
@@ -1538,7 +1541,7 @@ saveXR__LD(word xr, IOSTREAM *fd ARG_LD)
   } else if ( isString(xr) )
   { char *s;
     pl_wchar_t *w;
-    unsigned len;
+    size_t len;
 
     if ( (s = getCharsString(xr, &len)) )
     { Sputc(XR_STRING, fd);
@@ -1643,7 +1646,7 @@ do_save_qlf_term(Word t, IOSTREAM *fd ARG_LD)
   { functor_t f = functorTerm(*t);
 
     if ( f == FUNCTOR_var1 )
-    { int id = valInt(argTerm(*t, 0));
+    { int id = (int)valInt(argTerm(*t, 0));
 
       Sputc('v', fd);
       putNum(id, fd);
@@ -1776,12 +1779,12 @@ saveWicClause(Clause clause, IOSTREAM *fd)
       case CA1_STRING:
       { word m = *bp;
 	char *s = (char *)++bp;
-	int wn = wsizeofInd(m);
-	int l = wn*sizeof(word) - padHdr(m);
+	size_t wn = wsizeofInd(m);
+	size_t l = wn*sizeof(word) - padHdr(m);
 	bp += wn;
 
 	putNum(l, fd);
-	while(--l >= 0)
+	while(l-- > 0)
 	  Sputc(*s++&0xff, fd);
 	n++;
 	break;
@@ -1789,7 +1792,7 @@ saveWicClause(Clause clause, IOSTREAM *fd)
 #ifdef O_GMP
       case CA1_MPZ:
       { word m = *bp++;
-	int wn = wsizeofInd(m);
+	size_t wn = wsizeofInd(m);
 	int mpsize = (int)*bp;
 	int l = abs(mpsize)*sizeof(mp_limb_t);
 	char *s = (char*)&bp[1];
@@ -1969,8 +1972,8 @@ importWic(Procedure proc, IOSTREAM *fd ARG_LD)
 typedef struct source_mark *SourceMark;
 
 struct source_mark
-{ long	   file_index;
-  SourceMark next;
+{ long		file_index;
+  SourceMark	next;
 };
 
 static SourceMark source_mark_head = NULL;
@@ -1999,7 +2002,7 @@ sourceMark(IOSTREAM *s ARG_LD)
 
 static int
 writeSourceMarks(IOSTREAM *s ARG_LD)
-{ unsigned int n = 0;
+{ long n = 0;
   SourceMark pn, pm = source_mark_head;
 
   DEBUG(1, Sdprintf("Writing source marks: "));
@@ -2025,7 +2028,7 @@ qlfSourceInfo(IOSTREAM *s, long offset, term_t list ARG_LD)
 { char *str;
   term_t head = PL_new_term_ref();
 
-  if ( Sseek(s, offset, SIO_SEEK_SET) != offset )
+  if ( Sseek(s, offset, SIO_SEEK_SET) != 0 )
     return warning("%s: seek failed: %s", wicFile, OsError());
   if ( Getc(s) != 'F' || !(str=getString(s, NULL)) )
     return warning("QLF format error");
@@ -2072,12 +2075,12 @@ qlfInfo(const char *file,
 
   if ( Sseek(s, -4, SIO_SEEK_END) < 0 )	/* 4 bytes of putLong() */
     return warning("qlf_info/4: seek failed: %s", OsError());
-  nqlf = getWord(s);
+  nqlf = (int)getWord(s);
   DEBUG(1, Sdprintf("Found %d sources at", nqlf));
-  qlfstart = (long *)allocHeap(sizeof(long) * nqlf);
+  qlfstart = (long*)allocHeap(sizeof(long) * nqlf);
   Sseek(s, -4 * (nqlf+1), SIO_SEEK_END);
   for(i=0; i<nqlf; i++)
-  { qlfstart[i] = getWord(s);
+  { qlfstart[i] = getLong(s);
     DEBUG(1, Sdprintf(" %d", qlfstart[i]));
   }
   DEBUG(1, Sdprintf("\n"));
@@ -2094,7 +2097,7 @@ qlfInfo(const char *file,
 
 out:
   if ( qlfstart )
-    freeHeap(qlfstart, sizeof(long) * nqlf);
+    freeHeap(qlfstart, sizeof(intptr_t) * nqlf);
   if ( s )
     Sclose(s);
 
