@@ -1,4 +1,4 @@
-/*  $Id: pl-write.c,v 1.92 2006/08/28 13:32:19 jan Exp $
+/*  $Id: pl-write.c,v 1.95 2007/01/12 21:12:30 jan Exp $
 
     Part of SWI-Prolog
 
@@ -110,7 +110,7 @@ static int
 atomType(atom_t a, IOSTREAM *fd)
 { Atom atom = atomValue(a);
   char *s = atom->name;
-  int len = atom->length;
+  size_t len = atom->length;
 
   if ( len == 0 )
     return AT_QUOTE;
@@ -171,8 +171,8 @@ PutString(const char *str, IOSTREAM *s)
 
 
 static bool
-PutStringN(const char *str, unsigned int length, IOSTREAM *s)
-{ unsigned int i;
+PutStringN(const char *str, size_t length, IOSTREAM *s)
+{ size_t i;
   const unsigned char *q = (const unsigned char *)str;
 
   for(i=0; i<length; i++, q++)
@@ -215,7 +215,7 @@ PutToken(const char *s, IOSTREAM *stream)
 
 
 static bool
-PutTokenN(const char *s, unsigned int len, IOSTREAM *stream)
+PutTokenN(const char *s, size_t len, IOSTREAM *stream)
 { if ( len > 0 )
     return PutOpenToken(s[0], stream) && PutStringN(s, len, stream);
 
@@ -306,7 +306,7 @@ putQuoted(int c, int quote, int flags, IOSTREAM *stream)
 
 
 static bool
-writeQuoted(IOSTREAM *stream, const char *text, int len, int quote,
+writeQuoted(IOSTREAM *stream, const char *text, size_t len, int quote,
 	    write_options *options)
 { const unsigned char *s = (const unsigned char *)text;
 
@@ -437,7 +437,7 @@ int
 writeUCSAtom(IOSTREAM *fd, atom_t atom, int flags)
 { Atom a = atomValue(atom);
   pl_wchar_t *s = (pl_wchar_t*)a->name;
-  int len = a->length/sizeof(pl_wchar_t);
+  size_t len = a->length/sizeof(pl_wchar_t);
   pl_wchar_t *e = &s[len];
 
   if ( flags & PL_WRT_QUOTED )
@@ -540,7 +540,7 @@ such that it can be read as a float. This means using the conventions of
 the C locale and if the float happens to be integer as <int>.0.
 
 Switching the locale is no option as  locale handling is not thread-safe
-and may have unwanted  consequences  for   embedding.  There  is  a long
+and may have unwanted  consequences  for   embedding.  There  is  a intptr_t
 discussion on the very same topic on  the Python mailinglist. Many hacks
 are proposed, none is very satisfactory.   Richard  O'Keefe suggested to
 use ecvt(), fcvt() and gcvt(). These  are   not  thread-safe.  The GNU C
@@ -1025,6 +1025,7 @@ static const opt_spec write_term_options[] =
   { ATOM_module,	    OPT_ATOM },
   { ATOM_backquoted_string, OPT_BOOL },
   { ATOM_attributes,	    OPT_ATOM },
+  { ATOM_priority,	    OPT_INT },
   { NULL_ATOM,	     	    0 }
 };
 
@@ -1038,6 +1039,7 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   bool charescape = -1;			/* not set */
   atom_t mname    = ATOM_user;
   atom_t attr     = ATOM_nil;
+  int  priority   = 1200;
   IOSTREAM *s;
   write_options options;
 
@@ -1046,7 +1048,7 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   if ( !scan_options(opts, 0, ATOM_write_option, write_term_options,
 		     &quoted, &ignore_ops, &numbervars, &portray,
 		     &charescape, &options.max_depth, &mname,
-		     &bqstring, &attr) )
+		     &bqstring, &attr, &priority) )
     fail;
 
   if ( attr == ATOM_nil )
@@ -1059,7 +1061,12 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
     
     options.flags |= mask;
   }
+  if ( priority < 0 || priority > OP_MAXPRIORITY )
+  { term_t t = PL_new_term_ref();
+    PL_put_integer(t, priority);
 
+    return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_operator_priority, t);
+  }
   if ( !getOutputStream(stream, &s) )
     fail;
   
@@ -1080,10 +1087,10 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   PutOpenToken(EOF, s);			/* reset this */
   if ( (options.flags & PL_WRT_QUOTED) && !(s->flags&SIO_REPPL) )
   { s->flags |= SIO_REPPL;
-    writeTerm(term, 1200, &options);
+    writeTerm(term, priority, &options);
     s->flags &= ~SIO_REPPL;
   } else
-  { writeTerm(term, 1200, &options);
+  { writeTerm(term, priority, &options);
   }
 
   return streamStatus(s);
