@@ -1,4 +1,4 @@
-/*  $Id: plld.c,v 1.59 2004/09/21 14:22:26 jan Exp $
+/*  $Id: plld.c,v 1.78 2007/01/13 12:49:08 jan Exp $
 
     Part of SWI-Prolog
 
@@ -34,11 +34,25 @@ embedded application.
 
 #define UNQUOTED_PREFIX "\1"
 
-#ifdef WIN32
+#ifdef __WINDOWS__
+#if (_MSC_VER >= 1400)			/* VC8 */
+#ifdef WIN64
+#include <config/win64.h>
+#else
+#include <config/win32.h>
+#endif
+#endif
+
+#if (_MSC_VER < 1300)			/* <= VC6 */
+typedef long intptr_t;
+typedef unsigned long uintptr_t;
+#endif
+
 #include <process.h>
 #include <io.h>
-#define off_t long
-
+#if (_MSC_VER < 1400)
+#define off_t intptr_t
+#endif
 #define popen _popen
 #define pclose _pclose
 #define O_WRONLY _O_WRONLY
@@ -59,7 +73,7 @@ embedded application.
 #define LIB_PL_DEBUG "libplD.lib"
 #define EXT_OBJ "obj"
 #define OPT_DEBUG "/DEBUG"
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 #include "pl-incl.h"
 
 #ifndef PROG_PL
@@ -75,7 +89,7 @@ embedded application.
 #define SO_LDFLAGS "-shared"
 #endif
 
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -120,7 +134,7 @@ embedded application.
 #define streq(s, q)     (strcmp((s), (q)) == 0)
 #endif
 #define strprefix(s, p) (strncmp((s), (p), strlen(p)) == 0)
-#ifdef WIN32
+#ifdef __WINDOWS__
 #define strfeq(s, q)	(stricmp((s), (q)) == 0)
 #else
 #define strfeq(s, q)	streq(s, q)
@@ -304,7 +318,7 @@ strdup(const char *in)
 
 
 static char *
-strndup(const char *in, int len)
+strndup(const char *in, size_t len)
 { char *r = xmalloc(len+1);
  
   r[len] = '\0'; 
@@ -507,16 +521,16 @@ typedef struct
 
 static extdef extdefs[] =
 { { EXT_OBJ,	&ofiles },
-#ifdef WIN32
+#ifdef __WINDOWS__
   { "lib",	&libs },
 #else
-  { "a",	&ofiles },
+  { "a",	&libs },
 #endif
   { "c",	&cfiles },
   { "cpp",	&cppfiles },
   { "cxx",	&cppfiles },
   { "cc",	&cppfiles },
-#ifndef WIN32
+#ifndef __WINDOWS__
   { "C",	&cppfiles },
 #endif
   { "pl",	&plfiles },
@@ -572,7 +586,7 @@ usage()
 	  "usage: %s -help\n"
 	  "       %s [options] inputfile ...\n"
 	  "       %s -shared -o out inputfile ...\n"
-#ifdef WIN32
+#ifdef __WINDOWS__
 	  "       %s -dll -o out inputfile ...\n"
 #endif
 	  "       %s -E cppargument ...\n"
@@ -594,7 +608,7 @@ usage()
 	  "       -nostate         just relink the kernel\n"
 	  "       -shared          create target for load_foreign_library/2\n"
 	  "       -embed-shared    embed Prolog in a shared object/DLL\n"
-#ifdef WIN32
+#ifdef __WINDOWS__
 	  "       -dll             synonym for -embed-shared\n"
 #endif
 	  "       -fpic            compile small position-independent code\n"
@@ -648,14 +662,14 @@ parseOptions(int argc, char **argv)
     } else if ( streq(opt, "-g") )		/* -g */
     { appendArgList(&coptions, OPT_DEBUG);
       appendArgList(&cppoptions, OPT_DEBUG);
-#ifdef WIN32					/* MSVC DEBUG OPTIONS */
+#ifdef __WINDOWS__					/* MSVC DEBUG OPTIONS */
       appendArgList(&coptions, "/ZI");
       appendArgList(&coptions, "/Od");
       appendArgList(&cppoptions, "/ZI");
       appendArgList(&cppoptions, "/Od");
 #endif
       appendArgList(&ldoptions, OPT_DEBUG);
-#ifdef WIN32
+#ifdef __WINDOWS__
       pllib = LIB_PL_DEBUG;
 #endif
     } else if ( strprefix(opt, "-O") )		/* -O* */
@@ -671,7 +685,7 @@ parseOptions(int argc, char **argv)
     } else if ( streq(opt, "-dll") ||		/* -dll */
 		streq(opt, "-embed-shared") )   /* -embed-shared */
     { embed_shared = TRUE;
-#ifdef WIN32
+#ifdef __WINDOWS__
       appendArgList(&ldoptions, "/DLL");
 #else
 #ifdef SO_pic
@@ -794,9 +808,9 @@ parseOptions(int argc, char **argv)
     } else if ( strprefix(opt, "-L") )		/* -L<libdir> */
     { appendArgList(&libdirs, &opt[2]);
     } else if ( streq(opt, "-lccmalloc") )	/* -lccmalloc */
-    { appendArgList(&lastlibs, &opt[2]);
+    { appendArgList(&lastlibs, opt);
     } else if ( strprefix(opt, "-l") )		/* -l<lib> */
-    { appendArgList(&libs, &opt[2]);
+    { appendArgList(&libs, opt);
     }
   }
 }
@@ -860,7 +874,7 @@ fillDefaultOptions()
       ld = cxx;
   }
 
-#ifdef WIN32
+#ifdef __WINDOWS__
   if ( strcmp(LIB_PL_DEBUG,pllib) == 0 )
     ensureOption(&coptions, "/MDd");
   else ensureOption(&coptions, "/MD");
@@ -871,13 +885,13 @@ fillDefaultOptions()
 
   tmpPath(&ctmp,   "ctmp-");
   tmpPath(&pltmp,  "pltmp-");
-#if defined(__CYGWIN32__)
-/* Compile generates .exe files on CygWin */
+#if defined(__CYGWIN__)
+/* Compile generates .exe files on Cygwin */
   replaceExtension(ctmp, "exe", tmp);
   free(ctmp);
   ctmp = strdup(tmp);
 #endif
-#if defined(WIN32) || defined(__CYGWIN32__)
+#if defined(__WINDOWS__) || defined(__CYGWIN__)
 /* Saved states have the .exe extension under Windows */
   replaceExtension(pltmp, embed_shared ? "dll" : "exe", tmp);
   free(pltmp);
@@ -887,6 +901,14 @@ fillDefaultOptions()
   { fprintf(stderr, "%s: \"-o out\" required for linking shared object\n", plld);
     exit(1);
   }
+#if defined(__CYGWIN__)
+  /* Tack correct suffix to out, otherwise cat in createOutput fails. */
+  if ( out )
+  { replaceExtension(out, shared || embed_shared ? "dll" : "exe", tmp);
+    /* Dont call free(out), it could be an argv member. */
+    out = strdup(tmp);
+  }
+#endif
   defaultPath(&out, PROG_OUT);
 
   defaultProgram(&plgoal,     "$welcome");
@@ -894,7 +916,7 @@ fillDefaultOptions()
   defaultProgram(&plinitfile, "none");
   defaultProgram(&plsysinit,  "none");
 
-#ifdef WIN32
+#ifdef __WINDOWS__
   sprintf(tmp, "%s/lib", plbase);
 #else
   sprintf(tmp, "%s/lib/%s", plbase, plarch);
@@ -941,12 +963,20 @@ getPrologOptions()
 	  defaultPath(&plbase, v);
 	else if ( streq(name, "PLARCH") )
 	  defaultPath(&plarch, v);
+#ifdef __CYGWIN__
+	else if ( streq(name, "PLLIBS") )	/* Always required. */
+#else
 	else if ( streq(name, "PLLIBS") && !shared )
+#endif
 	  addOptionString(v);
 	else if ( streq(name, "PLLIB") )
 	  defaultProgram(&pllib, v);
-	else if ( streq(name, "PLLDFLAGS") && !shared )
+	else if ( streq(name, "PLLDFLAGS") )
 	  appendArgList(&ldoptions, v);
+	else if ( streq(name, "PLCFLAGS") )
+	{ appendArgList(&coptions, v);
+	  appendArgList(&cppoptions, v);
+	}
 	else if ( streq(name, "PLSOEXT") )
 	  soext = strdup(v);
 	else if ( streq(name, "PLTHREADS") && streq(v, "yes") )
@@ -960,13 +990,15 @@ getPrologOptions()
 	  continue;
 
 	if ( verbose )
-	  printf("\t\t%s=\"%s\"\n", name, v);
-      }	
+	  fprintf(stderr, "\t\t%s=\"%s\"\n", name, v);
+      }	else
+      { fprintf(stderr, "Unparsed Prolog option: %s\n", buf);
+      }
     }
 
     pclose(fd);
 
-#ifdef WIN32
+#ifdef __WINDOWS__
     sprintf(buf, "%s/bin/%s", plbase, PROG_PL);
 #else
     sprintf(buf, "%s/bin/%s/%s", plbase, plarch, PROG_PL);
@@ -1059,7 +1091,7 @@ compileFile(const char *compiler, arglist *options, const char *cfile)
     strcpy(ext, EXT_OBJ);
 
   prependArgList(args, "-c");
-#ifdef WIN32
+#ifdef __WINDOWS__
   appendArgList(args, "-DWIN32");
   appendArgList(args, "-D_WINDOWS");
 #endif
@@ -1090,7 +1122,7 @@ compileObjectFiles()
     compileFile(cxx, &cppoptions, cppfiles.list[n]);
 }
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 char *
 os_path(char *out, const char *in)
 { for(; *in; in++)
@@ -1131,14 +1163,14 @@ void
 linkBaseExecutable()
 { char *cout = out;
 
-#ifndef WIN32				/* bit of a hack ... */
+#ifndef __WINDOWS__				/* bit of a hack ... */
   if ( embed_shared )
   { linkSharedObject();
     return;
   } 
 #endif
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 { char tmp[MAXPATHLEN];
   sprintf(tmp, "/out:%s", cout);
   prependArgList(&ldoptions, tmp);
@@ -1148,19 +1180,19 @@ linkBaseExecutable()
   appendArgList(&ldoptions, pllib);		/* -lpl */
   concatArgList(&ldoptions, "", &libs);		/* libraries */
   concatArgList(&ldoptions, "", &lastlibs);	/* libraries */
-#else /*WIN32*/
+#else /*__WINDOWS__*/
   prependArgList(&ldoptions, cout);
   prependArgList(&ldoptions, "-o");		/* -o ctmp */
   concatArgList(&ldoptions, "", &ofiles);	/* object files */
   concatArgList(&ldoptions, "-L", &libdirs);    /* library directories */
   appendArgList(&ldoptions, pllib);		/* -lpl */
-  concatArgList(&ldoptions, "-l", &libs);	/* libraries */
-  concatArgList(&ldoptions, "-l", &lastlibs);	/* libraries */
+  concatArgList(&ldoptions, "", &libs);		/* libraries */
+  concatArgList(&ldoptions, "", &lastlibs);	/* libraries */
 #endif
 
   if ( !nostate )
   { 
-#ifdef WIN32
+#ifdef __WINDOWS__
     if ( !embed_shared )
     { char buf[MAXPATHLEN];
       appendArgList(&tmpfiles, replaceExtension(cout, "exp", buf));
@@ -1187,7 +1219,7 @@ linkSharedObject()
   { soout = replaceExtension(out, soext, soname);
   }
 
-#ifdef WIN32
+#ifdef __WINDOWS__
   prependArgList(&ldoptions, "/dll");
 { char tmp[MAXPATHLEN];
   sprintf(tmp, "/out:%s", soout);
@@ -1198,16 +1230,17 @@ linkSharedObject()
   appendArgList(&ldoptions, pllib);		/* libpl.lib */
   concatArgList(&ldoptions, "", &libs);		/* libraries */
   concatArgList(&ldoptions, "", &lastlibs);	/* libraries */
-#else /*WIN32*/
-#ifdef __CYGWIN32__
+#else /*__WINDOWS__*/
+#ifdef __CYGWIN__
+  prependArgList(&ldoptions, SO_LDFLAGS);
   prependArgList(&ldoptions, soout);
   prependArgList(&ldoptions, "-o");		/* -o ctmp */
   concatArgList(&ldoptions, "", &ofiles);	/* object files */
-  appendArgList(&ldoptions, "-lplimp");		/* kernel import library */
+  appendArgList(&ldoptions, pllib);		/* -lpl */
   concatArgList(&ldoptions, "-L", &libdirs);    /* library directories */
-  concatArgList(&ldoptions, "-l", &libs);	/* libraries */
-  concatArgList(&ldoptions, "-l", &lastlibs);	/* libraries */
-#else /*__CYGWIN32__*/
+  concatArgList(&ldoptions, "", &libs);		/* libraries */
+  concatArgList(&ldoptions, "", &lastlibs);	/* libraries */
+#else /*__CYGWIN__*/
 #ifdef SO_FORMAT_LDFLAGS			/* must specify output too */
   { char tmp[MAXPATHLEN];
     tmp[0] = UNQUOTED;
@@ -1221,15 +1254,18 @@ linkSharedObject()
 #endif /*SO_FORMAT_LDFLAGS*/
   concatArgList(&ldoptions, "", &ofiles);	/* object files */
   concatArgList(&ldoptions, "-L", &libdirs);    /* library directories */
+#ifndef O_SHARED_KERNEL
   if ( embed_shared )
-    appendArgList(&ldoptions, pllib);		/* -lpl */
-  concatArgList(&ldoptions, "-l", &libs);	/* libraries */
-  concatArgList(&ldoptions, "-l", &lastlibs);	/* libraries */
+#endif
+  { appendArgList(&ldoptions, pllib);		/* -lpl */
+  }
+  concatArgList(&ldoptions, "", &libs);		/* libraries */
+  concatArgList(&ldoptions, "", &lastlibs);	/* libraries */
 #ifdef __BEOS__
   appendArgList(&ldoptions, plexe);		/* last is executable */
 #endif
-#endif /*__CYGWIN32__*/
-#endif /*WIN32*/
+#endif /*__CYGWIN__*/
+#endif /*__WINDOWS__*/
 
   callprog(ld, &ldoptions);
 }
@@ -1301,7 +1337,7 @@ createSavedState()
   *e++ = ',';
   e = put_pl_option(e, "toplevel", pltoplevel);
   *e++ = ',';
-  e = put_pl_option(e, "initfile", plinitfile);
+  e = put_pl_option(e, "init_file", plinitfile);
   if ( plclass )
   { *e++ = ',';
     e = put_pl_option(e, "class", plclass);
@@ -1348,7 +1384,7 @@ copy_fd(int i, int o)
 }
 
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 void
 saveExportLib()
 { char ibuf[MAXPATHLEN];
@@ -1370,7 +1406,7 @@ saveExportLib()
     }
   }
 }
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 
 void
@@ -1379,7 +1415,7 @@ createOutput()
 
   if ( verbose )
   {
-#ifdef WIN32
+#ifdef __WINDOWS__
     printf("\tcopy /b %s+%s %s\n", out, pltmp, out);
 #else
     printf("\tcat %s >> %s\n", pltmp, out);
@@ -1461,7 +1497,9 @@ catchSignals()
 
 int
 main(int argc, char **argv)
-{ plld = argv[0];
+{ int special; 
+
+  plld = argv[0];
   
   argc--;
   argv++;
@@ -1476,16 +1514,35 @@ main(int argc, char **argv)
   putenv("PLLD=true");			/* for subprograms */
 
   verbose = FALSE;
-					/* behave as cpp */
-  if ( argc > 0 && streq(argv[0], "-E") )
+
+  if ( argc > 2 && streq(argv[0], "-pl") )
+    special = 2;
+  else
+    special = 0;
+					/* plld [-pl x] -E ...: behave as cpp */
+  if ( argc-special > 0 && streq(argv[special], "-E") )
   { arglist cppoptions;
+    int i;
 
     memset(&cppoptions, 0, sizeof(cppoptions));
-    for(argc--, argv++ ; argc > 0; argc--, argv++)
-      appendArgList(&cppoptions, argv[0]);
+    for(i=special+1 ; i < argc; i++)
+      appendArgList(&cppoptions, argv[i]);
 
     callprog(PROG_CPP, &cppoptions);
 
+    return 0;
+  }
+					  /* plld [-pl x] -v: verbose */
+  if ( argc-special == 1 && streq(argv[special], "-v") )
+  { arglist coptions;
+    int i;
+
+    memset(&coptions, 0, sizeof(coptions));
+    for(i=special; i < argc; i++)
+      appendArgList(&coptions, argv[i]);
+
+    callprog(PROG_CC, &coptions);
+
     return 0;
   }
 
