@@ -1,4 +1,4 @@
-/*  $Id: test.pl,v 1.129 2006/08/18 11:12:59 jan Exp $
+/*  $Id: test.pl,v 1.141 2007/01/14 21:02:58 jan Exp $
 
     Part of SWI-Prolog
 
@@ -302,6 +302,8 @@ arithmetic(arith-9) :-
 	4000 =:= integer(10000 * float_fractional_part(1e10 + 0.4)).
 arithmetic(arith-10) :-
 	-4000 =:= integer(10000 * float_fractional_part(-1e10 - 0.4)).
+arithmetic(float_fractional_part-1) :-
+	0 is float_fractional_part(4).
 arithmetic(arith-11) :-
 	1.0 is sin(pi/2),
 	\+ 1 is sin(pi/2).
@@ -327,6 +329,20 @@ arithmetic(truncate-1) :-
 	1 is truncate(1.9),
 	-1 is truncate(-1.1),
 	-1 is truncate(-1.9).
+:- if(current_prolog_flag(bounded, false)). 
+arithmetic(floor-2) :-
+	A is floor(9223372036854775808.000000),
+	A == 9223372036854775808.
+arithmetic(ceil-2) :-
+	A is ceil(9223372036854775808.000000),
+	A == 9223372036854775808.
+:- endif.
+arithmetic(round-2) :-
+	A is round(9223372036854775807.000000),
+	A == 9223372036854775807.
+arithmetic(integer-2) :-
+	A is integer(9223372036854775807.000000),
+	A == 9223372036854775807.
 
 
 		 /*******************************
@@ -518,6 +534,10 @@ gmp(shift-1) :-
 gmp(shift-2) :-
 	A is 1<<55, B is A<<8,
 	B =:= 9223372036854775808.
+gmp(shift-3) :-
+	var(A),
+	forall(between(1, 100, X),
+	       catch(A is 1<<(1<<X), error(resource_error(stack), _), true)).
 gmp(fac-1) :-
 	fac(25, X),
 	X == 15511210043330985984000000. % computed with bc
@@ -643,6 +663,13 @@ gmp(fmtf-1) :-
 gmp(idiv-1) :-
 	Qi is idiv(3 rdiv 2,2 rdiv 5),
 	Qi == 3.
+gmp(random) :-
+	A is random((1<<200)-((1<<200)-20)),
+	A < 20.
+gmp(length) :-
+	N is 1<<66,
+	catch(length(_L, N), Error, true),
+	Error = error(resource_error(stack), global).
 
 :- endif.
 
@@ -1075,6 +1102,10 @@ proc(current_predicate-1) :-
 	L == [0, 2].
 proc(compile_predicate-1) :-		% Bug#152
 	cp_one, !, cp_one.
+proc(erase-static) :-
+	clause(cpxx(_,_), true, Ref),
+	catch(erase(Ref), E, true),
+	nonvar(E).
 
 cp_one :-
 	assert(cp_foo(a)),
@@ -2010,6 +2041,25 @@ timeout(pipe-1) :-
 	prolog_load_context(file, File),
 	assert(testfile(File)).
 
+root(Root) :-
+	(   current_prolog_flag(windows, true)
+	->  atom_concat(L, ':/', Root),
+	    char_type(L, alpha)
+	;   Root == (/)
+	).
+
+file(canonise-1) :-
+	absolute_file_name('/foo/..', X),
+	root(X).
+file(canonise-2) :-
+	absolute_file_name('/foo/../..', X),
+	root(X).
+file(canonise-3) :-
+	absolute_file_name('/foo/../../..', X),
+	root(X).
+file(canonise-4) :-
+	absolute_file_name('/foo/../../../', X),
+	root(X).
 file(exists-1) :-
 	\+ exists_file(foobar26).
 file(exists-2) :-
