@@ -1,4 +1,4 @@
-/*  $Id: console.c,v 1.41 2005/11/21 16:09:53 jan Exp $
+/*  $Id: console.c,v 1.45 2007/01/10 12:39:51 jan Exp $
 
     Part of SWI-Prolog
 
@@ -71,8 +71,7 @@ Menus
 The current console provides a menu that can be extended from Prolog.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#ifdef _DEBUG
-#define O_DEBUG 1
+#if defined(_DEBUG) && !defined(WIN64)
 static void initHeapDebug(void);
 #include <crtdbg.h>
 #else
@@ -89,6 +88,10 @@ static void initHeapDebug(void);
 #define UNICODE_NOCHAR 0xFFFF
 #endif
 
+#if (_MSC_VER < 1400)
+typedef DWORD DWORD_PTR;
+#endif
+
 #include <stdlib.h>
 #include <io.h>
 #include <string.h>
@@ -177,7 +180,7 @@ static HICON    _rlc_hicon;		/* Global icon */
 		 *	     FUNCTIONS		*
 		 *******************************/
 
-static WINAPI	rlc_wnd_proc(HWND win, UINT msg, UINT wP, LONG lP);
+static LRESULT WINAPI rlc_wnd_proc(HWND win, UINT msg, WPARAM wP, LPARAM lP);
 
 static void	rlc_place_caret(RlcData b);
 static void	rlc_resize_pixel_units(RlcData b, int w, int h);
@@ -235,7 +238,7 @@ static HWND	emu_hwnd;		/* Emulating for this window */
 static void _rlc_create_kill_window(RlcData b);
 static DWORD WINAPI window_loop(LPVOID arg);	/* console window proc */
 
-#ifdef O_DEBUG
+#ifdef _DEBUG
 #include <stdarg.h>
 static void Dprintf(const TCHAR *fmt, ...);
 static void Dprint_lines(RlcData b, int from, int to);
@@ -350,7 +353,7 @@ rlc_window_class(HICON icon)
     wndClass.style		= CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS;
     wndClass.lpfnWndProc	= (LPVOID) rlc_wnd_proc;
     wndClass.cbClsExtra		= 0;
-    wndClass.cbWndExtra		= sizeof(long);
+    wndClass.cbWndExtra		= sizeof(intptr_t);
     wndClass.hInstance		= instance;
     if ( icon )
       wndClass.hIcon		= icon;
@@ -374,7 +377,7 @@ also available through GetCommandLine(), which  does include the command
 itself and returns LPTSTR (Unicode/ANSI). We assume the latter.
 
 Nevertheless, for backward compatibility as well  as easy to extract the
-full pathname of the  executable,  we   replace  argv[0]  with  the long
+full pathname of the  executable,  we   replace  argv[0]  with  the intptr_t
 filename version of the current module, so argv[0] is guaranteed to be a
 full path refering to the .exe file.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -536,7 +539,7 @@ rlc_progbase(TCHAR *path, TCHAR *base)
 		 *	  HIDDEN WINDOW		*
 		 *******************************/
 
-static WINAPI
+static LRESULT WINAPI
 rlc_kill_wnd_proc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 { switch(message)
   { case WM_DESTROY:
@@ -604,7 +607,7 @@ reg_save_int(HKEY key, const TCHAR *name, int value)
 static void
 reg_save_str(HKEY key, const TCHAR *name, TCHAR *value)
 { if ( RegSetValueEx(key, name, 0, REG_SZ,
-		     (LPBYTE)value, (_tcslen(value)+1)*sizeof(TCHAR)) != ERROR_SUCCESS )
+		     (LPBYTE)value, (DWORD)(_tcslen(value)+1)*sizeof(TCHAR)) != ERROR_SUCCESS )
     DEBUG(MessageBox(NULL, _T("Failed to save string setting"), _T("Error"), MB_OK));
 }
 
@@ -878,7 +881,7 @@ rlc_color(rlc_console con, int which, COLORREF c)
 
 static int
 rlc_kill(RlcData b)
-{ DWORD result;
+{ DWORD_PTR result;
 
   switch(b->closing++)
   { case 0:
@@ -959,8 +962,8 @@ IsDownKey(code)
 }
 
 
-static WINAPI
-rlc_wnd_proc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
+static LRESULT WINAPI
+rlc_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 { RlcData b = (RlcData) GetWindowLong(hwnd, GWL_DATA);
 
   switch(message)
@@ -1088,11 +1091,11 @@ rlc_wnd_proc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
       break;
     }
 	case WM_UNICHAR:
-	  chr = wParam;
+	  chr = (int)wParam;
 	  typed_char(b, chr);
 	  return 0;
     case WM_SYSCHAR:	typed_char(b, ESC); /* Play escape-something */
-    case WM_CHAR:	chr = wParam;
+    case WM_CHAR:	chr = (int)wParam;
 
       typed_char(b, chr);
 
@@ -1707,10 +1710,10 @@ rlc_copy(RlcData b)
 { TCHAR *sel = rlc_selection(b);
 
   if ( sel && b->window )
-  { int size = _tcslen(sel);
+  { size_t size = _tcslen(sel);
     HGLOBAL mem = GlobalAlloc(GMEM_MOVEABLE, (size + 1)*sizeof(TCHAR));
     TCHAR far *data;
-    int i;
+    size_t i;
 
     if ( !mem )
     { MessageBox(NULL, _T("Not enough memory to copy"), _T("Error"), MB_OK);
@@ -2249,7 +2252,7 @@ rlc_resize(RlcData b, int w, int h)
 	pl->text = rlc_realloc(pl->text, w * sizeof(TCHAR));
 	pl->size = w;
 	pl->adjusted = TRUE;
-	i = pl - b->lines;
+	i = (int)(pl - b->lines);
 	DEBUG(Dprint_lines(b, b->first, b->last));
       } else				/* put in next line */
       { TextLine nl;
@@ -2740,7 +2743,7 @@ rlc_get_mark(rlc_console c, RlcMark m)
 
 
 void
-rlc_goto_mark(rlc_console c, RlcMark m, const TCHAR *data, int offset)
+rlc_goto_mark(rlc_console c, RlcMark m, const TCHAR *data, size_t offset)
 { RlcData b = rlc_get_data(c);
   
   b->caret_x = m->mark_x;
@@ -3054,7 +3057,7 @@ rlc_resize_queue(RlcQueue q, int size)
       rlc_free(q->buffer);
     q->buffer = newbuf;
     q->first = 0;
-    q->last  = o-newbuf;
+    q->last  = (int)(o-newbuf);
     q->size  = size;
 
     return TRUE;
@@ -3123,10 +3126,10 @@ rlc_from_queue(RlcQueue q)
 When using UNICODE, count is in bytes!
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int
-rlc_read(rlc_console c, TCHAR *buf, unsigned int count)
+size_t
+rlc_read(rlc_console c, TCHAR *buf, size_t count)
 { RlcData d = rlc_get_data(c);
-  int give;
+  size_t give;
   MSG msg;
 
   if ( d->closing )
@@ -3226,9 +3229,9 @@ rlc_flush_output(rlc_console c)
 }
 
 
-int
-rlc_write(rlc_console c, TCHAR *buf, unsigned int count)
-{ DWORD result;
+size_t
+rlc_write(rlc_console c, TCHAR *buf, size_t count)
+{ DWORD_PTR result;
   TCHAR *e, *s;
   RlcData b = rlc_get_data(c);
 
@@ -3285,7 +3288,7 @@ free_rlc_data(RlcData b)
 rlc_close() tries to gracefully get rid of   the console thread. It does
 so by posting WM_RLC_CLOSEWIN and then waiting for a WM_RLC_READY reply.
 It waits for a maximum of  1.5  second,   which  should  be  fine as the
-console thread should not have long-lasting activities.
+console thread should not have intptr_t-lasting activities.
 
 If the timeout expires it hopes for the best. This was the old situation
 and proved to be sound on Windows-NT, but not on 95 and '98.
@@ -3371,7 +3374,12 @@ rlc_title(rlc_console c, TCHAR *title, TCHAR *old, int size)
 
 void
 rlc_icon(rlc_console c, HICON icon)
-{ SetClassLong(rlc_hwnd(c), GCL_HICON, (LONG) icon);
+{
+#ifdef WIN64
+  SetClassLong(rlc_hwnd(c), GCLP_HICON, (LONG) icon);
+#else
+  SetClassLong(rlc_hwnd(c), GCL_HICON, (LONG) icon);
+#endif
 }
 
 
@@ -3490,7 +3498,7 @@ rlc_message_hook(RlcMessageHook new)
 
 
 int
-rlc_set(rlc_console c, int what, unsigned long data, RlcFreeDataHook hook)
+rlc_set(rlc_console c, int what, uintptr_t data, RlcFreeDataHook hook)
 { RlcData b = rlc_get_data(c);
 
   switch(what)
@@ -3507,7 +3515,7 @@ rlc_set(rlc_console c, int what, unsigned long data, RlcFreeDataHook hook)
 
 
 int
-rlc_get(rlc_console c, int what, unsigned long *data)
+rlc_get(rlc_console c, int what, uintptr_t *data)
 { RlcData b = (RlcData)c;
 
   if ( !b )
@@ -3515,10 +3523,10 @@ rlc_get(rlc_console c, int what, unsigned long *data)
 
   switch(what)
   { case RLC_APPLICATION_THREAD:
-      *data = (unsigned long)b->application_thread;
+      *data = (uintptr_t)b->application_thread;
       return TRUE;
     case RLC_APPLICATION_THREAD_ID:
-      *data = (unsigned long)b->application_thread_id;
+      *data = (uintptr_t)b->application_thread_id;
       return TRUE;
     default:
       if ( what >= RLC_VALUE(0) &&
@@ -3540,7 +3548,7 @@ free_user_data(RlcData b)
   { RlcFreeDataHook hook;
 
     if ( (hook=d->hook) )
-    { unsigned long data = d->data;
+    { uintptr_t data = d->data;
       d->hook = NULL;
       d->data = 0L;
       (*hook)(data);
@@ -3561,7 +3569,7 @@ noMemory()
 
 
 void *
-rlc_malloc(int size)
+rlc_malloc(size_t size)
 { void *ptr = malloc(size);
 
   if ( !ptr && size > 0 )
@@ -3575,7 +3583,7 @@ rlc_malloc(int size)
 
 
 void *
-rlc_realloc(void *ptr, int size)
+rlc_realloc(void *ptr, size_t size)
 { void *ptr2 = realloc(ptr, size);
 
   if ( !ptr2 && size > 0 )
@@ -3590,7 +3598,7 @@ rlc_free(void *ptr)
 { free(ptr);
 }
 
-#ifdef O_DEBUG
+#ifndef initHeapDebug
 
 		 /*******************************
 		 *	       DEBUG		*
@@ -3645,4 +3653,4 @@ Dprint_lines(RlcData b, int from, int to)
   }
 }
 
-#endif /*O_DEBUG*/
+#endif /*initHeapDebug*/
