@@ -1,4 +1,4 @@
-/*  $Id: menu.c,v 1.8 2004/12/01 10:43:39 jan Exp $
+/*  $Id: menu.c,v 1.10 2007/01/10 12:39:51 jan Exp $
 
     Part of SWI-Prolog
 
@@ -57,7 +57,7 @@ static struct rl_item
 static UINT
 lookupMenuLabel(const TCHAR *label)
 { int i;
-  int llen;
+  size_t llen;
   struct rl_item *builtin;
 
   for(builtin = rl_items; builtin->id; builtin++)
@@ -124,7 +124,7 @@ insertMenu(HMENU in, const TCHAR *label, const TCHAR *before)
       info.fMask |= MIIM_ID;
       info.wID = lookupMenuLabel(label);
       info.dwTypeData = (TCHAR *)label;
-      info.cch = _tcslen(label);
+      info.cch = (int)_tcslen(label);
     } else
     { info.fType = MFT_SEPARATOR;
     }
@@ -219,7 +219,7 @@ rlc_add_menu_bar(HWND cwin)
 #define MEN_MAGIC 0x6c4a58e0
 
 typedef struct menu_data
-{ long magic;				/* safety */
+{ intptr_t magic;				/* safety */
   const TCHAR *menu;			/* menu to operate on */
   const TCHAR *label;			/* new label */
   const TCHAR *before;			/* add before this one */
@@ -263,7 +263,7 @@ rlc_menu_action(rlc_console c, menu_data *data)
       info.fType = MFT_STRING;
       info.hSubMenu = CreatePopupMenu();
       info.dwTypeData = (TCHAR *)data->label;
-      info.cch = _tcslen(data->label);
+      info.cch = (int)_tcslen(data->label);
       
       InsertMenuItem(mb, bid, TRUE, &info);
 					/* force redraw; not automatic! */
