@@ -1,4 +1,4 @@
-/*  $Id: plregtry.c,v 1.5 2003/09/24 09:29:23 jan Exp $
+/*  $Id: plregtry.c,v 1.7 2007/01/13 12:49:09 jan Exp $
 
     Part of SWI-Prolog
 
@@ -517,7 +517,8 @@ foreign_t
 pl_reg_set_value(term_t h, term_t name, term_t value)
 { HKEY k;
   char *vname;
-  DWORD rval, type, len;
+  DWORD rval, type;
+  size_t len;
   BYTE *data;
 
   if ( !(k = to_key(h)) || !PL_get_atom_chars(name, &vname) )
@@ -530,9 +531,12 @@ pl_reg_set_value(term_t h, term_t name, term_t value)
       type = REG_SZ;
       break;
     case PL_STRING:
-      PL_get_string(value, &data, &len);
+    { size_t l;
+      PL_get_string(value, &data, &l);
+      len = l;
       type = REG_SZ;
       break;
+    }
     case PL_INTEGER:
     { DWORD i;
       PL_get_long(value, &i);
@@ -573,7 +577,7 @@ pl_reg_set_value(term_t h, term_t name, term_t value)
   }
 
 
-  rval = RegSetValueEx(k, vname, 0L, type, data, len);
+  rval = RegSetValueEx(k, vname, 0L, type, data, (DWORD)len);
   if ( rval == ERROR_SUCCESS )
     PL_succeed;
 
